searchData={"items":[{"type":"module","title":"ExLLM","doc":"ExLLM - Unified Elixir client library for Large Language Models.\n\nExLLM provides a consistent interface across multiple LLM providers including\nOpenAI, Anthropic Claude, Ollama, and others. It features configuration injection,\nstandardized error handling, and streaming support.","ref":"ExLLM.html"},{"type":"module","title":"Quick Start - ExLLM","doc":"# Using environment variables\n    messages = [%{role: \"user\", content: \"Hello!\"}]\n    {:ok, response} = ExLLM.chat(:anthropic, messages)\n    IO.puts(response.content)\n\n    # Using static configuration\n    config = %{anthropic: %{api_key: \"your-key\"}}\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    {:ok, response} = ExLLM.chat(:anthropic, messages, config_provider: provider)","ref":"ExLLM.html#module-quick-start"},{"type":"module","title":"Supported Providers - ExLLM","doc":"- `:anthropic` - Anthropic Claude models\n- `:openai` - OpenAI GPT models\n- `:groq` - Groq (fast inference)\n- `:lmstudio` - Local models via LM Studio (desktop app)\n- `:mistral` - Mistral AI models (European AI)\n- `:perplexity` - Perplexity AI models (search-augmented)\n- `:openrouter` - OpenRouter (300+ models from multiple providers)\n- `:ollama` - Local models via Ollama\n- `:bedrock` - AWS Bedrock (multiple providers)\n- `:gemini` - Google Gemini models\n- `:xai` - X.AI Grok models\n- `:bumblebee` - Local models via Bumblebee (Phi-2, Llama 2, Mistral, etc.)\n- `:mock` - Mock adapter for testing","ref":"ExLLM.html#module-supported-providers"},{"type":"module","title":"Features - ExLLM","doc":"- **Unified Interface**: Same API across all providers\n- **Configuration Injection**: Flexible config management\n- **Streaming Support**: Real-time response streaming with error recovery\n- **Error Standardization**: Consistent error handling\n- **Function Calling**: Unified interface for tool use across providers\n- **Model Discovery**: Query and compare model capabilities\n- **Automatic Retries**: Exponential backoff with provider-specific policies\n- **Mock Adapter**: Built-in testing support without API calls\n- **Cost Tracking**: Automatic usage and cost calculation\n- **Context Management**: Automatic message truncation for model limits\n- **Session Management**: Conversation state tracking\n- **Structured Outputs**: Schema validation via instructor integration\n- **No Process Dependencies**: Pure functional core\n- **Extensible**: Easy to add new providers","ref":"ExLLM.html#module-features"},{"type":"module","title":"Configuration - ExLLM","doc":"ExLLM supports multiple configuration methods:","ref":"ExLLM.html#module-configuration"},{"type":"module","title":"Environment Variables - ExLLM","doc":"export ANTHROPIC_API_KEY=\"api-...\"\n    export OPENAI_API_KEY=\"sk-...\"\n    export GROQ_API_KEY=\"gsk-...\"\n    export OPENROUTER_API_KEY=\"sk-or-...\"\n    export OLLAMA_API_BASE=\"http://localhost:11434\"\n    export GOOGLE_API_KEY=\"your-key\"\n    export XAI_API_KEY=\"xai-...\"\n    export AWS_ACCESS_KEY_ID=\"your-key\"\n    export AWS_SECRET_ACCESS_KEY=\"your-secret\"","ref":"ExLLM.html#module-environment-variables"},{"type":"module","title":"Static Configuration - ExLLM","doc":"config = %{\n      anthropic: %{api_key: \"api-...\", model: \"claude-3-5-sonnet-20241022\"},\n      openai: %{api_key: \"sk-...\", model: \"gpt-4\"},\n      openrouter: %{api_key: \"sk-or-...\", model: \"openai/gpt-4o\"},\n      ollama: %{base_url: \"http://localhost:11434\", model: \"llama2\"},\n      bedrock: %{access_key_id: \"...\", secret_access_key: \"...\", region: \"us-east-1\"},\n      gemini: %{api_key: \"...\", model: \"gemini-pro\"},\n      bumblebee: %{model: \"microsoft/phi-2\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)","ref":"ExLLM.html#module-static-configuration"},{"type":"module","title":"Custom Configuration - ExLLM","doc":"defmodule MyConfigProvider do\n      @behaviour ExLLM.ConfigProvider\n      \n      def get([:anthropic, :api_key]), do: MyApp.get_secret(\"anthropic_key\")\n      def get(_), do: nil\n      \n      def get_all(), do: %{}\n    end","ref":"ExLLM.html#module-custom-configuration"},{"type":"module","title":"Examples - ExLLM","doc":"# Simple chat\n    {:ok, response} = ExLLM.chat(:anthropic, [\n      %{role: \"user\", content: \"What is Elixir?\"}\n    ])\n\n    # With options\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      model: \"claude-3-haiku-20240307\",\n      temperature: 0.7,\n      max_tokens: 1000\n    )\n\n    # Streaming\n    {:ok, stream} = ExLLM.stream_chat(:anthropic, messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end\n\n    # Check if provider is configured\n    if ExLLM.configured?(:anthropic) do\n      {:ok, response} = ExLLM.chat(:anthropic, messages)\n    end\n\n    # List available models\n    {:ok, models} = ExLLM.list_models(:anthropic)\n    Enum.each(models, fn model ->\n      IO.puts(model.name)\n    end)","ref":"ExLLM.html#module-examples"},{"type":"function","title":"ExLLM.add_session_message/4","doc":"Add a message to a session.","ref":"ExLLM.html#add_session_message/4"},{"type":"function","title":"Parameters - ExLLM.add_session_message/4","doc":"- `session` - The session to update\n- `role` - Message role (\"user\", \"assistant\", etc.)\n- `content` - Message content\n- `opts` - Additional message metadata","ref":"ExLLM.html#add_session_message/4-parameters"},{"type":"function","title":"Returns - ExLLM.add_session_message/4","doc":"Updated session.","ref":"ExLLM.html#add_session_message/4-returns"},{"type":"function","title":"Examples - ExLLM.add_session_message/4","doc":"session = ExLLM.add_session_message(session, \"user\", \"What is Elixir?\")","ref":"ExLLM.html#add_session_message/4-examples"},{"type":"function","title":"ExLLM.analyze_images/4","doc":"Analyze images with a specific prompt.","ref":"ExLLM.html#analyze_images/4"},{"type":"function","title":"Examples - ExLLM.analyze_images/4","doc":"{:ok, analysis} = ExLLM.analyze_images(:anthropic,\n      [\"chart1.png\", \"chart2.png\"],\n      \"Compare these two charts and identify key differences\"\n    )","ref":"ExLLM.html#analyze_images/4-examples"},{"type":"function","title":"ExLLM.calculate_cost/3","doc":"Calculate cost for token usage.","ref":"ExLLM.html#calculate_cost/3"},{"type":"function","title":"Parameters - ExLLM.calculate_cost/3","doc":"- `provider` - LLM provider name\n- `model` - Model name\n- `token_usage` - Map with `:input_tokens` and `:output_tokens`","ref":"ExLLM.html#calculate_cost/3-parameters"},{"type":"function","title":"Returns - ExLLM.calculate_cost/3","doc":"Cost calculation result or error map.","ref":"ExLLM.html#calculate_cost/3-returns"},{"type":"function","title":"Examples - ExLLM.calculate_cost/3","doc":"usage = %{input_tokens: 1000, output_tokens: 500}\n    cost = ExLLM.calculate_cost(\"openai\", \"gpt-4\", usage)\n    # => %{total_cost: 0.06, ...}","ref":"ExLLM.html#calculate_cost/3-examples"},{"type":"function","title":"ExLLM.chat/3","doc":"Send a chat completion request to the specified LLM provider.","ref":"ExLLM.html#chat/3"},{"type":"function","title":"Parameters - ExLLM.chat/3","doc":"- `provider` - The LLM provider (`:anthropic`, `:openai`, `:groq`, etc.) or a model string like \"groq/llama3-70b\"\n- `messages` - List of conversation messages\n- `options` - Options for the request (see module docs)","ref":"ExLLM.html#chat/3-parameters"},{"type":"function","title":"Options - ExLLM.chat/3","doc":"- `:model` - Override the default model\n- `:temperature` - Temperature setting (0.0 to 1.0)\n- `:max_tokens` - Maximum tokens in response or context\n- `:config_provider` - Configuration provider module or pid\n- `:track_cost` - Whether to track costs (default: true)\n- `:strategy` - Context truncation strategy (default: :sliding_window)\n  - `:sliding_window` - Keep most recent messages\n  - `:smart` - Preserve system messages and recent context\n- `:preserve_messages` - Number of recent messages to always preserve (default: 5)\n- `:response_model` - Ecto schema or type spec for structured output (requires instructor)\n- `:max_retries` - Number of retries for structured output validation\n- `:functions` - List of available functions for function calling\n- `:function_call` - Control function calling: \"auto\", \"none\", or specific function\n- `:tools` - Alternative to functions for providers that use tools API\n- `:retry` - Enable automatic retry (default: true)\n- `:retry_count` - Number of retry attempts (default: 3)\n- `:retry_delay` - Initial retry delay in ms (default: 1000)\n- `:retry_backoff` - Backoff strategy: :exponential or :linear (default: :exponential)\n- `:retry_jitter` - Add jitter to retry delays (default: true)\n- `:stream_recovery` - Enable stream recovery (default: false)\n- `:recovery_strategy` - Recovery strategy: :exact, :paragraph, or :summarize\n- `:cache` - Enable caching for this request (default: false unless globally enabled)\n- `:cache_ttl` - Cache TTL in milliseconds (default: 15 minutes)\n- `:timeout` - Request timeout in milliseconds (provider-specific defaults)\n  - Ollama default: 120000 (2 minutes)\n  - Other providers use their client library defaults\n- Mock adapter options:\n  - `:mock_response` - Static response or response map\n  - `:mock_handler` - Function to generate dynamic responses\n  - `:mock_error` - Simulate an error\n  - `:mock_chunks` - List of chunks for streaming\n  - `:chunk_delay` - Delay between chunks in ms\n  - `:capture_requests` - Capture requests for testing","ref":"ExLLM.html#chat/3-options"},{"type":"function","title":"Returns - ExLLM.chat/3","doc":"`{:ok, %ExLLM.Types.LLMResponse{}}` on success, or `{:ok, struct}` when using\nresponse_model. Returns `{:error, reason}` on failure.","ref":"ExLLM.html#chat/3-returns"},{"type":"function","title":"Examples - ExLLM.chat/3","doc":"# Simple usage\n    {:ok, response} = ExLLM.chat(:anthropic, [\n      %{role: \"user\", content: \"Hello!\"}\n    ])\n    \n    # Using provider/model syntax\n    {:ok, response} = ExLLM.chat(\"groq/llama3-70b\", [\n      %{role: \"user\", content: \"Hello!\"}\n    ])\n    \n    # Groq with specific model\n    {:ok, response} = ExLLM.chat(:groq, messages, model: \"mixtral-8x7b-32768\")\n\n    # With custom configuration\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(%{\n      anthropic: %{api_key: \"your-key\"}\n    })\n    {:ok, response} = ExLLM.chat(:anthropic, messages, config_provider: provider)\n\n    # With model override\n    {:ok, response} = ExLLM.chat(:openai, messages, model: \"gpt-4-turbo\")\n\n    # With context management\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      max_tokens: 4000,\n      strategy: :smart\n    )\n\n    # With structured output (requires instructor)\n    {:ok, classification} = ExLLM.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 3\n    )\n    \n    # With function calling\n    functions = [\n      %{\n        name: \"get_weather\",\n        description: \"Get current weather\",\n        parameters: %{\n          type: \"object\",\n          properties: %{location: %{type: \"string\"}},\n          required: [\"location\"]\n        }\n      }\n    ]\n    \n    {:ok, response} = ExLLM.chat(:openai, messages,\n      functions: functions,\n      function_call: \"auto\"\n    )","ref":"ExLLM.html#chat/3-examples"},{"type":"function","title":"ExLLM.chat_with_session/3","doc":"Send a chat request using a session, automatically tracking messages and usage.","ref":"ExLLM.html#chat_with_session/3"},{"type":"function","title":"Parameters - ExLLM.chat_with_session/3","doc":"- `session` - The session to use\n- `content` - The user message content\n- `options` - Chat options (same as `chat/3`)","ref":"ExLLM.html#chat_with_session/3-parameters"},{"type":"function","title":"Returns - ExLLM.chat_with_session/3","doc":"`{:ok, {response, updated_session}}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#chat_with_session/3-returns"},{"type":"function","title":"Examples - ExLLM.chat_with_session/3","doc":"session = ExLLM.new_session(:anthropic)\n    {:ok, {response, session}} = ExLLM.chat_with_session(session, \"Hello!\")\n    # Session now contains the conversation history","ref":"ExLLM.html#chat_with_session/3-examples"},{"type":"function","title":"ExLLM.clear_session/1","doc":"Clear messages from a session while preserving metadata.","ref":"ExLLM.html#clear_session/1"},{"type":"function","title":"Parameters - ExLLM.clear_session/1","doc":"- `session` - The session to clear","ref":"ExLLM.html#clear_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.clear_session/1","doc":"Updated session with no messages.","ref":"ExLLM.html#clear_session/1-returns"},{"type":"function","title":"Examples - ExLLM.clear_session/1","doc":"session = ExLLM.clear_session(session)","ref":"ExLLM.html#clear_session/1-examples"},{"type":"function","title":"ExLLM.compare_models/1","doc":"Compare capabilities across multiple models.","ref":"ExLLM.html#compare_models/1"},{"type":"function","title":"Examples - ExLLM.compare_models/1","doc":"comparison = ExLLM.compare_models([\n      {:openai, \"gpt-4-turbo\"},\n      {:anthropic, \"claude-3-5-sonnet-20241022\"},\n      {:gemini, \"gemini-pro\"}\n    ])\n    \n    # See which features each model supports\n    comparison.features[:vision]\n    # => [%{supported: true}, %{supported: true}, %{supported: false}]","ref":"ExLLM.html#compare_models/1-examples"},{"type":"function","title":"ExLLM.compare_providers/1","doc":"Compare capabilities across multiple providers.","ref":"ExLLM.html#compare_providers/1"},{"type":"function","title":"Examples - ExLLM.compare_providers/1","doc":"comparison = ExLLM.compare_providers([:openai, :anthropic, :ollama])\n    \n    # See all features across providers\n    comparison.features\n    # => [:streaming, :function_calling, :vision, ...]\n    \n    # Check specific provider capabilities\n    comparison.comparison.openai.features\n    # => [:streaming, :function_calling, :cost_tracking, ...]","ref":"ExLLM.html#compare_providers/1-examples"},{"type":"function","title":"ExLLM.configured?/2","doc":"Check if the specified provider is properly configured.","ref":"ExLLM.html#configured?/2"},{"type":"function","title":"Parameters - ExLLM.configured?/2","doc":"- `provider` - The LLM provider to check\n- `options` - Options including configuration provider","ref":"ExLLM.html#configured?/2-parameters"},{"type":"function","title":"Returns - ExLLM.configured?/2","doc":"`true` if configured, `false` otherwise.","ref":"ExLLM.html#configured?/2-returns"},{"type":"function","title":"Examples - ExLLM.configured?/2","doc":"if ExLLM.configured?(:anthropic) do\n      {:ok, response} = ExLLM.chat(:anthropic, messages)\n    else\n      IO.puts(\"Anthropic not configured\")\n    end","ref":"ExLLM.html#configured?/2-examples"},{"type":"function","title":"ExLLM.context_stats/1","doc":"Get statistics about message context usage.","ref":"ExLLM.html#context_stats/1"},{"type":"function","title":"Parameters - ExLLM.context_stats/1","doc":"- `messages` - List of conversation messages","ref":"ExLLM.html#context_stats/1-parameters"},{"type":"function","title":"Returns - ExLLM.context_stats/1","doc":"Map with context statistics.","ref":"ExLLM.html#context_stats/1-returns"},{"type":"function","title":"Examples - ExLLM.context_stats/1","doc":"stats = ExLLM.context_stats(messages)\n    # => %{total_tokens: 1500, message_count: 10, ...}","ref":"ExLLM.html#context_stats/1-examples"},{"type":"function","title":"ExLLM.context_window_size/2","doc":"Get default model for a provider.","ref":"ExLLM.html#context_window_size/2"},{"type":"function","title":"Parameters - ExLLM.context_window_size/2","doc":"- `provider` - LLM provider name\n- `model` - Model name","ref":"ExLLM.html#context_window_size/2-parameters"},{"type":"function","title":"Returns - ExLLM.context_window_size/2","doc":"Context window size in tokens or nil if unknown.","ref":"ExLLM.html#context_window_size/2-returns"},{"type":"function","title":"Examples - ExLLM.context_window_size/2","doc":"tokens = ExLLM.context_window_size(:anthropic, \"claude-3-5-sonnet-20241022\")\n    # => 200000","ref":"ExLLM.html#context_window_size/2-examples"},{"type":"function","title":"ExLLM.cosine_similarity/2","doc":"Calculate similarity between two embeddings.\n\nUses cosine similarity: 1.0 = identical, 0.0 = orthogonal, -1.0 = opposite","ref":"ExLLM.html#cosine_similarity/2"},{"type":"function","title":"Examples - ExLLM.cosine_similarity/2","doc":"similarity = ExLLM.cosine_similarity(embedding1, embedding2)\n    # => 0.87","ref":"ExLLM.html#cosine_similarity/2-examples"},{"type":"function","title":"ExLLM.default_model/1","doc":"Get the default model for the specified provider.","ref":"ExLLM.html#default_model/1"},{"type":"function","title":"Parameters - ExLLM.default_model/1","doc":"- `provider` - The LLM provider","ref":"ExLLM.html#default_model/1-parameters"},{"type":"function","title":"Returns - ExLLM.default_model/1","doc":"String model identifier.","ref":"ExLLM.html#default_model/1-returns"},{"type":"function","title":"Examples - ExLLM.default_model/1","doc":"model = ExLLM.default_model(:anthropic)\n    # => \"claude-sonnet-4-20250514\"","ref":"ExLLM.html#default_model/1-examples"},{"type":"function","title":"ExLLM.embeddings/3","doc":"Generate embeddings for text inputs.\n\nEmbeddings are numerical representations of text that can be used for:\n- Semantic search\n- Clustering\n- Recommendations\n- Anomaly detection\n- Classification","ref":"ExLLM.html#embeddings/3"},{"type":"function","title":"Parameters - ExLLM.embeddings/3","doc":"- `provider` - The LLM provider (`:openai`, `:anthropic`, etc.)\n- `inputs` - List of text strings to embed\n- `options` - Options including `:model`, `:dimensions`, etc.","ref":"ExLLM.html#embeddings/3-parameters"},{"type":"function","title":"Options - ExLLM.embeddings/3","doc":"- `:model` - Embedding model to use (provider-specific)\n- `:dimensions` - Desired embedding dimensions (if supported)\n- `:cache` - Enable caching for embeddings\n- `:cache_ttl` - Cache TTL in milliseconds\n- `:track_cost` - Track embedding costs (default: true)","ref":"ExLLM.html#embeddings/3-options"},{"type":"function","title":"Examples - ExLLM.embeddings/3","doc":"# Single embedding\n    {:ok, response} = ExLLM.embeddings(:openai, [\"Hello, world!\"])\n    [embedding] = response.embeddings\n    # => [0.0123, -0.0456, 0.0789, ...]\n    \n    # Multiple embeddings\n    texts = [\"First text\", \"Second text\", \"Third text\"]\n    {:ok, response} = ExLLM.embeddings(:openai, texts,\n      model: \"text-embedding-3-small\",\n      dimensions: 256  # Reduce dimensions for storage\n    )\n    \n    # With caching\n    {:ok, response} = ExLLM.embeddings(:openai, texts,\n      cache: true,\n      cache_ttl: :timer.hours(24)\n    )","ref":"ExLLM.html#embeddings/3-examples"},{"type":"function","title":"ExLLM.estimate_tokens/1","doc":"Estimate token count for text.","ref":"ExLLM.html#estimate_tokens/1"},{"type":"function","title":"Parameters - ExLLM.estimate_tokens/1","doc":"- `text` - Text to analyze (string, message map, or list)","ref":"ExLLM.html#estimate_tokens/1-parameters"},{"type":"function","title":"Returns - ExLLM.estimate_tokens/1","doc":"Estimated token count.","ref":"ExLLM.html#estimate_tokens/1-returns"},{"type":"function","title":"Examples - ExLLM.estimate_tokens/1","doc":"tokens = ExLLM.estimate_tokens(\"Hello, world!\")\n    # => 4","ref":"ExLLM.html#estimate_tokens/1-examples"},{"type":"function","title":"ExLLM.execute_function/2","doc":"Execute a function call with available functions.","ref":"ExLLM.html#execute_function/2"},{"type":"function","title":"Examples - ExLLM.execute_function/2","doc":"functions = [\n      %{\n        name: \"get_weather\",\n        description: \"Get weather\",\n        parameters: %{...},\n        handler: fn args -> get_weather_impl(args) end\n      }\n    ]\n    \n    {:ok, result} = ExLLM.execute_function(function_call, functions)","ref":"ExLLM.html#execute_function/2-examples"},{"type":"function","title":"ExLLM.extract_text_from_image/3","doc":"Extract text from an image using vision capabilities.\n\nThis is a convenience function for OCR-like tasks.","ref":"ExLLM.html#extract_text_from_image/3"},{"type":"function","title":"Examples - ExLLM.extract_text_from_image/3","doc":"{:ok, text} = ExLLM.extract_text_from_image(:anthropic, \"document.png\")\n    IO.puts(text)\n    \n    # With options\n    {:ok, text} = ExLLM.extract_text_from_image(:openai, \"handwriting.jpg\",\n      model: \"gpt-4-turbo\",\n      prompt: \"Extract all text, preserving formatting\"\n    )","ref":"ExLLM.html#extract_text_from_image/3-examples"},{"type":"function","title":"ExLLM.find_models_with_features/1","doc":"Find models that support specific features.","ref":"ExLLM.html#find_models_with_features/1"},{"type":"function","title":"Examples - ExLLM.find_models_with_features/1","doc":"# Find models with vision and function calling\n    models = ExLLM.find_models_with_features([:vision, :function_calling])\n    # => [{:openai, \"gpt-4-turbo\"}, {:anthropic, \"claude-3-opus-20240229\"}, ...]\n    \n    # Find models that support streaming and have large context\n    models = ExLLM.find_models_with_features([:streaming])\n    |> Enum.filter(fn {provider, model} ->\n      {:ok, info} = ExLLM.get_model_info(provider, model)\n      info.context_window >= 100_000\n    end)","ref":"ExLLM.html#find_models_with_features/1-examples"},{"type":"function","title":"ExLLM.find_providers_with_features/1","doc":"Find providers that support all specified features.","ref":"ExLLM.html#find_providers_with_features/1"},{"type":"function","title":"Examples - ExLLM.find_providers_with_features/1","doc":"# Find providers with embeddings and streaming\n    providers = ExLLM.find_providers_with_features([:embeddings, :streaming])\n    # => [:openai, :ollama]\n    \n    # Find providers with vision and function calling\n    providers = ExLLM.find_providers_with_features([:vision, :function_calling])\n    # => [:openai, :anthropic, :gemini]","ref":"ExLLM.html#find_providers_with_features/1-examples"},{"type":"function","title":"ExLLM.find_similar/3","doc":"Find the most similar items from a list of embeddings.","ref":"ExLLM.html#find_similar/3"},{"type":"function","title":"Options - ExLLM.find_similar/3","doc":"- `:top_k` - Number of results to return (default: 5)\n- `:threshold` - Minimum similarity score (default: 0.0)","ref":"ExLLM.html#find_similar/3-options"},{"type":"function","title":"Examples - ExLLM.find_similar/3","doc":"query_embedding = get_embedding(\"search query\")\n    \n    items = [\n      %{id: 1, text: \"First doc\", embedding: [...]},\n      %{id: 2, text: \"Second doc\", embedding: [...]},\n      # ...\n    ]\n    \n    results = ExLLM.find_similar(query_embedding, items,\n      top_k: 10,\n      threshold: 0.7\n    )\n    # => [%{item: %{id: 2, ...}, similarity: 0.92}, ...]","ref":"ExLLM.html#find_similar/3-examples"},{"type":"function","title":"ExLLM.format_cost/1","doc":"Format cost for display.","ref":"ExLLM.html#format_cost/1"},{"type":"function","title":"Parameters - ExLLM.format_cost/1","doc":"- `cost` - Cost in dollars","ref":"ExLLM.html#format_cost/1-parameters"},{"type":"function","title":"Returns - ExLLM.format_cost/1","doc":"Formatted cost string.","ref":"ExLLM.html#format_cost/1-returns"},{"type":"function","title":"Examples - ExLLM.format_cost/1","doc":"ExLLM.format_cost(0.0035)\n    # => \"$0.003500\"","ref":"ExLLM.html#format_cost/1-examples"},{"type":"function","title":"ExLLM.format_function_result/2","doc":"Format function result for conversation continuation.","ref":"ExLLM.html#format_function_result/2"},{"type":"function","title":"Examples - ExLLM.format_function_result/2","doc":"result = %FunctionCalling.FunctionResult{\n      name: \"get_weather\",\n      result: %{temperature: 72, condition: \"sunny\"}\n    }\n    \n    formatted = ExLLM.format_function_result(result, :openai)\n    \n    # Continue conversation with result\n    messages = messages ++ [formatted]\n    {:ok, response} = ExLLM.chat(:openai, messages)","ref":"ExLLM.html#format_function_result/2-examples"},{"type":"function","title":"ExLLM.get_model_info/2","doc":"Get complete capability information for a model.","ref":"ExLLM.html#get_model_info/2"},{"type":"function","title":"Examples - ExLLM.get_model_info/2","doc":"{:ok, info} = ExLLM.get_model_info(:openai, \"gpt-4-turbo\")\n    \n    # Check capabilities\n    info.capabilities[:vision].supported\n    # => true\n    \n    # Get context window\n    info.context_window\n    # => 128000","ref":"ExLLM.html#get_model_info/2-examples"},{"type":"function","title":"ExLLM.get_provider_capabilities/1","doc":"Get provider-level capabilities.\n\nProvider capabilities are API-level features that are independent of specific models.\nThis includes available endpoints, authentication methods, and provider limitations.","ref":"ExLLM.html#get_provider_capabilities/1"},{"type":"function","title":"Examples - ExLLM.get_provider_capabilities/1","doc":"{:ok, caps} = ExLLM.get_provider_capabilities(:openai)\n    caps.endpoints\n    # => [:chat, :embeddings, :images, :audio, :completions, :fine_tuning, :files]\n    \n    caps.features\n    # => [:streaming, :function_calling, :cost_tracking, :usage_tracking, ...]","ref":"ExLLM.html#get_provider_capabilities/1-examples"},{"type":"function","title":"ExLLM.get_session_messages/2","doc":"Get messages from a session.","ref":"ExLLM.html#get_session_messages/2"},{"type":"function","title":"Parameters - ExLLM.get_session_messages/2","doc":"- `session` - The session to query\n- `limit` - Optional message limit","ref":"ExLLM.html#get_session_messages/2-parameters"},{"type":"function","title":"Returns - ExLLM.get_session_messages/2","doc":"List of messages.","ref":"ExLLM.html#get_session_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.get_session_messages/2","doc":"messages = ExLLM.get_session_messages(session)\n    last_10 = ExLLM.get_session_messages(session, 10)","ref":"ExLLM.html#get_session_messages/2-examples"},{"type":"function","title":"ExLLM.is_local_provider?/1","doc":"Check if a provider is local (no API calls).","ref":"ExLLM.html#is_local_provider?/1"},{"type":"function","title":"Examples - ExLLM.is_local_provider?/1","doc":"ExLLM.is_local_provider?(:ollama)\n    # => true\n    \n    ExLLM.is_local_provider?(:openai)\n    # => false","ref":"ExLLM.html#is_local_provider?/1-examples"},{"type":"function","title":"ExLLM.list_embedding_models/2","doc":"List available embedding models for a provider.","ref":"ExLLM.html#list_embedding_models/2"},{"type":"function","title":"Examples - ExLLM.list_embedding_models/2","doc":"{:ok, models} = ExLLM.list_embedding_models(:openai)\n    Enum.each(models, fn m ->\n      IO.puts(\"#{m.name} - #{m.dimensions} dimensions\")\n    end)","ref":"ExLLM.html#list_embedding_models/2-examples"},{"type":"function","title":"ExLLM.list_model_features/0","doc":"List all trackable model features.","ref":"ExLLM.html#list_model_features/0"},{"type":"function","title":"Examples - ExLLM.list_model_features/0","doc":"features = ExLLM.list_model_features()\n    # => [:streaming, :function_calling, :vision, :audio, ...]","ref":"ExLLM.html#list_model_features/0-examples"},{"type":"function","title":"ExLLM.list_models/2","doc":"List available models for the specified provider.","ref":"ExLLM.html#list_models/2"},{"type":"function","title":"Parameters - ExLLM.list_models/2","doc":"- `provider` - The LLM provider\n- `options` - Options including configuration provider","ref":"ExLLM.html#list_models/2-parameters"},{"type":"function","title":"Returns - ExLLM.list_models/2","doc":"`{:ok, [%ExLLM.Types.Model{}]}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#list_models/2-returns"},{"type":"function","title":"Examples - ExLLM.list_models/2","doc":"{:ok, models} = ExLLM.list_models(:anthropic)\n    Enum.each(models, fn model ->\n      IO.puts(model.name)\n    end)","ref":"ExLLM.html#list_models/2-examples"},{"type":"function","title":"ExLLM.list_providers/0","doc":"List all available providers.","ref":"ExLLM.html#list_providers/0"},{"type":"function","title":"Examples - ExLLM.list_providers/0","doc":"providers = ExLLM.list_providers()\n    # => [:anthropic, :bedrock, :bumblebee, :gemini, :groq, :lmstudio, :mistral, :mock, :ollama, :openai, :openrouter, :perplexity, :xai]","ref":"ExLLM.html#list_providers/0-examples"},{"type":"function","title":"ExLLM.list_recoverable_streams/0","doc":"List all recoverable streams.","ref":"ExLLM.html#list_recoverable_streams/0"},{"type":"function","title":"ExLLM.load_image/2","doc":"Load an image from file for use in vision requests.","ref":"ExLLM.html#load_image/2"},{"type":"function","title":"Examples - ExLLM.load_image/2","doc":"{:ok, image_part} = ExLLM.load_image(\"photo.jpg\")\n    \n    message = %{\n      role: \"user\",\n      content: [\n        ExLLM.Vision.text(\"What's in this image?\"),\n        image_part\n      ]\n    }","ref":"ExLLM.html#load_image/2-examples"},{"type":"function","title":"ExLLM.load_session/1","doc":"Load a session from JSON.","ref":"ExLLM.html#load_session/1"},{"type":"function","title":"Parameters - ExLLM.load_session/1","doc":"- `json` - JSON string containing session data","ref":"ExLLM.html#load_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.load_session/1","doc":"`{:ok, session}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#load_session/1-returns"},{"type":"function","title":"Examples - ExLLM.load_session/1","doc":"json = File.read!(\"session.json\")\n    {:ok, session} = ExLLM.load_session(json)","ref":"ExLLM.html#load_session/1-examples"},{"type":"type","title":"ExLLM.messages/0","doc":"","ref":"ExLLM.html#t:messages/0"},{"type":"function","title":"ExLLM.model_supports?/3","doc":"Check if a model supports a specific feature.","ref":"ExLLM.html#model_supports?/3"},{"type":"function","title":"Examples - ExLLM.model_supports?/3","doc":"ExLLM.model_supports?(:anthropic, \"claude-3-opus-20240229\", :vision)\n    # => true\n    \n    ExLLM.model_supports?(:openai, \"gpt-3.5-turbo\", :vision)\n    # => false","ref":"ExLLM.html#model_supports?/3-examples"},{"type":"function","title":"ExLLM.models_by_capability/1","doc":"Get models grouped by a specific capability.","ref":"ExLLM.html#models_by_capability/1"},{"type":"function","title":"Examples - ExLLM.models_by_capability/1","doc":"vision_models = ExLLM.models_by_capability(:vision)\n    # => %{\n    #   supported: [{:openai, \"gpt-4-turbo\"}, {:anthropic, \"claude-3-opus-20240229\"}, ...],\n    #   not_supported: [{:openai, \"gpt-3.5-turbo\"}, ...]\n    # }","ref":"ExLLM.html#models_by_capability/1-examples"},{"type":"function","title":"ExLLM.new_session/2","doc":"Create a new conversation session.","ref":"ExLLM.html#new_session/2"},{"type":"function","title":"Parameters - ExLLM.new_session/2","doc":"- `provider` - LLM provider to use for the session\n- `opts` - Session options (`:name` for session name)","ref":"ExLLM.html#new_session/2-parameters"},{"type":"function","title":"Returns - ExLLM.new_session/2","doc":"A new session struct.","ref":"ExLLM.html#new_session/2-returns"},{"type":"function","title":"Examples - ExLLM.new_session/2","doc":"session = ExLLM.new_session(:anthropic)\n    session = ExLLM.new_session(:openai, name: \"Customer Support\")","ref":"ExLLM.html#new_session/2-examples"},{"type":"type","title":"ExLLM.options/0","doc":"","ref":"ExLLM.html#t:options/0"},{"type":"function","title":"ExLLM.parse_function_calls/2","doc":"Parse function calls from an LLM response.","ref":"ExLLM.html#parse_function_calls/2"},{"type":"function","title":"Examples - ExLLM.parse_function_calls/2","doc":"case ExLLM.parse_function_calls(response, :openai) do\n      {:ok, [function_call | _]} ->\n        # Execute the function\n        execute_function(function_call)\n        \n      {:ok, []} ->\n        # No function calls in response\n        response.content\n    end","ref":"ExLLM.html#parse_function_calls/2-examples"},{"type":"function","title":"ExLLM.prepare_messages/2","doc":"Prepare messages for sending to a provider with context management.","ref":"ExLLM.html#prepare_messages/2"},{"type":"function","title":"Parameters - ExLLM.prepare_messages/2","doc":"- `messages` - List of conversation messages\n- `options` - Options for context management","ref":"ExLLM.html#prepare_messages/2-parameters"},{"type":"function","title":"Options - ExLLM.prepare_messages/2","doc":"- `:max_tokens` - Maximum tokens for context (default: model-specific)\n- `:strategy` - Context truncation strategy (default: :sliding_window)\n- `:preserve_messages` - Number of recent messages to preserve (default: 5)","ref":"ExLLM.html#prepare_messages/2-options"},{"type":"function","title":"Returns - ExLLM.prepare_messages/2","doc":"Prepared messages list that fits within context window.","ref":"ExLLM.html#prepare_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.prepare_messages/2","doc":"messages = ExLLM.prepare_messages(long_conversation,\n      max_tokens: 4000,\n      strategy: :smart\n    )","ref":"ExLLM.html#prepare_messages/2-examples"},{"type":"type","title":"ExLLM.provider/0","doc":"","ref":"ExLLM.html#t:provider/0"},{"type":"function","title":"ExLLM.provider_requires_auth?/1","doc":"Check if a provider requires authentication.","ref":"ExLLM.html#provider_requires_auth?/1"},{"type":"function","title":"Examples - ExLLM.provider_requires_auth?/1","doc":"ExLLM.provider_requires_auth?(:openai)\n    # => true\n    \n    ExLLM.provider_requires_auth?(:bumblebee)\n    # => false","ref":"ExLLM.html#provider_requires_auth?/1-examples"},{"type":"function","title":"ExLLM.provider_supports?/2","doc":"Check if a provider supports a specific feature or endpoint.","ref":"ExLLM.html#provider_supports?/2"},{"type":"function","title":"Examples - ExLLM.provider_supports?/2","doc":"ExLLM.provider_supports?(:openai, :embeddings)\n    # => true\n    \n    ExLLM.provider_supports?(:ollama, :cost_tracking)\n    # => false\n    \n    ExLLM.provider_supports?(:anthropic, :computer_use)\n    # => true","ref":"ExLLM.html#provider_supports?/2-examples"},{"type":"function","title":"ExLLM.recommend_models/1","doc":"Get recommended models based on requirements.","ref":"ExLLM.html#recommend_models/1"},{"type":"function","title":"Options - ExLLM.recommend_models/1","doc":"- `:features` - Required features (list of atoms)\n- `:min_context_window` - Minimum context window size\n- `:max_cost_per_1k_tokens` - Maximum acceptable cost\n- `:prefer_local` - Prefer local models\n- `:limit` - Number of recommendations (default: 5)","ref":"ExLLM.html#recommend_models/1-options"},{"type":"function","title":"Examples - ExLLM.recommend_models/1","doc":"# Find best models for vision tasks with large context\n    recommendations = ExLLM.recommend_models(\n      features: [:vision, :streaming],\n      min_context_window: 50_000,\n      prefer_local: false\n    )\n    \n    # Find cheapest models for basic chat\n    recommendations = ExLLM.recommend_models(\n      features: [:multi_turn, :system_messages],\n      max_cost_per_1k_tokens: 1.0\n    )","ref":"ExLLM.html#recommend_models/1-examples"},{"type":"function","title":"ExLLM.recommend_providers/1","doc":"Get provider recommendations based on requirements.","ref":"ExLLM.html#recommend_providers/1"},{"type":"function","title":"Parameters - ExLLM.recommend_providers/1","doc":"- `requirements` - Map with:\n  - `:required_features` - Features that must be supported\n  - `:preferred_features` - Nice-to-have features\n  - `:required_endpoints` - Required API endpoints\n  - `:exclude_providers` - Providers to exclude\n  - `:prefer_local` - Prefer local providers (default: false)\n  - `:prefer_free` - Prefer free providers (default: false)","ref":"ExLLM.html#recommend_providers/1-parameters"},{"type":"function","title":"Examples - ExLLM.recommend_providers/1","doc":"# Find best providers for multimodal AI\n    recommendations = ExLLM.recommend_providers(%{\n      required_features: [:vision, :streaming],\n      preferred_features: [:audio_input, :function_calling],\n      exclude_providers: [:mock]\n    })\n    # => [\n    #   %{provider: :openai, score: 0.95, matched_features: [...], missing_features: []},\n    #   %{provider: :anthropic, score: 0.80, matched_features: [...], missing_features: [...]}\n    # ]\n    \n    # Find free local providers\n    recommendations = ExLLM.recommend_providers(%{\n      required_features: [:chat],\n      prefer_local: true,\n      prefer_free: true\n    })","ref":"ExLLM.html#recommend_providers/1-examples"},{"type":"function","title":"ExLLM.resume_stream/2","doc":"Resume a previously interrupted stream.","ref":"ExLLM.html#resume_stream/2"},{"type":"function","title":"Options - ExLLM.resume_stream/2","doc":"- `:strategy` - Recovery strategy (:exact, :paragraph, :summarize)","ref":"ExLLM.html#resume_stream/2-options"},{"type":"function","title":"Examples - ExLLM.resume_stream/2","doc":"{:ok, resumed_stream} = ExLLM.resume_stream(recovery_id)","ref":"ExLLM.html#resume_stream/2-examples"},{"type":"function","title":"ExLLM.save_session/1","doc":"Save a session to JSON.","ref":"ExLLM.html#save_session/1"},{"type":"function","title":"Parameters - ExLLM.save_session/1","doc":"- `session` - The session to save","ref":"ExLLM.html#save_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.save_session/1","doc":"`{:ok, json}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#save_session/1-returns"},{"type":"function","title":"Examples - ExLLM.save_session/1","doc":"{:ok, json} = ExLLM.save_session(session)\n    File.write!(\"session.json\", json)","ref":"ExLLM.html#save_session/1-examples"},{"type":"function","title":"ExLLM.session_token_usage/1","doc":"Get total token usage for a session.","ref":"ExLLM.html#session_token_usage/1"},{"type":"function","title":"Parameters - ExLLM.session_token_usage/1","doc":"- `session` - The session to analyze","ref":"ExLLM.html#session_token_usage/1-parameters"},{"type":"function","title":"Returns - ExLLM.session_token_usage/1","doc":"Total token count.","ref":"ExLLM.html#session_token_usage/1-returns"},{"type":"function","title":"Examples - ExLLM.session_token_usage/1","doc":"tokens = ExLLM.session_token_usage(session)\n    # => 2500","ref":"ExLLM.html#session_token_usage/1-examples"},{"type":"function","title":"ExLLM.stream_chat/3","doc":"Send a streaming chat completion request to the specified LLM provider.","ref":"ExLLM.html#stream_chat/3"},{"type":"function","title":"Parameters - ExLLM.stream_chat/3","doc":"- `provider` - The LLM provider (`:anthropic`, `:openai`, `:ollama`)\n- `messages` - List of conversation messages\n- `options` - Options for the request (see module docs)","ref":"ExLLM.html#stream_chat/3-parameters"},{"type":"function","title":"Options - ExLLM.stream_chat/3","doc":"Same as `chat/3`, plus:\n- `:on_chunk` - Callback function for each chunk\n- `:stream_recovery` - Enable automatic stream recovery (default: false)\n- `:recovery_strategy` - How to resume: :exact, :paragraph, or :summarize\n- `:recovery_id` - Custom ID for recovery (auto-generated if not provided)","ref":"ExLLM.html#stream_chat/3-options"},{"type":"function","title":"Returns - ExLLM.stream_chat/3","doc":"`{:ok, stream}` on success where stream yields `%ExLLM.Types.StreamChunk{}` structs,\n`{:error, reason}` on failure.","ref":"ExLLM.html#stream_chat/3-returns"},{"type":"function","title":"Examples - ExLLM.stream_chat/3","doc":"{:ok, stream} = ExLLM.stream_chat(:anthropic, messages)\n    \n    # Process the stream\n    for chunk <- stream do\n      case chunk do\n        %{content: content} when content != nil ->\n          IO.write(content)\n        %{finish_reason: \"stop\"} ->\n          IO.puts(\"\\nDone!\")\n        _ ->\n          :continue\n      end\n    end\n\n    # With context management\n    {:ok, stream} = ExLLM.stream_chat(:anthropic, messages,\n      max_tokens: 4000,\n      strategy: :smart\n    )","ref":"ExLLM.html#stream_chat/3-examples"},{"type":"function","title":"ExLLM.supported_providers/0","doc":"Get list of supported providers.","ref":"ExLLM.html#supported_providers/0"},{"type":"function","title":"Returns - ExLLM.supported_providers/0","doc":"List of provider atoms.","ref":"ExLLM.html#supported_providers/0-returns"},{"type":"function","title":"Examples - ExLLM.supported_providers/0","doc":"providers = ExLLM.supported_providers()\n    # => [:anthropic, :openai, :ollama]","ref":"ExLLM.html#supported_providers/0-examples"},{"type":"function","title":"ExLLM.supports_vision?/2","doc":"Check if a provider and model support vision inputs.","ref":"ExLLM.html#supports_vision?/2"},{"type":"function","title":"Examples - ExLLM.supports_vision?/2","doc":"ExLLM.supports_vision?(:anthropic, \"claude-3-opus-20240229\")\n    # => true\n    \n    ExLLM.supports_vision?(:openai, \"gpt-3.5-turbo\")\n    # => false","ref":"ExLLM.html#supports_vision?/2-examples"},{"type":"function","title":"ExLLM.validate_context/2","doc":"Validate that messages fit within a model's context window.","ref":"ExLLM.html#validate_context/2"},{"type":"function","title":"Parameters - ExLLM.validate_context/2","doc":"- `messages` - List of conversation messages\n- `options` - Options including model info","ref":"ExLLM.html#validate_context/2-parameters"},{"type":"function","title":"Returns - ExLLM.validate_context/2","doc":"`{:ok, token_count}` if valid, `{:error, reason}` if too large.","ref":"ExLLM.html#validate_context/2-returns"},{"type":"function","title":"ExLLM.vision_message/3","doc":"Create a vision-enabled message with text and images.","ref":"ExLLM.html#vision_message/3"},{"type":"function","title":"Examples - ExLLM.vision_message/3","doc":"# With image URLs\n    {:ok, message} = ExLLM.vision_message(\"What's in these images?\", [\n      \"https://example.com/image1.jpg\",\n      \"https://example.com/image2.png\"\n    ])\n    \n    {:ok, response} = ExLLM.chat(:anthropic, [message])\n    \n    # With local images\n    {:ok, message} = ExLLM.vision_message(\"Describe these photos\", [\n      \"/path/to/photo1.jpg\",\n      \"/path/to/photo2.png\"\n    ])\n    \n    # With options\n    {:ok, message} = ExLLM.vision_message(\"Analyze this chart\", \n      [\"chart.png\"],\n      role: \"user\",\n      detail: :high  # High detail for complex images\n    )","ref":"ExLLM.html#vision_message/3-examples"},{"type":"behaviour","title":"ExLLM.Adapter","doc":"Behaviour for LLM backend adapters.\n\nThis defines the common interface that all LLM adapters must implement,\nenabling a unified API across different providers.","ref":"ExLLM.Adapter.html"},{"type":"behaviour","title":"Example Implementation - ExLLM.Adapter","doc":"defmodule MyAdapter do\n      @behaviour ExLLM.Adapter\n      alias ExLLM.Types\n      \n      @impl true\n      def chat(messages, options \\ []) do\n        # Implementation here\n        {:ok, %Types.LLMResponse{content: \"Hello\"}}\n      end\n      \n      @impl true\n      def stream_chat(messages, options \\ []) do\n        # Implementation here\n        {:ok, stream}\n      end\n      \n      @impl true\n      def configured?(options \\ []) do\n        # Check if adapter is configured\n        true\n      end\n      \n      @impl true\n      def default_model() do\n        \"my-model\"\n      end\n      \n      @impl true\n      def list_models(options \\ []) do\n        {:ok, [\"model1\", \"model2\"]}\n      end\n    end","ref":"ExLLM.Adapter.html#module-example-implementation"},{"type":"behaviour","title":"Options - ExLLM.Adapter","doc":"All adapter functions accept an options keyword list that can include:\n\n- `:config_provider` - Configuration provider for API keys and settings\n- `:model` - Override the default model for this request\n- `:temperature` - Control randomness (0.0 to 1.0)\n- `:max_tokens` - Maximum tokens in response\n- `:timeout` - Request timeout in milliseconds","ref":"ExLLM.Adapter.html#module-options"},{"type":"behaviour","title":"Configuration Injection - ExLLM.Adapter","doc":"Adapters support configuration injection through the `:config_provider` option:\n\n    # Use environment variables\n    ExLLM.OpenAI.chat(messages, config_provider: ExLLM.ConfigProvider.Env)\n    \n    # Use static config\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(%{\n      openai: %{api_key: \"sk-...\"}\n    })\n    ExLLM.OpenAI.chat(messages, config_provider: provider)","ref":"ExLLM.Adapter.html#module-configuration-injection"},{"type":"callback","title":"ExLLM.Adapter.chat/2","doc":"Send a chat completion request to the LLM.","ref":"ExLLM.Adapter.html#c:chat/2"},{"type":"callback","title":"Parameters - ExLLM.Adapter.chat/2","doc":"- `messages` - List of conversation messages\n- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:chat/2-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.chat/2","doc":"`{:ok, %ExLLM.Types.LLMResponse{}}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:chat/2-returns"},{"type":"callback","title":"ExLLM.Adapter.configured?/1","doc":"Check if the adapter is properly configured and ready to use.","ref":"ExLLM.Adapter.html#c:configured?/1"},{"type":"callback","title":"Parameters - ExLLM.Adapter.configured?/1","doc":"- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:configured?/1-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.configured?/1","doc":"`true` if configured, `false` otherwise.","ref":"ExLLM.Adapter.html#c:configured?/1-returns"},{"type":"callback","title":"ExLLM.Adapter.default_model/0","doc":"Get the default model for this adapter.","ref":"ExLLM.Adapter.html#c:default_model/0"},{"type":"callback","title":"Returns - ExLLM.Adapter.default_model/0","doc":"String model identifier.","ref":"ExLLM.Adapter.html#c:default_model/0-returns"},{"type":"callback","title":"ExLLM.Adapter.embeddings/2","doc":"Generate embeddings for the given inputs.","ref":"ExLLM.Adapter.html#c:embeddings/2"},{"type":"callback","title":"Parameters - ExLLM.Adapter.embeddings/2","doc":"- `inputs` - List of text strings to embed (max length varies by model)\n- `options` - Adapter options including `:model`","ref":"ExLLM.Adapter.html#c:embeddings/2-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.embeddings/2","doc":"`{:ok, %ExLLM.Types.EmbeddingResponse{}}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:embeddings/2-returns"},{"type":"callback","title":"ExLLM.Adapter.list_embedding_models/1","doc":"List available embedding models for this adapter.","ref":"ExLLM.Adapter.html#c:list_embedding_models/1"},{"type":"callback","title":"Parameters - ExLLM.Adapter.list_embedding_models/1","doc":"- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:list_embedding_models/1-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.list_embedding_models/1","doc":"`{:ok, [%ExLLM.Types.EmbeddingModel{}]}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:list_embedding_models/1-returns"},{"type":"callback","title":"ExLLM.Adapter.list_models/1","doc":"List available models for this adapter.","ref":"ExLLM.Adapter.html#c:list_models/1"},{"type":"callback","title":"Parameters - ExLLM.Adapter.list_models/1","doc":"- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:list_models/1-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.list_models/1","doc":"`{:ok, [%ExLLM.Types.Model{}]}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:list_models/1-returns"},{"type":"callback","title":"ExLLM.Adapter.stream_chat/2","doc":"Send a streaming chat completion request to the LLM.","ref":"ExLLM.Adapter.html#c:stream_chat/2"},{"type":"callback","title":"Parameters - ExLLM.Adapter.stream_chat/2","doc":"- `messages` - List of conversation messages  \n- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:stream_chat/2-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.stream_chat/2","doc":"`{:ok, stream}` on success where stream yields `%ExLLM.Types.StreamChunk{}` structs,\n`{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:stream_chat/2-returns"},{"type":"module","title":"ExLLM.Adapters.Anthropic","doc":"Anthropic Claude API adapter for ExLLM.","ref":"ExLLM.Adapters.Anthropic.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Anthropic","doc":"This adapter requires an Anthropic API key and optionally a base URL.","ref":"ExLLM.Adapters.Anthropic.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Anthropic","doc":"# Set environment variables\n    export ANTHROPIC_API_KEY=\"your-api-key\"\n    export ANTHROPIC_MODEL=\"claude-3-5-sonnet-20241022\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Anthropic.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Anthropic.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Anthropic","doc":"config = %{\n      anthropic: %{\n        api_key: \"your-api-key\",\n        model: \"claude-3-5-sonnet-20241022\",\n        base_url: \"https://api.anthropic.com/v1\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Anthropic.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Anthropic.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Anthropic","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Anthropic.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Anthropic.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Anthropic.html#module-example-usage"},{"type":"module","title":"ExLLM.Adapters.Bedrock","doc":"AWS Bedrock adapter for ExLLM.\nSupports multiple providers including Claude, Titan, Llama, Cohere, AI21, and Mistral through Bedrock.","ref":"ExLLM.Adapters.Bedrock.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Bedrock","doc":"The adapter supports multiple credential sources:\n1. Explicit credentials in configuration\n2. Environment variables (AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, AWS_SESSION_TOKEN)\n3. AWS profiles from ~/.aws/credentials\n4. EC2 instance metadata\n5. ECS task role credentials\n\nConfiguration options:\n- `:access_key_id` - AWS access key\n- `:secret_access_key` - AWS secret key\n- `:session_token` - AWS session token (optional)\n- `:region` - AWS region (defaults to \"us-east-1\")\n- `:profile` - AWS profile name\n- `:model` - Default model to use","ref":"ExLLM.Adapters.Bedrock.html#module-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Bedrock","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Bedrock.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Bedrock.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Bedrock.html#module-example-usage"},{"type":"module","title":"Available Models - ExLLM.Adapters.Bedrock","doc":"Common model aliases:\n- \"nova-lite\" - Amazon Nova Lite (cost-effective default)\n- \"nova-pro\" - Amazon Nova Pro\n- \"claude-opus-4\" - Anthropic Claude 4 Opus\n- \"claude-3-5-sonnet\" - Anthropic Claude 3.5 Sonnet\n- \"llama-3.3-70b\" - Meta Llama 3.3 70B\n- \"palmyra-x5\" - Writer Palmyra X5\n- \"deepseek-r1\" - DeepSeek R1","ref":"ExLLM.Adapters.Bedrock.html#module-available-models"},{"type":"module","title":"ExLLM.Adapters.Bumblebee","doc":"Bumblebee adapter for on-device LLM inference.\n\nThis adapter enables running language models locally using Bumblebee and EXLA/EMLX backends.\nIt supports Apple Silicon (via EMLX), NVIDIA GPUs (via CUDA), and CPU inference.","ref":"ExLLM.Adapters.Bumblebee.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Bumblebee","doc":"The Bumblebee adapter doesn't require API keys but may need backend configuration:\n\n    # For Apple Silicon (automatic detection)\n    {:ok, response} = ExLLM.chat(:bumblebee, messages)\n    \n    # With specific model\n    {:ok, response} = ExLLM.chat(:bumblebee, messages, model: \"microsoft/phi-2\")","ref":"ExLLM.Adapters.Bumblebee.html#module-configuration"},{"type":"module","title":"Available Models - ExLLM.Adapters.Bumblebee","doc":"- `microsoft/phi-2` - Phi-2 (2.7B) - Default\n- `meta-llama/Llama-2-7b-hf` - Llama 2 (7B)\n- `mistralai/Mistral-7B-v0.1` - Mistral (7B)\n- `EleutherAI/gpt-neo-1.3B` - GPT-Neo (1.3B)\n- `google/flan-t5-base` - Flan-T5 Base","ref":"ExLLM.Adapters.Bumblebee.html#module-available-models"},{"type":"module","title":"Features - ExLLM.Adapters.Bumblebee","doc":"- On-device inference with no API calls\n- Automatic hardware acceleration detection\n- Support for Apple Silicon, NVIDIA GPUs, and CPUs\n- Model caching for faster subsequent loads\n- Streaming support for real-time generation","ref":"ExLLM.Adapters.Bumblebee.html#module-features"},{"type":"module","title":"ExLLM.Adapters.Gemini","doc":"Google Gemini API adapter for ExLLM.\n\nSupports Gemini 2.5, 2.0, and 1.5 models including Flash and Pro variants.","ref":"ExLLM.Adapters.Gemini.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Gemini","doc":"This adapter requires a Google API key.","ref":"ExLLM.Adapters.Gemini.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Gemini","doc":"# Set environment variables\n    export GOOGLE_API_KEY=\"your-api-key\"\n    export GEMINI_MODEL=\"gemini-2.0-flash\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Gemini.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Gemini.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Gemini","doc":"config = %{\n      gemini: %{\n        api_key: \"your-api-key\",\n        model: \"gemini-2.5-flash-preview-05-20\"\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Gemini.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Gemini.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Gemini","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Gemini.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Gemini.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Gemini.html#module-example-usage"},{"type":"module","title":"Safety Settings - ExLLM.Adapters.Gemini","doc":"You can customize safety settings:\n\n    options = [\n      safety_settings: [\n        %{category: \"HARM_CATEGORY_HARASSMENT\", threshold: \"BLOCK_ONLY_HIGH\"},\n        %{category: \"HARM_CATEGORY_HATE_SPEECH\", threshold: \"BLOCK_ONLY_HIGH\"}\n      ]\n    ]\n    {:ok, response} = ExLLM.Adapters.Gemini.chat(messages, options)","ref":"ExLLM.Adapters.Gemini.html#module-safety-settings"},{"type":"module","title":"ExLLM.Adapters.Groq","doc":"Groq adapter for ExLLM.\n\nGroq provides extremely fast inference for open-source models using their\ncustom LPU (Language Processing Unit) hardware.","ref":"ExLLM.Adapters.Groq.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Groq","doc":"config :ex_llm,\n      groq: [\n        api_key: System.get_env(\"GROQ_API_KEY\"),\n        base_url: \"https://api.groq.com/openai/v1\"  # optional\n      ]","ref":"ExLLM.Adapters.Groq.html#module-configuration"},{"type":"module","title":"Supported Models - ExLLM.Adapters.Groq","doc":"- llama-3.3-70b-versatile\n- llama-3.1-70b-versatile\n- llama-3.1-8b-instant\n- llama3-70b-8192\n- llama3-8b-8192\n- mixtral-8x7b-32768\n- gemma2-9b-it\n- gemma-7b-it","ref":"ExLLM.Adapters.Groq.html#module-supported-models"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Groq","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n    \n    {:ok, response} = ExLLM.Adapters.Groq.chat(messages, model: \"llama3-70b-8192\")","ref":"ExLLM.Adapters.Groq.html#module-example-usage"},{"type":"module","title":"ExLLM.Adapters.LMStudio","doc":"LM Studio adapter for local LLM inference.\n\nThis adapter provides integration with LM Studio, a desktop application for running\nlocal LLMs with an OpenAI-compatible API. LM Studio supports models from Hugging Face\nand provides both GUI and server modes for local inference.","ref":"ExLLM.Adapters.LMStudio.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.LMStudio","doc":"LM Studio runs a local server with OpenAI-compatible endpoints. By default, it listens\non `http://localhost:1234` with API key `\"lm-studio\"`.\n\n    # Basic usage\n    {:ok, response} = ExLLM.chat(:lmstudio, messages)\n    \n    # With custom endpoint\n    {:ok, response} = ExLLM.chat(:lmstudio, messages, \n      host: \"192.168.1.100\", \n      port: 8080\n    )","ref":"ExLLM.Adapters.LMStudio.html#module-configuration"},{"type":"module","title":"Features - ExLLM.Adapters.LMStudio","doc":"- OpenAI-compatible API (`/v1/chat/completions`, `/v1/models`, `/v1/embeddings`)\n- Native LM Studio REST API (`/api/v0/*`) with enhanced model information\n- Model loading status and quantization details\n- TTL (Time-To-Live) parameter for automatic model unloading\n- Support for both llama.cpp and MLX engines on Apple Silicon\n- Streaming chat completions","ref":"ExLLM.Adapters.LMStudio.html#module-features"},{"type":"module","title":"Requirements - ExLLM.Adapters.LMStudio","doc":"1. Install LM Studio from https://lmstudio.ai\n2. Download and load at least one model in LM Studio\n3. Start the local server (usually localhost:1234)\n4. Ensure the server is running when using this adapter","ref":"ExLLM.Adapters.LMStudio.html#module-requirements"},{"type":"module","title":"API Endpoints - ExLLM.Adapters.LMStudio","doc":"This adapter uses both OpenAI-compatible and native LM Studio endpoints:\n\n- **OpenAI Compatible**: `/v1/chat/completions`, `/v1/models`, `/v1/embeddings`\n- **Native API**: `/api/v0/models`, `/api/v0/chat/completions` (enhanced features)\n\nThe native API provides additional information like model loading status,\nquantization details, architecture information, and performance metrics.","ref":"ExLLM.Adapters.LMStudio.html#module-api-endpoints"},{"type":"module","title":"ExLLM.Adapters.Mistral","doc":"Mistral AI API adapter for ExLLM.\n\nMistral AI provides state-of-the-art language models including:\n- Mistral Large: Most capable model for complex reasoning\n- Mistral Medium: Balanced model for most use cases  \n- Mistral Small: Fast and cost-effective model\n- Codestral: Specialized model for code generation\n- Pixtral: Multimodal model with vision capabilities","ref":"ExLLM.Adapters.Mistral.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Mistral","doc":"This adapter requires a Mistral AI API key and optionally a base URL.","ref":"ExLLM.Adapters.Mistral.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Mistral","doc":"# Set environment variables\n    export MISTRAL_API_KEY=\"your-api-key\"\n    export MISTRAL_MODEL=\"mistral/mistral-tiny\"  # optional\n    export MISTRAL_API_BASE=\"https://api.mistral.ai/v1\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Mistral.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Mistral.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Mistral","doc":"config = %{\n      mistral: %{\n        api_key: \"your-api-key\",\n        model: \"mistral/mistral-small-latest\",\n        base_url: \"https://api.mistral.ai/v1\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Mistral.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Mistral.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Mistral","doc":"messages = [\n      %{role: \"user\", content: \"Explain quantum computing\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Mistral.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Mistral.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end\n\n    # Function calling\n    functions = [\n      %{\n        name: \"get_weather\",\n        description: \"Get weather information\",\n        parameters: %{\n          type: \"object\",\n          properties: %{\n            location: %{type: \"string\", description: \"City name\"}\n          }\n        }\n      }\n    ]\n\n    {:ok, response} = ExLLM.Adapters.Mistral.chat(messages, tools: functions)","ref":"ExLLM.Adapters.Mistral.html#module-example-usage"},{"type":"module","title":"ExLLM.Adapters.Mock","doc":"Mock adapter for testing ExLLM integrations.\n\nThis adapter provides predictable responses for testing without making actual API calls.\nIt supports all ExLLM features including streaming, function calling, and error simulation.","ref":"ExLLM.Adapters.Mock.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Mock","doc":"The mock adapter can be configured with predefined responses or response generators:\n\n    # Static response\n    ExLLM.Adapters.Mock.set_response(%{\n      content: \"Mock response\",\n      model: \"mock-model\",\n      usage: %{input_tokens: 10, output_tokens: 20}\n    })\n    \n    # Dynamic response based on input\n    ExLLM.Adapters.Mock.set_response_handler(fn messages, options ->\n      %{content: \"You said: #{List.last(messages).content}\"}\n    end)\n    \n    # Simulate errors\n    ExLLM.Adapters.Mock.set_error({:api_error, %{status: 500, body: \"Server error\"}})","ref":"ExLLM.Adapters.Mock.html#module-configuration"},{"type":"module","title":"Testing Features - ExLLM.Adapters.Mock","doc":"- Capture and inspect requests\n- Simulate various response types\n- Test error handling\n- Validate function calling\n- Test streaming behavior","ref":"ExLLM.Adapters.Mock.html#module-testing-features"},{"type":"function","title":"ExLLM.Adapters.Mock.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.Adapters.Mock.html#child_spec/1"},{"type":"function","title":"ExLLM.Adapters.Mock.get_cached_provider/0","doc":"Returns the currently configured cached provider, if any.","ref":"ExLLM.Adapters.Mock.html#get_cached_provider/0"},{"type":"function","title":"ExLLM.Adapters.Mock.get_last_request/0","doc":"Gets the last captured request.","ref":"ExLLM.Adapters.Mock.html#get_last_request/0"},{"type":"function","title":"ExLLM.Adapters.Mock.get_requests/0","doc":"Gets all captured requests.","ref":"ExLLM.Adapters.Mock.html#get_requests/0"},{"type":"function","title":"ExLLM.Adapters.Mock.list_cached_providers/0","doc":"Lists available cached providers.","ref":"ExLLM.Adapters.Mock.html#list_cached_providers/0"},{"type":"function","title":"ExLLM.Adapters.Mock.reset/0","doc":"Clears all captured requests and resets to defaults.","ref":"ExLLM.Adapters.Mock.html#reset/0"},{"type":"function","title":"ExLLM.Adapters.Mock.set_error/1","doc":"Sets an error response.","ref":"ExLLM.Adapters.Mock.html#set_error/1"},{"type":"function","title":"ExLLM.Adapters.Mock.set_function_call_response/1","doc":"Sets a function call response.","ref":"ExLLM.Adapters.Mock.html#set_function_call_response/1"},{"type":"function","title":"ExLLM.Adapters.Mock.set_response/1","doc":"Sets a static response for all requests.","ref":"ExLLM.Adapters.Mock.html#set_response/1"},{"type":"function","title":"ExLLM.Adapters.Mock.set_response_handler/1","doc":"Sets a dynamic response handler.","ref":"ExLLM.Adapters.Mock.html#set_response_handler/1"},{"type":"function","title":"ExLLM.Adapters.Mock.set_stream_chunks/1","doc":"Sets stream chunks for streaming responses.","ref":"ExLLM.Adapters.Mock.html#set_stream_chunks/1"},{"type":"function","title":"ExLLM.Adapters.Mock.start_link/1","doc":"Starts the mock adapter agent.","ref":"ExLLM.Adapters.Mock.html#start_link/1"},{"type":"function","title":"ExLLM.Adapters.Mock.use_cached_responses/1","doc":"Configures the mock adapter to use cached responses from a specific provider.\n\nThis allows the mock adapter to mimic the behavior of real providers using\npreviously cached responses.","ref":"ExLLM.Adapters.Mock.html#use_cached_responses/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Mock.use_cached_responses/1","doc":"# Use cached OpenAI responses\n    ExLLM.Adapters.Mock.use_cached_responses(:openai)\n    \n    # Use cached Anthropic responses  \n    ExLLM.Adapters.Mock.use_cached_responses(\"anthropic\")\n    \n    # Now mock calls will return realistic responses\n    {:ok, response} = ExLLM.chat(messages, provider: :mock)","ref":"ExLLM.Adapters.Mock.html#use_cached_responses/1-examples"},{"type":"module","title":"ExLLM.Adapters.Ollama","doc":"Ollama API adapter for ExLLM - provides local model inference via Ollama server.","ref":"ExLLM.Adapters.Ollama.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Ollama","doc":"This adapter requires a running Ollama server. By default, it connects to localhost:11434.","ref":"ExLLM.Adapters.Ollama.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Ollama","doc":"# Set environment variables\n    export OLLAMA_API_BASE=\"http://localhost:11434\"  # optional\n    export OLLAMA_MODEL=\"llama2\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Ollama.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Ollama.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Ollama","doc":"config = %{\n      ollama: %{\n        base_url: \"http://localhost:11434\",\n        model: \"llama2\"\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Ollama.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Ollama.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Ollama","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Ollama.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Ollama.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Ollama.html#module-example-usage"},{"type":"module","title":"Available Models - ExLLM.Adapters.Ollama","doc":"To see available models, ensure Ollama is running and use:\n\n    {:ok, models} = ExLLM.Adapters.Ollama.list_models()","ref":"ExLLM.Adapters.Ollama.html#module-available-models"},{"type":"module","title":"Configuration Management - ExLLM.Adapters.Ollama","doc":"The adapter provides functions to generate and update model configurations:\n\n    # Generate configuration for all installed models\n    {:ok, yaml} = ExLLM.Adapters.Ollama.generate_config()\n    \n    # Save the configuration\n    {:ok, path} = ExLLM.Adapters.Ollama.generate_config(save: true)\n    \n    # Update a specific model's configuration\n    {:ok, yaml} = ExLLM.Adapters.Ollama.update_model_config(\"llama3.1\")\n\nThis is useful for keeping your `config/models/ollama.yml` in sync with your\nlocally installed models and their actual capabilities.","ref":"ExLLM.Adapters.Ollama.html#module-configuration-management"},{"type":"function","title":"ExLLM.Adapters.Ollama.copy_model/3","doc":"Copy a model to a new name.\n\nUses Ollama's /api/copy endpoint to create a copy of an existing model\nwith a new name.","ref":"ExLLM.Adapters.Ollama.html#copy_model/3"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.copy_model/3","doc":"{:ok, _} = ExLLM.Adapters.Ollama.copy_model(\"llama3.1\", \"my-llama\")","ref":"ExLLM.Adapters.Ollama.html#copy_model/3-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.delete_model/2","doc":"Delete a model from Ollama.\n\nUses Ollama's /api/delete endpoint to remove a model from the local\nmodel store.","ref":"ExLLM.Adapters.Ollama.html#delete_model/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.delete_model/2","doc":"{:ok, _} = ExLLM.Adapters.Ollama.delete_model(\"old-model\")","ref":"ExLLM.Adapters.Ollama.html#delete_model/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.generate/2","doc":"Generate a completion for the given prompt using Ollama's /api/generate endpoint.\n\nThis is for non-chat completions, useful for base models or specific use cases.","ref":"ExLLM.Adapters.Ollama.html#generate/2"},{"type":"function","title":"Options - ExLLM.Adapters.Ollama.generate/2","doc":"- `:model` - The model to use (required)\n- `:prompt` - The prompt text (required)\n- `:suffix` - Text to append after the generation\n- `:images` - List of base64-encoded images for multimodal models\n- `:format` - Response format (e.g., \"json\")\n- `:options` - Model-specific options (temperature, seed, etc.)\n- `:context` - Context from previous request for maintaining conversation state\n- `:raw` - If true, no formatting will be applied to the prompt\n- `:keep_alive` - How long to keep the model loaded (e.g., \"5m\")\n- `:timeout` - Request timeout in milliseconds","ref":"ExLLM.Adapters.Ollama.html#generate/2-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.generate/2","doc":"# Simple completion\n    {:ok, response} = ExLLM.Adapters.Ollama.generate(\"Complete this: The sky is\", \n      model: \"llama3.1\")\n    \n    # With options\n    {:ok, response} = ExLLM.Adapters.Ollama.generate(\"Write a haiku about coding\",\n      model: \"llama3.1\",\n      options: %{temperature: 0.7, seed: 42})\n    \n    # Maintain conversation context\n    {:ok, response1} = ExLLM.Adapters.Ollama.generate(\"Hi, I'm learning Elixir\",\n      model: \"llama3.1\")\n    {:ok, response2} = ExLLM.Adapters.Ollama.generate(\"What should I learn first?\",\n      model: \"llama3.1\",\n      context: response1.context)","ref":"ExLLM.Adapters.Ollama.html#generate/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.generate_config/1","doc":"Generate YAML configuration for all locally installed Ollama models.\n\nThis function fetches information about all installed models and generates\na YAML configuration that can be saved to `config/models/ollama.yml`.","ref":"ExLLM.Adapters.Ollama.html#generate_config/1"},{"type":"function","title":"Options - ExLLM.Adapters.Ollama.generate_config/1","doc":"- `:save` - When true, saves the configuration to the file (default: false)\n- `:path` - Custom path for the YAML file (default: \"config/models/ollama.yml\")\n- `:merge` - When true, merges with existing configuration (default: true)","ref":"ExLLM.Adapters.Ollama.html#generate_config/1-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.generate_config/1","doc":"# Generate configuration and return as string\n    {:ok, yaml} = ExLLM.Adapters.Ollama.generate_config()\n    \n    # Save directly to config/models/ollama.yml\n    {:ok, path} = ExLLM.Adapters.Ollama.generate_config(save: true)\n    \n    # Save to custom location\n    {:ok, path} = ExLLM.Adapters.Ollama.generate_config(\n      save: true, \n      path: \"my_ollama_config.yml\"\n    )\n    \n    # Replace existing configuration instead of merging\n    {:ok, yaml} = ExLLM.Adapters.Ollama.generate_config(merge: false)","ref":"ExLLM.Adapters.Ollama.html#generate_config/1-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.list_running_models/1","doc":"List currently loaded models.\n\nUses Ollama's /api/ps endpoint to show which models are currently\nloaded in memory.","ref":"ExLLM.Adapters.Ollama.html#list_running_models/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.list_running_models/1","doc":"{:ok, loaded} = ExLLM.Adapters.Ollama.list_running_models()","ref":"ExLLM.Adapters.Ollama.html#list_running_models/1-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.pull_model/2","doc":"Pull a model from the Ollama library.\n\nUses Ollama's /api/pull endpoint to download a model. This returns\na stream of progress updates.","ref":"ExLLM.Adapters.Ollama.html#pull_model/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.pull_model/2","doc":"{:ok, stream} = ExLLM.Adapters.Ollama.pull_model(\"llama3.1:latest\")\n    for update <- stream do\n      IO.puts(\"Status: #{update[\"status\"]}\")\n    end","ref":"ExLLM.Adapters.Ollama.html#pull_model/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.push_model/2","doc":"Push a model to the Ollama library.\n\nUses Ollama's /api/push endpoint to upload a model.","ref":"ExLLM.Adapters.Ollama.html#push_model/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.push_model/2","doc":"{:ok, stream} = ExLLM.Adapters.Ollama.push_model(\"my-model:latest\")","ref":"ExLLM.Adapters.Ollama.html#push_model/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.show_model/2","doc":"Get detailed information about a specific model.\n\nUses Ollama's /api/show endpoint to retrieve model details including\nmodelfile, parameters, template, and more.","ref":"ExLLM.Adapters.Ollama.html#show_model/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.show_model/2","doc":"{:ok, info} = ExLLM.Adapters.Ollama.show_model(\"llama3.1\")","ref":"ExLLM.Adapters.Ollama.html#show_model/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.stream_generate/2","doc":"Stream a completion using the /api/generate endpoint.\n\nSimilar to `generate/2` but returns a stream of response chunks.","ref":"ExLLM.Adapters.Ollama.html#stream_generate/2"},{"type":"function","title":"ExLLM.Adapters.Ollama.update_model_config/2","doc":"Update configuration for a specific model in ollama.yml.\n\nThis function fetches the latest information for a specific model and\nupdates its entry in the configuration file.","ref":"ExLLM.Adapters.Ollama.html#update_model_config/2"},{"type":"function","title":"Options - ExLLM.Adapters.Ollama.update_model_config/2","doc":"- `:save` - When true, saves the configuration to the file (default: true)\n- `:path` - Custom path for the YAML file (default: \"config/models/ollama.yml\")","ref":"ExLLM.Adapters.Ollama.html#update_model_config/2-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.update_model_config/2","doc":"# Update a specific model's configuration\n    {:ok, yaml} = ExLLM.Adapters.Ollama.update_model_config(\"llama3.1\")\n    \n    # Update without saving (preview changes)\n    {:ok, yaml} = ExLLM.Adapters.Ollama.update_model_config(\"llama3.1\", save: false)","ref":"ExLLM.Adapters.Ollama.html#update_model_config/2-examples"},{"type":"function","title":"ExLLM.Adapters.Ollama.version/1","doc":"Get Ollama version information.","ref":"ExLLM.Adapters.Ollama.html#version/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Ollama.version/1","doc":"{:ok, version} = ExLLM.Adapters.Ollama.version()","ref":"ExLLM.Adapters.Ollama.html#version/1-examples"},{"type":"module","title":"ExLLM.Adapters.OpenAI","doc":"OpenAI GPT API adapter for ExLLM.","ref":"ExLLM.Adapters.OpenAI.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.OpenAI","doc":"This adapter requires an OpenAI API key and optionally a base URL.","ref":"ExLLM.Adapters.OpenAI.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.OpenAI","doc":"# Set environment variables\n    export OPENAI_API_KEY=\"your-api-key\"\n    export OPENAI_MODEL=\"gpt-4-turbo\"  # optional\n    export OPENAI_API_BASE=\"https://api.openai.com/v1\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.OpenAI.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.OpenAI.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.OpenAI","doc":"config = %{\n      openai: %{\n        api_key: \"your-api-key\",\n        model: \"gpt-4-turbo\",\n        base_url: \"https://api.openai.com/v1\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.OpenAI.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.OpenAI.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.OpenAI","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.OpenAI.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.OpenAI.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.OpenAI.html#module-example-usage"},{"type":"function","title":"ExLLM.Adapters.OpenAI.add_upload_part/3","doc":"Add a part to an existing upload.\n\nEach part can be at most 64 MB. Parts can be added in parallel.","ref":"ExLLM.Adapters.OpenAI.html#add_upload_part/3"},{"type":"function","title":"Parameters - ExLLM.Adapters.OpenAI.add_upload_part/3","doc":"- `upload_id` - The ID of the Upload\n- `data` - The chunk of bytes for this part\n- `options` - Additional options","ref":"ExLLM.Adapters.OpenAI.html#add_upload_part/3-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.add_upload_part/3","doc":"{:ok, part} = OpenAI.add_upload_part(\"upload_abc123\", chunk_data)\n    part[\"id\"] # => \"part_def456\"","ref":"ExLLM.Adapters.OpenAI.html#add_upload_part/3-examples"},{"type":"function","title":"ExLLM.Adapters.OpenAI.cancel_upload/2","doc":"Cancel an upload.\n\nNo parts may be added after an upload is cancelled.","ref":"ExLLM.Adapters.OpenAI.html#cancel_upload/2"},{"type":"function","title":"Parameters - ExLLM.Adapters.OpenAI.cancel_upload/2","doc":"- `upload_id` - The ID of the Upload to cancel\n- `options` - Additional options","ref":"ExLLM.Adapters.OpenAI.html#cancel_upload/2-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.cancel_upload/2","doc":"{:ok, cancelled} = OpenAI.cancel_upload(\"upload_abc123\")\n    cancelled[\"status\"] # => \"cancelled\"","ref":"ExLLM.Adapters.OpenAI.html#cancel_upload/2-examples"},{"type":"function","title":"ExLLM.Adapters.OpenAI.complete_upload/3","doc":"Complete an upload and create the final File object.","ref":"ExLLM.Adapters.OpenAI.html#complete_upload/3"},{"type":"function","title":"Parameters - ExLLM.Adapters.OpenAI.complete_upload/3","doc":"- `upload_id` - The ID of the Upload\n- `part_ids` - Ordered list of Part IDs\n- `options` - Additional options (can include `:md5` for checksum verification)","ref":"ExLLM.Adapters.OpenAI.html#complete_upload/3-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.complete_upload/3","doc":"{:ok, completed} = OpenAI.complete_upload(\n      \"upload_abc123\",\n      [\"part_def456\", \"part_ghi789\"]\n    )\n    \n    completed[\"status\"] # => \"completed\"\n    completed[\"file\"][\"id\"] # => \"file-xyz321\"","ref":"ExLLM.Adapters.OpenAI.html#complete_upload/3-examples"},{"type":"function","title":"ExLLM.Adapters.OpenAI.create_assistant/2","doc":"Create an assistant.","ref":"ExLLM.Adapters.OpenAI.html#create_assistant/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.create_batch/2","doc":"Create a batch request for processing multiple inputs.","ref":"ExLLM.Adapters.OpenAI.html#create_batch/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.create_upload/2","doc":"Create an Upload object for multipart file uploads.\n\nUse this for files larger than the regular file upload limit. \nAn Upload can accept at most 8 GB and expires after 1 hour.","ref":"ExLLM.Adapters.OpenAI.html#create_upload/2"},{"type":"function","title":"Parameters - ExLLM.Adapters.OpenAI.create_upload/2","doc":"- `bytes` - The total number of bytes to upload\n- `filename` - The name of the file\n- `mime_type` - The MIME type (e.g., \"text/jsonl\")\n- `purpose` - The intended purpose of the file\n- `options` - Additional options","ref":"ExLLM.Adapters.OpenAI.html#create_upload/2-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.create_upload/2","doc":"{:ok, upload} = OpenAI.create_upload(\n      bytes: 2_147_483_648,\n      filename: \"training_examples.jsonl\",\n      mime_type: \"text/jsonl\",\n      purpose: \"fine-tune\"\n    )\n    \n    upload[\"id\"] # => \"upload_abc123\"\n    upload[\"status\"] # => \"pending\"","ref":"ExLLM.Adapters.OpenAI.html#create_upload/2-examples"},{"type":"function","title":"ExLLM.Adapters.OpenAI.delete_file/2","doc":"Delete an uploaded file.","ref":"ExLLM.Adapters.OpenAI.html#delete_file/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.generate_image/2","doc":"Generate images using OpenAI's DALL-E API.","ref":"ExLLM.Adapters.OpenAI.html#generate_image/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.get_file/2","doc":"Get metadata for a specific file.","ref":"ExLLM.Adapters.OpenAI.html#get_file/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.list_files/1","doc":"List all uploaded files.","ref":"ExLLM.Adapters.OpenAI.html#list_files/1"},{"type":"function","title":"Options - ExLLM.Adapters.OpenAI.list_files/1","doc":"- `:after` - Cursor for pagination (object ID)\n- `:limit` - Number of objects to return (1-10000, default: 10000)\n- `:order` - Sort order: \"asc\" or \"desc\" (default: \"desc\")\n- `:purpose` - Filter by file purpose","ref":"ExLLM.Adapters.OpenAI.html#list_files/1-options"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.list_files/1","doc":"# List all files\n    {:ok, files} = OpenAI.list_files()\n    \n    # List with pagination\n    {:ok, files} = OpenAI.list_files(limit: 100, after: \"file-abc123\")\n    \n    # Filter by purpose\n    {:ok, files} = OpenAI.list_files(purpose: \"fine-tune\")","ref":"ExLLM.Adapters.OpenAI.html#list_files/1-examples"},{"type":"function","title":"ExLLM.Adapters.OpenAI.moderate_content/2","doc":"Moderate content using OpenAI's moderation API.","ref":"ExLLM.Adapters.OpenAI.html#moderate_content/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.parse_response/2","doc":"","ref":"ExLLM.Adapters.OpenAI.html#parse_response/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.retrieve_file_content/2","doc":"Retrieve the content of an uploaded file.","ref":"ExLLM.Adapters.OpenAI.html#retrieve_file_content/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.transcribe_audio/2","doc":"Transcribe audio using OpenAI's Whisper API.","ref":"ExLLM.Adapters.OpenAI.html#transcribe_audio/2"},{"type":"function","title":"ExLLM.Adapters.OpenAI.upload_file/3","doc":"Upload a file to OpenAI for use with assistants or other endpoints.","ref":"ExLLM.Adapters.OpenAI.html#upload_file/3"},{"type":"function","title":"Parameters - ExLLM.Adapters.OpenAI.upload_file/3","doc":"- `file_path` - Path to the file to upload\n- `purpose` - The intended purpose of the file. Supported values:\n  - `\"fine-tune\"` - For fine-tuning models\n  - `\"fine-tune-results\"` - Fine-tuning results (system-generated)\n  - `\"assistants\"` - For use with Assistants API\n  - `\"assistants_output\"` - Assistant outputs (system-generated)\n  - `\"batch\"` - For batch API input\n  - `\"batch_output\"` - Batch API results (system-generated)\n  - `\"vision\"` - For vision fine-tuning\n  - `\"user_data\"` - Flexible file type for any purpose\n  - `\"evals\"` - For evaluation datasets\n- `options` - Additional options including config_provider","ref":"ExLLM.Adapters.OpenAI.html#upload_file/3-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.OpenAI.upload_file/3","doc":"{:ok, file} = OpenAI.upload_file(\"/path/to/data.jsonl\", \"fine-tune\")\n    file[\"id\"] # => \"file-abc123\"\n    file[\"expires_at\"] # => 1680202602","ref":"ExLLM.Adapters.OpenAI.html#upload_file/3-examples"},{"type":"function","title":"File Object Structure - ExLLM.Adapters.OpenAI.upload_file/3","doc":"The returned file object contains:\n- `\"id\"` - The file identifier (e.g., \"file-abc123\")\n- `\"object\"` - Always \"file\"\n- `\"bytes\"` - Size of the file in bytes\n- `\"created_at\"` - Unix timestamp when created\n- `\"expires_at\"` - Unix timestamp when the file expires\n- `\"filename\"` - The name of the uploaded file\n- `\"purpose\"` - The intended purpose of the file\n- `\"status\"` - (Deprecated) Upload status\n- `\"status_details\"` - (Deprecated) Validation error details","ref":"ExLLM.Adapters.OpenAI.html#upload_file/3-file-object-structure"},{"type":"function","title":"ExLLM.Adapters.OpenAI.validate_functions_parameter/1","doc":"","ref":"ExLLM.Adapters.OpenAI.html#validate_functions_parameter/1"},{"type":"behaviour","title":"ExLLM.Adapters.OpenAICompatible","doc":"Base implementation for OpenAI-compatible API providers.\n\nMany LLM providers follow OpenAI's API format, allowing for shared implementation.\nThis module provides common functionality that can be reused across providers.","ref":"ExLLM.Adapters.OpenAICompatible.html"},{"type":"behaviour","title":"Usage - ExLLM.Adapters.OpenAICompatible","doc":"Create a new adapter by using this module:\n\n    defmodule ExLLM.Adapters.MyProvider do\n      use ExLLM.Adapters.OpenAICompatible,\n        provider: :my_provider,\n        base_url: \"https://api.myprovider.com/v1\",\n        models: [\"model-1\", \"model-2\"]\n        \n      # Override any functions as needed\n      defp transform_request(request, _options) do\n        # Custom request transformation\n        request\n      end\n    end","ref":"ExLLM.Adapters.OpenAICompatible.html#module-usage"},{"type":"function","title":"ExLLM.Adapters.OpenAICompatible.default_model_transformer/2","doc":"Default model transformer for configuration data.","ref":"ExLLM.Adapters.OpenAICompatible.html#default_model_transformer/2"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.filter_model/1","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:filter_model/1"},{"type":"function","title":"ExLLM.Adapters.OpenAICompatible.format_model_name/1","doc":"Formats a model ID into a human-readable name.\n\nDelegates to shared ModelUtils for consistency.","ref":"ExLLM.Adapters.OpenAICompatible.html#format_model_name/1"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.get_api_key/1","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:get_api_key/1"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.get_base_url/1","doc":"Defines callbacks for OpenAI-compatible adapters.","ref":"ExLLM.Adapters.OpenAICompatible.html#c:get_base_url/1"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.get_headers/2","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:get_headers/2"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.parse_error/1","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:parse_error/1"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.parse_model/1","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:parse_model/1"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.transform_request/2","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:transform_request/2"},{"type":"callback","title":"ExLLM.Adapters.OpenAICompatible.transform_response/2","doc":"","ref":"ExLLM.Adapters.OpenAICompatible.html#c:transform_response/2"},{"type":"module","title":"ExLLM.Adapters.OpenRouter","doc":"OpenRouter API adapter for ExLLM.\n\nOpenRouter provides access to 300+ AI models through a unified API that's compatible\nwith the OpenAI format. It offers intelligent routing, automatic fallbacks, and\nnormalized responses across different AI providers.","ref":"ExLLM.Adapters.OpenRouter.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.OpenRouter","doc":"This adapter requires an OpenRouter API key and optionally app identification headers.","ref":"ExLLM.Adapters.OpenRouter.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.OpenRouter","doc":"# Set environment variables\n    export OPENROUTER_API_KEY=\"your-api-key\"\n    export OPENROUTER_MODEL=\"openai/gpt-4o\"  # optional\n    export OPENROUTER_APP_NAME=\"MyApp\"       # optional\n    export OPENROUTER_APP_URL=\"https://myapp.com\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.OpenRouter.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.OpenRouter.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.OpenRouter","doc":"config = %{\n      openrouter: %{\n        api_key: \"your-api-key\",\n        model: \"openai/gpt-4o\",\n        app_name: \"MyApp\",\n        app_url: \"https://myapp.com\",\n        base_url: \"https://openrouter.ai/api/v1\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.OpenRouter.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.OpenRouter.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.OpenRouter","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.OpenRouter.chat(messages)\n    IO.puts(response.content)\n\n    # With specific model\n    {:ok, response} = ExLLM.Adapters.OpenRouter.chat(messages, model: \"anthropic/claude-3-5-sonnet\")\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.OpenRouter.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.OpenRouter.html#module-example-usage"},{"type":"module","title":"OpenRouter Features - ExLLM.Adapters.OpenRouter","doc":"- **Model Routing**: Access 300+ models from providers like OpenAI, Anthropic, Google, Meta, etc.\n- **Automatic Fallbacks**: Intelligent routing if primary model is unavailable\n- **Unified API**: OpenAI-compatible format for all models\n- **Streaming**: Real-time responses for all supported models\n- **Function Calling**: Tool/function calling support where available\n- **Multimodal**: Image and PDF input support for compatible models","ref":"ExLLM.Adapters.OpenRouter.html#module-openrouter-features"},{"type":"module","title":"ExLLM.Adapters.Perplexity","doc":"Perplexity AI API adapter for ExLLM.\n\nPerplexity AI provides search-augmented language models that combine LLM capabilities\nwith real-time web search. Their models include:","ref":"ExLLM.Adapters.Perplexity.html"},{"type":"module","title":"Search Models - ExLLM.Adapters.Perplexity","doc":"- Sonar: Lightweight, cost-effective search model\n- Sonar Pro: Advanced search with grounding for complex queries","ref":"ExLLM.Adapters.Perplexity.html#module-search-models"},{"type":"module","title":"Research Models - ExLLM.Adapters.Perplexity","doc":"- Sonar Deep Research: Expert-level research conducting exhaustive searches","ref":"ExLLM.Adapters.Perplexity.html#module-research-models"},{"type":"module","title":"Reasoning Models - ExLLM.Adapters.Perplexity","doc":"- Sonar Reasoning: Chain of thought reasoning with web search\n- Sonar Reasoning Pro: Premier reasoning powered by DeepSeek R1","ref":"ExLLM.Adapters.Perplexity.html#module-reasoning-models"},{"type":"module","title":"Standard Models - ExLLM.Adapters.Perplexity","doc":"- Various Llama, CodeLlama, and Mistral models without search capabilities","ref":"ExLLM.Adapters.Perplexity.html#module-standard-models"},{"type":"module","title":"Configuration - ExLLM.Adapters.Perplexity","doc":"This adapter requires a Perplexity API key and optionally a base URL.","ref":"ExLLM.Adapters.Perplexity.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Perplexity","doc":"# Set environment variables\n    export PERPLEXITY_API_KEY=\"pplx-your-api-key\"\n    export PERPLEXITY_MODEL=\"perplexity/sonar-pro\"  # optional\n    export PERPLEXITY_API_BASE=\"https://api.perplexity.ai\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Perplexity.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Perplexity.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Perplexity","doc":"config = %{\n      perplexity: %{\n        api_key: \"pplx-your-api-key\",\n        model: \"perplexity/sonar-pro\",\n        base_url: \"https://api.perplexity.ai\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Perplexity.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Perplexity.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Perplexity","doc":"messages = [\n      %{role: \"user\", content: \"What's the latest news in AI research?\"}\n    ]\n\n    # Simple search-augmented chat\n    {:ok, response} = ExLLM.Adapters.Perplexity.chat(messages, model: \"perplexity/sonar-pro\")\n    IO.puts(response.content)\n\n    # Academic search mode\n    {:ok, response} = ExLLM.Adapters.Perplexity.chat(messages, \n      model: \"perplexity/sonar-pro\",\n      search_mode: \"academic\",\n      web_search_options: %{search_context_size: \"medium\"}\n    )\n\n    # Deep research with high reasoning effort\n    {:ok, response} = ExLLM.Adapters.Perplexity.chat(messages,\n      model: \"perplexity/sonar-deep-research\", \n      reasoning_effort: \"high\"\n    )\n\n    # Streaming search results\n    {:ok, stream} = ExLLM.Adapters.Perplexity.stream_chat(messages,\n      model: \"perplexity/sonar\",\n      search_mode: \"news\"\n    )\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Perplexity.html#module-example-usage"},{"type":"function","title":"ExLLM.Adapters.Perplexity.supports_reasoning?/1","doc":"Checks if a model supports reasoning capabilities.","ref":"ExLLM.Adapters.Perplexity.html#supports_reasoning?/1"},{"type":"function","title":"ExLLM.Adapters.Perplexity.supports_web_search?/1","doc":"Checks if a model supports web search capabilities.","ref":"ExLLM.Adapters.Perplexity.html#supports_web_search?/1"},{"type":"function","title":"ExLLM.Adapters.Perplexity.validate_image_filters/1","doc":"Validates image filter parameters (domain or format filters).","ref":"ExLLM.Adapters.Perplexity.html#validate_image_filters/1"},{"type":"function","title":"ExLLM.Adapters.Perplexity.validate_reasoning_effort/1","doc":"Validates reasoning_effort parameter.","ref":"ExLLM.Adapters.Perplexity.html#validate_reasoning_effort/1"},{"type":"function","title":"ExLLM.Adapters.Perplexity.validate_search_mode/1","doc":"Validates search_mode parameter.","ref":"ExLLM.Adapters.Perplexity.html#validate_search_mode/1"},{"type":"module","title":"ExLLM.Adapters.Shared.ConfigHelper","doc":"Shared configuration management utilities for adapters.\n\nProvides common functionality for retrieving configuration,\nAPI keys, and default models across all adapters.","ref":"ExLLM.Adapters.Shared.ConfigHelper.html"},{"type":"function","title":"ExLLM.Adapters.Shared.ConfigHelper.ensure_default_model/1","doc":"Get the default model for an adapter, raising if not configured.","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#ensure_default_model/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ConfigHelper.ensure_default_model/1","doc":"iex> ConfigHelper.ensure_default_model(:openai)\n    \"gpt-3.5-turbo\"","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#ensure_default_model/1-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ConfigHelper.get_api_key/2","doc":"Get API key from config with environment variable fallback.","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#get_api_key/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ConfigHelper.get_api_key/2","doc":"iex> ConfigHelper.get_api_key(%{api_key: \"sk-123\"}, \"OPENAI_API_KEY\")\n    \"sk-123\"\n    \n    iex> ConfigHelper.get_api_key(%{}, \"OPENAI_API_KEY\")\n    \"env-api-key\"  # From environment","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#get_api_key/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ConfigHelper.get_config/2","doc":"Get configuration for a specific adapter from the config provider.","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#get_config/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ConfigHelper.get_config/2","doc":"iex> ConfigHelper.get_config(:anthropic, ExLLM.ConfigProvider.Env)\n    %{api_key: \"sk-...\", model: \"claude-3-5-sonnet\", ...}","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#get_config/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ConfigHelper.get_config_provider/1","doc":"Get the config provider from options with application default fallback.","ref":"ExLLM.Adapters.Shared.ConfigHelper.html#get_config_provider/1"},{"type":"module","title":"ExLLM.Adapters.Shared.ErrorHandler","doc":"Shared error handling utilities for ExLLM adapters.\n\nProvides consistent error handling patterns across all provider adapters,\nincluding provider-specific error parsing and standardization.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.extract_error_message/1","doc":"Extract error message from various response formats.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#extract_error_message/1"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.get_retry_after/1","doc":"Extract retry-after header value if present.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#get_retry_after/1"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.handle_provider_error/3","doc":"Handle provider-specific errors and convert to standard ExLLM errors.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#handle_provider_error/3"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ErrorHandler.handle_provider_error/3","doc":"# OpenAI-style error\n    ErrorHandler.handle_provider_error(:openai, 429, %{\n      \"error\" => %{\n        \"type\" => \"rate_limit_error\",\n        \"message\" => \"Rate limit exceeded\"\n      }\n    })\n    \n    # Anthropic-style error  \n    ErrorHandler.handle_provider_error(:anthropic, 400, %{\n      \"error\" => %{\n        \"type\" => \"invalid_request_error\",\n        \"message\" => \"max_tokens required\"\n      }\n    })","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#handle_provider_error/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.normalize_error/3","doc":"Normalize error responses across providers for consistent handling.\n\nReturns a standardized error tuple.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#normalize_error/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.retryable_error?/2","doc":"Check if an error is retryable based on status code and error type.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#retryable_error?/2"},{"type":"function","title":"ExLLM.Adapters.Shared.ErrorHandler.should_retry?/3","doc":"Check if an error should trigger a retry based on provider-specific rules.","ref":"ExLLM.Adapters.Shared.ErrorHandler.html#should_retry?/3"},{"type":"module","title":"ExLLM.Adapters.Shared.HTTPClient","doc":"Shared HTTP client utilities for ExLLM adapters.\n\nProvides common HTTP functionality including:\n- JSON API requests with proper headers\n- Server-Sent Events (SSE) streaming support\n- Standardized error handling\n- Response parsing utilities\n\nThis module abstracts common HTTP patterns to reduce duplication\nacross provider adapters.","ref":"ExLLM.Adapters.Shared.HTTPClient.html"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.add_custom_headers/2","doc":"Add custom headers specified by the user.\n\nAllows users to add arbitrary headers for specific use cases.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#add_custom_headers/2"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.add_idempotency_headers/3","doc":"Add idempotency headers for safe request retries.\n\nSome providers support idempotency keys to prevent duplicate operations.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#add_idempotency_headers/3"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.add_rate_limit_headers/3","doc":"Add rate limit headers to the request.\n\nSome providers support rate limit hints in request headers.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#add_rate_limit_headers/3"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.build_provider_headers/2","doc":"Build provider-specific headers for API requests.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#build_provider_headers/2"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.HTTPClient.build_provider_headers/2","doc":"- `:provider` - Provider name for specific headers\n- `:api_key` - API key for authorization\n- `:version` - API version header\n- `:organization` - Organization ID (OpenAI)\n- `:project` - Project ID (OpenAI)","ref":"ExLLM.Adapters.Shared.HTTPClient.html#build_provider_headers/2-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.build_provider_headers/2","doc":"HTTPClient.build_provider_headers(:openai, \n      api_key: \"sk-...\", \n      organization: \"org-123\"\n    )","ref":"ExLLM.Adapters.Shared.HTTPClient.html#build_provider_headers/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.get_json/3","doc":"Make a GET request to an API endpoint with proper caching support.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#get_json/3"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.HTTPClient.get_json/3","doc":"- `:timeout` - Request timeout in milliseconds (default: 60s)\n- `:provider` - Provider name for logging and caching","ref":"ExLLM.Adapters.Shared.HTTPClient.html#get_json/3-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.get_json/3","doc":"HTTPClient.get_json(\"https://api.example.com/v1/models\", \n      [{\"Authorization\", \"Bearer sk-...\"}],\n      provider: :openai\n    )","ref":"ExLLM.Adapters.Shared.HTTPClient.html#get_json/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.handle_api_error/2","doc":"Handle API error responses consistently.\n\nAttempts to parse error details from JSON responses.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#handle_api_error/2"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.parse_sse_chunks/2","doc":"Parse Server-Sent Events from a data buffer.\n\nReturns parsed events and remaining buffer.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#parse_sse_chunks/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.parse_sse_chunks/2","doc":"{events, new_buffer} = HTTPClient.parse_sse_chunks(\"data: {\"text\":\"Hi\"}\\n\\n\", \"\")","ref":"ExLLM.Adapters.Shared.HTTPClient.html#parse_sse_chunks/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.post_json/4","doc":"Make a POST request with JSON body to an API endpoint.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_json/4"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.HTTPClient.post_json/4","doc":"- `:timeout` - Request timeout in milliseconds (default: 60s)\n- `:recv_timeout` - Receive timeout for streaming (default: 5m)\n- `:stream` - Whether this is a streaming request","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_json/4-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.post_json/4","doc":"HTTPClient.post_json(\"https://api.example.com/v1/chat\", \n      %{messages: messages},\n      [{\"Authorization\", \"Bearer sk-...\"}],\n      timeout: 30_000\n    )","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_json/4-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.post_multipart/4","doc":"Make a multipart form POST request to upload files.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_multipart/4"},{"type":"function","title":"Parameters - ExLLM.Adapters.Shared.HTTPClient.post_multipart/4","doc":"- `url` - The endpoint URL\n- `form_data` - A keyword list of form fields, where file fields are {:file, path}\n- `headers` - Request headers\n- `opts` - Additional options","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_multipart/4-parameters"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.post_multipart/4","doc":"HTTPClient.post_multipart(\n      \"https://api.openai.com/v1/files\",\n      [\n        purpose: \"fine-tune\",\n        file: {:file, \"/path/to/file.jsonl\"}\n      ],\n      [{\"Authorization\", \"Bearer sk-...\"}]\n    )","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_multipart/4-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.post_stream/3","doc":"Make a streaming POST request using Req's into option.\n\nThis is used by StreamingCoordinator for unified streaming.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_stream/3"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.post_stream/3","doc":"HTTPClient.post_stream(url, body, [\n      headers: headers,\n      into: stream_collector_fn\n    ])","ref":"ExLLM.Adapters.Shared.HTTPClient.html#post_stream/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.prepare_headers/1","doc":"Prepare headers with common defaults.\n\nAdds Content-Type and User-Agent if not present.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#prepare_headers/1"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.stream_request/5","doc":"Make a streaming POST request that returns Server-Sent Events.\n\nThe callback function will be called for each chunk received.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#stream_request/5"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.HTTPClient.stream_request/5","doc":"HTTPClient.stream_request(url, body, headers, fn chunk ->\n      # Process each SSE chunk\n      IO.puts(\"Received: \" <> chunk)\n    end)","ref":"ExLLM.Adapters.Shared.HTTPClient.html#stream_request/5-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.HTTPClient.user_agent/0","doc":"Build a standardized User-Agent header for ExLLM.","ref":"ExLLM.Adapters.Shared.HTTPClient.html#user_agent/0"},{"type":"module","title":"ExLLM.Adapters.Shared.MessageFormatter","doc":"Shared message formatting utilities for ExLLM adapters.\n\nProvides common message transformations and validations used across\ndifferent LLM providers. Each provider has slightly different message\nformats, but this module provides the common patterns.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.add_system_message/2","doc":"Add a system message to the beginning of the messages list.\n\nHandles cases where system message might already exist.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#add_system_message/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.MessageFormatter.add_system_message/2","doc":"MessageFormatter.add_system_message(messages, \"You are a helpful assistant\")","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#add_system_message/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.ensure_alternating_roles/1","doc":"Ensure messages alternate between user and assistant roles.\n\nSome providers require strict alternation. This function adds\nempty assistant messages where needed.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#ensure_alternating_roles/1"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.estimate_token_count/1","doc":"Count tokens in messages (rough estimate).\n\nThis is a very rough estimate - actual token count depends on the\nspecific tokenizer used by each model.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#estimate_token_count/1"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.extract_system_message/1","doc":"Extract system message from messages list if present.\n\nReturns {system_message, remaining_messages}.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#extract_system_message/1"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.format_function_call/2","doc":"Format a function/tool call message for the provider.\n\nEach provider has different formats for function calls.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#format_function_call/2"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.format_function_result/2","doc":"Format a function result message for the provider.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#format_function_result/2"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.stringify_message_keys/1","doc":"Ensure all messages have string keys (some providers require this).","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#stringify_message_keys/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.MessageFormatter.stringify_message_keys/1","doc":"MessageFormatter.stringify_message_keys([\n      %{role: :user, content: \"Hello\"}\n    ])\n    # => [%{\"role\" => \"user\", \"content\" => \"Hello\"}]","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#stringify_message_keys/1-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.truncate_messages/2","doc":"Truncate messages to fit within a token limit.\n\nKeeps most recent messages, preserving system message if present.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#truncate_messages/2"},{"type":"function","title":"ExLLM.Adapters.Shared.MessageFormatter.validate_messages/1","doc":"Validate messages have required fields and proper structure.","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#validate_messages/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.MessageFormatter.validate_messages/1","doc":"MessageFormatter.validate_messages([\n      %{role: \"user\", content: \"Hello\"},\n      %{role: \"assistant\", content: \"Hi there!\"}\n    ])\n    # => :ok","ref":"ExLLM.Adapters.Shared.MessageFormatter.html#validate_messages/1-examples"},{"type":"behaviour","title":"ExLLM.Adapters.Shared.ModelFetcher","doc":"Unified behavior and utilities for fetching models from LLM provider APIs.\n\nThis module standardizes the pattern of:\n1. Fetching models from provider APIs\n2. Filtering out non-LLM models\n3. Parsing model metadata\n4. Transforming to common format\n5. Integrating with ModelLoader for caching","ref":"ExLLM.Adapters.Shared.ModelFetcher.html"},{"type":"behaviour","title":"Usage - ExLLM.Adapters.Shared.ModelFetcher","doc":"Adapters can implement the behavior callbacks or use the helper functions\nto reduce code duplication.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#module-usage"},{"type":"callback","title":"ExLLM.Adapters.Shared.ModelFetcher.fetch_models/1","doc":"Callback to fetch models from the provider's API.\n\nShould return {:ok, raw_models} or {:error, reason}.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#c:fetch_models/1"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.fetch_openai_compatible_models/3","doc":"Fetch models from a standard OpenAI-compatible endpoint.\n\nMany providers use the same /v1/models endpoint format.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#fetch_openai_compatible_models/3"},{"type":"callback","title":"ExLLM.Adapters.Shared.ModelFetcher.filter_model/1","doc":"Callback to filter out non-LLM models.\n\nSome providers return embeddings, whisper, etc. that we want to exclude.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#c:filter_model/1"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.list_models_with_loader/3","doc":"Standard implementation for list_models that integrates with ModelLoader.\n\nThis is the main entry point that adapters should use.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#list_models_with_loader/3"},{"type":"function","title":"Example - ExLLM.Adapters.Shared.ModelFetcher.list_models_with_loader/3","doc":"def list_models(options \\ []) do\n      ModelFetcher.list_models_with_loader(__MODULE__, :myprovider, options)\n    end","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#list_models_with_loader/3-example"},{"type":"callback","title":"ExLLM.Adapters.Shared.ModelFetcher.parse_api_model/1","doc":"Callback to parse a raw API model into Types.Model struct.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#c:parse_api_model/1"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.parse_standard_model/3","doc":"Parse a model from API response to Types.Model struct.\n\nProvides sensible defaults and uses ModelUtils for formatting.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#parse_standard_model/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.process_models/4","doc":"Helper to process raw models through filter and parse pipeline.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#process_models/4"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.standard_model_filter/2","doc":"Standard model filtering for common patterns.\n\nFilters out:\n- Embedding models\n- Whisper/audio models\n- Image generation models\n- Old deprecated models","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#standard_model_filter/2"},{"type":"callback","title":"ExLLM.Adapters.Shared.ModelFetcher.transform_model_config/2","doc":"Callback to transform config-based model data.\n\nUsed when loading from YAML configuration.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#c:transform_model_config/2"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelFetcher.transform_standard_config/3","doc":"Transform a model from YAML config to Types.Model struct.\n\nStandard implementation that most adapters can use.","ref":"ExLLM.Adapters.Shared.ModelFetcher.html#transform_standard_config/3"},{"type":"module","title":"ExLLM.Adapters.Shared.ModelUtils","doc":"Shared utilities for model management across adapters.\n\nProvides common functionality for:\n- Model name formatting and normalization\n- Description generation based on model characteristics\n- Capability inference from model IDs\n- Model categorization and filtering","ref":"ExLLM.Adapters.Shared.ModelUtils.html"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelUtils.format_model_name/1","doc":"Format a model ID into a human-readable name.","ref":"ExLLM.Adapters.Shared.ModelUtils.html#format_model_name/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ModelUtils.format_model_name/1","doc":"iex> ModelUtils.format_model_name(\"gpt-4-turbo\")\n    \"GPT 4 Turbo\"\n    \n    iex> ModelUtils.format_model_name(\"claude-3-5-sonnet\")\n    \"Claude 3 5 Sonnet\"","ref":"ExLLM.Adapters.Shared.ModelUtils.html#format_model_name/1-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelUtils.generate_description/2","doc":"Generate a model description based on its ID and characteristics.","ref":"ExLLM.Adapters.Shared.ModelUtils.html#generate_description/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ModelUtils.generate_description/2","doc":"iex> ModelUtils.generate_description(\"gpt-4-turbo\", :openai)\n    \"OpenAI's GPT-4 Turbo model with improved performance\"","ref":"ExLLM.Adapters.Shared.ModelUtils.html#generate_description/2-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelUtils.get_model_family/1","doc":"Extract model family from model ID.","ref":"ExLLM.Adapters.Shared.ModelUtils.html#get_model_family/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ModelUtils.get_model_family/1","doc":"iex> ModelUtils.get_model_family(\"gpt-4-turbo-preview\")\n    \"gpt-4\"\n    \n    iex> ModelUtils.get_model_family(\"claude-3-5-sonnet\")\n    \"claude-3\"","ref":"ExLLM.Adapters.Shared.ModelUtils.html#get_model_family/1-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelUtils.infer_capabilities/2","doc":"Infer model capabilities from its ID and provider.\n\nReturns a map with capability flags.","ref":"ExLLM.Adapters.Shared.ModelUtils.html#infer_capabilities/2"},{"type":"function","title":"ExLLM.Adapters.Shared.ModelUtils.is_chat_model?/1","doc":"Check if a model ID indicates a chat/conversational model.","ref":"ExLLM.Adapters.Shared.ModelUtils.html#is_chat_model?/1"},{"type":"behaviour","title":"ExLLM.Adapters.Shared.RequestBuilder","doc":"Unified request building for LLM providers.\n\nThis module provides common patterns for building API requests across\ndifferent LLM providers, reducing code duplication and ensuring consistency.\n\nFeatures:\n- Common request body construction\n- Optional parameter handling\n- Provider-specific extensions via callbacks\n- Function/tool formatting\n- Message formatting","ref":"ExLLM.Adapters.Shared.RequestBuilder.html"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.add_common_parameters/2","doc":"Add common optional parameters to a request.\n\nOnly adds parameters that are present in options.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#add_common_parameters/2"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.add_function_calling/2","doc":"Add function calling parameters to a request.\n\nHandles both the older `functions` format and newer `tools` format.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#add_function_calling/2"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.add_optional_param/4","doc":"Add an optional parameter to the request if it exists in options.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#add_optional_param/4"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.build_chat_request/3","doc":"Build a standard chat completion request.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_chat_request/3"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.RequestBuilder.build_chat_request/3","doc":"Common options supported across providers:\n- `:model` - Model to use\n- `:temperature` - Sampling temperature (0.0 to 2.0)\n- `:max_tokens` - Maximum tokens to generate\n- `:top_p` - Nucleus sampling parameter\n- `:frequency_penalty` - Frequency penalty (-2.0 to 2.0)\n- `:presence_penalty` - Presence penalty (-2.0 to 2.0)\n- `:stop` - Stop sequences\n- `:user` - User identifier\n- `:functions` - Function definitions for function calling\n- `:stream` - Whether to stream the response","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_chat_request/3-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.RequestBuilder.build_chat_request/3","doc":"RequestBuilder.build_chat_request(\n      messages,\n      \"gpt-4\",\n      temperature: 0.7,\n      max_tokens: 1000\n    )","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_chat_request/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.build_completion_request/3","doc":"Build a completion request (for non-chat models).","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_completion_request/3"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.RequestBuilder.build_completion_request/3","doc":"Similar to chat requests but with `prompt` instead of `messages`.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_completion_request/3-options"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.build_embeddings_request/3","doc":"Build an embeddings request.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_embeddings_request/3"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.RequestBuilder.build_embeddings_request/3","doc":"- `:model` - Embedding model to use\n- `:encoding_format` - Format for the embeddings (e.g., \"float\", \"base64\")\n- `:dimensions` - Number of dimensions for the embeddings\n- `:user` - User identifier","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_embeddings_request/3-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.RequestBuilder.build_embeddings_request/3","doc":"RequestBuilder.build_embeddings_request(\n      [\"Hello world\", \"How are you?\"],\n      \"text-embedding-3-small\",\n      dimensions: 512\n    )","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_embeddings_request/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.build_provider_request/4","doc":"Build a request with provider-specific transformations.\n\nThis is the main entry point for adapters that implement the transform_request callback.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_provider_request/4"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.RequestBuilder.build_provider_request/4","doc":"defmodule MyAdapter do\n      @behaviour ExLLM.Adapters.Shared.RequestBuilder\n      \n      def build_request(messages, model, options) do\n        RequestBuilder.build_provider_request(__MODULE__, messages, model, options)\n      end\n      \n      @impl true\n      def transform_request(request, options) do\n        # Add provider-specific fields\n        Map.put(request, \"custom_field\", \"value\")\n      end\n    end","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#build_provider_request/4-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.extract_system_message/1","doc":"Extract system message from a list of messages.\n\nReturns {system_content, other_messages} tuple.\nSome providers (like Anthropic) handle system messages differently.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#extract_system_message/1"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.format_messages/1","doc":"Format messages for API requests.\n\nEnsures messages have the correct structure and handles various input formats.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#format_messages/1"},{"type":"callback","title":"ExLLM.Adapters.Shared.RequestBuilder.format_messages_for_provider/1","doc":"Callback for provider-specific message formatting.\n\nSome providers need custom message structures.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#c:format_messages_for_provider/1"},{"type":"function","title":"ExLLM.Adapters.Shared.RequestBuilder.format_tools/1","doc":"Format functions as tools for the newer OpenAI tools API.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#format_tools/1"},{"type":"callback","title":"ExLLM.Adapters.Shared.RequestBuilder.transform_request/2","doc":"Callback for provider-specific request transformations.\n\nAllows adapters to modify the request after standard building.","ref":"ExLLM.Adapters.Shared.RequestBuilder.html#c:transform_request/2"},{"type":"module","title":"ExLLM.Adapters.Shared.ResponseBuilder","doc":"Shared utilities for building standardized responses across adapters.\n\nProvides consistent response construction for:\n- Chat completions\n- Streaming chunks\n- Function calls\n- Error responses\n- Embeddings","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_audio_response/3","doc":"Build an audio transcription response from provider data.\n\nUsed for Whisper and similar audio transcription endpoints.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_audio_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_chat_response/3","doc":"Build a standard chat response from provider-specific data.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_chat_response/3"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.ResponseBuilder.build_chat_response/3","doc":"- `:calculate_cost` - Whether to calculate cost (default: true)\n- `:provider` - Provider name for cost calculation","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_chat_response/3-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.ResponseBuilder.build_chat_response/3","doc":"ResponseBuilder.build_chat_response(%{\n      \"choices\" => [%{\"message\" => %{\"content\" => \"Hello!\"}}],\n      \"usage\" => %{\"prompt_tokens\" => 10, \"completion_tokens\" => 5}\n    }, \"gpt-4\", provider: :openai)","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_chat_response/3-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_completion_response/3","doc":"Build a completion response (non-chat format) from provider data.\n\nUsed for providers that support traditional completion endpoints.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_completion_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_embedding_response/3","doc":"Build an embedding response from provider data.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_embedding_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_error_response/3","doc":"Build an error response from provider data.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_error_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_image_response/3","doc":"Build an image generation response from provider data.\n\nUsed for DALL-E and similar image generation endpoints.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_image_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_moderation_response/3","doc":"Build a moderation response from provider data.\n\nUsed for content moderation endpoints.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_moderation_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_stream_chunk/2","doc":"Build a streaming chunk from provider data.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_stream_chunk/2"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.build_tool_call_response/3","doc":"Build a response with tool calls from provider data.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#build_tool_call_response/3"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.extract_metadata/2","doc":"Extract metadata from response data.\n\nIncludes timing information, model details, and provider-specific metadata.","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#extract_metadata/2"},{"type":"function","title":"ExLLM.Adapters.Shared.ResponseBuilder.extract_usage/1","doc":"Extract and normalize usage data from various formats.\n\nMaps provider-specific field names to standardized ExLLM format:\n- API field names: prompt_tokens, completion_tokens (OpenAI, OpenRouter, etc.)\n- ExLLM field names: input_tokens, output_tokens","ref":"ExLLM.Adapters.Shared.ResponseBuilder.html#extract_usage/1"},{"type":"behaviour","title":"ExLLM.Adapters.Shared.StreamingBehavior","doc":"Shared streaming behavior and utilities for ExLLM adapters.\n\nProvides common patterns for handling Server-Sent Events (SSE) streaming\nresponses from LLM APIs. This module can be used as a behavior or just\nfor its utility functions.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingBehavior.accumulate_chunks/1","doc":"Accumulate streaming chunks into a complete response.\n\nUseful for collecting all chunks before processing.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#accumulate_chunks/1"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingBehavior.create_function_chunk/3","doc":"Create a stream chunk for function calls.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#create_function_chunk/3"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingBehavior.create_text_chunk/2","doc":"Create a stream chunk for text content.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#create_text_chunk/2"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingBehavior.handle_stream/4","doc":"Common streaming response handler that works with HTTPoison async responses.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#handle_stream/4"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.StreamingBehavior.handle_stream/4","doc":"- `:timeout` - Stream timeout in milliseconds (default: 5 minutes)\n- `:buffer` - Initial buffer content (default: \"\")\n- `:on_error` - Error callback function","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#handle_stream/4-options"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.StreamingBehavior.handle_stream/4","doc":"StreamingBehavior.handle_stream(ref, MyAdapter, fn chunk ->\n      # Process each chunk\n      send(self(), {:chunk, chunk})\n    end)","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#handle_stream/4-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingBehavior.parse_sse_stream/2","doc":"Parse Server-Sent Events from a data stream.\n\nReturns a list of parsed events and the remaining buffer.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#parse_sse_stream/2"},{"type":"callback","title":"ExLLM.Adapters.Shared.StreamingBehavior.parse_stream_chunk/1","doc":"Callback for parsing a streaming chunk into an ExLLM StreamChunk.\n\nEach provider has a different chunk format, so adapters must implement this.","ref":"ExLLM.Adapters.Shared.StreamingBehavior.html#c:parse_stream_chunk/1"},{"type":"module","title":"ExLLM.Adapters.Shared.StreamingCoordinator","doc":"Unified streaming coordinator for all LLM adapters.\n\nThis module provides a consistent streaming implementation that eliminates\ncode duplication across adapters. It handles:\n\n- Task management for async streaming\n- SSE (Server-Sent Events) parsing\n- Chunk buffering and processing\n- Error recovery integration\n- Provider-agnostic streaming patterns","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.create_stream_collector/4","doc":"Create a stream collector function for Req's into option.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#create_stream_collector/4"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.execute_stream/7","doc":"Execute the actual streaming request with unified SSE handling.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#execute_stream/7"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.parse_sse_line/1","doc":"Parse a single SSE line according to the SSE specification.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#parse_sse_line/1"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.process_stream_data/8","doc":"Process streaming data with unified SSE parsing and buffering.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#process_stream_data/8"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.simple_stream/1","doc":"Create a simple streaming implementation for adapters.\n\nThis is a high-level function that adapters can use to implement\nstreaming with minimal boilerplate.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#simple_stream/1"},{"type":"function","title":"Example - ExLLM.Adapters.Shared.StreamingCoordinator.simple_stream/1","doc":"def stream_chat(messages, options, callback) do\n      base_url = \"https://api.openai.com/v1\"\n      api_key = \"your-api-key\"\n      \n      StreamingCoordinator.simple_stream(\n        url: \"#{base_url}/chat/completions\",\n        request: build_request(messages, options),\n        headers: build_headers(api_key),\n        callback: callback,\n        parse_chunk: &parse_openai_chunk/1,\n        options: options\n      )\n    end","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#simple_stream/1-example"},{"type":"function","title":"ExLLM.Adapters.Shared.StreamingCoordinator.start_stream/5","doc":"Start a streaming request with unified handling.","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#start_stream/5"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.StreamingCoordinator.start_stream/5","doc":"- `:parse_chunk_fn` - Function to parse provider-specific chunks (required)\n- `:recovery_id` - Optional ID for stream recovery\n- `:timeout` - Stream timeout in milliseconds (default: 5 minutes)\n- `:on_error` - Error callback function\n- `:on_metrics` - Metrics callback function (receives streaming metrics)\n- `:transform_chunk` - Optional function to transform chunks before callback\n- `:buffer_chunks` - Buffer size for chunk batching (default: 1)\n- `:validate_chunk` - Optional function to validate chunks\n- `:track_metrics` - Enable detailed metrics tracking (default: false)","ref":"ExLLM.Adapters.Shared.StreamingCoordinator.html#start_stream/5-options"},{"type":"module","title":"ExLLM.Adapters.Shared.Validation","doc":"Shared validation functions for ExLLM adapters.\n\nThis module provides common validation patterns used across different\nadapters to ensure consistency and reduce code duplication.","ref":"ExLLM.Adapters.Shared.Validation.html"},{"type":"function","title":"ExLLM.Adapters.Shared.Validation.validate_api_key/1","doc":"Validates an API key is present and not empty.","ref":"ExLLM.Adapters.Shared.Validation.html#validate_api_key/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.Validation.validate_api_key/1","doc":"iex> Validation.validate_api_key(\"sk-1234567890\")\n    {:ok, :valid}\n    \n    iex> Validation.validate_api_key(nil)\n    {:error, \"API key not configured\"}\n    \n    iex> Validation.validate_api_key(\"\")\n    {:error, \"API key not configured\"}","ref":"ExLLM.Adapters.Shared.Validation.html#validate_api_key/1-examples"},{"type":"function","title":"ExLLM.Adapters.Shared.Validation.validate_base_url/1","doc":"Validates a base URL is properly formatted.","ref":"ExLLM.Adapters.Shared.Validation.html#validate_base_url/1"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.Validation.validate_base_url/1","doc":"iex> Validation.validate_base_url(\"https://api.example.com\")\n    {:ok, :valid}\n    \n    iex> Validation.validate_base_url(\"not-a-url\")\n    {:error, \"Invalid base URL format\"}","ref":"ExLLM.Adapters.Shared.Validation.html#validate_base_url/1-examples"},{"type":"behaviour","title":"ExLLM.Adapters.Shared.VisionFormatter","doc":"Unified vision/multimodal content formatting for LLM providers.\n\nThis module standardizes the handling of images and other multimodal\ncontent across different providers, reducing code duplication.\n\nFeatures:\n- Image format validation and detection\n- Base64 encoding/decoding\n- Provider-specific formatting\n- URL and file handling\n- Content type detection","ref":"ExLLM.Adapters.Shared.VisionFormatter.html"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.base64_image/3","doc":"Create a base64 image content part.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#base64_image/3"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.detect_media_type/2","doc":"Detect media type from file extension or magic bytes.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#detect_media_type/2"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.format_messages/2","doc":"Format messages containing vision content for a specific provider.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#format_messages/2"},{"type":"function","title":"Examples - ExLLM.Adapters.Shared.VisionFormatter.format_messages/2","doc":"messages = [\n      %{\n        role: \"user\",\n        content: [\n          %{type: \"text\", text: \"What's in this image?\"},\n          %{type: \"image_url\", image_url: %{url: \"https://example.com/cat.jpg\"}}\n        ]\n      }\n    ]\n    \n    formatted = VisionFormatter.format_messages(messages, :anthropic)","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#format_messages/2-examples"},{"type":"callback","title":"ExLLM.Adapters.Shared.VisionFormatter.format_vision_content/1","doc":"Callback for provider-specific vision content formatting.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#c:format_vision_content/1"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.has_vision_content?/1","doc":"Check if messages contain vision content.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#has_vision_content?/1"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.image_url/2","doc":"Create an image URL content part.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#image_url/2"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.load_image_file/2","doc":"Load an image from a file path and encode it.\n\nReturns a content part ready for inclusion in a message.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#load_image_file/2"},{"type":"function","title":"Options - ExLLM.Adapters.Shared.VisionFormatter.load_image_file/2","doc":"- `:format` - Force output format (:base64 or :url)\n- `:max_size` - Maximum file size in bytes\n- `:detail` - Image detail level (\"low\", \"high\", \"auto\")","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#load_image_file/2-options"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.message_has_vision_content?/1","doc":"Check if a single message has vision content.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#message_has_vision_content?/1"},{"type":"callback","title":"ExLLM.Adapters.Shared.VisionFormatter.model_supports_vision?/1","doc":"Callback to check if a model supports vision.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#c:model_supports_vision?/1"},{"type":"function","title":"ExLLM.Adapters.Shared.VisionFormatter.validate_media_type/1","doc":"Validate that an image format is supported.","ref":"ExLLM.Adapters.Shared.VisionFormatter.html#validate_media_type/1"},{"type":"module","title":"ExLLM.Adapters.XAI","doc":"Adapter for X.AI's Grok models.\n\nX.AI provides access to the Grok family of models including Grok-3, Grok-2,\nand their variants with different capabilities (vision, reasoning, etc.).","ref":"ExLLM.Adapters.XAI.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.XAI","doc":"The adapter requires an API key to be configured:\n\n    config :ex_llm, :xai,\n      api_key: System.get_env(\"XAI_API_KEY\")\n\nOr using environment variables:\n- `XAI_API_KEY` - Your X.AI API key","ref":"ExLLM.Adapters.XAI.html#module-configuration"},{"type":"module","title":"Supported Models - ExLLM.Adapters.XAI","doc":"- `grok-beta` - Grok Beta with 131K context\n- `grok-2-vision-1212` - Grok 2 with vision support\n- `grok-3-beta` - Grok 3 Beta with reasoning capabilities\n- `grok-3-mini-beta` - Smaller, faster Grok 3 variant\n\nSee `config/models/xai.yml` for the full list of models.","ref":"ExLLM.Adapters.XAI.html#module-supported-models"},{"type":"module","title":"Features - ExLLM.Adapters.XAI","doc":"- ✅ Streaming support\n- ✅ Function calling\n- ✅ Vision support (for vision models)\n- ✅ Web search capabilities\n- ✅ Structured outputs\n- ✅ Tool choice\n- ✅ Reasoning (Grok-3 models)","ref":"ExLLM.Adapters.XAI.html#module-features"},{"type":"module","title":"Example - ExLLM.Adapters.XAI","doc":"# Basic chat\n    {:ok, response} = ExLLM.chat(:xai, [\n      %{role: \"user\", content: \"What is the meaning of life?\"}\n    ])\n\n    # With vision\n    {:ok, response} = ExLLM.chat(:xai, [\n      %{\n        role: \"user\",\n        content: [\n          %{type: \"text\", text: \"What's in this image?\"},\n          %{type: \"image_url\", image_url: %{url: \"data:image/jpeg;base64,...\"}}\n        ]\n      }\n    ], model: \"grok-2-vision-1212\")\n\n    # Function calling\n    {:ok, response} = ExLLM.chat(:xai, messages,\n      model: \"grok-beta\",\n      tools: [weather_tool],\n      tool_choice: \"auto\"\n    )","ref":"ExLLM.Adapters.XAI.html#module-example"},{"type":"function","title":"ExLLM.Adapters.XAI.function_call/3","doc":"","ref":"ExLLM.Adapters.XAI.html#function_call/3"},{"type":"module","title":"ExLLM.Bumblebee.EXLAConfig","doc":"Configuration module for EXLA/EMLX backend optimization.\n\nProvides optimal settings for CPU and GPU inference, including Apple Silicon support.\nThis module automatically detects available hardware acceleration and configures\nthe appropriate backend for best performance.","ref":"ExLLM.Bumblebee.EXLAConfig.html"},{"type":"module","title":"Supported Backends - ExLLM.Bumblebee.EXLAConfig","doc":"- **EMLX** - Apple Silicon (Metal) acceleration\n- **CUDA** - NVIDIA GPU acceleration\n- **ROCm** - AMD GPU acceleration  \n- **CPU** - Optimized CPU inference","ref":"ExLLM.Bumblebee.EXLAConfig.html#module-supported-backends"},{"type":"module","title":"Features - ExLLM.Bumblebee.EXLAConfig","doc":"- Automatic hardware detection\n- Mixed precision support\n- Memory optimization\n- Dynamic batch sizing\n- Parallel execution configuration","ref":"ExLLM.Bumblebee.EXLAConfig.html#module-features"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.acceleration_info/0","doc":"Get information about available acceleration.\n\nReturns a map with acceleration details including type, name, and capabilities.","ref":"ExLLM.Bumblebee.EXLAConfig.html#acceleration_info/0"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.configure_backend/0","doc":"Configure EXLA/EMLX backend with optimal settings based on available hardware.\n\nReturns `{:ok, backend}` where backend is `:emlx`, `:cuda`, `:rocm`, `:cpu`, or `:binary`.","ref":"ExLLM.Bumblebee.EXLAConfig.html#configure_backend/0"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.determine_backend_options/0","doc":"Determine optimal backend options based on available hardware.\n\nReturns a map of backend configuration options.","ref":"ExLLM.Bumblebee.EXLAConfig.html#determine_backend_options/0"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.enable_mixed_precision/0","doc":"Enable mixed precision training/inference for better performance.","ref":"ExLLM.Bumblebee.EXLAConfig.html#enable_mixed_precision/0"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.optimize_memory/0","doc":"Optimize memory usage for large models.","ref":"ExLLM.Bumblebee.EXLAConfig.html#optimize_memory/0"},{"type":"function","title":"ExLLM.Bumblebee.EXLAConfig.serving_options/0","doc":"Get optimal compiler options for model serving.\n\nReturns keyword list of options for Bumblebee serving configuration.","ref":"ExLLM.Bumblebee.EXLAConfig.html#serving_options/0"},{"type":"module","title":"ExLLM.Bumblebee.ModelLoader","doc":"Handles loading and caching of Bumblebee models for local inference.\n\nThis GenServer manages the lifecycle of loaded models, ensuring efficient\nmemory usage and providing fast access to cached models.","ref":"ExLLM.Bumblebee.ModelLoader.html"},{"type":"module","title":"Features - ExLLM.Bumblebee.ModelLoader","doc":"- Automatic model downloading from HuggingFace\n- Model caching to avoid reloading\n- Memory management with model unloading\n- Hardware acceleration detection\n- Support for local model paths","ref":"ExLLM.Bumblebee.ModelLoader.html#module-features"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.Bumblebee.ModelLoader.html#child_spec/1"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.get_acceleration_info/0","doc":"Get information about hardware acceleration.","ref":"ExLLM.Bumblebee.ModelLoader.html#get_acceleration_info/0"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.get_model_info/1","doc":"Get information about a loaded model.","ref":"ExLLM.Bumblebee.ModelLoader.html#get_model_info/1"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"ExLLM.Bumblebee.ModelLoader.html#init/1"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.list_loaded_models/0","doc":"List all loaded models.","ref":"ExLLM.Bumblebee.ModelLoader.html#list_loaded_models/0"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.load_model/1","doc":"Load a model by name or path. Returns {:ok, model_info} or {:error, reason}.","ref":"ExLLM.Bumblebee.ModelLoader.html#load_model/1"},{"type":"function","title":"Examples - ExLLM.Bumblebee.ModelLoader.load_model/1","doc":"{:ok, model} = ModelLoader.load_model(\"microsoft/phi-2\")\n    {:ok, model} = ModelLoader.load_model(\"/path/to/local/model\")","ref":"ExLLM.Bumblebee.ModelLoader.html#load_model/1-examples"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.start_link/1","doc":"","ref":"ExLLM.Bumblebee.ModelLoader.html#start_link/1"},{"type":"function","title":"ExLLM.Bumblebee.ModelLoader.unload_model/1","doc":"Unload a model from memory.","ref":"ExLLM.Bumblebee.ModelLoader.html#unload_model/1"},{"type":"module","title":"ExLLM.Bumblebee.TokenCounter","doc":"Token counting utilities for local models.\n\nProvides accurate token counting when Bumblebee is available,\nwith fallback to heuristic estimation.","ref":"ExLLM.Bumblebee.TokenCounter.html"},{"type":"function","title":"ExLLM.Bumblebee.TokenCounter.count_messages/2","doc":"Count tokens for a list of messages.","ref":"ExLLM.Bumblebee.TokenCounter.html#count_messages/2"},{"type":"function","title":"Examples - ExLLM.Bumblebee.TokenCounter.count_messages/2","doc":"messages = [\n      %{role: \"user\", content: \"Hello\"},\n      %{role: \"assistant\", content: \"Hi there!\"}\n    ]\n    {:ok, count} = TokenCounter.count_messages(messages, model: \"microsoft/phi-2\")","ref":"ExLLM.Bumblebee.TokenCounter.html#count_messages/2-examples"},{"type":"function","title":"ExLLM.Bumblebee.TokenCounter.count_tokens/2","doc":"Count tokens for the given text using the specified model's tokenizer.\n\nFalls back to heuristic estimation if model is not loaded or Bumblebee\nis not available.","ref":"ExLLM.Bumblebee.TokenCounter.html#count_tokens/2"},{"type":"function","title":"Examples - ExLLM.Bumblebee.TokenCounter.count_tokens/2","doc":"{:ok, count} = TokenCounter.count_tokens(\"Hello world\", model: \"microsoft/phi-2\")\n    # => {:ok, 2}\n    \n    # Fallback when model not loaded\n    {:ok, estimate} = TokenCounter.count_tokens(\"Hello world\")\n    # => {:ok, 3}","ref":"ExLLM.Bumblebee.TokenCounter.html#count_tokens/2-examples"},{"type":"module","title":"ExLLM.Cache","doc":"Unified caching system for ExLLM providing both runtime performance caching\nand optional disk persistence for development/testing.","ref":"ExLLM.Cache.html"},{"type":"module","title":"Features - ExLLM.Cache","doc":"- Fast ETS-based runtime caching with TTL\n- Optional one-way disk persistence for test scenario collection\n- Automatic cache expiration and cleanup\n- Cache statistics and monitoring\n- Mock adapter integration via persisted responses","ref":"ExLLM.Cache.html#module-features"},{"type":"module","title":"Usage - ExLLM.Cache","doc":"# Runtime caching only (default)\n    {:ok, response} = ExLLM.chat(:anthropic, messages, cache: true)\n    \n    # With custom TTL\n    {:ok, response} = ExLLM.chat(:anthropic, messages, \n      cache: true,\n      cache_ttl: :timer.minutes(30)\n    )\n    \n    # Skip cache for this request\n    {:ok, response} = ExLLM.chat(:anthropic, messages, cache: false)","ref":"ExLLM.Cache.html#module-usage"},{"type":"module","title":"Disk Persistence - ExLLM.Cache","doc":"Enable disk persistence to automatically save cached responses for testing:\n\n    # Environment variable\n    export EX_LLM_CACHE_PERSIST=true\n    export EX_LLM_CACHE_DIR=\"/path/to/cache\"  # Optional\n    \n    # Or application config\n    config :ex_llm,\n      cache_persist_disk: true,\n      cache_disk_path: \"/tmp/ex_llm_cache\"\n\nWhen enabled, all cached responses are also written to disk and can be used\nby the Mock adapter for realistic testing without API calls.","ref":"ExLLM.Cache.html#module-disk-persistence"},{"type":"module","title":"Configuration - ExLLM.Cache","doc":"config :ex_llm, :cache,\n      enabled: true,\n      storage: {ExLLM.Cache.Storage.ETS, []},\n      default_ttl: :timer.minutes(15),\n      cleanup_interval: :timer.minutes(5),\n      persist_disk: false,\n      disk_path: \"/tmp/ex_llm_cache\"","ref":"ExLLM.Cache.html#module-configuration"},{"type":"function","title":"ExLLM.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.Cache.html#child_spec/1"},{"type":"function","title":"ExLLM.Cache.clear/0","doc":"Clear all cache entries.","ref":"ExLLM.Cache.html#clear/0"},{"type":"function","title":"ExLLM.Cache.configure_disk_persistence/2","doc":"Update disk persistence configuration at runtime.","ref":"ExLLM.Cache.html#configure_disk_persistence/2"},{"type":"function","title":"ExLLM.Cache.delete/1","doc":"Delete a specific cache entry.","ref":"ExLLM.Cache.html#delete/1"},{"type":"function","title":"ExLLM.Cache.generate_cache_key/3","doc":"Generate a cache key for a chat request.\n\nThe key is based on:\n- Provider\n- Model\n- Messages content\n- Relevant options (temperature, max_tokens, etc.)","ref":"ExLLM.Cache.html#generate_cache_key/3"},{"type":"function","title":"ExLLM.Cache.get/1","doc":"Get a cached response if available and not expired.","ref":"ExLLM.Cache.html#get/1"},{"type":"function","title":"ExLLM.Cache.put/3","doc":"Store a response in the cache with TTL.","ref":"ExLLM.Cache.html#put/3"},{"type":"function","title":"ExLLM.Cache.should_cache?/1","doc":"Check if caching should be used for this request.\n\nReturns false for:\n- Streaming requests\n- Requests with functions/tools\n- Explicitly disabled caching","ref":"ExLLM.Cache.html#should_cache?/1"},{"type":"function","title":"ExLLM.Cache.start_link/1","doc":"","ref":"ExLLM.Cache.html#start_link/1"},{"type":"function","title":"ExLLM.Cache.stats/0","doc":"Get cache statistics.","ref":"ExLLM.Cache.html#stats/0"},{"type":"function","title":"ExLLM.Cache.with_cache/3","doc":"Wrap a cache-aware function execution.\n\nThis is the main integration point for ExLLM modules.","ref":"ExLLM.Cache.html#with_cache/3"},{"type":"module","title":"ExLLM.Cache.Stats","doc":"Cache statistics.","ref":"ExLLM.Cache.Stats.html"},{"type":"type","title":"ExLLM.Cache.Stats.t/0","doc":"","ref":"ExLLM.Cache.Stats.html#t:t/0"},{"type":"behaviour","title":"ExLLM.Cache.Storage","doc":"Behaviour for cache storage backends.\n\nThis allows ExLLM to support different cache storage mechanisms\nlike ETS, Redis, Memcached, etc.","ref":"ExLLM.Cache.Storage.html"},{"type":"callback","title":"ExLLM.Cache.Storage.clear/1","doc":"Clear all entries from the cache.","ref":"ExLLM.Cache.Storage.html#c:clear/1"},{"type":"callback","title":"ExLLM.Cache.Storage.delete/2","doc":"Delete a key from the cache.","ref":"ExLLM.Cache.Storage.html#c:delete/2"},{"type":"callback","title":"ExLLM.Cache.Storage.get/2","doc":"Get a value from the cache.","ref":"ExLLM.Cache.Storage.html#c:get/2"},{"type":"callback","title":"ExLLM.Cache.Storage.info/1","doc":"Get storage info/stats (optional).","ref":"ExLLM.Cache.Storage.html#c:info/1"},{"type":"callback","title":"ExLLM.Cache.Storage.init/1","doc":"Initialize the storage backend.","ref":"ExLLM.Cache.Storage.html#c:init/1"},{"type":"callback","title":"ExLLM.Cache.Storage.list_keys/2","doc":"Get all keys matching a pattern (optional).","ref":"ExLLM.Cache.Storage.html#c:list_keys/2"},{"type":"callback","title":"ExLLM.Cache.Storage.put/4","doc":"Put a value in the cache with expiration time.","ref":"ExLLM.Cache.Storage.html#c:put/4"},{"type":"module","title":"ExLLM.Cache.Storage.ETS","doc":"ETS-based storage backend for ExLLM cache.\n\nProvides fast in-memory caching with automatic expiration.","ref":"ExLLM.Cache.Storage.ETS.html"},{"type":"module","title":"ExLLM.Cache.Storage.TestCache","doc":"Specialized storage backend for timestamp-based test response caching.\n\nThis module provides hierarchical organization by provider/test module/scenario\nwith timestamp-based file naming, rich metadata indexing, content deduplication,\nand smart fallback strategies for test response caching.","ref":"ExLLM.Cache.Storage.TestCache.html"},{"type":"type","title":"ExLLM.Cache.Storage.TestCache.cache_entry/0","doc":"","ref":"ExLLM.Cache.Storage.TestCache.html#t:cache_entry/0"},{"type":"type","title":"ExLLM.Cache.Storage.TestCache.cache_key/0","doc":"","ref":"ExLLM.Cache.Storage.TestCache.html#t:cache_key/0"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.clear/1","doc":"Clear cache entries for a specific cache key or pattern.","ref":"ExLLM.Cache.Storage.TestCache.html#clear/1"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.exists?/1","doc":"Check if a cached response exists and is valid.","ref":"ExLLM.Cache.Storage.TestCache.html#exists?/1"},{"type":"type","title":"ExLLM.Cache.Storage.TestCache.fallback_strategy/0","doc":"","ref":"ExLLM.Cache.Storage.TestCache.html#t:fallback_strategy/0"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.get/2","doc":"Retrieve a cached response using TTL and fallback strategy.","ref":"ExLLM.Cache.Storage.TestCache.html#get/2"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.get_stats/1","doc":"Get cache statistics for a specific key or all keys.","ref":"ExLLM.Cache.Storage.TestCache.html#get_stats/1"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.list_cache_keys/0","doc":"List all available cache keys.","ref":"ExLLM.Cache.Storage.TestCache.html#list_cache_keys/0"},{"type":"function","title":"ExLLM.Cache.Storage.TestCache.store/3","doc":"Store a test response with timestamp-based naming.","ref":"ExLLM.Cache.Storage.TestCache.html#store/3"},{"type":"module","title":"ExLLM.CachingInterceptor","doc":"Interceptor module for automatically caching provider responses.\n\nThis module provides functions to wrap adapter calls and automatically\ncache their responses for later use in testing with the Mock adapter.","ref":"ExLLM.CachingInterceptor.html"},{"type":"module","title":"Usage - ExLLM.CachingInterceptor","doc":"","ref":"ExLLM.CachingInterceptor.html#module-usage"},{"type":"module","title":"Environment-based Auto-caching - ExLLM.CachingInterceptor","doc":"# Enable caching for all providers\n    export EX_LLM_CACHE_RESPONSES=true\n    export EX_LLM_CACHE_DIR=\"/path/to/cache\"\n    \n    # Normal ExLLM usage will automatically cache responses\n    {:ok, response} = ExLLM.chat(messages, provider: :openai)","ref":"ExLLM.CachingInterceptor.html#module-environment-based-auto-caching"},{"type":"module","title":"Manual Caching Wrapper - ExLLM.CachingInterceptor","doc":"# Wrap a specific call to cache its response\n    {:ok, response} = ExLLM.CachingInterceptor.with_caching(:openai, fn ->\n      ExLLM.Adapters.OpenAI.chat(messages)\n    end)","ref":"ExLLM.CachingInterceptor.html#module-manual-caching-wrapper"},{"type":"module","title":"Batch Response Collection - ExLLM.CachingInterceptor","doc":"# Collect responses for testing scenarios\n    ExLLM.CachingInterceptor.collect_test_responses(:openai, [\n      {[%{role: \"user\", content: \"Hello\"}], []},\n      {[%{role: \"user\", content: \"What is 2+2?\"}], [max_tokens: 10]},\n      {[%{role: \"user\", content: \"Tell me a joke\"}], [temperature: 0.8]}\n    ])","ref":"ExLLM.CachingInterceptor.html#module-batch-response-collection"},{"type":"function","title":"ExLLM.CachingInterceptor.collect_test_responses/2","doc":"Collects responses for common test scenarios.\n\nThis function executes a list of test cases and caches their responses\nfor later use in testing.","ref":"ExLLM.CachingInterceptor.html#collect_test_responses/2"},{"type":"function","title":"ExLLM.CachingInterceptor.create_test_collection/1","doc":"Creates a comprehensive test response collection for a provider.","ref":"ExLLM.CachingInterceptor.html#create_test_collection/1"},{"type":"function","title":"ExLLM.CachingInterceptor.enable_auto_caching/1","doc":"Enables automatic caching for a specific provider.\n\nThis modifies the adapter's behavior to automatically cache all responses.","ref":"ExLLM.CachingInterceptor.html#enable_auto_caching/1"},{"type":"function","title":"ExLLM.CachingInterceptor.with_caching/2","doc":"Wraps an adapter call to automatically cache the response.","ref":"ExLLM.CachingInterceptor.html#with_caching/2"},{"type":"function","title":"ExLLM.CachingInterceptor.with_streaming_cache/4","doc":"Wraps a streaming adapter call to cache the complete stream.","ref":"ExLLM.CachingInterceptor.html#with_streaming_cache/4"},{"type":"module","title":"ExLLM.Capabilities","doc":"Unified capability querying with automatic normalization.\n\nThis is the main interface for checking capabilities across providers and models.\nIt handles normalization of different capability names used by various providers.","ref":"ExLLM.Capabilities.html"},{"type":"module","title":"Examples - ExLLM.Capabilities","doc":"# Check if a provider supports a capability\n    ExLLM.Capabilities.supports?(:openai, :function_calling)\n    # => true\n    \n    # Works with provider-specific names too\n    ExLLM.Capabilities.supports?(:anthropic, :tool_use)  # normalized to :function_calling\n    # => true\n    \n    # Check if a specific model supports a capability\n    ExLLM.Capabilities.model_supports?(:openai, \"gpt-4o\", :vision)\n    # => true\n    \n    # Find all providers that support a capability\n    ExLLM.Capabilities.find_providers(:image_generation)\n    # => [:openai]","ref":"ExLLM.Capabilities.html#module-examples"},{"type":"function","title":"ExLLM.Capabilities.display_name/1","doc":"Get human-readable name for a capability.","ref":"ExLLM.Capabilities.html#display_name/1"},{"type":"function","title":"ExLLM.Capabilities.find_models/1","doc":"Find all models that support a capability (normalized).","ref":"ExLLM.Capabilities.html#find_models/1"},{"type":"function","title":"ExLLM.Capabilities.find_providers/1","doc":"Find all providers that support a capability (normalized).","ref":"ExLLM.Capabilities.html#find_providers/1"},{"type":"function","title":"ExLLM.Capabilities.get_provider_capability_summary/1","doc":"Get detailed capability information for a provider.\n\nReturns both provider-level and model-level capabilities with normalization applied.","ref":"ExLLM.Capabilities.html#get_provider_capability_summary/1"},{"type":"function","title":"ExLLM.Capabilities.list_capabilities/0","doc":"List all normalized capability names.","ref":"ExLLM.Capabilities.html#list_capabilities/0"},{"type":"function","title":"ExLLM.Capabilities.model_supports?/2","doc":"Check if any model from a provider supports a capability.","ref":"ExLLM.Capabilities.html#model_supports?/2"},{"type":"function","title":"ExLLM.Capabilities.model_supports?/3","doc":"Check if a specific model supports a capability (normalized).","ref":"ExLLM.Capabilities.html#model_supports?/3"},{"type":"function","title":"ExLLM.Capabilities.normalize_capability/1","doc":"Get normalized capability name.","ref":"ExLLM.Capabilities.html#normalize_capability/1"},{"type":"function","title":"ExLLM.Capabilities.supports?/2","doc":"Check if a provider supports a capability (normalized).\n\nThis checks both provider-level capabilities and model-level capabilities.","ref":"ExLLM.Capabilities.html#supports?/2"},{"type":"behaviour","title":"ExLLM.ConfigProvider","doc":"Behaviour for configuration providers.\n\nThis allows modules to receive configuration through dependency injection\nrather than directly accessing application configuration, making them more \nportable and testable.","ref":"ExLLM.ConfigProvider.html"},{"type":"behaviour","title":"Example - ExLLM.ConfigProvider","doc":"# Using static configuration\n    config = %{\n      openai: %{api_key: \"sk-...\"},\n      anthropic: %{api_key: \"api-...\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.OpenAI.chat(messages, config_provider: provider)\n\n    # Using environment-based configuration\n    ExLLM.OpenAI.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.ConfigProvider.html#module-example"},{"type":"callback","title":"ExLLM.ConfigProvider.get/2","doc":"Gets configuration value for a provider and key.","ref":"ExLLM.ConfigProvider.html#c:get/2"},{"type":"callback","title":"ExLLM.ConfigProvider.get_all/1","doc":"Gets all configuration for a provider.","ref":"ExLLM.ConfigProvider.html#c:get_all/1"},{"type":"module","title":"ExLLM.ConfigProvider.Default","doc":"Default configuration provider that reads from environment variables.\nThis is an alias for the Env provider for backward compatibility.","ref":"ExLLM.ConfigProvider.Default.html"},{"type":"module","title":"ExLLM.ConfigProvider.Env","doc":"Environment-based configuration provider.\n\nReads configuration from environment variables using standard naming:\n- `OPENAI_API_KEY` for OpenAI\n- `ANTHROPIC_API_KEY` for Anthropic\n- etc.","ref":"ExLLM.ConfigProvider.Env.html"},{"type":"module","title":"ExLLM.ConfigProvider.Static","doc":"Static configuration provider for testing and library usage.","ref":"ExLLM.ConfigProvider.Static.html"},{"type":"module","title":"Usage - ExLLM.ConfigProvider.Static","doc":"config = %{\n      openai: %{api_key: \"sk-test\", model: \"gpt-4\"},\n      anthropic: %{api_key: \"api-test\", model: \"claude-3\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.OpenAI.chat(messages, config_provider: provider)","ref":"ExLLM.ConfigProvider.Static.html#module-usage"},{"type":"function","title":"ExLLM.ConfigProvider.Static.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.ConfigProvider.Static.html#child_spec/1"},{"type":"function","title":"ExLLM.ConfigProvider.Static.start_link/1","doc":"","ref":"ExLLM.ConfigProvider.Static.html#start_link/1"},{"type":"module","title":"ExLLM.Context","doc":"Context management for LLM conversations.\n\nProvides utilities for managing conversation context windows, including:\n- Token counting and estimation\n- Message truncation strategies\n- Context window validation\n- System prompt preservation","ref":"ExLLM.Context.html"},{"type":"function","title":"ExLLM.Context.get_context_window/2","doc":"Get the context window size for a given provider and model.\n\nReturns the maximum number of tokens the model can handle.\nRaises an error if the model is not found in configuration.","ref":"ExLLM.Context.html#get_context_window/2"},{"type":"function","title":"ExLLM.Context.get_token_allocation/3","doc":"Get optimal token allocation for different message types.\n\nReturns a map with recommended token allocations.","ref":"ExLLM.Context.html#get_token_allocation/3"},{"type":"function","title":"ExLLM.Context.truncate_messages/4","doc":"Truncate messages to fit within context window.\n\nSupports different truncation strategies:\n- `:sliding_window` - Remove old messages from the beginning\n- `:smart` - Preserve system message and recent messages, remove from middle","ref":"ExLLM.Context.html#truncate_messages/4"},{"type":"function","title":"ExLLM.Context.validate_context/4","doc":"Check if messages fit within the context window for a given model.\n\nReturns `{:ok, token_count}` if messages fit, or `{:error, reason}` if not.","ref":"ExLLM.Context.html#validate_context/4"},{"type":"module","title":"ExLLM.Context.Strategies","doc":"Message truncation strategies for context window management.\n\nProvides different algorithms for fitting messages within token limits:\n- Sliding window: Remove oldest messages first\n- Smart: Preserve system messages and prioritize recent messages\n- Summary: Replace old messages with summaries (future)","ref":"ExLLM.Context.Strategies.html"},{"type":"function","title":"ExLLM.Context.Strategies.calculate_distribution/2","doc":"Calculate token distribution for different message types.\n\nReturns suggested token allocations for system, conversation, and response.","ref":"ExLLM.Context.Strategies.html#calculate_distribution/2"},{"type":"function","title":"ExLLM.Context.Strategies.group_by_role/1","doc":"Group consecutive messages by role for better token efficiency.\n\nSome models handle grouped messages more efficiently.","ref":"ExLLM.Context.Strategies.html#group_by_role/1"},{"type":"function","title":"ExLLM.Context.Strategies.sliding_window/2","doc":"Sliding window truncation - keep most recent messages.","ref":"ExLLM.Context.Strategies.html#sliding_window/2"},{"type":"function","title":"ExLLM.Context.Strategies.smart_truncate/2","doc":"Smart truncation - preserve system messages and recent conversation.","ref":"ExLLM.Context.Strategies.html#smart_truncate/2"},{"type":"function","title":"ExLLM.Context.Strategies.split_system_messages/1","doc":"Split messages into system and non-system messages.","ref":"ExLLM.Context.Strategies.html#split_system_messages/1"},{"type":"function","title":"ExLLM.Context.Strategies.truncate/3","doc":"Apply a truncation strategy to fit messages within token limit.","ref":"ExLLM.Context.Strategies.html#truncate/3"},{"type":"function","title":"Strategies - ExLLM.Context.Strategies.truncate/3","doc":"- `:sliding_window` - Remove messages from the beginning\n- `:smart` - Keep system messages and recent messages\n- `:fifo` - First in, first out (alias for sliding_window)\n- `:lifo` - Last in, first out (keep oldest messages)","ref":"ExLLM.Context.Strategies.html#truncate/3-strategies"},{"type":"function","title":"Examples - ExLLM.Context.Strategies.truncate/3","doc":"Strategies.truncate(messages, 1000, :sliding_window)","ref":"ExLLM.Context.Strategies.html#truncate/3-examples"},{"type":"module","title":"ExLLM.Cost","doc":"Cost calculation functionality for ExLLM.\n\nProvides token estimation, cost calculation, and pricing information\nfor all supported LLM providers.","ref":"ExLLM.Cost.html"},{"type":"function","title":"ExLLM.Cost.calculate/3","doc":"Calculate cost for token usage.","ref":"ExLLM.Cost.html#calculate/3"},{"type":"function","title":"ExLLM.Cost.compare/2","doc":"Compare costs across different providers for the same usage.","ref":"ExLLM.Cost.html#compare/2"},{"type":"function","title":"ExLLM.Cost.estimate_tokens/1","doc":"Estimate token count for text using heuristic analysis.","ref":"ExLLM.Cost.html#estimate_tokens/1"},{"type":"function","title":"ExLLM.Cost.format/1","doc":"Format cost for human-readable display.","ref":"ExLLM.Cost.html#format/1"},{"type":"function","title":"ExLLM.Cost.get_pricing/2","doc":"Get pricing for a specific provider and model.","ref":"ExLLM.Cost.html#get_pricing/2"},{"type":"function","title":"ExLLM.Cost.list_pricing/0","doc":"List all available models and their pricing.","ref":"ExLLM.Cost.html#list_pricing/0"},{"type":"module","title":"ExLLM.Error","doc":"Standardized error types and utilities for ExLLM.\n\nThis module defines consistent error patterns used throughout ExLLM\nto provide predictable error handling for library consumers.","ref":"ExLLM.Error.html"},{"type":"module","title":"Error Patterns - ExLLM.Error","doc":"All functions should return either `{:ok, result}` or `{:error, reason}`.\n\nError reasons follow these patterns:","ref":"ExLLM.Error.html#module-error-patterns"},{"type":"module","title":"Simple Errors (atoms) - ExLLM.Error","doc":"- `:not_found` - Resource not found\n- `:not_connected` - Connection not established\n- `:invalid_config` - Configuration is invalid\n- `:not_configured` - Required configuration missing\n- `:timeout` - Operation timed out\n- `:unauthorized` - Authentication failed","ref":"ExLLM.Error.html#module-simple-errors-atoms"},{"type":"module","title":"Complex Errors (tuples) - ExLLM.Error","doc":"- `{:api_error, details}` - API call failed\n- `{:validation, field, message}` - Validation failed\n- `{:connection_failed, reason}` - Connection failure\n- `{:json_parse_error, reason}` - JSON parsing failed","ref":"ExLLM.Error.html#module-complex-errors-tuples"},{"type":"module","title":"Examples - ExLLM.Error","doc":"# Simple error\n    {:error, :not_found}\n\n    # API error with details\n    {:error, {:api_error, %{status: 404, body: \"Not found\"}}}\n\n    # Validation error\n    {:error, {:validation, :name, \"cannot be empty\"}}\n\n    # Connection error\n    {:error, {:connection_failed, :timeout}}","ref":"ExLLM.Error.html#module-examples"},{"type":"function","title":"ExLLM.Error.api_error/2","doc":"Creates a standardized API error.","ref":"ExLLM.Error.html#api_error/2"},{"type":"function","title":"Parameters - ExLLM.Error.api_error/2","doc":"- `status` - HTTP status code\n- `body` - Response body (string or map)","ref":"ExLLM.Error.html#api_error/2-parameters"},{"type":"function","title":"Returns - ExLLM.Error.api_error/2","doc":"API error tuple","ref":"ExLLM.Error.html#api_error/2-returns"},{"type":"function","title":"ExLLM.Error.authentication_error/1","doc":"Creates a standardized authentication error.","ref":"ExLLM.Error.html#authentication_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.authentication_error/1","doc":"- `message` - Error message","ref":"ExLLM.Error.html#authentication_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.authentication_error/1","doc":"Authentication error","ref":"ExLLM.Error.html#authentication_error/1-returns"},{"type":"type","title":"ExLLM.Error.complex_error/0","doc":"","ref":"ExLLM.Error.html#t:complex_error/0"},{"type":"function","title":"ExLLM.Error.connection_error/1","doc":"Creates a standardized connection failure error.","ref":"ExLLM.Error.html#connection_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.connection_error/1","doc":"- `reason` - Underlying failure reason","ref":"ExLLM.Error.html#connection_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.connection_error/1","doc":"Connection error tuple","ref":"ExLLM.Error.html#connection_error/1-returns"},{"type":"function","title":"ExLLM.Error.error?/1","doc":"Checks if a value is an error tuple.","ref":"ExLLM.Error.html#error?/1"},{"type":"function","title":"Parameters - ExLLM.Error.error?/1","doc":"- `value` - Value to check","ref":"ExLLM.Error.html#error?/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.error?/1","doc":"Boolean indicating if value is an error tuple","ref":"ExLLM.Error.html#error?/1-returns"},{"type":"type","title":"ExLLM.Error.error_reason/0","doc":"","ref":"ExLLM.Error.html#t:error_reason/0"},{"type":"function","title":"ExLLM.Error.get_error_reason/1","doc":"Extracts error reason from error tuple.","ref":"ExLLM.Error.html#get_error_reason/1"},{"type":"function","title":"Parameters - ExLLM.Error.get_error_reason/1","doc":"- `error_tuple` - Error tuple","ref":"ExLLM.Error.html#get_error_reason/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.get_error_reason/1","doc":"Error reason or nil if not an error tuple","ref":"ExLLM.Error.html#get_error_reason/1-returns"},{"type":"function","title":"ExLLM.Error.json_parse_error/1","doc":"Creates a standardized JSON parsing error.","ref":"ExLLM.Error.html#json_parse_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.json_parse_error/1","doc":"- `reason` - JSON parsing failure reason","ref":"ExLLM.Error.html#json_parse_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.json_parse_error/1","doc":"JSON parse error tuple","ref":"ExLLM.Error.html#json_parse_error/1-returns"},{"type":"function","title":"ExLLM.Error.rate_limit_error/1","doc":"Creates a standardized rate limit error.","ref":"ExLLM.Error.html#rate_limit_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.rate_limit_error/1","doc":"- `message` - Error message","ref":"ExLLM.Error.html#rate_limit_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.rate_limit_error/1","doc":"Rate limit error","ref":"ExLLM.Error.html#rate_limit_error/1-returns"},{"type":"type","title":"ExLLM.Error.result/1","doc":"","ref":"ExLLM.Error.html#t:result/1"},{"type":"function","title":"ExLLM.Error.service_unavailable/1","doc":"Creates a service unavailable error.","ref":"ExLLM.Error.html#service_unavailable/1"},{"type":"function","title":"Parameters - ExLLM.Error.service_unavailable/1","doc":"- `message` - Error message","ref":"ExLLM.Error.html#service_unavailable/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.service_unavailable/1","doc":"Service unavailable error","ref":"ExLLM.Error.html#service_unavailable/1-returns"},{"type":"type","title":"ExLLM.Error.simple_error/0","doc":"","ref":"ExLLM.Error.html#t:simple_error/0"},{"type":"function","title":"ExLLM.Error.unknown_error/1","doc":"Creates a standardized unknown error.","ref":"ExLLM.Error.html#unknown_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.unknown_error/1","doc":"- `reason` - Error reason","ref":"ExLLM.Error.html#unknown_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.unknown_error/1","doc":"Unknown error","ref":"ExLLM.Error.html#unknown_error/1-returns"},{"type":"function","title":"ExLLM.Error.validation_error/2","doc":"Creates a standardized validation error.","ref":"ExLLM.Error.html#validation_error/2"},{"type":"function","title":"Parameters - ExLLM.Error.validation_error/2","doc":"- `field` - Field name (atom)\n- `message` - Error message (string)","ref":"ExLLM.Error.html#validation_error/2-parameters"},{"type":"function","title":"Returns - ExLLM.Error.validation_error/2","doc":"Validation error tuple","ref":"ExLLM.Error.html#validation_error/2-returns"},{"type":"module","title":"ExLLM.FunctionCalling","doc":"Function calling support for ExLLM.\n\nProvides a unified interface for function/tool calling across different LLM providers.\nEach provider has slightly different implementations, but this module normalizes them\ninto a consistent API.","ref":"ExLLM.FunctionCalling.html"},{"type":"module","title":"Features - ExLLM.FunctionCalling","doc":"- Unified function definition format\n- Automatic parameter validation\n- Type conversion and coercion\n- Function execution with safety controls\n- Streaming function call support","ref":"ExLLM.FunctionCalling.html#module-features"},{"type":"module","title":"Usage - ExLLM.FunctionCalling","doc":"# Define available functions\n    functions = [\n      %{\n        name: \"get_weather\",\n        description: \"Get the current weather for a location\",\n        parameters: %{\n          type: \"object\",\n          properties: %{\n            location: %{type: \"string\", description: \"City and state\"},\n            unit: %{type: \"string\", enum: [\"celsius\", \"fahrenheit\"]}\n          },\n          required: [\"location\"]\n        }\n      }\n    ]\n    \n    # Chat with function calling\n    {:ok, response} = ExLLM.chat(:openai, messages,\n      functions: functions,\n      function_call: \"auto\"  # or \"none\" or %{name: \"specific_function\"}\n    )\n    \n    # Handle function calls in response\n    case response do\n      %{function_call: %{name: name, arguments: args}} ->\n        result = execute_function(name, args)\n        # Continue conversation with function result\n        \n      %{content: content} ->\n        # Normal response\n    end","ref":"ExLLM.FunctionCalling.html#module-usage"},{"type":"function","title":"ExLLM.FunctionCalling.execute_function/2","doc":"Executes a function call safely.","ref":"ExLLM.FunctionCalling.html#execute_function/2"},{"type":"function","title":"ExLLM.FunctionCalling.format_for_provider/2","doc":"Converts unified function format to provider-specific format.","ref":"ExLLM.FunctionCalling.html#format_for_provider/2"},{"type":"function","title":"ExLLM.FunctionCalling.format_function_result/2","doc":"Formats function results for conversation continuation.","ref":"ExLLM.FunctionCalling.html#format_function_result/2"},{"type":"function","title":"ExLLM.FunctionCalling.normalize_function/2","doc":"Normalizes a single function to unified format.","ref":"ExLLM.FunctionCalling.html#normalize_function/2"},{"type":"function","title":"ExLLM.FunctionCalling.normalize_functions/2","doc":"Converts provider-specific function format to unified format.","ref":"ExLLM.FunctionCalling.html#normalize_functions/2"},{"type":"function","title":"ExLLM.FunctionCalling.parse_arguments/1","doc":"Parse function call arguments from JSON string to map.","ref":"ExLLM.FunctionCalling.html#parse_arguments/1"},{"type":"function","title":"Parameters - ExLLM.FunctionCalling.parse_arguments/1","doc":"- `arguments` - JSON string or map containing function arguments","ref":"ExLLM.FunctionCalling.html#parse_arguments/1-parameters"},{"type":"function","title":"Returns - ExLLM.FunctionCalling.parse_arguments/1","doc":"- `{:ok, args_map}` on success\n- `{:error, reason}` on failure","ref":"ExLLM.FunctionCalling.html#parse_arguments/1-returns"},{"type":"function","title":"Examples - ExLLM.FunctionCalling.parse_arguments/1","doc":"{:ok, args} = ExLLM.FunctionCalling.parse_arguments(\"{\"location\": \"NYC\"}\")\n    # => {:ok, %{\"location\" => \"NYC\"}}","ref":"ExLLM.FunctionCalling.html#parse_arguments/1-examples"},{"type":"function","title":"ExLLM.FunctionCalling.parse_function_calls/2","doc":"Parses function calls from LLM response.","ref":"ExLLM.FunctionCalling.html#parse_function_calls/2"},{"type":"function","title":"ExLLM.FunctionCalling.validate_arguments/2","doc":"Validates function call arguments against schema.","ref":"ExLLM.FunctionCalling.html#validate_arguments/2"},{"type":"module","title":"ExLLM.FunctionCalling.Function","doc":"Represents a callable function/tool.","ref":"ExLLM.FunctionCalling.Function.html"},{"type":"type","title":"ExLLM.FunctionCalling.Function.t/0","doc":"","ref":"ExLLM.FunctionCalling.Function.html#t:t/0"},{"type":"module","title":"ExLLM.FunctionCalling.FunctionCall","doc":"Represents a function call request from the LLM.","ref":"ExLLM.FunctionCalling.FunctionCall.html"},{"type":"type","title":"ExLLM.FunctionCalling.FunctionCall.t/0","doc":"","ref":"ExLLM.FunctionCalling.FunctionCall.html#t:t/0"},{"type":"module","title":"ExLLM.FunctionCalling.FunctionResult","doc":"Represents the result of executing a function.","ref":"ExLLM.FunctionCalling.FunctionResult.html"},{"type":"type","title":"ExLLM.FunctionCalling.FunctionResult.t/0","doc":"","ref":"ExLLM.FunctionCalling.FunctionResult.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Auth","doc":"OAuth2 authentication helper for Google Gemini APIs that require user authentication.\n\nThis module provides utilities for obtaining and managing OAuth2 tokens for APIs\nlike the Permissions API that don't support API key authentication.","ref":"ExLLM.Gemini.Auth.html"},{"type":"module","title":"Usage - ExLLM.Gemini.Auth","doc":"# Option 1: Service Account (recommended for servers)\n    {:ok, token} = ExLLM.Gemini.Auth.get_service_account_token()\n    \n    # Option 2: User OAuth2 flow\n    {:ok, auth_url} = ExLLM.Gemini.Auth.get_authorization_url()\n    # ... user authorizes ...\n    {:ok, tokens} = ExLLM.Gemini.Auth.exchange_code(auth_code)\n    \n    # Use with Permissions API\n    ExLLM.Gemini.Permissions.list_permissions(\"tunedModels/my-model\",\n      oauth_token: token\n    )","ref":"ExLLM.Gemini.Auth.html#module-usage"},{"type":"function","title":"ExLLM.Gemini.Auth.cli_flow/1","doc":"Simple CLI flow for obtaining OAuth2 tokens.\n\nThis starts a local web server to receive the OAuth2 callback.\nUseful for CLI tools and development.","ref":"ExLLM.Gemini.Auth.html#cli_flow/1"},{"type":"function","title":"Options - ExLLM.Gemini.Auth.cli_flow/1","doc":"* `:port` - Local server port (default: 8080)\n* `:client_id` - Google OAuth2 client ID\n* `:client_secret` - Google OAuth2 client secret\n* `:scopes` - List of OAuth2 scopes","ref":"ExLLM.Gemini.Auth.html#cli_flow/1-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.cli_flow/1","doc":"{:ok, tokens} = ExLLM.Gemini.Auth.cli_flow(\n      client_id: \"your-client-id\",\n      client_secret: \"your-client-secret\"\n    )","ref":"ExLLM.Gemini.Auth.html#cli_flow/1-examples"},{"type":"function","title":"ExLLM.Gemini.Auth.exchange_code/2","doc":"Exchanges an authorization code for access and refresh tokens.","ref":"ExLLM.Gemini.Auth.html#exchange_code/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Auth.exchange_code/2","doc":"* `code` - The authorization code from the OAuth2 callback\n* `opts` - Options including client credentials and redirect URI","ref":"ExLLM.Gemini.Auth.html#exchange_code/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.exchange_code/2","doc":"{:ok, tokens} = ExLLM.Gemini.Auth.exchange_code(auth_code,\n      client_id: \"your-client-id\",\n      client_secret: \"your-client-secret\",\n      redirect_uri: \"http://localhost:8080/callback\"\n    )","ref":"ExLLM.Gemini.Auth.html#exchange_code/2-examples"},{"type":"function","title":"ExLLM.Gemini.Auth.get_authorization_url/1","doc":"Gets the authorization URL for the OAuth2 flow.","ref":"ExLLM.Gemini.Auth.html#get_authorization_url/1"},{"type":"function","title":"Options - ExLLM.Gemini.Auth.get_authorization_url/1","doc":"* `:client_id` - Google OAuth2 client ID (required)\n* `:redirect_uri` - Callback URL (default: \"http://localhost:8080/callback\")\n* `:scopes` - List of OAuth2 scopes (default: generative language scope)\n* `:access_type` - \"online\" or \"offline\" (default: \"offline\" for refresh tokens)\n* `:prompt` - \"none\", \"consent\", or \"select_account\" (default: \"consent\")\n* `:state` - Optional state parameter for CSRF protection","ref":"ExLLM.Gemini.Auth.html#get_authorization_url/1-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.get_authorization_url/1","doc":"{:ok, url} = ExLLM.Gemini.Auth.get_authorization_url(\n      client_id: \"your-client-id\",\n      scopes: [:generative_language, :tuning]\n    )","ref":"ExLLM.Gemini.Auth.html#get_authorization_url/1-examples"},{"type":"function","title":"ExLLM.Gemini.Auth.get_service_account_token/1","doc":"Gets a service account token using Application Default Credentials.\n\nThis requires either:\n1. GOOGLE_APPLICATION_CREDENTIALS environment variable pointing to a service account JSON file\n2. Running on Google Cloud with appropriate service account attached\n3. Using gcloud auth application-default login","ref":"ExLLM.Gemini.Auth.html#get_service_account_token/1"},{"type":"function","title":"Options - ExLLM.Gemini.Auth.get_service_account_token/1","doc":"* `:scopes` - List of OAuth2 scopes (default: generative language scope)","ref":"ExLLM.Gemini.Auth.html#get_service_account_token/1-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.get_service_account_token/1","doc":"# Requires Goth library to be added to your project\n    {:ok, token} = ExLLM.Gemini.Auth.get_service_account_token()","ref":"ExLLM.Gemini.Auth.html#get_service_account_token/1-examples"},{"type":"type","title":"ExLLM.Gemini.Auth.oauth_config/0","doc":"","ref":"ExLLM.Gemini.Auth.html#t:oauth_config/0"},{"type":"function","title":"ExLLM.Gemini.Auth.refresh_token/2","doc":"Refreshes an access token using a refresh token.","ref":"ExLLM.Gemini.Auth.html#refresh_token/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Auth.refresh_token/2","doc":"* `refresh_token` - The refresh token\n* `opts` - Options including client credentials","ref":"ExLLM.Gemini.Auth.html#refresh_token/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.refresh_token/2","doc":"{:ok, new_token} = ExLLM.Gemini.Auth.refresh_token(stored_refresh_token)","ref":"ExLLM.Gemini.Auth.html#refresh_token/2-examples"},{"type":"function","title":"ExLLM.Gemini.Auth.revoke_token/2","doc":"Revokes a token (access token or refresh token).","ref":"ExLLM.Gemini.Auth.html#revoke_token/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Auth.revoke_token/2","doc":"* `token` - The token to revoke\n* `opts` - Options (currently unused)","ref":"ExLLM.Gemini.Auth.html#revoke_token/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Auth.revoke_token/2","doc":":ok = ExLLM.Gemini.Auth.revoke_token(access_token)","ref":"ExLLM.Gemini.Auth.html#revoke_token/2-examples"},{"type":"type","title":"ExLLM.Gemini.Auth.token_response/0","doc":"","ref":"ExLLM.Gemini.Auth.html#t:token_response/0"},{"type":"function","title":"ExLLM.Gemini.Auth.validate_token/1","doc":"Validates if a token is still valid by checking with Google's tokeninfo endpoint.","ref":"ExLLM.Gemini.Auth.html#validate_token/1"},{"type":"function","title":"Parameters - ExLLM.Gemini.Auth.validate_token/1","doc":"* `access_token` - The access token to validate","ref":"ExLLM.Gemini.Auth.html#validate_token/1-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Auth.validate_token/1","doc":"* `{:ok, token_info}` - Token is valid, returns info including expiry\n* `{:error, reason}` - Token is invalid or expired","ref":"ExLLM.Gemini.Auth.html#validate_token/1-returns"},{"type":"module","title":"ExLLM.Gemini.Base","doc":"Base HTTP request functionality for Gemini API modules.","ref":"ExLLM.Gemini.Base.html"},{"type":"function","title":"ExLLM.Gemini.Base.request/1","doc":"Makes an HTTP request to the Gemini API.","ref":"ExLLM.Gemini.Base.html#request/1"},{"type":"type","title":"ExLLM.Gemini.Base.request_opts/0","doc":"","ref":"ExLLM.Gemini.Base.html#t:request_opts/0"},{"type":"function","title":"ExLLM.Gemini.Base.stream_request/2","doc":"Makes a streaming HTTP request to the Gemini API.\nReturns a function that takes a callback and streams responses.","ref":"ExLLM.Gemini.Base.html#stream_request/2"},{"type":"module","title":"ExLLM.Gemini.Caching","doc":"Google Gemini Context Caching API implementation.\n\nProvides functionality to cache and reuse large contexts across multiple\nrequests, reducing costs and improving performance for repeated queries\non the same content.","ref":"ExLLM.Gemini.Caching.html"},{"type":"function","title":"ExLLM.Gemini.Caching.create_cached_content/2","doc":"Creates a new cached content resource.","ref":"ExLLM.Gemini.Caching.html#create_cached_content/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Caching.create_cached_content/2","doc":"* `request` - Map containing:\n    * `:contents` - Content to cache\n    * `:model` - Model to use (required)\n    * `:ttl` or `:expire_time` - Expiration (one required)\n    * `:display_name` - Optional display name\n    * `:system_instruction` - Optional system instruction\n    * `:tools` - Optional tools\n    * `:tool_config` - Optional tool config\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Caching.html#create_cached_content/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Caching.create_cached_content/2","doc":"request = %{\n      contents: [%Content{role: \"user\", parts: [%Part{text: \"Large context\"}]}],\n      model: \"models/gemini-2.0-flash\",\n      ttl: \"3600s\",\n      display_name: \"My Cache\"\n    }\n    {:ok, cached} = ExLLM.Gemini.Caching.create_cached_content(request)","ref":"ExLLM.Gemini.Caching.html#create_cached_content/2-examples"},{"type":"type","title":"ExLLM.Gemini.Caching.create_options/0","doc":"","ref":"ExLLM.Gemini.Caching.html#t:create_options/0"},{"type":"function","title":"ExLLM.Gemini.Caching.delete_cached_content/2","doc":"Deletes cached content.","ref":"ExLLM.Gemini.Caching.html#delete_cached_content/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Caching.delete_cached_content/2","doc":"* `name` - The cached content name\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Caching.html#delete_cached_content/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Caching.delete_cached_content/2","doc":":ok = ExLLM.Gemini.Caching.delete_cached_content(\"cachedContents/abc-123\")","ref":"ExLLM.Gemini.Caching.html#delete_cached_content/2-examples"},{"type":"function","title":"ExLLM.Gemini.Caching.get_cached_content/2","doc":"Gets a specific cached content.","ref":"ExLLM.Gemini.Caching.html#get_cached_content/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Caching.get_cached_content/2","doc":"* `name` - The cached content name (e.g., \"cachedContents/abc-123\")\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Caching.html#get_cached_content/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Caching.get_cached_content/2","doc":"{:ok, cached} = ExLLM.Gemini.Caching.get_cached_content(\"cachedContents/abc-123\")","ref":"ExLLM.Gemini.Caching.html#get_cached_content/2-examples"},{"type":"function","title":"ExLLM.Gemini.Caching.list_cached_contents/1","doc":"Lists cached contents.","ref":"ExLLM.Gemini.Caching.html#list_cached_contents/1"},{"type":"function","title":"Parameters - ExLLM.Gemini.Caching.list_cached_contents/1","doc":"* `opts` - Options including `:page_size`, `:page_token`, and `:config_provider`","ref":"ExLLM.Gemini.Caching.html#list_cached_contents/1-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Caching.list_cached_contents/1","doc":"{:ok, %{cached_contents: contents}} = ExLLM.Gemini.Caching.list_cached_contents(page_size: 10)","ref":"ExLLM.Gemini.Caching.html#list_cached_contents/1-examples"},{"type":"type","title":"ExLLM.Gemini.Caching.list_options/0","doc":"","ref":"ExLLM.Gemini.Caching.html#t:list_options/0"},{"type":"function","title":"ExLLM.Gemini.Caching.update_cached_content/3","doc":"Updates cached content (only expiration can be updated).","ref":"ExLLM.Gemini.Caching.html#update_cached_content/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Caching.update_cached_content/3","doc":"* `name` - The cached content name\n  * `update` - Map containing either `:ttl` or `:expire_time`\n  * `opts` - Options including `:update_mask` and `:config_provider`","ref":"ExLLM.Gemini.Caching.html#update_cached_content/3-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Caching.update_cached_content/3","doc":"{:ok, updated} = ExLLM.Gemini.Caching.update_cached_content(\n      \"cachedContents/abc-123\",\n      %{ttl: \"7200s\"}\n    )","ref":"ExLLM.Gemini.Caching.html#update_cached_content/3-examples"},{"type":"type","title":"ExLLM.Gemini.Caching.update_options/0","doc":"","ref":"ExLLM.Gemini.Caching.html#t:update_options/0"},{"type":"module","title":"ExLLM.Gemini.Caching.CachedContent","doc":"Represents cached content that can be reused across requests.","ref":"ExLLM.Gemini.Caching.CachedContent.html"},{"type":"function","title":"ExLLM.Gemini.Caching.CachedContent.from_api/1","doc":"Converts API response to CachedContent struct.","ref":"ExLLM.Gemini.Caching.CachedContent.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Caching.CachedContent.t/0","doc":"","ref":"ExLLM.Gemini.Caching.CachedContent.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Caching.UsageMetadata","doc":"Token usage information for cached content.","ref":"ExLLM.Gemini.Caching.UsageMetadata.html"},{"type":"function","title":"ExLLM.Gemini.Caching.UsageMetadata.from_api/1","doc":"Converts API response to UsageMetadata struct.","ref":"ExLLM.Gemini.Caching.UsageMetadata.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Caching.UsageMetadata.t/0","doc":"","ref":"ExLLM.Gemini.Caching.UsageMetadata.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Chunk","doc":"Chunk management for Gemini's Semantic Retrieval API.\n\nA Chunk is a subpart of a Document that is treated as an independent unit for \nthe purposes of vector representation and storage. A Corpus can have a maximum \nof 1 million Chunks.","ref":"ExLLM.Gemini.Chunk.html"},{"type":"module","title":"Authentication - ExLLM.Gemini.Chunk","doc":"Chunk operations require authentication. Both API key and OAuth2 are supported:\n\n- **API key**: Most operations work with API keys\n- **OAuth2**: Required for some operations, especially those involving user-specific data","ref":"ExLLM.Gemini.Chunk.html#module-authentication"},{"type":"module","title":"Examples - ExLLM.Gemini.Chunk","doc":"# Create a chunk\n    {:ok, chunk} = Chunk.create_chunk(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{data: %{string_value: \"This is chunk content.\"}},\n      api_key: \"your-api-key\"\n    )\n    \n    # List chunks with pagination\n    {:ok, result} = Chunk.list_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      page_size: 20,\n      api_key: \"your-api-key\"\n    )\n    \n    # Update chunk content\n    {:ok, updated} = Chunk.update_chunk(\n      \"corpora/my-corpus/documents/my-doc/chunks/my-chunk\",\n      %{data: %{string_value: \"Updated content\"}},\n      \"data\",\n      api_key: \"your-api-key\"\n    )\n    \n    # Batch create chunks\n    {:ok, result} = Chunk.batch_create_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      [\n        %{chunk: %{data: %{string_value: \"First chunk\"}}},\n        %{chunk: %{data: %{string_value: \"Second chunk\"}}}\n      ],\n      api_key: \"your-api-key\"\n    )\n    \n    # Delete chunk\n    :ok = Chunk.delete_chunk(\n      \"corpora/my-corpus/documents/my-doc/chunks/my-chunk\",\n      api_key: \"your-api-key\"\n    )","ref":"ExLLM.Gemini.Chunk.html#module-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.batch_create_chunks/3","doc":"Batch create Chunks in a Document.","ref":"ExLLM.Gemini.Chunk.html#batch_create_chunks/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.batch_create_chunks/3","doc":"- `parent` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `chunk_requests` - List of chunk creation requests (max 100)\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#batch_create_chunks/3-parameters"},{"type":"function","title":"Chunk Request Format - ExLLM.Gemini.Chunk.batch_create_chunks/3","doc":"Each request should have:\n- `:parent` - The document name (must match parent parameter)\n- `:chunk` - The chunk data (same format as create_chunk params)","ref":"ExLLM.Gemini.Chunk.html#batch_create_chunks/3-chunk-request-format"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.batch_create_chunks/3","doc":"{:ok, result} = Chunk.batch_create_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      [\n        %{\n          parent: \"corpora/my-corpus/documents/my-doc\",\n          chunk: %{data: %{string_value: \"First chunk content\"}}\n        },\n        %{\n          parent: \"corpora/my-corpus/documents/my-doc\",\n          chunk: %{\n            name: \"corpora/my-corpus/documents/my-doc/chunks/special-chunk\",\n            data: %{string_value: \"Second chunk with custom name\"},\n            custom_metadata: [%{key: \"type\", string_value: \"important\"}]\n          }\n        }\n      ]\n    )","ref":"ExLLM.Gemini.Chunk.html#batch_create_chunks/3-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.batch_delete_chunks/3","doc":"Batch delete Chunks from a Document.","ref":"ExLLM.Gemini.Chunk.html#batch_delete_chunks/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.batch_delete_chunks/3","doc":"- `parent` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `delete_requests` - List of chunk names to delete\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#batch_delete_chunks/3-parameters"},{"type":"function","title":"Delete Request Format - ExLLM.Gemini.Chunk.batch_delete_chunks/3","doc":"Each request should have:\n- `:name` - The full chunk name to delete","ref":"ExLLM.Gemini.Chunk.html#batch_delete_chunks/3-delete-request-format"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.batch_delete_chunks/3","doc":":ok = Chunk.batch_delete_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      [\n        %{name: \"corpora/my-corpus/documents/my-doc/chunks/chunk-1\"},\n        %{name: \"corpora/my-corpus/documents/my-doc/chunks/chunk-2\"}\n      ]\n    )","ref":"ExLLM.Gemini.Chunk.html#batch_delete_chunks/3-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.batch_update_chunks/3","doc":"Batch update Chunks in a Document.","ref":"ExLLM.Gemini.Chunk.html#batch_update_chunks/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.batch_update_chunks/3","doc":"- `parent` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `update_requests` - List of chunk update requests (max 100)\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#batch_update_chunks/3-parameters"},{"type":"function","title":"Update Request Format - ExLLM.Gemini.Chunk.batch_update_chunks/3","doc":"Each request should have:\n- `:chunk` - The chunk data with name and fields to update\n- `:update_mask` - Fields to update (\"data\", \"customMetadata\")","ref":"ExLLM.Gemini.Chunk.html#batch_update_chunks/3-update-request-format"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.batch_update_chunks/3","doc":"{:ok, result} = Chunk.batch_update_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      [\n        %{\n          chunk: %{\n            name: \"corpora/my-corpus/documents/my-doc/chunks/chunk-1\",\n            data: %{string_value: \"Updated content\"}\n          },\n          update_mask: \"data\"\n        },\n        %{\n          chunk: %{\n            name: \"corpora/my-corpus/documents/my-doc/chunks/chunk-2\",\n            custom_metadata: [%{key: \"status\", string_value: \"reviewed\"}]\n          },\n          update_mask: \"customMetadata\"\n        }\n      ]\n    )","ref":"ExLLM.Gemini.Chunk.html#batch_update_chunks/3-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.create_chunk/3","doc":"Creates a Chunk in the specified Document.","ref":"ExLLM.Gemini.Chunk.html#create_chunk/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.create_chunk/3","doc":"- `parent` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `params` - Chunk creation parameters\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#create_chunk/3-parameters"},{"type":"function","title":"Params - ExLLM.Gemini.Chunk.create_chunk/3","doc":"- `:name` - Optional custom chunk name\n- `:data` - Required. The chunk content (max 2043 tokens)\n- `:custom_metadata` - List of key-value metadata (max 20 items)","ref":"ExLLM.Gemini.Chunk.html#create_chunk/3-params"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.create_chunk/3","doc":"# Create with auto-generated name\n    {:ok, chunk} = Chunk.create_chunk(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{data: %{string_value: \"This is the content.\"}}\n    )\n    \n    # Create with custom name and metadata\n    {:ok, chunk} = Chunk.create_chunk(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{\n        name: \"corpora/my-corpus/documents/my-doc/chunks/chapter-1\",\n        data: %{string_value: \"Chapter 1 content.\"},\n        custom_metadata: [\n          %{key: \"chapter\", numeric_value: 1},\n          %{key: \"keywords\", string_list_value: %{values: [\"intro\", \"overview\"]}}\n        ]\n      }\n    )","ref":"ExLLM.Gemini.Chunk.html#create_chunk/3-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.delete_chunk/2","doc":"Deletes a Chunk.","ref":"ExLLM.Gemini.Chunk.html#delete_chunk/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.delete_chunk/2","doc":"- `name` - The chunk name in format \"corpora/{corpus_id}/documents/{document_id}/chunks/{chunk_id}\"\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#delete_chunk/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.delete_chunk/2","doc":":ok = Chunk.delete_chunk(\"corpora/my-corpus/documents/my-doc/chunks/my-chunk\")","ref":"ExLLM.Gemini.Chunk.html#delete_chunk/2-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.get_chunk/2","doc":"Gets information about a specific Chunk.","ref":"ExLLM.Gemini.Chunk.html#get_chunk/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.get_chunk/2","doc":"- `name` - The chunk name in format \"corpora/{corpus_id}/documents/{document_id}/chunks/{chunk_id}\"\n- `opts` - Keyword list of options:\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#get_chunk/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.get_chunk/2","doc":"{:ok, chunk} = Chunk.get_chunk(\"corpora/my-corpus/documents/my-doc/chunks/my-chunk\")","ref":"ExLLM.Gemini.Chunk.html#get_chunk/2-examples"},{"type":"function","title":"ExLLM.Gemini.Chunk.list_chunks/2","doc":"Lists all Chunks in a Document with pagination support.","ref":"ExLLM.Gemini.Chunk.html#list_chunks/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.list_chunks/2","doc":"- `parent` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `opts` - Keyword list of options:\n  - `:page_size` - Maximum chunks per page (1-100, default 10)\n  - `:page_token` - Token for pagination\n  - `:api_key` - API key for authentication\n  - `:oauth_token` - OAuth2 token for authentication","ref":"ExLLM.Gemini.Chunk.html#list_chunks/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.list_chunks/2","doc":"# List all chunks\n    {:ok, result} = Chunk.list_chunks(\"corpora/my-corpus/documents/my-doc\")\n    \n    # List with pagination\n    {:ok, result} = Chunk.list_chunks(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{page_size: 50, page_token: \"next-page-token\"}\n    )","ref":"ExLLM.Gemini.Chunk.html#list_chunks/2-examples"},{"type":"type","title":"ExLLM.Gemini.Chunk.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Chunk.update_chunk/4","doc":"Updates a Chunk.","ref":"ExLLM.Gemini.Chunk.html#update_chunk/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.Chunk.update_chunk/4","doc":"- `name` - The chunk name in format \"corpora/{corpus_id}/documents/{document_id}/chunks/{chunk_id}\"\n- `updates` - Fields to update\n- `update_mask` - Required. Fields to update (\"data\", \"customMetadata\")\n- `opts` - Authentication options","ref":"ExLLM.Gemini.Chunk.html#update_chunk/4-parameters"},{"type":"function","title":"Updates - ExLLM.Gemini.Chunk.update_chunk/4","doc":"- `:data` - New chunk content\n- `:custom_metadata` - New custom metadata","ref":"ExLLM.Gemini.Chunk.html#update_chunk/4-updates"},{"type":"function","title":"Examples - ExLLM.Gemini.Chunk.update_chunk/4","doc":"# Update content\n    {:ok, chunk} = Chunk.update_chunk(\n      \"corpora/my-corpus/documents/my-doc/chunks/my-chunk\",\n      %{data: %{string_value: \"New content\"}},\n      \"data\"\n    )\n    \n    # Update metadata\n    {:ok, chunk} = Chunk.update_chunk(\n      \"corpora/my-corpus/documents/my-doc/chunks/my-chunk\",\n      %{custom_metadata: [%{key: \"status\", string_value: \"reviewed\"}]},\n      \"customMetadata\"\n    )","ref":"ExLLM.Gemini.Chunk.html#update_chunk/4-examples"},{"type":"module","title":"ExLLM.Gemini.Chunk.BatchResult","doc":"Response from batch operations containing a list of chunks.","ref":"ExLLM.Gemini.Chunk.BatchResult.html"},{"type":"type","title":"ExLLM.Gemini.Chunk.BatchResult.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.BatchResult.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Chunk.ChunkData","doc":"Extracted data that represents the Chunk content.","ref":"ExLLM.Gemini.Chunk.ChunkData.html"},{"type":"type","title":"ExLLM.Gemini.Chunk.ChunkData.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.ChunkData.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Chunk.CustomMetadata","doc":"User provided metadata stored as key-value pairs.","ref":"ExLLM.Gemini.Chunk.CustomMetadata.html"},{"type":"type","title":"ExLLM.Gemini.Chunk.CustomMetadata.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.CustomMetadata.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Chunk.ListResult","doc":"Response from listing chunks with pagination support.","ref":"ExLLM.Gemini.Chunk.ListResult.html"},{"type":"type","title":"ExLLM.Gemini.Chunk.ListResult.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.ListResult.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Chunk.StringList","doc":"User provided string values assigned to a single metadata key.","ref":"ExLLM.Gemini.Chunk.StringList.html"},{"type":"type","title":"ExLLM.Gemini.Chunk.StringList.t/0","doc":"","ref":"ExLLM.Gemini.Chunk.StringList.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content","doc":"Google Gemini Content Generation API implementation.\n\nProvides functionality for generating content using Gemini models including\ntext generation, streaming, multimodal inputs, function calling, and structured outputs.","ref":"ExLLM.Gemini.Content.html"},{"type":"function","title":"ExLLM.Gemini.Content.content_to_json/1","doc":"Converts Content struct to JSON format for API requests.","ref":"ExLLM.Gemini.Content.html#content_to_json/1"},{"type":"function","title":"ExLLM.Gemini.Content.generate_content/3","doc":"Generates content using a Gemini model.","ref":"ExLLM.Gemini.Content.html#generate_content/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Content.generate_content/3","doc":"* `model` - The model name (e.g., \"gemini-2.0-flash\")\n  * `request` - A GenerateContentRequest struct\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Content.html#generate_content/3-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Content.generate_content/3","doc":"request = %GenerateContentRequest{\n      contents: [\n        %Content{\n          role: \"user\",\n          parts: [%Part{text: \"Hello!\"}]\n        }\n      ]\n    }\n    \n    {:ok, response} = ExLLM.Gemini.Content.generate_content(\"gemini-2.0-flash\", request)","ref":"ExLLM.Gemini.Content.html#generate_content/3-examples"},{"type":"type","title":"ExLLM.Gemini.Content.options/0","doc":"","ref":"ExLLM.Gemini.Content.html#t:options/0"},{"type":"function","title":"ExLLM.Gemini.Content.stream_generate_content/3","doc":"Streams content generation using a Gemini model.\n\nReturns a stream of GenerateContentResponse chunks.","ref":"ExLLM.Gemini.Content.html#stream_generate_content/3"},{"type":"module","title":"ExLLM.Gemini.Content.Candidate","doc":"A generated candidate response.","ref":"ExLLM.Gemini.Content.Candidate.html"},{"type":"type","title":"ExLLM.Gemini.Content.Candidate.t/0","doc":"","ref":"ExLLM.Gemini.Content.Candidate.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.Content","doc":"Represents content with a role and parts.","ref":"ExLLM.Gemini.Content.Content.html"},{"type":"type","title":"ExLLM.Gemini.Content.Content.t/0","doc":"","ref":"ExLLM.Gemini.Content.Content.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.GenerateContentRequest","doc":"Request structure for content generation.","ref":"ExLLM.Gemini.Content.GenerateContentRequest.html"},{"type":"type","title":"ExLLM.Gemini.Content.GenerateContentRequest.t/0","doc":"","ref":"ExLLM.Gemini.Content.GenerateContentRequest.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Content.GenerateContentRequest.to_json/1","doc":"Converts GenerateContentRequest struct to JSON format for API requests.","ref":"ExLLM.Gemini.Content.GenerateContentRequest.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Content.GenerateContentResponse","doc":"Response from content generation.","ref":"ExLLM.Gemini.Content.GenerateContentResponse.html"},{"type":"type","title":"ExLLM.Gemini.Content.GenerateContentResponse.t/0","doc":"","ref":"ExLLM.Gemini.Content.GenerateContentResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.GenerationConfig","doc":"Configuration for content generation.","ref":"ExLLM.Gemini.Content.GenerationConfig.html"},{"type":"type","title":"ExLLM.Gemini.Content.GenerationConfig.t/0","doc":"","ref":"ExLLM.Gemini.Content.GenerationConfig.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.Part","doc":"Represents a content part which can be text, inline data, or function call/response.","ref":"ExLLM.Gemini.Content.Part.html"},{"type":"type","title":"ExLLM.Gemini.Content.Part.t/0","doc":"","ref":"ExLLM.Gemini.Content.Part.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Content.Part.to_json/1","doc":"Converts Part struct to JSON format for API requests.","ref":"ExLLM.Gemini.Content.Part.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Content.SafetySetting","doc":"Safety settings for content generation.","ref":"ExLLM.Gemini.Content.SafetySetting.html"},{"type":"type","title":"ExLLM.Gemini.Content.SafetySetting.t/0","doc":"","ref":"ExLLM.Gemini.Content.SafetySetting.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.Tool","doc":"Tool definitions for function calling.","ref":"ExLLM.Gemini.Content.Tool.html"},{"type":"type","title":"ExLLM.Gemini.Content.Tool.t/0","doc":"","ref":"ExLLM.Gemini.Content.Tool.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.ToolConfig","doc":"Configuration for tool usage.","ref":"ExLLM.Gemini.Content.ToolConfig.html"},{"type":"type","title":"ExLLM.Gemini.Content.ToolConfig.t/0","doc":"","ref":"ExLLM.Gemini.Content.ToolConfig.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Content.UsageMetadata","doc":"Token usage information.","ref":"ExLLM.Gemini.Content.UsageMetadata.html"},{"type":"type","title":"ExLLM.Gemini.Content.UsageMetadata.t/0","doc":"","ref":"ExLLM.Gemini.Content.UsageMetadata.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus","doc":"Google Gemini Corpus Management API implementation.\n\nCorpora are collections of Documents used for semantic retrieval. A project can \ncreate up to 5 corpora.","ref":"ExLLM.Gemini.Corpus.html"},{"type":"module","title":"Usage - ExLLM.Gemini.Corpus","doc":"# Create a corpus\n    {:ok, corpus} = ExLLM.Gemini.Corpus.create_corpus(%{\n      display_name: \"My Knowledge Base\"\n    }, oauth_token: \"your-oauth-token\")\n    \n    # List corpora\n    {:ok, response} = ExLLM.Gemini.Corpus.list_corpora(%{\n      page_size: 10\n    }, oauth_token: \"your-oauth-token\")\n    \n    # Query a corpus\n    {:ok, response} = ExLLM.Gemini.Corpus.query_corpus(\n      \"corpora/my-corpus\",\n      \"search query\",\n      %{results_count: 5},\n      oauth_token: \"your-oauth-token\"\n    )\n    \n    # Update corpus\n    {:ok, corpus} = ExLLM.Gemini.Corpus.update_corpus(\n      \"corpora/my-corpus\",\n      %{display_name: \"Updated Name\"},\n      [\"displayName\"],\n      oauth_token: \"your-oauth-token\"\n    )\n    \n    # Delete corpus\n    :ok = ExLLM.Gemini.Corpus.delete_corpus(\n      \"corpora/my-corpus\",\n      oauth_token: \"your-oauth-token\"\n    )","ref":"ExLLM.Gemini.Corpus.html#module-usage"},{"type":"function","title":"ExLLM.Gemini.Corpus.build_create_corpus_request/1","doc":"Builds a CreateCorpusRequest struct from the given parameters.","ref":"ExLLM.Gemini.Corpus.html#build_create_corpus_request/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.build_list_corpora_request/1","doc":"Builds a ListCorporaRequest struct from the given parameters.","ref":"ExLLM.Gemini.Corpus.html#build_list_corpora_request/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.build_query_corpus_request/3","doc":"Builds a QueryCorpusRequest struct from the given parameters.","ref":"ExLLM.Gemini.Corpus.html#build_query_corpus_request/3"},{"type":"function","title":"ExLLM.Gemini.Corpus.build_update_corpus_request/3","doc":"Builds an UpdateCorpusRequest struct from the given parameters.","ref":"ExLLM.Gemini.Corpus.html#build_update_corpus_request/3"},{"type":"function","title":"ExLLM.Gemini.Corpus.create_corpus/2","doc":"Creates a new corpus.","ref":"ExLLM.Gemini.Corpus.html#create_corpus/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.create_corpus/2","doc":"* `corpus_data` - Map containing corpus information\n  * `:name` - Optional corpus name (will be auto-generated if not provided)\n  * `:display_name` - Optional human-readable display name\n* `opts` - Options including OAuth2 token","ref":"ExLLM.Gemini.Corpus.html#create_corpus/2-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Corpus.create_corpus/2","doc":"* `:oauth_token` - OAuth2 token (required for corpus operations)","ref":"ExLLM.Gemini.Corpus.html#create_corpus/2-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.create_corpus/2","doc":"# Create corpus with auto-generated name\n    {:ok, corpus} = ExLLM.Gemini.Corpus.create_corpus(%{\n      display_name: \"My Knowledge Base\"\n    }, oauth_token: \"your-oauth-token\")\n    \n    # Create corpus with specific name\n    {:ok, corpus} = ExLLM.Gemini.Corpus.create_corpus(%{\n      name: \"corpora/my-custom-corpus\",\n      display_name: \"Custom Corpus\"\n    }, oauth_token: \"your-oauth-token\")","ref":"ExLLM.Gemini.Corpus.html#create_corpus/2-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.delete_corpus/2","doc":"Deletes a corpus.","ref":"ExLLM.Gemini.Corpus.html#delete_corpus/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.delete_corpus/2","doc":"* `corpus_name` - The name of the corpus to delete\n* `opts` - Options including OAuth2 token and force flag\n  * `:force` - If true, delete all documents in the corpus as well","ref":"ExLLM.Gemini.Corpus.html#delete_corpus/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.delete_corpus/2","doc":"# Delete empty corpus\n    :ok = ExLLM.Gemini.Corpus.delete_corpus(\n      \"corpora/my-corpus\",\n      oauth_token: \"your-oauth-token\"\n    )\n    \n    # Force delete corpus with documents\n    :ok = ExLLM.Gemini.Corpus.delete_corpus(\n      \"corpora/my-corpus\",\n      oauth_token: \"your-oauth-token\",\n      force: true\n    )","ref":"ExLLM.Gemini.Corpus.html#delete_corpus/2-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.format_operator/1","doc":"Formats an operator string from the API to an atom.","ref":"ExLLM.Gemini.Corpus.html#format_operator/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.get_corpus/2","doc":"Gets information about a specific corpus.","ref":"ExLLM.Gemini.Corpus.html#get_corpus/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.get_corpus/2","doc":"* `corpus_name` - The name of the corpus (e.g., \"corpora/my-corpus\")\n* `opts` - Options including OAuth2 token","ref":"ExLLM.Gemini.Corpus.html#get_corpus/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.get_corpus/2","doc":"{:ok, corpus} = ExLLM.Gemini.Corpus.get_corpus(\n      \"corpora/my-corpus\",\n      oauth_token: \"your-oauth-token\"\n    )","ref":"ExLLM.Gemini.Corpus.html#get_corpus/2-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.list_corpora/2","doc":"Lists corpora owned by the user.","ref":"ExLLM.Gemini.Corpus.html#list_corpora/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.list_corpora/2","doc":"* `list_options` - Map containing pagination options\n  * `:page_size` - Optional maximum number of corpora per page (1-20)\n  * `:page_token` - Optional page token for pagination\n* `opts` - Options including OAuth2 token","ref":"ExLLM.Gemini.Corpus.html#list_corpora/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.list_corpora/2","doc":"# List all corpora\n    {:ok, response} = ExLLM.Gemini.Corpus.list_corpora(%{}, \n      oauth_token: \"your-oauth-token\")\n    \n    # List with pagination\n    {:ok, response} = ExLLM.Gemini.Corpus.list_corpora(%{\n      page_size: 5,\n      page_token: \"next_page_token\"\n    }, oauth_token: \"your-oauth-token\")","ref":"ExLLM.Gemini.Corpus.html#list_corpora/2-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.operator_to_api_string/1","doc":"Converts an operator atom to the API string format.","ref":"ExLLM.Gemini.Corpus.html#operator_to_api_string/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.parse_corpus_response/1","doc":"Parses a corpus response from the API.","ref":"ExLLM.Gemini.Corpus.html#parse_corpus_response/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.parse_list_corpora_response/1","doc":"Parses a list corpora response from the API.","ref":"ExLLM.Gemini.Corpus.html#parse_list_corpora_response/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.parse_query_corpus_response/1","doc":"Parses a query corpus response from the API.","ref":"ExLLM.Gemini.Corpus.html#parse_query_corpus_response/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.query_corpus/4","doc":"Performs semantic search over a corpus.","ref":"ExLLM.Gemini.Corpus.html#query_corpus/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.query_corpus/4","doc":"* `corpus_name` - The name of the corpus to query\n* `query` - The search query string\n* `query_options` - Map containing query options\n  * `:results_count` - Maximum number of chunks to return (1-100)\n  * `:metadata_filters` - List of metadata filters\n* `opts` - Options including OAuth2 token","ref":"ExLLM.Gemini.Corpus.html#query_corpus/4-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.query_corpus/4","doc":"# Simple query\n    {:ok, response} = ExLLM.Gemini.Corpus.query_corpus(\n      \"corpora/my-corpus\",\n      \"artificial intelligence\",\n      %{results_count: 10},\n      oauth_token: \"your-oauth-token\"\n    )\n    \n    # Query with metadata filters\n    {:ok, response} = ExLLM.Gemini.Corpus.query_corpus(\n      \"corpora/my-corpus\",\n      \"machine learning\",\n      %{\n        results_count: 5,\n        metadata_filters: [\n          %{\n            key: \"document.custom_metadata.category\",\n            conditions: [\n              %{string_value: \"technology\", operation: \"EQUAL\"}\n            ]\n          }\n        ]\n      },\n      oauth_token: \"your-oauth-token\"\n    )","ref":"ExLLM.Gemini.Corpus.html#query_corpus/4-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.update_corpus/4","doc":"Updates a corpus.","ref":"ExLLM.Gemini.Corpus.html#update_corpus/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.Corpus.update_corpus/4","doc":"* `corpus_name` - The name of the corpus to update\n* `update_data` - Map containing fields to update\n  * `:display_name` - New display name\n* `update_mask` - List of fields to update (currently only [\"displayName\"] is supported)\n* `opts` - Options including OAuth2 token","ref":"ExLLM.Gemini.Corpus.html#update_corpus/4-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Corpus.update_corpus/4","doc":"{:ok, corpus} = ExLLM.Gemini.Corpus.update_corpus(\n      \"corpora/my-corpus\",\n      %{display_name: \"Updated Name\"},\n      [\"displayName\"],\n      oauth_token: \"your-oauth-token\"\n    )","ref":"ExLLM.Gemini.Corpus.html#update_corpus/4-examples"},{"type":"function","title":"ExLLM.Gemini.Corpus.valid_corpus_name?/1","doc":"Validates if a corpus name has the correct format.","ref":"ExLLM.Gemini.Corpus.html#valid_corpus_name?/1"},{"type":"function","title":"ExLLM.Gemini.Corpus.valid_display_name?/1","doc":"Validates if a display name has a valid length.","ref":"ExLLM.Gemini.Corpus.html#valid_display_name?/1"},{"type":"module","title":"ExLLM.Gemini.Corpus.Condition","doc":"Filter condition for metadata values.","ref":"ExLLM.Gemini.Corpus.Condition.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.Condition.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.Condition.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.CorpusInfo","doc":"Information about a corpus.","ref":"ExLLM.Gemini.Corpus.CorpusInfo.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.CorpusInfo.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.CorpusInfo.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.CreateCorpusRequest","doc":"Request structure for creating a corpus.","ref":"ExLLM.Gemini.Corpus.CreateCorpusRequest.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.CreateCorpusRequest.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.CreateCorpusRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.ListCorporaRequest","doc":"Request structure for listing corpora.","ref":"ExLLM.Gemini.Corpus.ListCorporaRequest.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.ListCorporaRequest.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.ListCorporaRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.ListCorporaResponse","doc":"Response from listing corpora.","ref":"ExLLM.Gemini.Corpus.ListCorporaResponse.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.ListCorporaResponse.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.ListCorporaResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.MetadataFilter","doc":"Filter for chunk and document metadata.","ref":"ExLLM.Gemini.Corpus.MetadataFilter.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.MetadataFilter.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.MetadataFilter.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.QueryCorpusRequest","doc":"Request structure for querying a corpus.","ref":"ExLLM.Gemini.Corpus.QueryCorpusRequest.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.QueryCorpusRequest.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.QueryCorpusRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.QueryCorpusResponse","doc":"Response from querying a corpus.","ref":"ExLLM.Gemini.Corpus.QueryCorpusResponse.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.QueryCorpusResponse.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.QueryCorpusResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.RelevantChunk","doc":"A chunk relevant to a query with its relevance score.","ref":"ExLLM.Gemini.Corpus.RelevantChunk.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.RelevantChunk.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.RelevantChunk.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Corpus.UpdateCorpusRequest","doc":"Request structure for updating a corpus.","ref":"ExLLM.Gemini.Corpus.UpdateCorpusRequest.html"},{"type":"type","title":"ExLLM.Gemini.Corpus.UpdateCorpusRequest.t/0","doc":"","ref":"ExLLM.Gemini.Corpus.UpdateCorpusRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document","doc":"Document management for Gemini's Semantic Retrieval API.\n\nA Document is a collection of Chunks. A Corpus can have a maximum of 10,000 Documents.\nDocuments are used to organize and query related content within a corpus.","ref":"ExLLM.Gemini.Document.html"},{"type":"module","title":"Authentication - ExLLM.Gemini.Document","doc":"Document operations require authentication. Both API key and OAuth2 are supported:\n\n- **API key**: Most operations work with API keys\n- **OAuth2**: Required for some operations, especially those involving user-specific data","ref":"ExLLM.Gemini.Document.html#module-authentication"},{"type":"module","title":"Examples - ExLLM.Gemini.Document","doc":"# Create a document\n    {:ok, document} = Document.create_document(\n      \"corpora/my-corpus\",\n      %{display_name: \"Research Papers\"}\n    )\n    \n    # List documents with pagination\n    {:ok, result} = Document.list_documents(\n      \"corpora/my-corpus\",\n      %{page_size: 10}\n    )\n    \n    # Query a document\n    {:ok, results} = Document.query_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      \"artificial intelligence\",\n      %{results_count: 5}\n    )\n    \n    # Update document\n    {:ok, updated} = Document.update_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{display_name: \"Updated Name\"},\n      %{update_mask: \"displayName\"}\n    )\n    \n    # Delete document\n    :ok = Document.delete_document(\"corpora/my-corpus/documents/my-doc\")","ref":"ExLLM.Gemini.Document.html#module-examples"},{"type":"function","title":"ExLLM.Gemini.Document.create_document/3","doc":"Creates an empty Document in the specified corpus.","ref":"ExLLM.Gemini.Document.html#create_document/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.create_document/3","doc":"- `parent` - The corpus name in format \"corpora/{corpus_id}\"\n- `params` - Document creation parameters\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#create_document/3-parameters"},{"type":"function","title":"Params - ExLLM.Gemini.Document.create_document/3","doc":"- `:name` - Optional custom document name\n- `:display_name` - Human-readable display name (max 512 characters)\n- `:custom_metadata` - List of key-value metadata (max 20 items)","ref":"ExLLM.Gemini.Document.html#create_document/3-params"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.create_document/3","doc":"# Create with auto-generated name\n    {:ok, doc} = Document.create_document(\n      \"corpora/my-corpus\",\n      %{display_name: \"Research Papers\"}\n    )\n    \n    # Create with custom name and metadata\n    {:ok, doc} = Document.create_document(\n      \"corpora/my-corpus\",\n      %{\n        name: \"corpora/my-corpus/documents/research-2024\",\n        display_name: \"Research Papers 2024\",\n        custom_metadata: [\n          %{key: \"category\", string_value: \"research\"},\n          %{key: \"year\", numeric_value: 2024}\n        ]\n      }\n    )","ref":"ExLLM.Gemini.Document.html#create_document/3-examples"},{"type":"function","title":"ExLLM.Gemini.Document.delete_document/2","doc":"Deletes a Document.","ref":"ExLLM.Gemini.Document.html#delete_document/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.delete_document/2","doc":"- `name` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `opts` - Delete options\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#delete_document/2-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Document.delete_document/2","doc":"- `:force` - If true, delete related chunks. If false (default), fail if chunks exist.","ref":"ExLLM.Gemini.Document.html#delete_document/2-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.delete_document/2","doc":"# Delete document (fails if chunks exist)\n    :ok = Document.delete_document(\"corpora/my-corpus/documents/my-doc\")\n    \n    # Force delete with chunks\n    :ok = Document.delete_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{force: true}\n    )","ref":"ExLLM.Gemini.Document.html#delete_document/2-examples"},{"type":"function","title":"ExLLM.Gemini.Document.get_document/2","doc":"Gets information about a specific Document.","ref":"ExLLM.Gemini.Document.html#get_document/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.get_document/2","doc":"- `name` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#get_document/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.get_document/2","doc":"{:ok, document} = Document.get_document(\"corpora/my-corpus/documents/my-doc\")","ref":"ExLLM.Gemini.Document.html#get_document/2-examples"},{"type":"function","title":"ExLLM.Gemini.Document.list_documents/2","doc":"Lists all Documents in a Corpus with pagination support.","ref":"ExLLM.Gemini.Document.html#list_documents/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.list_documents/2","doc":"- `parent` - The corpus name in format \"corpora/{corpus_id}\"\n- `opts` - Optional listing parameters\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#list_documents/2-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Document.list_documents/2","doc":"- `:page_size` - Maximum documents per page (1-20, default 10)\n- `:page_token` - Token for pagination","ref":"ExLLM.Gemini.Document.html#list_documents/2-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.list_documents/2","doc":"# List all documents\n    {:ok, result} = Document.list_documents(\"corpora/my-corpus\")\n    \n    # List with pagination\n    {:ok, result} = Document.list_documents(\n      \"corpora/my-corpus\",\n      %{page_size: 20, page_token: \"next-page-token\"}\n    )","ref":"ExLLM.Gemini.Document.html#list_documents/2-examples"},{"type":"function","title":"ExLLM.Gemini.Document.query_document/4","doc":"Performs semantic search over a Document.","ref":"ExLLM.Gemini.Document.html#query_document/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.query_document/4","doc":"- `name` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `query` - Query string to perform semantic search\n- `opts` - Query options\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#query_document/4-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Document.query_document/4","doc":"- `:results_count` - Maximum chunks to return (1-100, default 10)\n- `:metadata_filters` - List of metadata filters for chunk filtering","ref":"ExLLM.Gemini.Document.html#query_document/4-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.query_document/4","doc":"# Simple query\n    {:ok, result} = Document.query_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      \"artificial intelligence\"\n    )\n    \n    # Query with filters\n    {:ok, result} = Document.query_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      \"machine learning\",\n      %{\n        results_count: 5,\n        metadata_filters: [\n          %{\n            key: \"chunk.custom_metadata.category\",\n            conditions: [%{operation: \"EQUAL\", string_value: \"research\"}]\n          }\n        ]\n      }\n    )","ref":"ExLLM.Gemini.Document.html#query_document/4-examples"},{"type":"type","title":"ExLLM.Gemini.Document.t/0","doc":"","ref":"ExLLM.Gemini.Document.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Document.update_document/4","doc":"Updates a Document.","ref":"ExLLM.Gemini.Document.html#update_document/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.Document.update_document/4","doc":"- `name` - The document name in format \"corpora/{corpus_id}/documents/{document_id}\"\n- `updates` - Fields to update\n- `opts` - Update options including update_mask\n- `auth` - Authentication (API key or OAuth2 token)","ref":"ExLLM.Gemini.Document.html#update_document/4-parameters"},{"type":"function","title":"Updates - ExLLM.Gemini.Document.update_document/4","doc":"- `:display_name` - New display name\n- `:custom_metadata` - New custom metadata","ref":"ExLLM.Gemini.Document.html#update_document/4-updates"},{"type":"function","title":"Options - ExLLM.Gemini.Document.update_document/4","doc":"- `:update_mask` - Required. Fields to update (\"displayName\", \"customMetadata\")","ref":"ExLLM.Gemini.Document.html#update_document/4-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Document.update_document/4","doc":"# Update display name\n    {:ok, doc} = Document.update_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{display_name: \"New Name\"},\n      %{update_mask: \"displayName\"}\n    )\n    \n    # Update metadata\n    {:ok, doc} = Document.update_document(\n      \"corpora/my-corpus/documents/my-doc\",\n      %{custom_metadata: [%{key: \"status\", string_value: \"published\"}]},\n      %{update_mask: \"customMetadata\"}\n    )","ref":"ExLLM.Gemini.Document.html#update_document/4-examples"},{"type":"module","title":"ExLLM.Gemini.Document.Condition","doc":"Filter condition applicable to a single key.","ref":"ExLLM.Gemini.Document.Condition.html"},{"type":"type","title":"ExLLM.Gemini.Document.Condition.t/0","doc":"","ref":"ExLLM.Gemini.Document.Condition.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.CustomMetadata","doc":"User provided metadata stored as key-value pairs.","ref":"ExLLM.Gemini.Document.CustomMetadata.html"},{"type":"type","title":"ExLLM.Gemini.Document.CustomMetadata.t/0","doc":"","ref":"ExLLM.Gemini.Document.CustomMetadata.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.ListResult","doc":"Response from listing documents with pagination support.","ref":"ExLLM.Gemini.Document.ListResult.html"},{"type":"type","title":"ExLLM.Gemini.Document.ListResult.t/0","doc":"","ref":"ExLLM.Gemini.Document.ListResult.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.MetadataFilter","doc":"User provided filter to limit retrieval based on Chunk or Document level metadata values.","ref":"ExLLM.Gemini.Document.MetadataFilter.html"},{"type":"type","title":"ExLLM.Gemini.Document.MetadataFilter.t/0","doc":"","ref":"ExLLM.Gemini.Document.MetadataFilter.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.QueryResult","doc":"Response from document query containing a list of relevant chunks.","ref":"ExLLM.Gemini.Document.QueryResult.html"},{"type":"type","title":"ExLLM.Gemini.Document.QueryResult.t/0","doc":"","ref":"ExLLM.Gemini.Document.QueryResult.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.RelevantChunk","doc":"The information for a chunk relevant to a query.","ref":"ExLLM.Gemini.Document.RelevantChunk.html"},{"type":"type","title":"ExLLM.Gemini.Document.RelevantChunk.t/0","doc":"","ref":"ExLLM.Gemini.Document.RelevantChunk.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Document.StringList","doc":"User provided string values assigned to a single metadata key.","ref":"ExLLM.Gemini.Document.StringList.html"},{"type":"type","title":"ExLLM.Gemini.Document.StringList.t/0","doc":"","ref":"ExLLM.Gemini.Document.StringList.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Embeddings","doc":"Google Gemini Embeddings API implementation.\n\nProvides functionality to generate text embeddings using Gemini embedding models.\nSupports both single and batch embedding generation with various task types\nand configuration options.","ref":"ExLLM.Gemini.Embeddings.html"},{"type":"function","title":"ExLLM.Gemini.Embeddings.batch_embed_contents/3","doc":"Generates multiple embedding vectors from a batch of content.","ref":"ExLLM.Gemini.Embeddings.html#batch_embed_contents/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Embeddings.batch_embed_contents/3","doc":"* `model` - The model to use\n  * `requests` - List of EmbedContentRequest\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Embeddings.html#batch_embed_contents/3-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Embeddings.batch_embed_contents/3","doc":"requests = [\n      %EmbedContentRequest{\n        model: \"models/text-embedding-004\",\n        content: %Content{role: \"user\", parts: [%Part{text: \"First text\"}]}\n      },\n      %EmbedContentRequest{\n        model: \"models/text-embedding-004\",\n        content: %Content{role: \"user\", parts: [%Part{text: \"Second text\"}]}\n      }\n    ]\n    {:ok, embeddings} = ExLLM.Gemini.Embeddings.batch_embed_contents(\"models/text-embedding-004\", requests)","ref":"ExLLM.Gemini.Embeddings.html#batch_embed_contents/3-examples"},{"type":"function","title":"ExLLM.Gemini.Embeddings.embed_content/3","doc":"Generates a text embedding vector from the input Content.","ref":"ExLLM.Gemini.Embeddings.html#embed_content/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Embeddings.embed_content/3","doc":"* `model` - The model to use (e.g., \"models/text-embedding-004\")\n  * `request` - EmbedContentRequest with content and options\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Embeddings.html#embed_content/3-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Embeddings.embed_content/3","doc":"request = %EmbedContentRequest{\n      content: %Content{\n        role: \"user\",\n        parts: [%Part{text: \"Hello world\"}]\n      },\n      task_type: :retrieval_query\n    }\n    {:ok, embedding} = ExLLM.Gemini.Embeddings.embed_content(\"models/text-embedding-004\", request)","ref":"ExLLM.Gemini.Embeddings.html#embed_content/3-examples"},{"type":"function","title":"ExLLM.Gemini.Embeddings.embed_text/3","doc":"Convenience function to embed a single text string.","ref":"ExLLM.Gemini.Embeddings.html#embed_text/3"},{"type":"function","title":"Examples - ExLLM.Gemini.Embeddings.embed_text/3","doc":"{:ok, embedding} = ExLLM.Gemini.Embeddings.embed_text(\"models/text-embedding-004\", \"Hello world\")","ref":"ExLLM.Gemini.Embeddings.html#embed_text/3-examples"},{"type":"function","title":"ExLLM.Gemini.Embeddings.embed_texts/3","doc":"Convenience function to embed multiple text strings.","ref":"ExLLM.Gemini.Embeddings.html#embed_texts/3"},{"type":"function","title":"Examples - ExLLM.Gemini.Embeddings.embed_texts/3","doc":"texts = [\"First text\", \"Second text\", \"Third text\"]\n    {:ok, embeddings} = ExLLM.Gemini.Embeddings.embed_texts(\"models/text-embedding-004\", texts)","ref":"ExLLM.Gemini.Embeddings.html#embed_texts/3-examples"},{"type":"type","title":"ExLLM.Gemini.Embeddings.options/0","doc":"","ref":"ExLLM.Gemini.Embeddings.html#t:options/0"},{"type":"module","title":"ExLLM.Gemini.Embeddings.ContentEmbedding","doc":"A list of floats representing an embedding.","ref":"ExLLM.Gemini.Embeddings.ContentEmbedding.html"},{"type":"function","title":"ExLLM.Gemini.Embeddings.ContentEmbedding.from_api/1","doc":"Converts API response to ContentEmbedding struct.","ref":"ExLLM.Gemini.Embeddings.ContentEmbedding.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Embeddings.ContentEmbedding.t/0","doc":"","ref":"ExLLM.Gemini.Embeddings.ContentEmbedding.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Embeddings.EmbedContentRequest","doc":"Request containing the Content for the model to embed.","ref":"ExLLM.Gemini.Embeddings.EmbedContentRequest.html"},{"type":"type","title":"ExLLM.Gemini.Embeddings.EmbedContentRequest.t/0","doc":"","ref":"ExLLM.Gemini.Embeddings.EmbedContentRequest.html#t:t/0"},{"type":"type","title":"ExLLM.Gemini.Embeddings.EmbedContentRequest.task_type/0","doc":"","ref":"ExLLM.Gemini.Embeddings.EmbedContentRequest.html#t:task_type/0"},{"type":"module","title":"ExLLM.Gemini.Files","doc":"Google Gemini Files API implementation.\n\nProvides functionality to upload, manage, and delete files that can be used\nwith Gemini models for multimodal generation.","ref":"ExLLM.Gemini.Files.html"},{"type":"function","title":"ExLLM.Gemini.Files.delete_file/2","doc":"Deletes a file.","ref":"ExLLM.Gemini.Files.html#delete_file/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Files.delete_file/2","doc":"* `file_name` - The file name (e.g., \"files/abc-123\")\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Files.html#delete_file/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Files.delete_file/2","doc":":ok = ExLLM.Gemini.Files.delete_file(\"files/abc-123\")","ref":"ExLLM.Gemini.Files.html#delete_file/2-examples"},{"type":"function","title":"ExLLM.Gemini.Files.get_file/2","doc":"Gets metadata for a specific file.","ref":"ExLLM.Gemini.Files.html#get_file/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Files.get_file/2","doc":"* `file_name` - The file name (e.g., \"files/abc-123\")\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Files.html#get_file/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Files.get_file/2","doc":"{:ok, file} = ExLLM.Gemini.Files.get_file(\"files/abc-123\")","ref":"ExLLM.Gemini.Files.html#get_file/2-examples"},{"type":"function","title":"ExLLM.Gemini.Files.list_files/1","doc":"Lists files owned by the requesting project.","ref":"ExLLM.Gemini.Files.html#list_files/1"},{"type":"function","title":"Parameters - ExLLM.Gemini.Files.list_files/1","doc":"* `opts` - Options including `:page_size`, `:page_token`, and `:config_provider`","ref":"ExLLM.Gemini.Files.html#list_files/1-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Files.list_files/1","doc":"{:ok, %{files: files, next_page_token: token}} = ExLLM.Gemini.Files.list_files(page_size: 10)","ref":"ExLLM.Gemini.Files.html#list_files/1-examples"},{"type":"type","title":"ExLLM.Gemini.Files.list_options/0","doc":"","ref":"ExLLM.Gemini.Files.html#t:list_options/0"},{"type":"function","title":"ExLLM.Gemini.Files.upload_file/2","doc":"Uploads a file to the Gemini API using resumable upload.","ref":"ExLLM.Gemini.Files.html#upload_file/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Files.upload_file/2","doc":"* `file_path` - Path to the file to upload\n  * `opts` - Upload options including `:display_name` and `:config_provider`","ref":"ExLLM.Gemini.Files.html#upload_file/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Files.upload_file/2","doc":"{:ok, file} = ExLLM.Gemini.Files.upload_file(\"/path/to/image.png\", display_name: \"My Image\")","ref":"ExLLM.Gemini.Files.html#upload_file/2-examples"},{"type":"type","title":"ExLLM.Gemini.Files.upload_options/0","doc":"","ref":"ExLLM.Gemini.Files.html#t:upload_options/0"},{"type":"function","title":"ExLLM.Gemini.Files.wait_for_file/2","doc":"Waits for a file to become active (processed).","ref":"ExLLM.Gemini.Files.html#wait_for_file/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Files.wait_for_file/2","doc":"* `file_name` - The file name (e.g., \"files/abc-123\")\n  * `opts` - Options including `:timeout`, `:poll_interval`, and `:config_provider`","ref":"ExLLM.Gemini.Files.html#wait_for_file/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Files.wait_for_file/2","doc":"{:ok, file} = ExLLM.Gemini.Files.wait_for_file(\"files/abc-123\", timeout: 30_000)","ref":"ExLLM.Gemini.Files.html#wait_for_file/2-examples"},{"type":"type","title":"ExLLM.Gemini.Files.wait_options/0","doc":"","ref":"ExLLM.Gemini.Files.html#t:wait_options/0"},{"type":"module","title":"ExLLM.Gemini.Files.File","doc":"Represents an uploaded file in the Gemini API.","ref":"ExLLM.Gemini.Files.File.html"},{"type":"function","title":"ExLLM.Gemini.Files.File.from_api/1","doc":"Converts API response to File struct.","ref":"ExLLM.Gemini.Files.File.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Files.File.source/0","doc":"","ref":"ExLLM.Gemini.Files.File.html#t:source/0"},{"type":"type","title":"ExLLM.Gemini.Files.File.state/0","doc":"","ref":"ExLLM.Gemini.Files.File.html#t:state/0"},{"type":"type","title":"ExLLM.Gemini.Files.File.t/0","doc":"","ref":"ExLLM.Gemini.Files.File.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Files.Status","doc":"Error status information.","ref":"ExLLM.Gemini.Files.Status.html"},{"type":"function","title":"ExLLM.Gemini.Files.Status.from_api/1","doc":"Converts API response to Status struct.","ref":"ExLLM.Gemini.Files.Status.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Files.Status.t/0","doc":"","ref":"ExLLM.Gemini.Files.Status.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Files.VideoFileMetadata","doc":"Metadata for video files.","ref":"ExLLM.Gemini.Files.VideoFileMetadata.html"},{"type":"function","title":"ExLLM.Gemini.Files.VideoFileMetadata.from_api/1","doc":"Converts API response to VideoFileMetadata struct.","ref":"ExLLM.Gemini.Files.VideoFileMetadata.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Files.VideoFileMetadata.t/0","doc":"","ref":"ExLLM.Gemini.Files.VideoFileMetadata.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live","doc":"Google Gemini Live API implementation using WebSockets.\n\nThe Live API enables real-time bidirectional communication with Gemini models,\nsupporting text, audio, and video inputs with streaming responses.","ref":"ExLLM.Gemini.Live.html"},{"type":"module","title":"Features - ExLLM.Gemini.Live","doc":"- Real-time text, audio, and video streaming\n- Bidirectional communication with interruption support\n- Tool/function calling in real-time sessions\n- Session resumption capabilities\n- Activity detection and management\n- Audio transcription for both input and output","ref":"ExLLM.Gemini.Live.html#module-features"},{"type":"module","title":"Usage - ExLLM.Gemini.Live","doc":"# Start a live session\n    config = %{\n      model: \"models/gemini-2.5-flash-preview-05-20\",\n      generation_config: %{\n        temperature: 0.7,\n        response_modalities: [\"TEXT\", \"AUDIO\"]\n      },\n      system_instruction: \"You are a helpful assistant.\"\n    }\n    \n    {:ok, session} = Live.start_session(config, api_key: \"your-api-key\")\n    \n    # Send text message\n    :ok = Live.send_text(session, \"Hello, how are you?\")\n    \n    # Send audio data\n    :ok = Live.send_audio(session, audio_chunk)\n    \n    # Listen for responses\n    receive do\n      {:live_response, :server_content, content} ->\n        IO.puts(\"Model response: #{content.model_turn_content}\")\n      {:live_response, :tool_call, tool_call} ->\n        # Handle tool call\n        response = execute_tool(tool_call)\n        Live.send_tool_response(session, response)\n    end\n    \n    # Close session\n    Live.close_session(session)","ref":"ExLLM.Gemini.Live.html#module-usage"},{"type":"module","title":"Authentication - ExLLM.Gemini.Live","doc":"The Live API supports both API key and OAuth2 authentication:\n\n- **API key**: Passed as query parameter in WebSocket URL\n- **OAuth2**: Passed as Authorization header during WebSocket handshake","ref":"ExLLM.Gemini.Live.html#module-authentication"},{"type":"function","title":"ExLLM.Gemini.Live.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.Gemini.Live.html#child_spec/1"},{"type":"function","title":"ExLLM.Gemini.Live.close_session/1","doc":"Closes the Live API session.","ref":"ExLLM.Gemini.Live.html#close_session/1"},{"type":"function","title":"ExLLM.Gemini.Live.send_activity_end/1","doc":"Sends activity end signal to the session.\n\nThis is used when automatic activity detection is disabled.","ref":"ExLLM.Gemini.Live.html#send_activity_end/1"},{"type":"function","title":"ExLLM.Gemini.Live.send_activity_start/1","doc":"Sends activity start signal to the session.\n\nThis is used when automatic activity detection is disabled.","ref":"ExLLM.Gemini.Live.html#send_activity_start/1"},{"type":"function","title":"ExLLM.Gemini.Live.send_audio/2","doc":"Sends audio data to the session.","ref":"ExLLM.Gemini.Live.html#send_audio/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Live.send_audio/2","doc":"- `session` - Session process pid\n- `audio_data` - Binary audio data","ref":"ExLLM.Gemini.Live.html#send_audio/2-parameters"},{"type":"function","title":"ExLLM.Gemini.Live.send_realtime_text/2","doc":"Sends real-time text input to the session.\n\nThis is different from send_text/3 as it's designed for streaming text input\nthat doesn't interrupt model generation.","ref":"ExLLM.Gemini.Live.html#send_realtime_text/2"},{"type":"function","title":"ExLLM.Gemini.Live.send_text/3","doc":"Sends a text message to the session.","ref":"ExLLM.Gemini.Live.html#send_text/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Live.send_text/3","doc":"- `session` - Session process pid\n- `text` - Text content to send\n- `opts` - Options","ref":"ExLLM.Gemini.Live.html#send_text/3-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Live.send_text/3","doc":"- `:turn_complete` - Whether this completes the user's turn (default: true)","ref":"ExLLM.Gemini.Live.html#send_text/3-options"},{"type":"function","title":"ExLLM.Gemini.Live.send_tool_response/2","doc":"Sends tool/function response to the session.","ref":"ExLLM.Gemini.Live.html#send_tool_response/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Live.send_tool_response/2","doc":"- `session` - Session process pid\n- `function_responses` - List of function response objects","ref":"ExLLM.Gemini.Live.html#send_tool_response/2-parameters"},{"type":"function","title":"ExLLM.Gemini.Live.send_video/2","doc":"Sends video data to the session.","ref":"ExLLM.Gemini.Live.html#send_video/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Live.send_video/2","doc":"- `session` - Session process pid\n- `video_data` - Binary video data","ref":"ExLLM.Gemini.Live.html#send_video/2-parameters"},{"type":"function","title":"ExLLM.Gemini.Live.start_session/2","doc":"Starts a new Live API session.","ref":"ExLLM.Gemini.Live.html#start_session/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Live.start_session/2","doc":"- `config` - Session configuration containing model and parameters\n- `opts` - Authentication and options","ref":"ExLLM.Gemini.Live.html#start_session/2-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.Live.start_session/2","doc":"- `:api_key` - Google API key for authentication\n- `:oauth_token` - OAuth2 token for authentication (alternative to API key)\n- `:owner_pid` - Process to receive session messages (defaults to caller)","ref":"ExLLM.Gemini.Live.html#start_session/2-options"},{"type":"function","title":"Returns - ExLLM.Gemini.Live.start_session/2","doc":"- `{:ok, session_pid}` - Session GenServer process\n- `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Live.html#start_session/2-returns"},{"type":"function","title":"Examples - ExLLM.Gemini.Live.start_session/2","doc":"config = %{\n      model: \"models/gemini-2.5-flash-preview-05-20\",\n      generation_config: %{\n        temperature: 0.7,\n        response_modalities: [\"TEXT\", \"AUDIO\"]\n      }\n    }\n    \n    {:ok, session} = Live.start_session(config, api_key: \"your-api-key\")","ref":"ExLLM.Gemini.Live.html#start_session/2-examples"},{"type":"type","title":"ExLLM.Gemini.Live.t/0","doc":"","ref":"ExLLM.Gemini.Live.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.ClientContentMessage","doc":"Client content message","ref":"ExLLM.Gemini.Live.ClientContentMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.ClientContentMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.ClientContentMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.Content","doc":"Content structure for Live API","ref":"ExLLM.Gemini.Live.Content.html"},{"type":"type","title":"ExLLM.Gemini.Live.Content.t/0","doc":"","ref":"ExLLM.Gemini.Live.Content.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.FunctionCall","doc":"Function call details","ref":"ExLLM.Gemini.Live.FunctionCall.html"},{"type":"type","title":"ExLLM.Gemini.Live.FunctionCall.t/0","doc":"","ref":"ExLLM.Gemini.Live.FunctionCall.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.GenerationConfig","doc":"Generation configuration for Live API","ref":"ExLLM.Gemini.Live.GenerationConfig.html"},{"type":"type","title":"ExLLM.Gemini.Live.GenerationConfig.t/0","doc":"","ref":"ExLLM.Gemini.Live.GenerationConfig.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.GoAwayMessage","doc":"Server disconnect notification","ref":"ExLLM.Gemini.Live.GoAwayMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.GoAwayMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.GoAwayMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.RealtimeInputMessage","doc":"Real-time input message","ref":"ExLLM.Gemini.Live.RealtimeInputMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.RealtimeInputMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.RealtimeInputMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.ServerContentMessage","doc":"Server content message","ref":"ExLLM.Gemini.Live.ServerContentMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.ServerContentMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.ServerContentMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.SetupMessage","doc":"Initial session setup message","ref":"ExLLM.Gemini.Live.SetupMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.SetupMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.SetupMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.ToolCallMessage","doc":"Tool call message from server","ref":"ExLLM.Gemini.Live.ToolCallMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.ToolCallMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.ToolCallMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Live.ToolResponseMessage","doc":"Tool response message","ref":"ExLLM.Gemini.Live.ToolResponseMessage.html"},{"type":"type","title":"ExLLM.Gemini.Live.ToolResponseMessage.t/0","doc":"","ref":"ExLLM.Gemini.Live.ToolResponseMessage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Models","doc":"Google Gemini Models API implementation.\n\nProvides functionality to list available models and retrieve model metadata\nincluding token limits, supported features, and generation parameters.","ref":"ExLLM.Gemini.Models.html"},{"type":"function","title":"ExLLM.Gemini.Models.get_model/2","doc":"Gets information about a specific model.","ref":"ExLLM.Gemini.Models.html#get_model/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Models.get_model/2","doc":"* `model_name` - The model name (e.g., \"gemini-2.0-flash\")\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Models.html#get_model/2-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Models.get_model/2","doc":"{:ok, model} = ExLLM.Gemini.Models.get_model(\"gemini-2.0-flash\")","ref":"ExLLM.Gemini.Models.html#get_model/2-examples"},{"type":"function","title":"ExLLM.Gemini.Models.list_models/1","doc":"Lists available Gemini models.","ref":"ExLLM.Gemini.Models.html#list_models/1"},{"type":"function","title":"Options - ExLLM.Gemini.Models.list_models/1","doc":"* `:page_size` - Maximum number of models to return (max 1000)\n  * `:page_token` - Token for retrieving the next page\n  * `:config_provider` - Configuration provider (defaults to Application config)","ref":"ExLLM.Gemini.Models.html#list_models/1-options"},{"type":"function","title":"Examples - ExLLM.Gemini.Models.list_models/1","doc":"# List all models\n    {:ok, response} = ExLLM.Gemini.Models.list_models()\n    \n    # List with pagination\n    {:ok, response} = ExLLM.Gemini.Models.list_models(page_size: 10)\n    \n    # Get next page\n    {:ok, next_page} = ExLLM.Gemini.Models.list_models(page_token: response.next_page_token)","ref":"ExLLM.Gemini.Models.html#list_models/1-examples"},{"type":"type","title":"ExLLM.Gemini.Models.list_response/0","doc":"","ref":"ExLLM.Gemini.Models.html#t:list_response/0"},{"type":"type","title":"ExLLM.Gemini.Models.options/0","doc":"","ref":"ExLLM.Gemini.Models.html#t:options/0"},{"type":"module","title":"ExLLM.Gemini.Models.Model","doc":"Represents a Gemini model with its capabilities and metadata.","ref":"ExLLM.Gemini.Models.Model.html"},{"type":"function","title":"ExLLM.Gemini.Models.Model.from_api/1","doc":"Converts API response data to a Model struct.","ref":"ExLLM.Gemini.Models.Model.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Models.Model.t/0","doc":"","ref":"ExLLM.Gemini.Models.Model.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Permissions","doc":"Google Gemini Permissions API implementation.\n\nThis module provides functionality for managing permissions on tuned models\nand corpora in the Gemini API. Permissions control access levels (reader, writer, owner)\nfor users, groups, or everyone.","ref":"ExLLM.Gemini.Permissions.html"},{"type":"function","title":"ExLLM.Gemini.Permissions.create_permission/3","doc":"Creates a permission for a tuned model or corpus.","ref":"ExLLM.Gemini.Permissions.html#create_permission/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.create_permission/3","doc":"* `parent` - The parent resource (e.g., \"tunedModels/my-model\" or \"corpora/my-corpus\")\n* `permission` - The permission to create containing:\n  * `:grantee_type` - Type of grantee (USER, GROUP, or EVERYONE)\n  * `:email_address` - Email for USER or GROUP types\n  * `:role` - Permission level (READER, WRITER, or OWNER)\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Permissions.html#create_permission/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.create_permission/3","doc":"* `{:ok, permission}` - The created permission\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#create_permission/3-returns"},{"type":"function","title":"ExLLM.Gemini.Permissions.delete_permission/2","doc":"Deletes a permission.","ref":"ExLLM.Gemini.Permissions.html#delete_permission/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.delete_permission/2","doc":"* `name` - The resource name of the permission\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Permissions.html#delete_permission/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.delete_permission/2","doc":"* `{:ok, %{}}` - Empty response on success\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#delete_permission/2-returns"},{"type":"function","title":"ExLLM.Gemini.Permissions.get_permission/2","doc":"Gets information about a specific permission.","ref":"ExLLM.Gemini.Permissions.html#get_permission/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.get_permission/2","doc":"* `name` - The resource name (e.g., \"tunedModels/my-model/permissions/perm-123\")\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Permissions.html#get_permission/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.get_permission/2","doc":"* `{:ok, permission}` - The permission details\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#get_permission/2-returns"},{"type":"type","title":"ExLLM.Gemini.Permissions.grantee_type/0","doc":"","ref":"ExLLM.Gemini.Permissions.html#t:grantee_type/0"},{"type":"function","title":"ExLLM.Gemini.Permissions.list_permissions/2","doc":"Lists permissions for a tuned model or corpus.","ref":"ExLLM.Gemini.Permissions.html#list_permissions/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.list_permissions/2","doc":"* `parent` - The parent resource\n* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:page_size` - Maximum number of permissions to return\n  * `:page_token` - Token for pagination","ref":"ExLLM.Gemini.Permissions.html#list_permissions/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.list_permissions/2","doc":"* `{:ok, response}` - List of permissions\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#list_permissions/2-returns"},{"type":"type","title":"ExLLM.Gemini.Permissions.options/0","doc":"","ref":"ExLLM.Gemini.Permissions.html#t:options/0"},{"type":"type","title":"ExLLM.Gemini.Permissions.role/0","doc":"","ref":"ExLLM.Gemini.Permissions.html#t:role/0"},{"type":"function","title":"ExLLM.Gemini.Permissions.transfer_ownership/3","doc":"Transfers ownership of a tuned model.\n\nThe current owner will be downgraded to writer role.","ref":"ExLLM.Gemini.Permissions.html#transfer_ownership/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.transfer_ownership/3","doc":"* `name` - The resource name of the tuned model\n* `request` - Map containing `:email_address` of new owner\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Permissions.html#transfer_ownership/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.transfer_ownership/3","doc":"* `{:ok, %{}}` - Empty response on success\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#transfer_ownership/3-returns"},{"type":"function","title":"ExLLM.Gemini.Permissions.update_permission/3","doc":"Updates a permission's role.","ref":"ExLLM.Gemini.Permissions.html#update_permission/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Permissions.update_permission/3","doc":"* `name` - The resource name of the permission\n* `update` - Map containing `:role` to update\n* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:update_mask` - Required field mask (should be \"role\")","ref":"ExLLM.Gemini.Permissions.html#update_permission/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Permissions.update_permission/3","doc":"* `{:ok, permission}` - The updated permission\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Permissions.html#update_permission/3-returns"},{"type":"module","title":"ExLLM.Gemini.Permissions.ListPermissionsResponse","doc":"Response from listing permissions.","ref":"ExLLM.Gemini.Permissions.ListPermissionsResponse.html"},{"type":"function","title":"ExLLM.Gemini.Permissions.ListPermissionsResponse.from_json/1","doc":"","ref":"ExLLM.Gemini.Permissions.ListPermissionsResponse.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Permissions.ListPermissionsResponse.t/0","doc":"","ref":"ExLLM.Gemini.Permissions.ListPermissionsResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Permissions.Permission","doc":"Permission resource that grants access to a tuned model or corpus.","ref":"ExLLM.Gemini.Permissions.Permission.html"},{"type":"function","title":"ExLLM.Gemini.Permissions.Permission.from_json/1","doc":"","ref":"ExLLM.Gemini.Permissions.Permission.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Permissions.Permission.t/0","doc":"","ref":"ExLLM.Gemini.Permissions.Permission.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Permissions.Permission.to_json/1","doc":"","ref":"ExLLM.Gemini.Permissions.Permission.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Permissions.TransferOwnershipRequest","doc":"Request to transfer ownership of a tuned model.","ref":"ExLLM.Gemini.Permissions.TransferOwnershipRequest.html"},{"type":"type","title":"ExLLM.Gemini.Permissions.TransferOwnershipRequest.t/0","doc":"","ref":"ExLLM.Gemini.Permissions.TransferOwnershipRequest.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Permissions.TransferOwnershipRequest.to_json/1","doc":"","ref":"ExLLM.Gemini.Permissions.TransferOwnershipRequest.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.QA","doc":"Google Gemini Question Answering API implementation.\n\nThe Semantic Retrieval API provides a hosted question answering service for building \nRetrieval Augmented Generation (RAG) systems using Google's infrastructure.","ref":"ExLLM.Gemini.QA.html"},{"type":"module","title":"Usage - ExLLM.Gemini.QA","doc":"# With inline passages\n    contents = [\n      %{\n        parts: [%{text: \"What is the capital of France?\"}],\n        role: \"user\"\n      }\n    ]\n    \n    passages = [\n      %{\n        id: \"france_info\",\n        content: %{\n          parts: [%{text: \"France is a country in Europe. Paris is the capital of France.\"}]\n        }\n      }\n    ]\n    \n    {:ok, response} = ExLLM.Gemini.QA.generate_answer(\n      \"models/gemini-1.5-flash\",\n      contents,\n      :abstractive,\n      inline_passages: passages,\n      temperature: 0.1,\n      api_key: \"your-api-key\"\n    )\n    \n    # With semantic retriever\n    {:ok, response} = ExLLM.Gemini.QA.generate_answer(\n      \"models/gemini-1.5-flash\",\n      contents,\n      :verbose,\n      semantic_retriever: %{\n        source: \"corpora/my_corpus\",\n        query: %{parts: [%{text: \"capital of France\"}]},\n        max_chunks_count: 5\n      },\n      oauth_token: \"your-oauth-token\"\n    )","ref":"ExLLM.Gemini.QA.html#module-usage"},{"type":"function","title":"ExLLM.Gemini.QA.build_generate_answer_request/3","doc":"Builds a GenerateAnswerRequest struct from the given parameters.","ref":"ExLLM.Gemini.QA.html#build_generate_answer_request/3"},{"type":"function","title":"ExLLM.Gemini.QA.format_answer_style/1","doc":"Formats an answer style atom to the API string format.","ref":"ExLLM.Gemini.QA.html#format_answer_style/1"},{"type":"function","title":"ExLLM.Gemini.QA.format_block_reason/1","doc":"Formats a block reason string to an atom.","ref":"ExLLM.Gemini.QA.html#format_block_reason/1"},{"type":"function","title":"ExLLM.Gemini.QA.generate_answer/4","doc":"Generates a grounded answer from the model given an input.","ref":"ExLLM.Gemini.QA.html#generate_answer/4"},{"type":"function","title":"Parameters - ExLLM.Gemini.QA.generate_answer/4","doc":"* `model` - The name of the model to use (e.g., \"models/gemini-1.5-flash\")\n* `contents` - List of conversation content (messages)\n* `answer_style` - Style for the answer (:abstractive, :extractive, :verbose)\n* `opts` - Additional options","ref":"ExLLM.Gemini.QA.html#generate_answer/4-parameters"},{"type":"function","title":"Options - ExLLM.Gemini.QA.generate_answer/4","doc":"* `:inline_passages` - List of passages to use for grounding\n* `:semantic_retriever` - Configuration for semantic retrieval\n* `:temperature` - Controls randomness (0.0-1.0)\n* `:safety_settings` - List of safety settings\n* `:api_key` - Gemini API key\n* `:oauth_token` - OAuth2 token (alternative to API key)","ref":"ExLLM.Gemini.QA.html#generate_answer/4-options"},{"type":"function","title":"Examples - ExLLM.Gemini.QA.generate_answer/4","doc":"{:ok, response} = ExLLM.Gemini.QA.generate_answer(\n      \"models/gemini-1.5-flash\",\n      [%{parts: [%{text: \"What is AI?\"}], role: \"user\"}],\n      :abstractive,\n      inline_passages: [\n        %{\n          id: \"ai_definition\",\n          content: %{parts: [%{text: \"AI is artificial intelligence...\"}]}\n        }\n      ],\n      temperature: 0.1,\n      api_key: \"your-api-key\"\n    )","ref":"ExLLM.Gemini.QA.html#generate_answer/4-examples"},{"type":"function","title":"ExLLM.Gemini.QA.parse_generate_answer_response/1","doc":"Parses a GenerateAnswerResponse from the API response body.","ref":"ExLLM.Gemini.QA.html#parse_generate_answer_response/1"},{"type":"module","title":"ExLLM.Gemini.QA.GenerateAnswerRequest","doc":"Request structure for generating grounded answers.","ref":"ExLLM.Gemini.QA.GenerateAnswerRequest.html"},{"type":"type","title":"ExLLM.Gemini.QA.GenerateAnswerRequest.t/0","doc":"","ref":"ExLLM.Gemini.QA.GenerateAnswerRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.QA.GenerateAnswerResponse","doc":"Response structure for grounded answers.","ref":"ExLLM.Gemini.QA.GenerateAnswerResponse.html"},{"type":"type","title":"ExLLM.Gemini.QA.GenerateAnswerResponse.t/0","doc":"","ref":"ExLLM.Gemini.QA.GenerateAnswerResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.QA.GroundingPassage","doc":"A single passage included inline with a grounding configuration.","ref":"ExLLM.Gemini.QA.GroundingPassage.html"},{"type":"type","title":"ExLLM.Gemini.QA.GroundingPassage.t/0","doc":"","ref":"ExLLM.Gemini.QA.GroundingPassage.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.QA.GroundingPassages","doc":"A list of passages provided inline with the request.","ref":"ExLLM.Gemini.QA.GroundingPassages.html"},{"type":"type","title":"ExLLM.Gemini.QA.GroundingPassages.t/0","doc":"","ref":"ExLLM.Gemini.QA.GroundingPassages.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.QA.InputFeedback","doc":"Feedback related to the input data used to answer the question.","ref":"ExLLM.Gemini.QA.InputFeedback.html"},{"type":"type","title":"ExLLM.Gemini.QA.InputFeedback.t/0","doc":"","ref":"ExLLM.Gemini.QA.InputFeedback.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.QA.SemanticRetrieverConfig","doc":"Configuration for retrieving grounding content from a Corpus or Document \ncreated using the Semantic Retriever API.","ref":"ExLLM.Gemini.QA.SemanticRetrieverConfig.html"},{"type":"type","title":"ExLLM.Gemini.QA.SemanticRetrieverConfig.t/0","doc":"","ref":"ExLLM.Gemini.QA.SemanticRetrieverConfig.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tokens","doc":"Google Gemini Token Counting API implementation.\n\nProvides functionality to count tokens for content and generate content requests\nbefore sending them to the model. This helps estimate costs and ensure requests\nfit within model limits.","ref":"ExLLM.Gemini.Tokens.html"},{"type":"function","title":"ExLLM.Gemini.Tokens.count_tokens/3","doc":"Counts tokens for the given content or generate content request.","ref":"ExLLM.Gemini.Tokens.html#count_tokens/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tokens.count_tokens/3","doc":"* `model` - The model name (e.g., \"gemini-2.0-flash\")\n  * `request` - CountTokensRequest with either contents or generate_content_request\n  * `opts` - Options including `:config_provider`","ref":"ExLLM.Gemini.Tokens.html#count_tokens/3-parameters"},{"type":"function","title":"Examples - ExLLM.Gemini.Tokens.count_tokens/3","doc":"# Count tokens for simple content\n    request = %CountTokensRequest{\n      contents: [\n        %Content{\n          role: \"user\",\n          parts: [%Part{text: \"Hello world\"}]\n        }\n      ]\n    }\n    {:ok, response} = ExLLM.Gemini.Tokens.count_tokens(\"gemini-2.0-flash\", request)\n    \n    # Count tokens for full generate request\n    request = %CountTokensRequest{\n      generate_content_request: %GenerateContentRequest{\n        contents: [...],\n        system_instruction: %Content{...}\n      }\n    }\n    {:ok, response} = ExLLM.Gemini.Tokens.count_tokens(\"gemini-2.0-flash\", request)","ref":"ExLLM.Gemini.Tokens.html#count_tokens/3-examples"},{"type":"function","title":"ExLLM.Gemini.Tokens.normalize_model_name/1","doc":"Normalizes model name to include proper prefix.","ref":"ExLLM.Gemini.Tokens.html#normalize_model_name/1"},{"type":"type","title":"ExLLM.Gemini.Tokens.options/0","doc":"","ref":"ExLLM.Gemini.Tokens.html#t:options/0"},{"type":"function","title":"ExLLM.Gemini.Tokens.to_json/1","doc":"Converts a CountTokensRequest to JSON format for the API.","ref":"ExLLM.Gemini.Tokens.html#to_json/1"},{"type":"function","title":"ExLLM.Gemini.Tokens.validate_request/1","doc":"Validates that the request has either contents or generate_content_request, but not both.","ref":"ExLLM.Gemini.Tokens.html#validate_request/1"},{"type":"module","title":"ExLLM.Gemini.Tokens.CountTokensRequest","doc":"Request structure for counting tokens.\n\nEither `contents` or `generate_content_request` must be provided, but not both.","ref":"ExLLM.Gemini.Tokens.CountTokensRequest.html"},{"type":"type","title":"ExLLM.Gemini.Tokens.CountTokensRequest.t/0","doc":"","ref":"ExLLM.Gemini.Tokens.CountTokensRequest.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tokens.CountTokensResponse","doc":"Response from the token counting API.","ref":"ExLLM.Gemini.Tokens.CountTokensResponse.html"},{"type":"function","title":"ExLLM.Gemini.Tokens.CountTokensResponse.from_api/1","doc":"Converts API response to CountTokensResponse struct.","ref":"ExLLM.Gemini.Tokens.CountTokensResponse.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Tokens.CountTokensResponse.t/0","doc":"","ref":"ExLLM.Gemini.Tokens.CountTokensResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tokens.ModalityTokenCount","doc":"Token count breakdown by modality (TEXT, IMAGE, AUDIO, VIDEO).","ref":"ExLLM.Gemini.Tokens.ModalityTokenCount.html"},{"type":"function","title":"ExLLM.Gemini.Tokens.ModalityTokenCount.from_api/1","doc":"Converts API response to ModalityTokenCount struct.","ref":"ExLLM.Gemini.Tokens.ModalityTokenCount.html#from_api/1"},{"type":"type","title":"ExLLM.Gemini.Tokens.ModalityTokenCount.t/0","doc":"","ref":"ExLLM.Gemini.Tokens.ModalityTokenCount.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tuning","doc":"Google Gemini Fine-tuning API implementation.\n\nThis module provides functionality for creating and managing tuned models\nusing the Gemini API.","ref":"ExLLM.Gemini.Tuning.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.create_tuned_model/2","doc":"Creates a tuned model.","ref":"ExLLM.Gemini.Tuning.html#create_tuned_model/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.create_tuned_model/2","doc":"* `request` - The tuned model creation request containing:\n  * `:base_model` - The base model to tune (required)\n  * `:tuning_task` - The tuning task configuration (required)\n  * `:display_name` - Optional display name\n  * `:description` - Optional description\n  * `:temperature` - Optional temperature setting\n  * `:top_p` - Optional nucleus sampling parameter\n  * `:top_k` - Optional top-k sampling parameter\n* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:tuned_model_id` - Optional custom ID for the tuned model","ref":"ExLLM.Gemini.Tuning.html#create_tuned_model/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.create_tuned_model/2","doc":"* `{:ok, operation}` - The long-running operation for tracking tuning progress\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#create_tuned_model/2-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.delete_tuned_model/2","doc":"Deletes a tuned model.","ref":"ExLLM.Gemini.Tuning.html#delete_tuned_model/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.delete_tuned_model/2","doc":"* `name` - The resource name of the model\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Tuning.html#delete_tuned_model/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.delete_tuned_model/2","doc":"* `{:ok, %{}}` - Empty response on success\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#delete_tuned_model/2-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.generate_content/3","doc":"Generates content using a tuned model.","ref":"ExLLM.Gemini.Tuning.html#generate_content/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.generate_content/3","doc":"* `model` - The tuned model name (e.g., \"tunedModels/my-model\")\n* `request` - The generation request\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Tuning.html#generate_content/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.generate_content/3","doc":"* `{:ok, response}` - The generation response\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#generate_content/3-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.get_tuned_model/2","doc":"Gets information about a specific tuned model.","ref":"ExLLM.Gemini.Tuning.html#get_tuned_model/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.get_tuned_model/2","doc":"* `name` - The resource name of the model (e.g., \"tunedModels/my-model-id\")\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Tuning.html#get_tuned_model/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.get_tuned_model/2","doc":"* `{:ok, model}` - The tuned model details\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#get_tuned_model/2-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.list_tuned_models/1","doc":"Lists tuned models.","ref":"ExLLM.Gemini.Tuning.html#list_tuned_models/1"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.list_tuned_models/1","doc":"* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:page_size` - Maximum number of models to return\n  * `:page_token` - Token for pagination\n  * `:filter` - Filter expression","ref":"ExLLM.Gemini.Tuning.html#list_tuned_models/1-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.list_tuned_models/1","doc":"* `{:ok, response}` - List of tuned models\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#list_tuned_models/1-returns"},{"type":"type","title":"ExLLM.Gemini.Tuning.options/0","doc":"","ref":"ExLLM.Gemini.Tuning.html#t:options/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.stream_generate_content/3","doc":"Streams content generation using a tuned model.","ref":"ExLLM.Gemini.Tuning.html#stream_generate_content/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.stream_generate_content/3","doc":"* `model` - The tuned model name\n* `request` - The generation request\n* `opts` - Options including `:api_key`","ref":"ExLLM.Gemini.Tuning.html#stream_generate_content/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.stream_generate_content/3","doc":"* `{:ok, stream}` - Stream of response chunks\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#stream_generate_content/3-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.update_tuned_model/3","doc":"Updates a tuned model.","ref":"ExLLM.Gemini.Tuning.html#update_tuned_model/3"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.update_tuned_model/3","doc":"* `name` - The resource name of the model\n* `update` - Map of fields to update\n* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:update_mask` - Field mask specifying which fields to update","ref":"ExLLM.Gemini.Tuning.html#update_tuned_model/3-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.update_tuned_model/3","doc":"* `{:ok, model}` - The updated model\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#update_tuned_model/3-returns"},{"type":"function","title":"ExLLM.Gemini.Tuning.wait_for_tuning/2","doc":"Waits for a tuning operation to complete.","ref":"ExLLM.Gemini.Tuning.html#wait_for_tuning/2"},{"type":"function","title":"Parameters - ExLLM.Gemini.Tuning.wait_for_tuning/2","doc":"* `operation_name` - The operation name from create_tuned_model\n* `opts` - Options including:\n  * `:api_key` - Google API key\n  * `:timeout` - Maximum time to wait in milliseconds (default: 300_000)","ref":"ExLLM.Gemini.Tuning.html#wait_for_tuning/2-parameters"},{"type":"function","title":"Returns - ExLLM.Gemini.Tuning.wait_for_tuning/2","doc":"* `{:ok, model}` - The completed tuned model\n* `{:error, reason}` - Error details","ref":"ExLLM.Gemini.Tuning.html#wait_for_tuning/2-returns"},{"type":"module","title":"ExLLM.Gemini.Tuning.Dataset","doc":"Dataset for training or validation.","ref":"ExLLM.Gemini.Tuning.Dataset.html"},{"type":"type","title":"ExLLM.Gemini.Tuning.Dataset.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.Dataset.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.Dataset.to_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.Dataset.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Tuning.Hyperparameters","doc":"Hyperparameters controlling the tuning process.","ref":"ExLLM.Gemini.Tuning.Hyperparameters.html"},{"type":"type","title":"ExLLM.Gemini.Tuning.Hyperparameters.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.Hyperparameters.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.Hyperparameters.to_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.Hyperparameters.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Tuning.ListTunedModelsResponse","doc":"Response from listing tuned models.","ref":"ExLLM.Gemini.Tuning.ListTunedModelsResponse.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.ListTunedModelsResponse.from_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.ListTunedModelsResponse.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Tuning.ListTunedModelsResponse.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.ListTunedModelsResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tuning.TunedModel","doc":"A fine-tuned model created using the tuning API.","ref":"ExLLM.Gemini.Tuning.TunedModel.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.TunedModel.from_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TunedModel.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Tuning.TunedModel.state/0","doc":"","ref":"ExLLM.Gemini.Tuning.TunedModel.html#t:state/0"},{"type":"type","title":"ExLLM.Gemini.Tuning.TunedModel.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TunedModel.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tuning.TunedModelSource","doc":"Tuned model as a source for training a new model.","ref":"ExLLM.Gemini.Tuning.TunedModelSource.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.TunedModelSource.from_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TunedModelSource.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Tuning.TunedModelSource.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TunedModelSource.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tuning.TuningExample","doc":"A single example for tuning.","ref":"ExLLM.Gemini.Tuning.TuningExample.html"},{"type":"type","title":"ExLLM.Gemini.Tuning.TuningExample.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TuningExample.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.TuningExample.to_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TuningExample.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Tuning.TuningExamples","doc":"A set of tuning examples.","ref":"ExLLM.Gemini.Tuning.TuningExamples.html"},{"type":"type","title":"ExLLM.Gemini.Tuning.TuningExamples.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TuningExamples.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.TuningExamples.to_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TuningExamples.html#to_json/1"},{"type":"module","title":"ExLLM.Gemini.Tuning.TuningSnapshot","doc":"Record for a single tuning step.","ref":"ExLLM.Gemini.Tuning.TuningSnapshot.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.TuningSnapshot.from_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TuningSnapshot.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Tuning.TuningSnapshot.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TuningSnapshot.html#t:t/0"},{"type":"module","title":"ExLLM.Gemini.Tuning.TuningTask","doc":"Tuning task that creates tuned models.","ref":"ExLLM.Gemini.Tuning.TuningTask.html"},{"type":"function","title":"ExLLM.Gemini.Tuning.TuningTask.from_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TuningTask.html#from_json/1"},{"type":"type","title":"ExLLM.Gemini.Tuning.TuningTask.t/0","doc":"","ref":"ExLLM.Gemini.Tuning.TuningTask.html#t:t/0"},{"type":"function","title":"ExLLM.Gemini.Tuning.TuningTask.to_json/1","doc":"","ref":"ExLLM.Gemini.Tuning.TuningTask.html#to_json/1"},{"type":"module","title":"ExLLM.Instructor","doc":"Structured output support for ExLLM using instructor_ex.\n\nThis module provides integration with the `instructor` library to enable\nstructured outputs with validation when using ExLLM. It allows you to\ndefine expected response structures using Ecto schemas and automatically\nvalidates and retries LLM responses.","ref":"ExLLM.Instructor.html"},{"type":"module","title":"Supported Providers - ExLLM.Instructor","doc":"The following providers support structured outputs through Instructor:\n- `:anthropic` - Claude models\n- `:openai` - GPT models\n- `:gemini` - Google Gemini models\n- `:ollama` - Local Ollama models\n- `:groq` - Groq cloud models\n- `:xai` - X.AI Grok models\n\nOther providers (`:bedrock`, `:openrouter`, `:bumblebee`) do not currently\nsupport structured outputs through Instructor.","ref":"ExLLM.Instructor.html#module-supported-providers"},{"type":"module","title":"Requirements - ExLLM.Instructor","doc":"This module requires the `instructor` dependency:\n\n    {:instructor, \"~> 0.1.0\"}","ref":"ExLLM.Instructor.html#module-requirements"},{"type":"module","title":"Usage - ExLLM.Instructor","doc":"","ref":"ExLLM.Instructor.html#module-usage"},{"type":"module","title":"Basic Example - ExLLM.Instructor","doc":"defmodule EmailClassification do\n      use Ecto.Schema\n      use Instructor.Validator\n\n      @llm_doc \"Classification of an email as spam or not spam\"\n      \n      @primary_key false\n      embedded_schema do\n        field :classification, Ecto.Enum, values: [:spam, :not_spam]\n        field :confidence, :float\n        field :reason, :string\n      end\n\n      @impl true\n      def validate_changeset(changeset) do\n        changeset\n        |> Ecto.Changeset.validate_required([:classification, :confidence, :reason])\n        |> Ecto.Changeset.validate_number(:confidence, \n            greater_than_or_equal_to: 0.0,\n            less_than_or_equal_to: 1.0\n          )\n      end\n    end\n\n    # Use with ExLLM\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, [\n      %{role: \"user\", content: \"Is this spam? 'You won a million dollars!'\"}\n    ], response_model: EmailClassification)\n\n    # result is now an EmailClassification struct\n    # %EmailClassification{classification: :spam, confidence: 0.95, reason: \"...\"}","ref":"ExLLM.Instructor.html#module-basic-example"},{"type":"module","title":"With Retries - ExLLM.Instructor","doc":"{:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: UserProfile,\n      max_retries: 3,\n      temperature: 0.7\n    )","ref":"ExLLM.Instructor.html#module-with-retries"},{"type":"module","title":"With Simple Maps - ExLLM.Instructor","doc":"# Define expected structure\n    response_model = %{\n      name: :string,\n      age: :integer,\n      tags: {:array, :string}\n    }\n\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: response_model\n    )","ref":"ExLLM.Instructor.html#module-with-simple-maps"},{"type":"module","title":"Integration with ExLLM - ExLLM.Instructor","doc":"The structured output functionality is also available through the main\nExLLM module when instructor is installed:\n\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 2\n    )","ref":"ExLLM.Instructor.html#module-integration-with-exllm"},{"type":"function","title":"ExLLM.Instructor.available?/0","doc":"Check if instructor is available.\n\nSince instructor is now a required dependency, this always returns `true`.","ref":"ExLLM.Instructor.html#available?/0"},{"type":"function","title":"Returns - ExLLM.Instructor.available?/0","doc":"`true` (always)","ref":"ExLLM.Instructor.html#available?/0-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.available?/0","doc":"# This is now always true, but kept for backwards compatibility\n    if ExLLM.Instructor.available?() do\n      # Use structured outputs\n    end","ref":"ExLLM.Instructor.html#available?/0-examples"},{"type":"function","title":"ExLLM.Instructor.chat/3","doc":"Send a chat request with structured output validation.","ref":"ExLLM.Instructor.html#chat/3"},{"type":"function","title":"Parameters - ExLLM.Instructor.chat/3","doc":"- `provider` - The LLM provider to use\n- `messages` - List of conversation messages\n- `options` - Options including `:response_model` and standard chat options","ref":"ExLLM.Instructor.html#chat/3-parameters"},{"type":"function","title":"Options - ExLLM.Instructor.chat/3","doc":"- `:response_model` - Required. Ecto schema module or simple type specification\n- `:max_retries` - Number of retries for validation errors (default: 0)\n- All standard ExLLM.chat/3 options","ref":"ExLLM.Instructor.html#chat/3-options"},{"type":"function","title":"Returns - ExLLM.Instructor.chat/3","doc":"- `{:ok, struct}` where struct matches the response_model\n- `{:error, reason}` on failure","ref":"ExLLM.Instructor.html#chat/3-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.chat/3","doc":"# With Ecto schema\n    {:ok, classification} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 3\n    )\n\n    # With simple type spec\n    {:ok, data} = ExLLM.Instructor.chat(:openai, messages,\n      response_model: %{name: :string, age: :integer}\n    )","ref":"ExLLM.Instructor.html#chat/3-examples"},{"type":"function","title":"ExLLM.Instructor.parse_response/2","doc":"Transform a regular ExLLM response into a structured output.\n\nThis function is useful when you already have a response from ExLLM\nand want to parse it into a structured format.","ref":"ExLLM.Instructor.html#parse_response/2"},{"type":"function","title":"Parameters - ExLLM.Instructor.parse_response/2","doc":"- `response` - An ExLLM.Types.LLMResponse struct\n- `response_model` - The expected structure (Ecto schema or type spec)","ref":"ExLLM.Instructor.html#parse_response/2-parameters"},{"type":"function","title":"Returns - ExLLM.Instructor.parse_response/2","doc":"- `{:ok, struct}` on successful parsing and validation\n- `{:error, reason}` on failure","ref":"ExLLM.Instructor.html#parse_response/2-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.parse_response/2","doc":"{:ok, response} = ExLLM.chat(:anthropic, messages)\n    {:ok, structured} = ExLLM.Instructor.parse_response(response, UserProfile)","ref":"ExLLM.Instructor.html#parse_response/2-examples"},{"type":"function","title":"ExLLM.Instructor.simple_schema/2","doc":"Create a simple schema module at runtime.\n\nThis is a convenience function for creating simple schemas without\ndefining a full module.","ref":"ExLLM.Instructor.html#simple_schema/2"},{"type":"function","title":"Parameters - ExLLM.Instructor.simple_schema/2","doc":"- `fields` - Map of field names to types\n- `validations` - Optional list of validation functions","ref":"ExLLM.Instructor.html#simple_schema/2-parameters"},{"type":"function","title":"Examples - ExLLM.Instructor.simple_schema/2","doc":"schema = ExLLM.Instructor.simple_schema(%{\n      name: :string,\n      age: :integer,\n      email: :string\n    }, [\n      {:validate_format, :email, ~r/@/}\n    ])\n\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: schema\n    )","ref":"ExLLM.Instructor.html#simple_schema/2-examples"},{"type":"module","title":"ExLLM.Logger","doc":"Unified logging for ExLLM with automatic context and security features.\n\nThis module provides a simple, unified logging interface that combines\nthe simplicity of Elixir's Logger with ExLLM-specific features like\ncontext tracking, security redaction, and component filtering.","ref":"ExLLM.Logger.html"},{"type":"module","title":"Usage - ExLLM.Logger","doc":"Simple logging (like Elixir's Logger):\n\n    ExLLM.Logger.info(\"Model loaded successfully\")\n    ExLLM.Logger.error(\"Failed to connect\", error: reason)\n    ExLLM.Logger.debug(\"Processing chunk\", size: byte_size(data))\n\nWith provider context:\n\n    ExLLM.Logger.with_context(provider: :openai, operation: :chat) do\n      ExLLM.Logger.info(\"Starting chat request\")\n      # ... do work ...\n      ExLLM.Logger.info(\"Chat completed\", tokens: 150)\n    end\n\nStructured logging for specific events:\n\n    ExLLM.Logger.log_request(:openai, url, body, headers)\n    ExLLM.Logger.log_response(:openai, response, duration_ms)\n    ExLLM.Logger.log_retry(:anthropic, attempt, max_attempts, reason)","ref":"ExLLM.Logger.html#module-usage"},{"type":"module","title":"Configuration - ExLLM.Logger","doc":"config :ex_llm,\n      log_level: :info,  # :debug, :info, :warn, :error, :none\n      log_components: %{\n        requests: true,\n        responses: true,\n        streaming: false,\n        retries: true,\n        cache: false,\n        models: true\n      },\n      log_redaction: %{\n        api_keys: true,\n        content: false\n      }","ref":"ExLLM.Logger.html#module-configuration"},{"type":"function","title":"ExLLM.Logger.clear_context/0","doc":"Clear all context.","ref":"ExLLM.Logger.html#clear_context/0"},{"type":"function","title":"ExLLM.Logger.debug/2","doc":"Log a debug message with optional metadata.","ref":"ExLLM.Logger.html#debug/2"},{"type":"function","title":"ExLLM.Logger.error/2","doc":"Log an error message with optional metadata.","ref":"ExLLM.Logger.html#error/2"},{"type":"function","title":"ExLLM.Logger.info/2","doc":"Log an info message with optional metadata.","ref":"ExLLM.Logger.html#info/2"},{"type":"function","title":"ExLLM.Logger.log_cache_event/3","doc":"Log a cache event.","ref":"ExLLM.Logger.html#log_cache_event/3"},{"type":"function","title":"ExLLM.Logger.log_model_event/3","doc":"Log a model event.","ref":"ExLLM.Logger.html#log_model_event/3"},{"type":"function","title":"ExLLM.Logger.log_request/4","doc":"Log an API request with automatic redaction.","ref":"ExLLM.Logger.html#log_request/4"},{"type":"function","title":"ExLLM.Logger.log_response/3","doc":"Log an API response.","ref":"ExLLM.Logger.html#log_response/3"},{"type":"function","title":"ExLLM.Logger.log_retry/5","doc":"Log a retry attempt.","ref":"ExLLM.Logger.html#log_retry/5"},{"type":"function","title":"ExLLM.Logger.log_stream_event/3","doc":"Log a streaming event.","ref":"ExLLM.Logger.html#log_stream_event/3"},{"type":"function","title":"ExLLM.Logger.put_context/1","doc":"Add context that persists for the rest of the process.","ref":"ExLLM.Logger.html#put_context/1"},{"type":"function","title":"ExLLM.Logger.warn/2","doc":"Log a warning message with optional metadata.","ref":"ExLLM.Logger.html#warn/2"},{"type":"function","title":"ExLLM.Logger.with_context/2","doc":"Set context for all logs within the given function.","ref":"ExLLM.Logger.html#with_context/2"},{"type":"function","title":"Examples - ExLLM.Logger.with_context/2","doc":"ExLLM.Logger.with_context(provider: :openai, operation: :chat) do\n      ExLLM.Logger.info(\"Starting request\")\n      # All logs in here will include provider and operation\n    end","ref":"ExLLM.Logger.html#with_context/2-examples"},{"type":"module","title":"ExLLM.ModelCapabilities","doc":"Model capability discovery and management for ExLLM.\n\nThis module provides information about what features and capabilities\neach model supports across different providers. It helps users make\ninformed decisions about which model to use for their specific needs.","ref":"ExLLM.ModelCapabilities.html"},{"type":"module","title":"Features - ExLLM.ModelCapabilities","doc":"- Automatic capability detection\n- Provider-specific feature mapping\n- Capability comparison across models\n- Feature availability checking\n- Model recommendation based on requirements","ref":"ExLLM.ModelCapabilities.html#module-features"},{"type":"module","title":"Usage - ExLLM.ModelCapabilities","doc":"# Get capabilities for a specific model\n    {:ok, caps} = ExLLM.ModelCapabilities.get_capabilities(:openai, \"gpt-4-turbo\")\n    \n    # Check if a model supports a specific feature\n    ExLLM.ModelCapabilities.supports?(:anthropic, \"claude-3-opus-20240229\", :vision)\n    \n    # Find models that support specific features\n    models = ExLLM.ModelCapabilities.find_models_with_features([:function_calling, :streaming])\n    \n    # Compare models\n    comparison = ExLLM.ModelCapabilities.compare_models([\n      {:openai, \"gpt-4\"},\n      {:anthropic, \"claude-3-sonnet-20240229\"}\n    ])","ref":"ExLLM.ModelCapabilities.html#module-usage"},{"type":"function","title":"ExLLM.ModelCapabilities.compare_models/1","doc":"Compare capabilities across multiple models.","ref":"ExLLM.ModelCapabilities.html#compare_models/1"},{"type":"function","title":"ExLLM.ModelCapabilities.find_models_with_features/1","doc":"Find all models that support specific features.\n\nThis function searches both the hardcoded model database and dynamically\nloads models from YAML configuration files to provide comprehensive results.","ref":"ExLLM.ModelCapabilities.html#find_models_with_features/1"},{"type":"function","title":"ExLLM.ModelCapabilities.get_capabilities/2","doc":"Get complete capability information for a model.","ref":"ExLLM.ModelCapabilities.html#get_capabilities/2"},{"type":"function","title":"ExLLM.ModelCapabilities.get_capability_details/3","doc":"Get detailed information about a specific capability.","ref":"ExLLM.ModelCapabilities.html#get_capability_details/3"},{"type":"function","title":"ExLLM.ModelCapabilities.get_model_info/2","doc":"Get model information for a specific provider and model.","ref":"ExLLM.ModelCapabilities.html#get_model_info/2"},{"type":"function","title":"Parameters - ExLLM.ModelCapabilities.get_model_info/2","doc":"- `provider` - Provider atom (e.g., :openai, :anthropic)\n- `model` - Model identifier","ref":"ExLLM.ModelCapabilities.html#get_model_info/2-parameters"},{"type":"function","title":"Returns - ExLLM.ModelCapabilities.get_model_info/2","doc":"- `{:ok, model_info}` on success\n- `{:error, :not_found}` if model not found","ref":"ExLLM.ModelCapabilities.html#get_model_info/2-returns"},{"type":"function","title":"Examples - ExLLM.ModelCapabilities.get_model_info/2","doc":"{:ok, info} = ExLLM.ModelCapabilities.get_model_info(:openai, \"gpt-4o\")","ref":"ExLLM.ModelCapabilities.html#get_model_info/2-examples"},{"type":"function","title":"ExLLM.ModelCapabilities.list_features/0","doc":"Get all available features.","ref":"ExLLM.ModelCapabilities.html#list_features/0"},{"type":"function","title":"ExLLM.ModelCapabilities.models_by_capability/1","doc":"Get models grouped by capability.\n\nThis function searches both the hardcoded model database and dynamically\nloads models from YAML configuration files to provide comprehensive results.","ref":"ExLLM.ModelCapabilities.html#models_by_capability/1"},{"type":"function","title":"ExLLM.ModelCapabilities.recommend_models/1","doc":"Get recommended models based on requirements.","ref":"ExLLM.ModelCapabilities.html#recommend_models/1"},{"type":"function","title":"ExLLM.ModelCapabilities.supports?/3","doc":"Check if a model supports a specific feature.","ref":"ExLLM.ModelCapabilities.html#supports?/3"},{"type":"module","title":"ExLLM.ModelCapabilities.Capability","doc":"Represents a model capability or feature.","ref":"ExLLM.ModelCapabilities.Capability.html"},{"type":"type","title":"ExLLM.ModelCapabilities.Capability.t/0","doc":"","ref":"ExLLM.ModelCapabilities.Capability.html#t:t/0"},{"type":"module","title":"ExLLM.ModelCapabilities.ModelInfo","doc":"Complete information about a model's capabilities.","ref":"ExLLM.ModelCapabilities.ModelInfo.html"},{"type":"type","title":"ExLLM.ModelCapabilities.ModelInfo.t/0","doc":"","ref":"ExLLM.ModelCapabilities.ModelInfo.html#t:t/0"},{"type":"module","title":"ExLLM.ModelConfig","doc":"Model configuration loader for ExLLM.\n\nLoads model information from external YAML configuration files including:\n- Model pricing information\n- Context window sizes\n- Model capabilities\n- Default models per provider\n\nConfiguration files are located in `config/models/` and organized by provider.","ref":"ExLLM.ModelConfig.html"},{"type":"function","title":"ExLLM.ModelConfig.ensure_cache_table/0","doc":"","ref":"ExLLM.ModelConfig.html#ensure_cache_table/0"},{"type":"function","title":"ExLLM.ModelConfig.get_all_context_windows/1","doc":"Gets all context window information for a provider.\n\nReturns a map of model names to context window sizes.","ref":"ExLLM.ModelConfig.html#get_all_context_windows/1"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_all_context_windows/1","doc":"iex> contexts = ExLLM.ModelConfig.get_all_context_windows(:anthropic)\n    iex> contexts[\"claude-3-5-sonnet-20241022\"]\n    200000","ref":"ExLLM.ModelConfig.html#get_all_context_windows/1-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_all_models/1","doc":"Gets all models for a provider.\n\nReturns a map of model names to their configurations.","ref":"ExLLM.ModelConfig.html#get_all_models/1"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_all_models/1","doc":"iex> models = ExLLM.ModelConfig.get_all_models(:anthropic)\n    iex> Map.keys(models)\n    [\"claude-3-5-sonnet-20241022\", \"claude-3-5-haiku-20241022\", ...]","ref":"ExLLM.ModelConfig.html#get_all_models/1-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_all_pricing/1","doc":"Gets all pricing information for a provider.\n\nReturns a map of model names to pricing maps (with `:input` and `:output` keys).","ref":"ExLLM.ModelConfig.html#get_all_pricing/1"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_all_pricing/1","doc":"iex> pricing = ExLLM.ModelConfig.get_all_pricing(:openai)\n    iex> pricing[\"gpt-4o\"]\n    %{input: 2.50, output: 10.00}","ref":"ExLLM.ModelConfig.html#get_all_pricing/1-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_capabilities/2","doc":"Gets the capabilities for a specific provider and model.\n\nReturns a list of capability atoms, or `nil` if not found.","ref":"ExLLM.ModelConfig.html#get_capabilities/2"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_capabilities/2","doc":"iex> ExLLM.ModelConfig.get_capabilities(:openai, \"gpt-4o\")\n    [:text, :vision, :function_calling, :streaming]","ref":"ExLLM.ModelConfig.html#get_capabilities/2-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_context_window/2","doc":"Gets the context window size for a specific provider and model.\n\nReturns the context window size in tokens, or `nil` if not found.","ref":"ExLLM.ModelConfig.html#get_context_window/2"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_context_window/2","doc":"iex> ExLLM.ModelConfig.get_context_window(:openai, \"gpt-4o\")\n    128000\n\n    iex> ExLLM.ModelConfig.get_context_window(:anthropic, \"claude-3-5-sonnet-20241022\")\n    200000","ref":"ExLLM.ModelConfig.html#get_context_window/2-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_default_model/1","doc":"Gets the default model for a provider.\n\nReturns the default model name as a string, or `nil` if not found.","ref":"ExLLM.ModelConfig.html#get_default_model/1"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_default_model/1","doc":"iex> ExLLM.ModelConfig.get_default_model(:openai)\n    \"gpt-4.1-mini\"\n\n    iex> ExLLM.ModelConfig.get_default_model(:anthropic)\n    \"claude-sonnet-4-20250514\"","ref":"ExLLM.ModelConfig.html#get_default_model/1-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_model_config/2","doc":"Gets the full configuration for a specific model.\n\nReturns the model configuration map or nil if not found.","ref":"ExLLM.ModelConfig.html#get_model_config/2"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_model_config/2","doc":"iex> ExLLM.ModelConfig.get_model_config(:openai, \"gpt-4o\")\n    %{\n      context_window: 128000,\n      pricing: %{input: 2.50, output: 10.00},\n      capabilities: [:streaming, :function_calling]\n    }","ref":"ExLLM.ModelConfig.html#get_model_config/2-examples"},{"type":"function","title":"ExLLM.ModelConfig.get_pricing/2","doc":"Gets the pricing information for a specific provider and model.\n\nReturns a map with `:input` and `:output` pricing per 1M tokens,\nor `nil` if the model is not found.","ref":"ExLLM.ModelConfig.html#get_pricing/2"},{"type":"function","title":"Examples - ExLLM.ModelConfig.get_pricing/2","doc":"iex> ExLLM.ModelConfig.get_pricing(:openai, \"gpt-4o\")\n    %{input: 2.50, output: 10.00}\n\n    iex> ExLLM.ModelConfig.get_pricing(:anthropic, \"claude-3-5-sonnet-20241022\")\n    %{input: 3.00, output: 15.00}\n\n    iex> ExLLM.ModelConfig.get_pricing(:unknown_provider, \"model\")\n    nil","ref":"ExLLM.ModelConfig.html#get_pricing/2-examples"},{"type":"function","title":"ExLLM.ModelConfig.reload_config/0","doc":"Reloads all configuration from files.\n\nClears the cache and forces a reload of all configuration files.\nUseful during development or when configuration files are updated.","ref":"ExLLM.ModelConfig.html#reload_config/0"},{"type":"module","title":"ExLLM.ModelLoader","doc":"Dynamic model loader for ExLLM adapters.\n\nProvides functionality to:\n1. Fetch models from provider APIs where available\n2. Fall back to YAML configuration files\n3. Cache results to avoid repeated API calls","ref":"ExLLM.ModelLoader.html"},{"type":"function","title":"ExLLM.ModelLoader.clear_cache/1","doc":"Clears the model cache for a specific provider or all providers.","ref":"ExLLM.ModelLoader.html#clear_cache/1"},{"type":"function","title":"ExLLM.ModelLoader.load_models/2","doc":"Loads models for a provider, attempting API first then falling back to config.","ref":"ExLLM.ModelLoader.html#load_models/2"},{"type":"function","title":"Options - ExLLM.ModelLoader.load_models/2","doc":"- `:force_refresh` - Skip cache and fetch fresh data\n- `:api_fetcher` - Function to fetch from API: fn(options) -> {:ok, models} | {:error, reason}\n- `:config_transformer` - Function to transform config data to Types.Model structs","ref":"ExLLM.ModelLoader.html#load_models/2-options"},{"type":"module","title":"ExLLM.ProviderCapabilities","doc":"Provider-level capability tracking for ExLLM.\n\nThis module provides information about what features and capabilities\neach provider supports at the API level, independent of specific models.\nIt helps users understand what operations are available with each provider\nand assists in provider selection based on required features.","ref":"ExLLM.ProviderCapabilities.html"},{"type":"module","title":"Features - ExLLM.ProviderCapabilities","doc":"- Provider endpoint discovery (chat, embeddings, images, etc.)\n- Authentication method tracking\n- Cost tracking availability\n- API feature support\n- Provider metadata and limitations\n- Feature-based provider recommendations","ref":"ExLLM.ProviderCapabilities.html#module-features"},{"type":"module","title":"Usage - ExLLM.ProviderCapabilities","doc":"# Check if a provider supports a specific feature\n    ExLLM.ProviderCapabilities.supports?(:openai, :embeddings)\n    \n    # Get all capabilities for a provider\n    {:ok, caps} = ExLLM.ProviderCapabilities.get_capabilities(:anthropic)\n    \n    # Find providers that support specific features\n    providers = ExLLM.ProviderCapabilities.find_providers_with_features([:embeddings, :streaming])\n    \n    # Get provider recommendations based on requirements\n    recommended = ExLLM.ProviderCapabilities.recommend_providers(%{\n      required_features: [:streaming, :function_calling],\n      preferred_features: [:vision, :cost_tracking],\n      exclude_providers: [:mock]\n    })","ref":"ExLLM.ProviderCapabilities.html#module-usage"},{"type":"function","title":"ExLLM.ProviderCapabilities.compare_providers/1","doc":"Compare capabilities across multiple providers.","ref":"ExLLM.ProviderCapabilities.html#compare_providers/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.compare_providers/1","doc":"- `providers` - List of provider atoms to compare","ref":"ExLLM.ProviderCapabilities.html#compare_providers/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.compare_providers/1","doc":"- Map with comparison data","ref":"ExLLM.ProviderCapabilities.html#compare_providers/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.compare_providers/1","doc":"comparison = ExLLM.ProviderCapabilities.compare_providers([:openai, :anthropic, :ollama])","ref":"ExLLM.ProviderCapabilities.html#compare_providers/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.find_providers_with_features/1","doc":"Find providers that support all specified features.","ref":"ExLLM.ProviderCapabilities.html#find_providers_with_features/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.find_providers_with_features/1","doc":"- `features` - List of required features","ref":"ExLLM.ProviderCapabilities.html#find_providers_with_features/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.find_providers_with_features/1","doc":"- List of provider atoms that support all features","ref":"ExLLM.ProviderCapabilities.html#find_providers_with_features/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.find_providers_with_features/1","doc":"providers = ExLLM.ProviderCapabilities.find_providers_with_features([:embeddings, :streaming])\n    # => [:openai, :groq]","ref":"ExLLM.ProviderCapabilities.html#find_providers_with_features/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.get_adapter_module/1","doc":"Get a provider-specific adapter module.","ref":"ExLLM.ProviderCapabilities.html#get_adapter_module/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.get_adapter_module/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#get_adapter_module/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.get_adapter_module/1","doc":"- Module atom or nil","ref":"ExLLM.ProviderCapabilities.html#get_adapter_module/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.get_adapter_module/1","doc":"ExLLM.ProviderCapabilities.get_adapter_module(:openai)\n    # => ExLLM.Adapters.OpenAI","ref":"ExLLM.ProviderCapabilities.html#get_adapter_module/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.get_auth_methods/1","doc":"Get authentication methods for a provider.","ref":"ExLLM.ProviderCapabilities.html#get_auth_methods/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.get_auth_methods/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#get_auth_methods/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.get_auth_methods/1","doc":"- List of authentication methods or empty list","ref":"ExLLM.ProviderCapabilities.html#get_auth_methods/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.get_auth_methods/1","doc":"ExLLM.ProviderCapabilities.get_auth_methods(:openai)\n    # => [:api_key, :bearer_token]","ref":"ExLLM.ProviderCapabilities.html#get_auth_methods/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.get_capabilities/1","doc":"Get capabilities for a specific provider.","ref":"ExLLM.ProviderCapabilities.html#get_capabilities/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.get_capabilities/1","doc":"- `provider` - Provider atom (e.g., :openai, :anthropic)","ref":"ExLLM.ProviderCapabilities.html#get_capabilities/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.get_capabilities/1","doc":"- `{:ok, provider_info}` on success\n- `{:error, :not_found}` if provider not found","ref":"ExLLM.ProviderCapabilities.html#get_capabilities/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.get_capabilities/1","doc":"{:ok, info} = ExLLM.ProviderCapabilities.get_capabilities(:openai)","ref":"ExLLM.ProviderCapabilities.html#get_capabilities/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.get_endpoints/1","doc":"Get available endpoints for a provider.","ref":"ExLLM.ProviderCapabilities.html#get_endpoints/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.get_endpoints/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#get_endpoints/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.get_endpoints/1","doc":"- List of available endpoints or empty list","ref":"ExLLM.ProviderCapabilities.html#get_endpoints/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.get_endpoints/1","doc":"ExLLM.ProviderCapabilities.get_endpoints(:openai)\n    # => [:chat, :embeddings, :images, :audio, :completions, :fine_tuning, :files]","ref":"ExLLM.ProviderCapabilities.html#get_endpoints/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.get_limitations/1","doc":"Get provider limitations.","ref":"ExLLM.ProviderCapabilities.html#get_limitations/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.get_limitations/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#get_limitations/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.get_limitations/1","doc":"- Map of limitations or empty map","ref":"ExLLM.ProviderCapabilities.html#get_limitations/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.get_limitations/1","doc":"ExLLM.ProviderCapabilities.get_limitations(:ollama)\n    # => %{no_cost_tracking: true, performance_depends_on_hardware: true}","ref":"ExLLM.ProviderCapabilities.html#get_limitations/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.is_local?/1","doc":"Check if a provider is considered \"local\" (no API calls).","ref":"ExLLM.ProviderCapabilities.html#is_local?/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.is_local?/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#is_local?/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.is_local?/1","doc":"- Boolean","ref":"ExLLM.ProviderCapabilities.html#is_local?/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.is_local?/1","doc":"ExLLM.ProviderCapabilities.is_local?(:ollama)\n    # => true\n    \n    ExLLM.ProviderCapabilities.is_local?(:openai)\n    # => false","ref":"ExLLM.ProviderCapabilities.html#is_local?/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.list_providers/0","doc":"List all known providers.","ref":"ExLLM.ProviderCapabilities.html#list_providers/0"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.list_providers/0","doc":"- List of provider atoms","ref":"ExLLM.ProviderCapabilities.html#list_providers/0-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.list_providers/0","doc":"ExLLM.ProviderCapabilities.list_providers()\n    # => [:anthropic, :bedrock, :bumblebee, :gemini, :groq, :mock, :ollama, :openai, :openrouter, :xai]","ref":"ExLLM.ProviderCapabilities.html#list_providers/0-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.recommend_providers/1","doc":"Get provider recommendations based on requirements.","ref":"ExLLM.ProviderCapabilities.html#recommend_providers/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.recommend_providers/1","doc":"- `requirements` - Map with:\n  - `:required_features` - List of features that must be supported\n  - `:preferred_features` - List of features that are nice to have\n  - `:required_endpoints` - List of endpoints that must be available\n  - `:exclude_providers` - List of providers to exclude\n  - `:prefer_local` - Boolean to prefer local providers (default: false)\n  - `:prefer_free` - Boolean to prefer free providers (default: false)","ref":"ExLLM.ProviderCapabilities.html#recommend_providers/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.recommend_providers/1","doc":"- List of recommended providers sorted by match score","ref":"ExLLM.ProviderCapabilities.html#recommend_providers/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.recommend_providers/1","doc":"recommendations = ExLLM.ProviderCapabilities.recommend_providers(%{\n      required_features: [:streaming, :function_calling],\n      preferred_features: [:vision, :cost_tracking],\n      exclude_providers: [:mock]\n    })\n    # => [\n    #   %{provider: :openai, score: 1.0, matched_features: [...], missing_features: []},\n    #   %{provider: :anthropic, score: 0.85, matched_features: [...], missing_features: [...]}\n    # ]","ref":"ExLLM.ProviderCapabilities.html#recommend_providers/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.requires_auth?/1","doc":"Check if a provider requires authentication.","ref":"ExLLM.ProviderCapabilities.html#requires_auth?/1"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.requires_auth?/1","doc":"- `provider` - Provider atom","ref":"ExLLM.ProviderCapabilities.html#requires_auth?/1-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.requires_auth?/1","doc":"- Boolean","ref":"ExLLM.ProviderCapabilities.html#requires_auth?/1-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.requires_auth?/1","doc":"ExLLM.ProviderCapabilities.requires_auth?(:openai)\n    # => true\n    \n    ExLLM.ProviderCapabilities.requires_auth?(:ollama)\n    # => false","ref":"ExLLM.ProviderCapabilities.html#requires_auth?/1-examples"},{"type":"function","title":"ExLLM.ProviderCapabilities.supports?/2","doc":"Check if a provider supports a specific feature.","ref":"ExLLM.ProviderCapabilities.html#supports?/2"},{"type":"function","title":"Parameters - ExLLM.ProviderCapabilities.supports?/2","doc":"- `provider` - Provider atom\n- `feature` - Feature to check (endpoint or capability)","ref":"ExLLM.ProviderCapabilities.html#supports?/2-parameters"},{"type":"function","title":"Returns - ExLLM.ProviderCapabilities.supports?/2","doc":"- `true` if supported\n- `false` if not supported or provider not found","ref":"ExLLM.ProviderCapabilities.html#supports?/2-returns"},{"type":"function","title":"Examples - ExLLM.ProviderCapabilities.supports?/2","doc":"ExLLM.ProviderCapabilities.supports?(:openai, :embeddings)\n    # => true\n    \n    ExLLM.ProviderCapabilities.supports?(:ollama, :cost_tracking)\n    # => false","ref":"ExLLM.ProviderCapabilities.html#supports?/2-examples"},{"type":"module","title":"ExLLM.ProviderCapabilities.ProviderInfo","doc":"Represents provider-level information and capabilities.","ref":"ExLLM.ProviderCapabilities.ProviderInfo.html"},{"type":"type","title":"ExLLM.ProviderCapabilities.ProviderInfo.auth_method/0","doc":"","ref":"ExLLM.ProviderCapabilities.ProviderInfo.html#t:auth_method/0"},{"type":"type","title":"ExLLM.ProviderCapabilities.ProviderInfo.endpoint/0","doc":"","ref":"ExLLM.ProviderCapabilities.ProviderInfo.html#t:endpoint/0"},{"type":"type","title":"ExLLM.ProviderCapabilities.ProviderInfo.feature/0","doc":"","ref":"ExLLM.ProviderCapabilities.ProviderInfo.html#t:feature/0"},{"type":"type","title":"ExLLM.ProviderCapabilities.ProviderInfo.t/0","doc":"","ref":"ExLLM.ProviderCapabilities.ProviderInfo.html#t:t/0"},{"type":"module","title":"ExLLM.ResponseCache","doc":"Response caching system for collecting and storing real provider responses.\n\nThis module allows ExLLM to cache actual responses from providers like OpenAI,\nAnthropic, OpenRouter, etc., and later replay them in tests using the Mock adapter.","ref":"ExLLM.ResponseCache.html"},{"type":"module","title":"Usage - ExLLM.ResponseCache","doc":"","ref":"ExLLM.ResponseCache.html#module-usage"},{"type":"module","title":"Automatic Response Caching - ExLLM.ResponseCache","doc":"Enable caching in your environment:\n\n    # Cache all responses during testing\n    export EX_LLM_CACHE_RESPONSES=true\n    export EX_LLM_CACHE_DIR=\"/path/to/cache/responses\"","ref":"ExLLM.ResponseCache.html#module-automatic-response-caching"},{"type":"module","title":"Manual Response Caching - ExLLM.ResponseCache","doc":"# Store a response\n    ExLLM.ResponseCache.store_response(\"openai\", request_key, response_data)\n    \n    # Retrieve a cached response\n    cached = ExLLM.ResponseCache.get_response(\"openai\", request_key)\n    \n    # Load all responses for a provider\n    responses = ExLLM.ResponseCache.load_provider_responses(\"anthropic\")","ref":"ExLLM.ResponseCache.html#module-manual-response-caching"},{"type":"module","title":"Integration with Mock Adapter - ExLLM.ResponseCache","doc":"# Configure mock to use cached responses from OpenAI\n    ExLLM.ResponseCache.configure_mock_provider(\"openai\")\n    \n    # Use cached responses in tests\n    {:ok, response} = ExLLM.chat(messages, provider: :mock)","ref":"ExLLM.ResponseCache.html#module-integration-with-mock-adapter"},{"type":"module","title":"Cache Structure - ExLLM.ResponseCache","doc":"Responses are stored in JSON files organized by provider:\n\n    cache/\n    ├── openai/\n    │   ├── chat_completions.json\n    │   ├── embeddings.json\n    │   └── streaming.json\n    ├── anthropic/\n    │   ├── messages.json\n    │   └── streaming.json\n    └── openrouter/\n        ├── chat_completions.json\n        └── models.json\n\nEach file contains an array of cached request/response pairs with metadata.","ref":"ExLLM.ResponseCache.html#module-cache-structure"},{"type":"function","title":"ExLLM.ResponseCache.cache_dir/0","doc":"Returns the cache directory path.","ref":"ExLLM.ResponseCache.html#cache_dir/0"},{"type":"function","title":"ExLLM.ResponseCache.caching_enabled?/0","doc":"Returns true if response caching is enabled.","ref":"ExLLM.ResponseCache.html#caching_enabled?/0"},{"type":"function","title":"ExLLM.ResponseCache.clear_all_cache/0","doc":"Clears all cached responses.","ref":"ExLLM.ResponseCache.html#clear_all_cache/0"},{"type":"function","title":"ExLLM.ResponseCache.clear_provider_cache/1","doc":"Clears all cached responses for a provider.","ref":"ExLLM.ResponseCache.html#clear_provider_cache/1"},{"type":"function","title":"ExLLM.ResponseCache.configure_mock_provider/1","doc":"Configures the Mock adapter to use cached responses from a specific provider.","ref":"ExLLM.ResponseCache.html#configure_mock_provider/1"},{"type":"function","title":"ExLLM.ResponseCache.get_response/3","doc":"Retrieves a cached response matching the request.","ref":"ExLLM.ResponseCache.html#get_response/3"},{"type":"function","title":"ExLLM.ResponseCache.list_cached_providers/0","doc":"Lists all available providers with cached responses.","ref":"ExLLM.ResponseCache.html#list_cached_providers/0"},{"type":"function","title":"ExLLM.ResponseCache.load_provider_responses/1","doc":"Loads all cached responses for a provider.","ref":"ExLLM.ResponseCache.html#load_provider_responses/1"},{"type":"function","title":"ExLLM.ResponseCache.store_from_cache/6","doc":"Stores a response from the unified cache system.\n\nThis is called by ExLLM.Cache when disk persistence is enabled.","ref":"ExLLM.ResponseCache.html#store_from_cache/6"},{"type":"function","title":"ExLLM.ResponseCache.store_response/5","doc":"Stores a response in the cache.","ref":"ExLLM.ResponseCache.html#store_response/5"},{"type":"module","title":"ExLLM.Retry","doc":"Request retry logic with exponential backoff for ExLLM.\n\nProvides configurable retry mechanisms for failed API requests with\nsupport for different backoff strategies and retry conditions.","ref":"ExLLM.Retry.html"},{"type":"module","title":"Features - ExLLM.Retry","doc":"- Exponential backoff with jitter\n- Configurable retry conditions\n- Per-provider retry policies\n- Circuit breaker pattern\n- Request deduplication","ref":"ExLLM.Retry.html#module-features"},{"type":"module","title":"Usage - ExLLM.Retry","doc":"# With default retry policy\n    ExLLM.Retry.with_retry fn ->\n      ExLLM.chat(:openai, messages)\n    end\n    \n    # With custom retry options\n    ExLLM.Retry.with_retry fn ->\n      ExLLM.chat(:anthropic, messages)\n    end,\n      max_attempts: 5,\n      base_delay: 1000,\n      max_delay: 30_000,\n      jitter: true","ref":"ExLLM.Retry.html#module-usage"},{"type":"function","title":"ExLLM.Retry.calculate_delay/2","doc":"Calculates the delay before the next retry attempt.","ref":"ExLLM.Retry.html#calculate_delay/2"},{"type":"function","title":"ExLLM.Retry.get_provider_policy/1","doc":"Default retry policies for providers.","ref":"ExLLM.Retry.html#get_provider_policy/1"},{"type":"function","title":"ExLLM.Retry.should_retry?/3","doc":"Checks if an error should trigger a retry.","ref":"ExLLM.Retry.html#should_retry?/3"},{"type":"function","title":"ExLLM.Retry.with_provider_retry/3","doc":"Executes a function with retry logic for a specific provider.","ref":"ExLLM.Retry.html#with_provider_retry/3"},{"type":"function","title":"ExLLM.Retry.with_retry/2","doc":"Executes a function with retry logic.","ref":"ExLLM.Retry.html#with_retry/2"},{"type":"function","title":"Options - ExLLM.Retry.with_retry/2","doc":"- `:max_attempts` - Maximum number of attempts (default: 3)\n- `:base_delay` - Initial delay in milliseconds (default: 1000)\n- `:max_delay` - Maximum delay in milliseconds (default: 60000)\n- `:multiplier` - Backoff multiplier (default: 2)\n- `:jitter` - Add random jitter to delays (default: true)\n- `:retry_on` - Function to determine if error is retryable","ref":"ExLLM.Retry.html#with_retry/2-options"},{"type":"module","title":"ExLLM.Retry.CircuitBreaker","doc":"Circuit breaker to prevent cascading failures.","ref":"ExLLM.Retry.CircuitBreaker.html"},{"type":"type","title":"ExLLM.Retry.CircuitBreaker.state/0","doc":"","ref":"ExLLM.Retry.CircuitBreaker.html#t:state/0"},{"type":"type","title":"ExLLM.Retry.CircuitBreaker.t/0","doc":"","ref":"ExLLM.Retry.CircuitBreaker.html#t:t/0"},{"type":"module","title":"ExLLM.Retry.RetryPolicy","doc":"Defines a retry policy for a specific provider or operation.","ref":"ExLLM.Retry.RetryPolicy.html"},{"type":"type","title":"ExLLM.Retry.RetryPolicy.t/0","doc":"","ref":"ExLLM.Retry.RetryPolicy.html#t:t/0"},{"type":"module","title":"ExLLM.Session","doc":"Session management for ExLLM - handles conversation sessions with LLM providers.\n\nExLLM.Session provides pure functional operations for managing conversation sessions,\nincluding message history, token tracking, and session persistence. All operations\nare stateless and return updated session structs.","ref":"ExLLM.Session.html"},{"type":"module","title":"Quick Start - ExLLM.Session","doc":"# Create a new session\n    session = ExLLM.Session.new(\"anthropic\")\n\n    # Add messages\n    session = ExLLM.Session.add_message(session, \"user\", \"Hello!\")\n    session = ExLLM.Session.add_message(session, \"assistant\", \"Hi there!\")\n\n    # Get messages\n    messages = ExLLM.Session.get_messages(session)\n\n    # Update token usage\n    session = ExLLM.Session.update_token_usage(session, %{input_tokens: 10, output_tokens: 15})","ref":"ExLLM.Session.html#module-quick-start"},{"type":"module","title":"Features - ExLLM.Session","doc":"- **Pure Functional**: All operations are stateless and immutable\n- **Message Management**: Add, retrieve, and filter conversation messages\n- **Token Tracking**: Track input/output token usage across conversations\n- **Context Storage**: Store arbitrary metadata with sessions\n- **Session Persistence**: Serialize/deserialize sessions to/from JSON\n- **LLM Integration**: Seamlessly works with ExLLM providers","ref":"ExLLM.Session.html#module-features"},{"type":"module","title":"Session Structure - ExLLM.Session","doc":"Sessions are represented by `ExLLM.Session.Types.Session` structs containing:\n- `id` - Unique session identifier\n- `llm_backend` - LLM backend name (optional)\n- `messages` - List of conversation messages\n- `context` - Arbitrary metadata map\n- `created_at`/`updated_at` - Timestamps\n- `token_usage` - Token consumption tracking\n- `name` - Human-readable session name (optional)","ref":"ExLLM.Session.html#module-session-structure"},{"type":"function","title":"ExLLM.Session.add_message/4","doc":"Add a message to the session.","ref":"ExLLM.Session.html#add_message/4"},{"type":"function","title":"Parameters - ExLLM.Session.add_message/4","doc":"- `session` - The session to update\n- `role` - Message role (\"user\", \"assistant\", etc.)\n- `content` - Message content\n- `opts` - Additional message metadata","ref":"ExLLM.Session.html#add_message/4-parameters"},{"type":"function","title":"Returns - ExLLM.Session.add_message/4","doc":"Updated session with the new message.","ref":"ExLLM.Session.html#add_message/4-returns"},{"type":"function","title":"Examples - ExLLM.Session.add_message/4","doc":"session = ExLLM.Session.add_message(session, \"user\", \"Hello!\")\n    session = ExLLM.Session.add_message(session, \"assistant\", \"Hi there!\", timestamp: DateTime.utc_now())","ref":"ExLLM.Session.html#add_message/4-examples"},{"type":"function","title":"ExLLM.Session.clear_messages/1","doc":"Clear all messages from the session.","ref":"ExLLM.Session.html#clear_messages/1"},{"type":"function","title":"Parameters - ExLLM.Session.clear_messages/1","doc":"- `session` - The session to clear","ref":"ExLLM.Session.html#clear_messages/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.clear_messages/1","doc":"Updated session with empty message list.","ref":"ExLLM.Session.html#clear_messages/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.clear_messages/1","doc":"session = ExLLM.Session.clear_messages(session)","ref":"ExLLM.Session.html#clear_messages/1-examples"},{"type":"function","title":"ExLLM.Session.from_json/1","doc":"Deserialize session from JSON.","ref":"ExLLM.Session.html#from_json/1"},{"type":"function","title":"Parameters - ExLLM.Session.from_json/1","doc":"- `json_string` - JSON string to deserialize","ref":"ExLLM.Session.html#from_json/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.from_json/1","doc":"`{:ok, session}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Session.html#from_json/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.from_json/1","doc":"{:ok, session} = ExLLM.Session.from_json(json_string)","ref":"ExLLM.Session.html#from_json/1-examples"},{"type":"function","title":"ExLLM.Session.get_messages/2","doc":"Get messages from the session, optionally with a limit.","ref":"ExLLM.Session.html#get_messages/2"},{"type":"function","title":"Parameters - ExLLM.Session.get_messages/2","doc":"- `session` - The session to get messages from\n- `limit` - Maximum number of messages to return (optional)","ref":"ExLLM.Session.html#get_messages/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.get_messages/2","doc":"List of messages, most recent first if limited.","ref":"ExLLM.Session.html#get_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.get_messages/2","doc":"all_messages = ExLLM.Session.get_messages(session)\n    last_5 = ExLLM.Session.get_messages(session, 5)","ref":"ExLLM.Session.html#get_messages/2-examples"},{"type":"function","title":"ExLLM.Session.load_from_file/1","doc":"Load session from a JSON file.","ref":"ExLLM.Session.html#load_from_file/1"},{"type":"function","title":"Parameters - ExLLM.Session.load_from_file/1","doc":"- `file_path` - Path to the file containing session data","ref":"ExLLM.Session.html#load_from_file/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.load_from_file/1","doc":"- `{:ok, session}` on success\n- `{:error, reason}` on failure","ref":"ExLLM.Session.html#load_from_file/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.load_from_file/1","doc":"{:ok, session} = ExLLM.Session.load_from_file(\"my_session.json\")","ref":"ExLLM.Session.html#load_from_file/1-examples"},{"type":"function","title":"ExLLM.Session.new/2","doc":"Create a new session with the specified backend.","ref":"ExLLM.Session.html#new/2"},{"type":"function","title":"Parameters - ExLLM.Session.new/2","doc":"- `backend` - LLM backend to use (optional)\n- `opts` - Additional options (`:name` for session name)","ref":"ExLLM.Session.html#new/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.new/2","doc":"A new Session struct.","ref":"ExLLM.Session.html#new/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.new/2","doc":"session = ExLLM.Session.new(\"anthropic\")\n    session = ExLLM.Session.new(\"openai\", name: \"My Chat\")","ref":"ExLLM.Session.html#new/2-examples"},{"type":"function","title":"ExLLM.Session.save_to_file/2","doc":"Save session to a JSON file.","ref":"ExLLM.Session.html#save_to_file/2"},{"type":"function","title":"Parameters - ExLLM.Session.save_to_file/2","doc":"- `session` - The session to save\n- `file_path` - Path to the file where session will be saved","ref":"ExLLM.Session.html#save_to_file/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.save_to_file/2","doc":"- `:ok` on success\n- `{:error, reason}` on failure","ref":"ExLLM.Session.html#save_to_file/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.save_to_file/2","doc":"ExLLM.Session.save_to_file(session, \"my_session.json\")","ref":"ExLLM.Session.html#save_to_file/2-examples"},{"type":"function","title":"ExLLM.Session.set_context/2","doc":"Set context for the session.","ref":"ExLLM.Session.html#set_context/2"},{"type":"function","title":"Parameters - ExLLM.Session.set_context/2","doc":"- `session` - The session to update\n- `context` - Context map to set","ref":"ExLLM.Session.html#set_context/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.set_context/2","doc":"Updated session with new context.","ref":"ExLLM.Session.html#set_context/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.set_context/2","doc":"session = ExLLM.Session.set_context(session, %{temperature: 0.7, max_tokens: 1000})","ref":"ExLLM.Session.html#set_context/2-examples"},{"type":"function","title":"ExLLM.Session.set_name/2","doc":"Set the name of the session.","ref":"ExLLM.Session.html#set_name/2"},{"type":"function","title":"Parameters - ExLLM.Session.set_name/2","doc":"- `session` - The session to update\n- `name` - New session name","ref":"ExLLM.Session.html#set_name/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.set_name/2","doc":"Updated session with new name.","ref":"ExLLM.Session.html#set_name/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.set_name/2","doc":"session = ExLLM.Session.set_name(session, \"My Important Chat\")","ref":"ExLLM.Session.html#set_name/2-examples"},{"type":"function","title":"ExLLM.Session.to_json/1","doc":"Serialize session to JSON.","ref":"ExLLM.Session.html#to_json/1"},{"type":"function","title":"Parameters - ExLLM.Session.to_json/1","doc":"- `session` - The session to serialize","ref":"ExLLM.Session.html#to_json/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.to_json/1","doc":"`{:ok, json_string}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Session.html#to_json/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.to_json/1","doc":"{:ok, json} = ExLLM.Session.to_json(session)","ref":"ExLLM.Session.html#to_json/1-examples"},{"type":"function","title":"ExLLM.Session.total_tokens/1","doc":"Get the total token count for the session.","ref":"ExLLM.Session.html#total_tokens/1"},{"type":"function","title":"Parameters - ExLLM.Session.total_tokens/1","doc":"- `session` - The session to analyze","ref":"ExLLM.Session.html#total_tokens/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.total_tokens/1","doc":"Total token count (input + output).","ref":"ExLLM.Session.html#total_tokens/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.total_tokens/1","doc":"total = ExLLM.Session.total_tokens(session)","ref":"ExLLM.Session.html#total_tokens/1-examples"},{"type":"function","title":"ExLLM.Session.update_token_usage/2","doc":"Update the token usage for the session.","ref":"ExLLM.Session.html#update_token_usage/2"},{"type":"function","title":"Parameters - ExLLM.Session.update_token_usage/2","doc":"- `session` - The session to update\n- `usage` - Token usage map with `:input_tokens` and `:output_tokens`","ref":"ExLLM.Session.html#update_token_usage/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.update_token_usage/2","doc":"Updated session with new token usage.","ref":"ExLLM.Session.html#update_token_usage/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.update_token_usage/2","doc":"session = ExLLM.Session.update_token_usage(session, %{input_tokens: 100, output_tokens: 150})","ref":"ExLLM.Session.html#update_token_usage/2-examples"},{"type":"module","title":"ExLLM.Session.Types","doc":"Type definitions for ExLLM.Session.","ref":"ExLLM.Session.Types.html"},{"type":"type","title":"ExLLM.Session.Types.context/0","doc":"","ref":"ExLLM.Session.Types.html#t:context/0"},{"type":"type","title":"ExLLM.Session.Types.message/0","doc":"","ref":"ExLLM.Session.Types.html#t:message/0"},{"type":"type","title":"ExLLM.Session.Types.token_usage/0","doc":"","ref":"ExLLM.Session.Types.html#t:token_usage/0"},{"type":"module","title":"ExLLM.Session.Types.Session","doc":"Represents a conversation session with message history and metadata.","ref":"ExLLM.Session.Types.Session.html"},{"type":"type","title":"ExLLM.Session.Types.Session.t/0","doc":"","ref":"ExLLM.Session.Types.Session.html#t:t/0"},{"type":"module","title":"ExLLM.StreamRecovery","doc":"Streaming error recovery and resumption support for ExLLM.\n\nThis module provides functionality to recover from interrupted streaming\nresponses, allowing continuation from where the stream was interrupted.","ref":"ExLLM.StreamRecovery.html"},{"type":"module","title":"Features - ExLLM.StreamRecovery","doc":"- Automatic saving of partial responses during streaming\n- Detection of recoverable vs non-recoverable errors\n- Multiple resumption strategies\n- Configurable storage backends\n- Automatic cleanup of old partial responses","ref":"ExLLM.StreamRecovery.html#module-features"},{"type":"module","title":"Usage - ExLLM.StreamRecovery","doc":"# Enable recovery when streaming\n    {:ok, stream} = ExLLM.stream_chat(:anthropic, messages,\n      recovery: [\n        enabled: true,\n        strategy: :paragraph,\n        storage: :memory\n      ]\n    )\n    \n    # If stream is interrupted, resume it\n    {:ok, resumed_stream} = ExLLM.StreamRecovery.resume_stream(recovery_id)","ref":"ExLLM.StreamRecovery.html#module-usage"},{"type":"function","title":"ExLLM.StreamRecovery.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.StreamRecovery.html#child_spec/1"},{"type":"function","title":"ExLLM.StreamRecovery.cleanup_old_responses/1","doc":"Cleans up old partial responses.","ref":"ExLLM.StreamRecovery.html#cleanup_old_responses/1"},{"type":"function","title":"ExLLM.StreamRecovery.clear_partial_response/1","doc":"Clears a partial response from memory.","ref":"ExLLM.StreamRecovery.html#clear_partial_response/1"},{"type":"function","title":"ExLLM.StreamRecovery.complete_stream/1","doc":"Marks a stream as completed (no recovery needed).","ref":"ExLLM.StreamRecovery.html#complete_stream/1"},{"type":"function","title":"ExLLM.StreamRecovery.get_partial_response/1","doc":"Gets the partial response (chunks) for a recovery ID.","ref":"ExLLM.StreamRecovery.html#get_partial_response/1"},{"type":"function","title":"ExLLM.StreamRecovery.init_recovery/3","doc":"Initializes a recoverable stream.","ref":"ExLLM.StreamRecovery.html#init_recovery/3"},{"type":"function","title":"ExLLM.StreamRecovery.list_recoverable_streams/0","doc":"Lists all recoverable streams.","ref":"ExLLM.StreamRecovery.html#list_recoverable_streams/0"},{"type":"function","title":"ExLLM.StreamRecovery.record_chunk/2","doc":"Records a chunk for a recoverable stream.","ref":"ExLLM.StreamRecovery.html#record_chunk/2"},{"type":"function","title":"ExLLM.StreamRecovery.record_error/2","doc":"Records an error for potential recovery.","ref":"ExLLM.StreamRecovery.html#record_error/2"},{"type":"function","title":"ExLLM.StreamRecovery.recoverable_error?/1","doc":"Checks if an error is recoverable.","ref":"ExLLM.StreamRecovery.html#recoverable_error?/1"},{"type":"function","title":"ExLLM.StreamRecovery.resume_stream/2","doc":"Attempts to resume a previously interrupted stream.","ref":"ExLLM.StreamRecovery.html#resume_stream/2"},{"type":"function","title":"Strategies - ExLLM.StreamRecovery.resume_stream/2","doc":"- `:exact` - Continue from exact cutoff point\n- `:paragraph` - Regenerate from last complete paragraph\n- `:summarize` - Summarize received content and continue","ref":"ExLLM.StreamRecovery.html#resume_stream/2-strategies"},{"type":"function","title":"ExLLM.StreamRecovery.start_link/1","doc":"Starts the StreamRecovery GenServer.","ref":"ExLLM.StreamRecovery.html#start_link/1"},{"type":"module","title":"ExLLM.StreamRecovery.PartialResponse","doc":"Represents a partial streaming response that can be resumed.","ref":"ExLLM.StreamRecovery.PartialResponse.html"},{"type":"type","title":"ExLLM.StreamRecovery.PartialResponse.t/0","doc":"","ref":"ExLLM.StreamRecovery.PartialResponse.html#t:t/0"},{"type":"module","title":"ExLLM.TestCacheConfig","doc":"Centralized configuration system for automatic test response caching.\n\nThis module provides configuration management for timestamp-based test caching,\nincluding automatic test environment detection and hierarchical configuration\nloading from environment variables, application config, and defaults.","ref":"ExLLM.TestCacheConfig.html"},{"type":"function","title":"ExLLM.TestCacheConfig.cache_dir/0","doc":"Get the cache directory, ensuring it exists.","ref":"ExLLM.TestCacheConfig.html#cache_dir/0"},{"type":"type","title":"ExLLM.TestCacheConfig.config/0","doc":"","ref":"ExLLM.TestCacheConfig.html#t:config/0"},{"type":"function","title":"ExLLM.TestCacheConfig.enabled?/0","doc":"Check if test caching is enabled for the current environment.","ref":"ExLLM.TestCacheConfig.html#enabled?/0"},{"type":"type","title":"ExLLM.TestCacheConfig.fallback_strategy/0","doc":"","ref":"ExLLM.TestCacheConfig.html#t:fallback_strategy/0"},{"type":"function","title":"ExLLM.TestCacheConfig.get_config/0","doc":"Get the complete test cache configuration.\n\nConfiguration is loaded in this priority order:\n1. Environment variables (highest priority)\n2. Application configuration\n3. Default values (lowest priority)","ref":"ExLLM.TestCacheConfig.html#get_config/0"},{"type":"function","title":"ExLLM.TestCacheConfig.get_fallback_strategy/1","doc":"Get fallback strategy for specific test context.","ref":"ExLLM.TestCacheConfig.html#get_fallback_strategy/1"},{"type":"function","title":"ExLLM.TestCacheConfig.get_ttl/2","doc":"Get TTL for specific test context.","ref":"ExLLM.TestCacheConfig.html#get_ttl/2"},{"type":"type","title":"ExLLM.TestCacheConfig.hash_algorithm/0","doc":"","ref":"ExLLM.TestCacheConfig.html#t:hash_algorithm/0"},{"type":"type","title":"ExLLM.TestCacheConfig.timestamp_format/0","doc":"","ref":"ExLLM.TestCacheConfig.html#t:timestamp_format/0"},{"type":"module","title":"ExLLM.TestCacheDetector","doc":"Intelligent detection of test scenarios requiring automatic caching.\n\nThis module provides runtime detection of integration tests, OAuth2 tests,\nand other scenarios that should trigger automatic response caching.","ref":"ExLLM.TestCacheDetector.html"},{"type":"function","title":"ExLLM.TestCacheDetector.clear_test_context/0","doc":"Clear test context from the current process.","ref":"ExLLM.TestCacheDetector.html#clear_test_context/0"},{"type":"function","title":"ExLLM.TestCacheDetector.generate_cache_key/2","doc":"Generate a cache key based on the current test context.","ref":"ExLLM.TestCacheDetector.html#generate_cache_key/2"},{"type":"function","title":"ExLLM.TestCacheDetector.get_current_test_context/0","doc":"Get the current test context if available.","ref":"ExLLM.TestCacheDetector.html#get_current_test_context/0"},{"type":"function","title":"ExLLM.TestCacheDetector.integration_test_running?/0","doc":"Check if an integration test is currently running.","ref":"ExLLM.TestCacheDetector.html#integration_test_running?/0"},{"type":"function","title":"ExLLM.TestCacheDetector.is_destructive_operation?/0","doc":"Check if the current test involves destructive operations that shouldn't be cached.","ref":"ExLLM.TestCacheDetector.html#is_destructive_operation?/0"},{"type":"function","title":"ExLLM.TestCacheDetector.live_api_test_running?/0","doc":"Check if a live API test is currently running.","ref":"ExLLM.TestCacheDetector.html#live_api_test_running?/0"},{"type":"function","title":"ExLLM.TestCacheDetector.oauth2_test_running?/0","doc":"Check if an OAuth2 test is currently running.","ref":"ExLLM.TestCacheDetector.html#oauth2_test_running?/0"},{"type":"function","title":"ExLLM.TestCacheDetector.set_test_context/1","doc":"Store test context in the current process for later retrieval.","ref":"ExLLM.TestCacheDetector.html#set_test_context/1"},{"type":"function","title":"ExLLM.TestCacheDetector.should_cache_responses?/0","doc":"Check if test response caching should be enabled based on current context.","ref":"ExLLM.TestCacheDetector.html#should_cache_responses?/0"},{"type":"type","title":"ExLLM.TestCacheDetector.test_context/0","doc":"","ref":"ExLLM.TestCacheDetector.html#t:test_context/0"},{"type":"module","title":"ExLLM.TestCacheHelpers","doc":"Test helper functions for managing and working with the automatic test cache.\n\nThis module provides convenient functions for test suites to interact with\nthe caching system, including cache warming, clearing, and debugging.","ref":"ExLLM.TestCacheHelpers.html"},{"type":"function","title":"ExLLM.TestCacheHelpers.cleanup_old_timestamps/1","doc":"Clean up old cache timestamps.","ref":"ExLLM.TestCacheHelpers.html#cleanup_old_timestamps/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.clear_test_cache/1","doc":"Clear test cache with different scopes.","ref":"ExLLM.TestCacheHelpers.html#clear_test_cache/1"},{"type":"function","title":"Examples - ExLLM.TestCacheHelpers.clear_test_cache/1","doc":"# Clear all cache\n    clear_test_cache(:all)\n    \n    # Clear specific provider\n    clear_test_cache(\"anthropic\")\n    \n    # Clear specific test module\n    clear_test_cache(\"AnthropicIntegrationTest\")","ref":"ExLLM.TestCacheHelpers.html#clear_test_cache/1-examples"},{"type":"function","title":"ExLLM.TestCacheHelpers.deduplicate_cache_content/1","doc":"Deduplicate cache content to save space.","ref":"ExLLM.TestCacheHelpers.html#deduplicate_cache_content/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.disable_cache_debug/0","doc":"Disable debug logging for cache operations.","ref":"ExLLM.TestCacheHelpers.html#disable_cache_debug/0"},{"type":"function","title":"ExLLM.TestCacheHelpers.enable_cache_debug/0","doc":"Enable debug logging for cache operations.","ref":"ExLLM.TestCacheHelpers.html#enable_cache_debug/0"},{"type":"function","title":"ExLLM.TestCacheHelpers.force_cache_miss/1","doc":"Force cache miss for specific patterns.\n\nUseful for testing real API calls without disabling cache entirely.","ref":"ExLLM.TestCacheHelpers.html#force_cache_miss/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.force_cache_refresh/1","doc":"Force cache refresh for specific patterns.\n\nIgnores TTL and forces fresh API calls.","ref":"ExLLM.TestCacheHelpers.html#force_cache_refresh/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.get_cache_stats/1","doc":"Get cache statistics for specific test module.","ref":"ExLLM.TestCacheHelpers.html#get_cache_stats/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.list_cache_timestamps/1","doc":"List all cache timestamps for a pattern.","ref":"ExLLM.TestCacheHelpers.html#list_cache_timestamps/1"},{"type":"function","title":"Examples - ExLLM.TestCacheHelpers.list_cache_timestamps/1","doc":"list_cache_timestamps(\"anthropic/chat_basic\")","ref":"ExLLM.TestCacheHelpers.html#list_cache_timestamps/1-examples"},{"type":"function","title":"ExLLM.TestCacheHelpers.print_cache_summary/0","doc":"Print cache statistics summary.","ref":"ExLLM.TestCacheHelpers.html#print_cache_summary/0"},{"type":"function","title":"ExLLM.TestCacheHelpers.print_provider_stats/0","doc":"Print cache statistics by provider.","ref":"ExLLM.TestCacheHelpers.html#print_provider_stats/0"},{"type":"function","title":"ExLLM.TestCacheHelpers.restore_cache_timestamp/2","doc":"Restore cache from a specific timestamp.\n\nUseful for debugging or testing with older responses.","ref":"ExLLM.TestCacheHelpers.html#restore_cache_timestamp/2"},{"type":"function","title":"ExLLM.TestCacheHelpers.set_fallback_strategy/2","doc":"Set fallback strategy for specific test patterns.","ref":"ExLLM.TestCacheHelpers.html#set_fallback_strategy/2"},{"type":"function","title":"ExLLM.TestCacheHelpers.set_test_ttl/2","doc":"Set custom TTL for specific test patterns.","ref":"ExLLM.TestCacheHelpers.html#set_test_ttl/2"},{"type":"function","title":"ExLLM.TestCacheHelpers.setup_test_cache/1","doc":"Setup hook for test modules to initialize caching.","ref":"ExLLM.TestCacheHelpers.html#setup_test_cache/1"},{"type":"function","title":"Usage in test module: - ExLLM.TestCacheHelpers.setup_test_cache/1","doc":"setup do\n      ExLLM.TestCacheHelpers.setup_test_cache()\n    end","ref":"ExLLM.TestCacheHelpers.html#setup_test_cache/1-usage-in-test-module"},{"type":"function","title":"ExLLM.TestCacheHelpers.verify_cache_integrity/0","doc":"Verify cache integrity and check for issues.","ref":"ExLLM.TestCacheHelpers.html#verify_cache_integrity/0"},{"type":"function","title":"ExLLM.TestCacheHelpers.warm_test_cache/1","doc":"Warm test cache for specific test modules or patterns.\n\nThis pre-loads cache entries that might be expiring soon.","ref":"ExLLM.TestCacheHelpers.html#warm_test_cache/1"},{"type":"function","title":"ExLLM.TestCacheHelpers.with_test_cache/2","doc":"Run a test with specific cache configuration.","ref":"ExLLM.TestCacheHelpers.html#with_test_cache/2"},{"type":"function","title":"Examples - ExLLM.TestCacheHelpers.with_test_cache/2","doc":"with_test_cache([ttl: :timer.hours(1)], fn ->\n      # Your test code here\n    end)","ref":"ExLLM.TestCacheHelpers.html#with_test_cache/2-examples"},{"type":"module","title":"ExLLM.TestCacheIndex","doc":"Maintain index of timestamped cache entries with metadata.\n\nThis module provides efficient indexing and querying of timestamp-based\ncache entries, including content deduplication tracking, usage statistics,\nand cleanup coordination for the test caching system.","ref":"ExLLM.TestCacheIndex.html"},{"type":"function","title":"ExLLM.TestCacheIndex.add_entry/2","doc":"Add a new cache entry to the index.","ref":"ExLLM.TestCacheIndex.html#add_entry/2"},{"type":"function","title":"ExLLM.TestCacheIndex.add_entry/3","doc":"Add a new cache entry to the index and save to disk.","ref":"ExLLM.TestCacheIndex.html#add_entry/3"},{"type":"function","title":"ExLLM.TestCacheIndex.add_entry/4","doc":"Add a new cache entry with options like max_entries limit.","ref":"ExLLM.TestCacheIndex.html#add_entry/4"},{"type":"type","title":"ExLLM.TestCacheIndex.cache_entry/0","doc":"","ref":"ExLLM.TestCacheIndex.html#t:cache_entry/0"},{"type":"type","title":"ExLLM.TestCacheIndex.cache_index/0","doc":"","ref":"ExLLM.TestCacheIndex.html#t:cache_index/0"},{"type":"function","title":"ExLLM.TestCacheIndex.calculate_hit_rate/1","doc":"Calculate the hit rate for the index.","ref":"ExLLM.TestCacheIndex.html#calculate_hit_rate/1"},{"type":"function","title":"ExLLM.TestCacheIndex.cleanup_index/1","doc":"Clean up the index and apply retention policies.","ref":"ExLLM.TestCacheIndex.html#cleanup_index/1"},{"type":"function","title":"ExLLM.TestCacheIndex.cleanup_old_entries/3","doc":"Clean up old entries by age.","ref":"ExLLM.TestCacheIndex.html#cleanup_old_entries/3"},{"type":"function","title":"ExLLM.TestCacheIndex.find_duplicate_content/1","doc":"Find duplicate content using content hashes.","ref":"ExLLM.TestCacheIndex.html#find_duplicate_content/1"},{"type":"function","title":"ExLLM.TestCacheIndex.find_duplicates/1","doc":"Find duplicate entries based on content hash.","ref":"ExLLM.TestCacheIndex.html#find_duplicates/1"},{"type":"function","title":"ExLLM.TestCacheIndex.get_entries_by_status/2","doc":"Get entries that match the specified status.","ref":"ExLLM.TestCacheIndex.html#get_entries_by_status/2"},{"type":"function","title":"ExLLM.TestCacheIndex.get_entries_by_status/3","doc":"Get entries that match the specified status, with directory path for loading latest state.","ref":"ExLLM.TestCacheIndex.html#get_entries_by_status/3"},{"type":"function","title":"ExLLM.TestCacheIndex.get_entry_by_filename/2","doc":"Get entry by filename.","ref":"ExLLM.TestCacheIndex.html#get_entry_by_filename/2"},{"type":"function","title":"ExLLM.TestCacheIndex.get_statistics/1","doc":"Get cache statistics from the index.","ref":"ExLLM.TestCacheIndex.html#get_statistics/1"},{"type":"function","title":"ExLLM.TestCacheIndex.get_valid_entries/2","doc":"Get entries within the specified TTL.","ref":"ExLLM.TestCacheIndex.html#get_valid_entries/2"},{"type":"function","title":"ExLLM.TestCacheIndex.limit_entries/2","doc":"Keep only the specified number of most recent entries.","ref":"ExLLM.TestCacheIndex.html#limit_entries/2"},{"type":"function","title":"ExLLM.TestCacheIndex.load_index/1","doc":"Load cache index from disk or create a new one.","ref":"ExLLM.TestCacheIndex.html#load_index/1"},{"type":"function","title":"ExLLM.TestCacheIndex.rebuild_index/1","doc":"Rebuild index from existing cache files.","ref":"ExLLM.TestCacheIndex.html#rebuild_index/1"},{"type":"function","title":"ExLLM.TestCacheIndex.record_access/1","doc":"Record a cache access (hit or miss) for statistics.","ref":"ExLLM.TestCacheIndex.html#record_access/1"},{"type":"function","title":"ExLLM.TestCacheIndex.record_hit/1","doc":"Record a cache hit for statistics.","ref":"ExLLM.TestCacheIndex.html#record_hit/1"},{"type":"function","title":"ExLLM.TestCacheIndex.remove_entry/3","doc":"Remove a specific entry by filename.","ref":"ExLLM.TestCacheIndex.html#remove_entry/3"},{"type":"function","title":"ExLLM.TestCacheIndex.remove_old_entries/2","doc":"Remove cache entries older than the specified cutoff.","ref":"ExLLM.TestCacheIndex.html#remove_old_entries/2"},{"type":"function","title":"ExLLM.TestCacheIndex.save_index/2","doc":"Save cache index to disk.","ref":"ExLLM.TestCacheIndex.html#save_index/2"},{"type":"function","title":"ExLLM.TestCacheIndex.update_stats/3","doc":"Update cache statistics.","ref":"ExLLM.TestCacheIndex.html#update_stats/3"},{"type":"module","title":"ExLLM.TestCacheMatcher","doc":"Intelligent matching of requests to cached responses.\n\nThis module provides sophisticated request-to-cache matching strategies\nincluding exact matching, fuzzy matching, content-based matching, and\ntest context-aware matching for the automatic test response caching system.","ref":"ExLLM.TestCacheMatcher.html"},{"type":"type","title":"ExLLM.TestCacheMatcher.cached_request/0","doc":"","ref":"ExLLM.TestCacheMatcher.html#t:cached_request/0"},{"type":"function","title":"ExLLM.TestCacheMatcher.calculate_similarity/2","doc":"Calculate similarity score between two requests.","ref":"ExLLM.TestCacheMatcher.html#calculate_similarity/2"},{"type":"function","title":"ExLLM.TestCacheMatcher.context_match/3","doc":"Find context-aware match considering test module and tags.","ref":"ExLLM.TestCacheMatcher.html#context_match/3"},{"type":"function","title":"ExLLM.TestCacheMatcher.exact_match/2","doc":"Find exact match for a request in cached responses.","ref":"ExLLM.TestCacheMatcher.html#exact_match/2"},{"type":"function","title":"ExLLM.TestCacheMatcher.extract_message_content/1","doc":"Extract message content from request for semantic matching.","ref":"ExLLM.TestCacheMatcher.html#extract_message_content/1"},{"type":"function","title":"ExLLM.TestCacheMatcher.find_best_match/3","doc":"Find best match using comprehensive strategy.","ref":"ExLLM.TestCacheMatcher.html#find_best_match/3"},{"type":"function","title":"ExLLM.TestCacheMatcher.fuzzy_match/3","doc":"Find fuzzy match for a request with configurable tolerance.","ref":"ExLLM.TestCacheMatcher.html#fuzzy_match/3"},{"type":"function","title":"ExLLM.TestCacheMatcher.generate_request_signature/1","doc":"","ref":"ExLLM.TestCacheMatcher.html#generate_request_signature/1"},{"type":"type","title":"ExLLM.TestCacheMatcher.match_result/0","doc":"","ref":"ExLLM.TestCacheMatcher.html#t:match_result/0"},{"type":"function","title":"ExLLM.TestCacheMatcher.normalize_request/1","doc":"Normalize request for comparison.","ref":"ExLLM.TestCacheMatcher.html#normalize_request/1"},{"type":"function","title":"ExLLM.TestCacheMatcher.semantic_match/2","doc":"Find semantic match based on request content similarity.","ref":"ExLLM.TestCacheMatcher.html#semantic_match/2"},{"type":"module","title":"ExLLM.TestCacheStats","doc":"Track cache performance and cost savings with timestamp-based metrics.\n\nThis module provides comprehensive statistics tracking for the automatic\ntest response caching system, including hit/miss ratios, cost savings,\nperformance improvements, and storage optimization metrics.","ref":"ExLLM.TestCacheStats.html"},{"type":"type","title":"ExLLM.TestCacheStats.cache_stats/0","doc":"","ref":"ExLLM.TestCacheStats.html#t:cache_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.TestCacheStats.html#child_spec/1"},{"type":"function","title":"ExLLM.TestCacheStats.format_duration/1","doc":"Format duration in milliseconds.","ref":"ExLLM.TestCacheStats.html#format_duration/1"},{"type":"function","title":"ExLLM.TestCacheStats.format_percentage/1","doc":"Format a percentage value.","ref":"ExLLM.TestCacheStats.html#format_percentage/1"},{"type":"function","title":"ExLLM.TestCacheStats.get_cache_age_stats/0","doc":"Get cache age statistics.","ref":"ExLLM.TestCacheStats.html#get_cache_age_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_cache_key_stats/1","doc":"Get detailed statistics for a specific cache key.","ref":"ExLLM.TestCacheStats.html#get_cache_key_stats/1"},{"type":"function","title":"ExLLM.TestCacheStats.get_efficiency_metrics/0","doc":"Get cache efficiency metrics for monitoring.","ref":"ExLLM.TestCacheStats.html#get_efficiency_metrics/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_global_stats/0","doc":"Get comprehensive cache statistics for all cache keys.","ref":"ExLLM.TestCacheStats.html#get_global_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_provider_stats/1","doc":"Get statistics for a specific provider.","ref":"ExLLM.TestCacheStats.html#get_provider_stats/1"},{"type":"function","title":"ExLLM.TestCacheStats.get_refresh_reasons/0","doc":"Get refresh reasons statistics.","ref":"ExLLM.TestCacheStats.html#get_refresh_reasons/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_stats/0","doc":"Get basic cache statistics.","ref":"ExLLM.TestCacheStats.html#get_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_stats_by_provider/0","doc":"Get cache statistics by provider.","ref":"ExLLM.TestCacheStats.html#get_stats_by_provider/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_stats_by_test_suite/0","doc":"Get cache statistics by test suite/module.","ref":"ExLLM.TestCacheStats.html#get_stats_by_test_suite/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_storage_stats/0","doc":"Get storage statistics.","ref":"ExLLM.TestCacheStats.html#get_storage_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.get_test_stats/1","doc":"Get test-specific statistics.","ref":"ExLLM.TestCacheStats.html#get_test_stats/1"},{"type":"function","title":"ExLLM.TestCacheStats.print_cache_summary/0","doc":"Print a formatted cache summary to the console.","ref":"ExLLM.TestCacheStats.html#print_cache_summary/0"},{"type":"function","title":"ExLLM.TestCacheStats.print_provider_stats/0","doc":"Print cache statistics by provider.","ref":"ExLLM.TestCacheStats.html#print_provider_stats/0"},{"type":"type","title":"ExLLM.TestCacheStats.provider_stats/0","doc":"","ref":"ExLLM.TestCacheStats.html#t:provider_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.record_cost_saving/2","doc":"Record a cost saving event.","ref":"ExLLM.TestCacheStats.html#record_cost_saving/2"},{"type":"function","title":"ExLLM.TestCacheStats.record_cost_savings/1","doc":"Record cost savings from cache usage.","ref":"ExLLM.TestCacheStats.html#record_cost_savings/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_fallback/1","doc":"Record fallback usage.","ref":"ExLLM.TestCacheStats.html#record_fallback/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_hit/1","doc":"Record a cache hit event.","ref":"ExLLM.TestCacheStats.html#record_hit/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_miss/1","doc":"Record a cache miss event.","ref":"ExLLM.TestCacheStats.html#record_miss/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_operation/3","doc":"Record a cache operation for statistics.","ref":"ExLLM.TestCacheStats.html#record_operation/3"},{"type":"function","title":"ExLLM.TestCacheStats.record_refresh/1","doc":"Record a refresh event.","ref":"ExLLM.TestCacheStats.html#record_refresh/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_storage_usage/1","doc":"Record storage usage metrics.","ref":"ExLLM.TestCacheStats.html#record_storage_usage/1"},{"type":"function","title":"ExLLM.TestCacheStats.record_time_saved/1","doc":"Record time saved by using cache.","ref":"ExLLM.TestCacheStats.html#record_time_saved/1"},{"type":"function","title":"ExLLM.TestCacheStats.reset_stats/0","doc":"Reset all cache statistics.\n\nClears all in-memory statistics. Note that this doesn't affect\nthe actual cache files, only the statistics tracking.","ref":"ExLLM.TestCacheStats.html#reset_stats/0"},{"type":"function","title":"ExLLM.TestCacheStats.start_link/1","doc":"","ref":"ExLLM.TestCacheStats.html#start_link/1"},{"type":"type","title":"ExLLM.TestCacheStats.test_suite_stats/0","doc":"","ref":"ExLLM.TestCacheStats.html#t:test_suite_stats/0"},{"type":"module","title":"ExLLM.TestCacheStrategy","doc":"Implement cache-first strategy for test requests with timestamp selection.\n\nThis module orchestrates the complete cache-first request flow including\ncache checking, timestamp selection, fallback handling, and cache warming\nfor the automatic test response caching system.","ref":"ExLLM.TestCacheStrategy.html"},{"type":"function","title":"ExLLM.TestCacheStrategy.build_request_metadata/2","doc":"Build request metadata for tracking.","ref":"ExLLM.TestCacheStrategy.html#build_request_metadata/2"},{"type":"function","title":"ExLLM.TestCacheStrategy.execute/2","doc":"Execute cache-first strategy for a request.\n\nFlow:\n1. Check if test caching is enabled\n2. Generate cache key from request and test context\n3. Load cache index for the cache key\n4. Select best timestamp entry based on strategy\n5. If valid timestamp found: return cached response\n6. If no valid cache or expired: return proceed signal\n7. If real request fails: fallback to older timestamps if available","ref":"ExLLM.TestCacheStrategy.html#execute/2"},{"type":"function","title":"ExLLM.TestCacheStrategy.extract_provider/1","doc":"Extract provider from cache key or URL.","ref":"ExLLM.TestCacheStrategy.html#extract_provider/1"},{"type":"type","title":"ExLLM.TestCacheStrategy.fallback_options/0","doc":"","ref":"ExLLM.TestCacheStrategy.html#t:fallback_options/0"},{"type":"function","title":"ExLLM.TestCacheStrategy.generate_cache_key/1","doc":"Generate cache key for a request.","ref":"ExLLM.TestCacheStrategy.html#generate_cache_key/1"},{"type":"function","title":"ExLLM.TestCacheStrategy.get_cache_performance_stats/0","doc":"Get cache statistics for monitoring.","ref":"ExLLM.TestCacheStrategy.html#get_cache_performance_stats/0"},{"type":"function","title":"ExLLM.TestCacheStrategy.get_cached_response/3","doc":"Get cached response if available.","ref":"ExLLM.TestCacheStrategy.html#get_cached_response/3"},{"type":"function","title":"ExLLM.TestCacheStrategy.handle_request_failure/3","doc":"Handle fallback when real request fails.","ref":"ExLLM.TestCacheStrategy.html#handle_request_failure/3"},{"type":"function","title":"ExLLM.TestCacheStrategy.invalidate_cache/1","doc":"Invalidate cache entries matching a pattern.","ref":"ExLLM.TestCacheStrategy.html#invalidate_cache/1"},{"type":"function","title":"ExLLM.TestCacheStrategy.normalize_request_for_cache/1","doc":"Normalize request for cache key generation.","ref":"ExLLM.TestCacheStrategy.html#normalize_request_for_cache/1"},{"type":"function","title":"ExLLM.TestCacheStrategy.save_response/3","doc":"Save response to cache.","ref":"ExLLM.TestCacheStrategy.html#save_response/3"},{"type":"function","title":"ExLLM.TestCacheStrategy.should_use_cache?/1","doc":"Check if cache should be used for given options.","ref":"ExLLM.TestCacheStrategy.html#should_use_cache?/1"},{"type":"function","title":"ExLLM.TestCacheStrategy.should_warm_cache?/1","doc":"Check if cache warming is recommended for a test scenario.","ref":"ExLLM.TestCacheStrategy.html#should_warm_cache?/1"},{"type":"type","title":"ExLLM.TestCacheStrategy.strategy_result/0","doc":"","ref":"ExLLM.TestCacheStrategy.html#t:strategy_result/0"},{"type":"function","title":"ExLLM.TestCacheStrategy.warm_cache/1","doc":"Warm cache for upcoming test scenarios.","ref":"ExLLM.TestCacheStrategy.html#warm_cache/1"},{"type":"module","title":"ExLLM.TestCacheTTL","doc":"Handle cache selection and TTL logic for timestamp-based caching.\n\nThis module implements cache expiration checking, smart cache selection\nbased on fallback strategies, and TTL management for different test\nscenarios in the automatic test response caching system.","ref":"ExLLM.TestCacheTTL.html"},{"type":"function","title":"ExLLM.TestCacheTTL.api_version_compatible?/2","doc":"Check if a cached response is still compatible with current API version.","ref":"ExLLM.TestCacheTTL.html#api_version_compatible?/2"},{"type":"function","title":"ExLLM.TestCacheTTL.cache_expired?/2","doc":"Check if a cache entry is expired based on TTL.","ref":"ExLLM.TestCacheTTL.html#cache_expired?/2"},{"type":"type","title":"ExLLM.TestCacheTTL.cache_selection_result/0","doc":"","ref":"ExLLM.TestCacheTTL.html#t:cache_selection_result/0"},{"type":"function","title":"ExLLM.TestCacheTTL.calculate_ttl/2","doc":"Calculate TTL for specific test context and provider.","ref":"ExLLM.TestCacheTTL.html#calculate_ttl/2"},{"type":"type","title":"ExLLM.TestCacheTTL.fallback_strategy/0","doc":"","ref":"ExLLM.TestCacheTTL.html#t:fallback_strategy/0"},{"type":"function","title":"ExLLM.TestCacheTTL.force_refresh_for_test?/1","doc":"Check if a force refresh is required for the current test context.","ref":"ExLLM.TestCacheTTL.html#force_refresh_for_test?/1"},{"type":"function","title":"ExLLM.TestCacheTTL.get_cache_age/1","doc":"Get cache entry age in milliseconds.","ref":"ExLLM.TestCacheTTL.html#get_cache_age/1"},{"type":"function","title":"ExLLM.TestCacheTTL.get_entries_near_expiration/2","doc":"Get entries that are near expiration and should be refreshed.","ref":"ExLLM.TestCacheTTL.html#get_entries_near_expiration/2"},{"type":"function","title":"ExLLM.TestCacheTTL.get_latest_successful_entry/2","doc":"Get the latest successful entry from cache directory within TTL.","ref":"ExLLM.TestCacheTTL.html#get_latest_successful_entry/2"},{"type":"function","title":"ExLLM.TestCacheTTL.get_latest_valid_entry/2","doc":"Get the latest valid (non-expired) entry from cache directory.","ref":"ExLLM.TestCacheTTL.html#get_latest_valid_entry/2"},{"type":"function","title":"ExLLM.TestCacheTTL.get_refresh_priority/2","doc":"Get cache refresh priority based on usage and expiration.","ref":"ExLLM.TestCacheTTL.html#get_refresh_priority/2"},{"type":"function","title":"ExLLM.TestCacheTTL.select_cache_entry/3","doc":"Select the best cache entry based on TTL and fallback strategy.","ref":"ExLLM.TestCacheTTL.html#select_cache_entry/3"},{"type":"function","title":"ExLLM.TestCacheTTL.select_fallback_entry/2","doc":"Select fallback entry when primary cache is unavailable.","ref":"ExLLM.TestCacheTTL.html#select_fallback_entry/2"},{"type":"function","title":"ExLLM.TestCacheTTL.should_warm_cache?/2","doc":"Check if cache should be warmed up based on upcoming expiration.","ref":"ExLLM.TestCacheTTL.html#should_warm_cache?/2"},{"type":"module","title":"ExLLM.TestCacheTimestamp","doc":"Manage timestamped cache entries and cleanup policies.\n\nThis module handles timestamp generation, parsing, cleanup operations,\nand content deduplication for the timestamp-based caching system.","ref":"ExLLM.TestCacheTimestamp.html"},{"type":"function","title":"ExLLM.TestCacheTimestamp.cleanup_old_entries/3","doc":"Clean up old cache entries based on max entries and max age policies.","ref":"ExLLM.TestCacheTimestamp.html#cleanup_old_entries/3"},{"type":"type","title":"ExLLM.TestCacheTimestamp.cleanup_report/0","doc":"","ref":"ExLLM.TestCacheTimestamp.html#t:cleanup_report/0"},{"type":"type","title":"ExLLM.TestCacheTimestamp.dedup_report/0","doc":"","ref":"ExLLM.TestCacheTimestamp.html#t:dedup_report/0"},{"type":"function","title":"ExLLM.TestCacheTimestamp.deduplicate_content/1","doc":"Deduplicate content across timestamps using file hashes.","ref":"ExLLM.TestCacheTimestamp.html#deduplicate_content/1"},{"type":"function","title":"ExLLM.TestCacheTimestamp.generate_timestamp_filename/0","doc":"Generate a timestamp-based filename for the current moment.","ref":"ExLLM.TestCacheTimestamp.html#generate_timestamp_filename/0"},{"type":"function","title":"ExLLM.TestCacheTimestamp.generate_timestamp_filename/1","doc":"Generate a timestamp-based filename for a specific DateTime.","ref":"ExLLM.TestCacheTimestamp.html#generate_timestamp_filename/1"},{"type":"function","title":"ExLLM.TestCacheTimestamp.get_content_hash/1","doc":"Get content hash for a file.","ref":"ExLLM.TestCacheTimestamp.html#get_content_hash/1"},{"type":"function","title":"ExLLM.TestCacheTimestamp.list_cache_timestamps/1","doc":"List all cache timestamps for a given cache directory, sorted newest first.","ref":"ExLLM.TestCacheTimestamp.html#list_cache_timestamps/1"},{"type":"function","title":"ExLLM.TestCacheTimestamp.parse_timestamp_from_filename/1","doc":"Parse timestamp from filename.","ref":"ExLLM.TestCacheTimestamp.html#parse_timestamp_from_filename/1"},{"type":"module","title":"ExLLM.TestResponseInterceptor","doc":"Automatically intercept and cache responses during tests.\n\nThis module hooks into the HTTPClient request/response cycle to provide\nautomatic cache key generation, rich metadata capture, and streaming\nresponse reassembly for the test caching system.","ref":"ExLLM.TestResponseInterceptor.html"},{"type":"function","title":"ExLLM.TestResponseInterceptor.build_response_metadata/3","doc":"Build comprehensive metadata for cached responses.","ref":"ExLLM.TestResponseInterceptor.html#build_response_metadata/3"},{"type":"function","title":"ExLLM.TestResponseInterceptor.generate_cache_key/3","doc":"Generate cache key from request parameters and test context.","ref":"ExLLM.TestResponseInterceptor.html#generate_cache_key/3"},{"type":"function","title":"ExLLM.TestResponseInterceptor.generate_request_signature/2","doc":"Generate a unique signature for the request to differentiate similar requests.","ref":"ExLLM.TestResponseInterceptor.html#generate_request_signature/2"},{"type":"function","title":"ExLLM.TestResponseInterceptor.handle_streaming_response/2","doc":"Handle streaming response caching.","ref":"ExLLM.TestResponseInterceptor.html#handle_streaming_response/2"},{"type":"function","title":"ExLLM.TestResponseInterceptor.intercept_request/4","doc":"Intercept a request and check for cached response.","ref":"ExLLM.TestResponseInterceptor.html#intercept_request/4"},{"type":"type","title":"ExLLM.TestResponseInterceptor.intercept_result/0","doc":"","ref":"ExLLM.TestResponseInterceptor.html#t:intercept_result/0"},{"type":"function","title":"ExLLM.TestResponseInterceptor.record_cache_hit/2","doc":"Record cache hit for statistics.","ref":"ExLLM.TestResponseInterceptor.html#record_cache_hit/2"},{"type":"function","title":"ExLLM.TestResponseInterceptor.record_cache_miss/2","doc":"Record cache miss for statistics.","ref":"ExLLM.TestResponseInterceptor.html#record_cache_miss/2"},{"type":"function","title":"ExLLM.TestResponseInterceptor.save_response/3","doc":"Save response after successful request.","ref":"ExLLM.TestResponseInterceptor.html#save_response/3"},{"type":"function","title":"ExLLM.TestResponseInterceptor.should_intercept_request?/0","doc":"Check if request should be intercepted for caching.","ref":"ExLLM.TestResponseInterceptor.html#should_intercept_request?/0"},{"type":"module","title":"ExLLM.TestResponseMetadata","doc":"Capture comprehensive metadata for cached responses.\n\nThis module provides structured metadata capture for test response caching,\nincluding request information, response data, test context, timing, and\nusage tracking for comprehensive cache management.","ref":"ExLLM.TestResponseMetadata.html"},{"type":"function","title":"ExLLM.TestResponseMetadata.create_cache_key_metadata/1","doc":"Extract minimal metadata for cache key generation.","ref":"ExLLM.TestResponseMetadata.html#create_cache_key_metadata/1"},{"type":"function","title":"ExLLM.TestResponseMetadata.create_metadata/3","doc":"Create comprehensive metadata for a cached response.","ref":"ExLLM.TestResponseMetadata.html#create_metadata/3"},{"type":"function","title":"ExLLM.TestResponseMetadata.extract_cost_summary/1","doc":"Extract cost information from metadata.","ref":"ExLLM.TestResponseMetadata.html#extract_cost_summary/1"},{"type":"function","title":"ExLLM.TestResponseMetadata.extract_timing_info/1","doc":"Extract timing information from metadata.","ref":"ExLLM.TestResponseMetadata.html#extract_timing_info/1"},{"type":"type","title":"ExLLM.TestResponseMetadata.response_metadata/0","doc":"","ref":"ExLLM.TestResponseMetadata.html#t:response_metadata/0"},{"type":"function","title":"ExLLM.TestResponseMetadata.sanitize_for_storage/1","doc":"Sanitize metadata for storage (remove sensitive information).","ref":"ExLLM.TestResponseMetadata.html#sanitize_for_storage/1"},{"type":"function","title":"ExLLM.TestResponseMetadata.successful_response?/1","doc":"Check if metadata indicates a successful response.","ref":"ExLLM.TestResponseMetadata.html#successful_response?/1"},{"type":"function","title":"ExLLM.TestResponseMetadata.update_metadata/2","doc":"Update metadata with additional information.","ref":"ExLLM.TestResponseMetadata.html#update_metadata/2"},{"type":"module","title":"ExLLM.Types","doc":"Shared type definitions used across ExLLM modules.\n\nThis module contains struct definitions and types that are used by multiple\nmodules, helping to avoid circular dependencies.","ref":"ExLLM.Types.html"},{"type":"type","title":"ExLLM.Types.adapter_options/0","doc":"","ref":"ExLLM.Types.html#t:adapter_options/0"},{"type":"type","title":"ExLLM.Types.content_part/0","doc":"","ref":"ExLLM.Types.html#t:content_part/0"},{"type":"type","title":"ExLLM.Types.cost_result/0","doc":"","ref":"ExLLM.Types.html#t:cost_result/0"},{"type":"type","title":"ExLLM.Types.image_content/0","doc":"","ref":"ExLLM.Types.html#t:image_content/0"},{"type":"type","title":"ExLLM.Types.image_data/0","doc":"","ref":"ExLLM.Types.html#t:image_data/0"},{"type":"type","title":"ExLLM.Types.image_url/0","doc":"","ref":"ExLLM.Types.html#t:image_url/0"},{"type":"type","title":"ExLLM.Types.message/0","doc":"","ref":"ExLLM.Types.html#t:message/0"},{"type":"type","title":"ExLLM.Types.pricing/0","doc":"","ref":"ExLLM.Types.html#t:pricing/0"},{"type":"type","title":"ExLLM.Types.stream/0","doc":"","ref":"ExLLM.Types.html#t:stream/0"},{"type":"type","title":"ExLLM.Types.text_content/0","doc":"","ref":"ExLLM.Types.html#t:text_content/0"},{"type":"type","title":"ExLLM.Types.token_usage/0","doc":"","ref":"ExLLM.Types.html#t:token_usage/0"},{"type":"module","title":"ExLLM.Types.EmbeddingModel","doc":"Represents an available embedding model.","ref":"ExLLM.Types.EmbeddingModel.html"},{"type":"type","title":"ExLLM.Types.EmbeddingModel.t/0","doc":"","ref":"ExLLM.Types.EmbeddingModel.html#t:t/0"},{"type":"module","title":"ExLLM.Types.EmbeddingResponse","doc":"Represents an embedding response from an LLM provider.","ref":"ExLLM.Types.EmbeddingResponse.html"},{"type":"type","title":"ExLLM.Types.EmbeddingResponse.t/0","doc":"","ref":"ExLLM.Types.EmbeddingResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Types.LLMResponse","doc":"Standard response format from LLM adapters with integrated cost calculation.","ref":"ExLLM.Types.LLMResponse.html"},{"type":"type","title":"ExLLM.Types.LLMResponse.t/0","doc":"","ref":"ExLLM.Types.LLMResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Types.Model","doc":"Represents an available LLM model.","ref":"ExLLM.Types.Model.html"},{"type":"type","title":"ExLLM.Types.Model.t/0","doc":"","ref":"ExLLM.Types.Model.html#t:t/0"},{"type":"module","title":"ExLLM.Types.StreamChunk","doc":"Represents a chunk from a streaming LLM response.","ref":"ExLLM.Types.StreamChunk.html"},{"type":"type","title":"ExLLM.Types.StreamChunk.t/0","doc":"","ref":"ExLLM.Types.StreamChunk.html#t:t/0"},{"type":"module","title":"ExLLM.Vision","doc":"Vision and multimodal support for ExLLM.\n\nProvides utilities for handling images in LLM requests, including:\n- Image format validation\n- Base64 encoding/decoding\n- URL validation\n- Provider-specific formatting","ref":"ExLLM.Vision.html"},{"type":"module","title":"Supported Image Formats - ExLLM.Vision","doc":"- JPEG/JPG\n- PNG\n- GIF (static only for most providers)\n- WebP","ref":"ExLLM.Vision.html#module-supported-image-formats"},{"type":"module","title":"Usage - ExLLM.Vision","doc":"# With image URL\n    messages = [\n      %{\n        role: \"user\",\n        content: [\n          %{type: \"text\", text: \"What's in this image?\"},\n          %{type: \"image_url\", image_url: %{url: \"https://example.com/image.jpg\"}}\n        ]\n      }\n    ]\n    \n    # With base64 image\n    image_data = File.read!(\"photo.jpg\") |> Base.encode64()\n    messages = [\n      %{\n        role: \"user\", \n        content: [\n          %{type: \"text\", text: \"Describe this photo\"},\n          %{type: \"image\", image: %{data: image_data, media_type: \"image/jpeg\"}}\n        ]\n      }\n    ]\n    \n    {:ok, response} = ExLLM.chat(:anthropic, messages)","ref":"ExLLM.Vision.html#module-usage"},{"type":"function","title":"ExLLM.Vision.build_message/4","doc":"Build a vision message with text and images.","ref":"ExLLM.Vision.html#build_message/4"},{"type":"function","title":"Examples - ExLLM.Vision.build_message/4","doc":"message = ExLLM.Vision.build_message(\"user\", \"What's in these images?\", [\n      \"https://example.com/image1.jpg\",\n      \"/path/to/local/image2.png\"\n    ])","ref":"ExLLM.Vision.html#build_message/4-examples"},{"type":"function","title":"ExLLM.Vision.count_images/1","doc":"Count images in a message.","ref":"ExLLM.Vision.html#count_images/1"},{"type":"function","title":"ExLLM.Vision.format_for_provider/2","doc":"Format messages for a specific provider.\n\nSome providers have specific requirements for vision content.","ref":"ExLLM.Vision.html#format_for_provider/2"},{"type":"function","title":"ExLLM.Vision.has_vision_content?/1","doc":"Check if a message contains vision content.","ref":"ExLLM.Vision.html#has_vision_content?/1"},{"type":"function","title":"ExLLM.Vision.image_url/2","doc":"Create an image URL content part.","ref":"ExLLM.Vision.html#image_url/2"},{"type":"function","title":"Options - ExLLM.Vision.image_url/2","doc":"- `:detail` - Image detail level (:auto, :low, :high)","ref":"ExLLM.Vision.html#image_url/2-options"},{"type":"function","title":"ExLLM.Vision.load_image/2","doc":"Load an image from file and encode it for API use.\n\nReturns a content part ready to be included in a message.","ref":"ExLLM.Vision.html#load_image/2"},{"type":"function","title":"ExLLM.Vision.normalize_messages/1","doc":"Validate and normalize messages containing images.\n\nReturns `{:ok, normalized_messages}` or `{:error, reason}`.","ref":"ExLLM.Vision.html#normalize_messages/1"},{"type":"function","title":"ExLLM.Vision.supports_vision?/1","doc":"Check if a provider supports vision/multimodal inputs.","ref":"ExLLM.Vision.html#supports_vision?/1"},{"type":"function","title":"ExLLM.Vision.text/1","doc":"Create a text content part.","ref":"ExLLM.Vision.html#text/1"},{"type":"task","title":"mix ex_llm.cache","doc":"Mix tasks for managing the ExLLM test response cache.","ref":"Mix.Tasks.ExLlm.Cache.html"},{"type":"task","title":"Available Commands - mix ex_llm.cache","doc":"mix ex_llm.cache.stats          # Show cache statistics\n    mix ex_llm.cache.clear          # Clear all cache\n    mix ex_llm.cache.clear --provider openai  # Clear specific provider\n    mix ex_llm.cache.cleanup        # Clean up old entries\n    mix ex_llm.cache.deduplicate    # Deduplicate content\n    mix ex_llm.cache.list           # List cache keys\n    mix ex_llm.cache.verify         # Verify cache integrity","ref":"Mix.Tasks.ExLlm.Cache.html#module-available-commands"},{"type":"task","title":"mix ex_llm.cache.cleanup","doc":"Clean up old cache entries","ref":"Mix.Tasks.ExLlm.Cache.Cleanup.html"},{"type":"function","title":"Mix.Tasks.ExLlm.Cache.Cleanup.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.ExLlm.Cache.Cleanup.html#run/1"},{"type":"task","title":"mix ex_llm.cache.clear","doc":"Clear test cache","ref":"Mix.Tasks.ExLlm.Cache.Clear.html"},{"type":"function","title":"Mix.Tasks.ExLlm.Cache.Clear.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.ExLlm.Cache.Clear.html#run/1"},{"type":"task","title":"mix ex_llm.cache.stats","doc":"Show test cache statistics","ref":"Mix.Tasks.ExLlm.Cache.Stats.html"},{"type":"function","title":"Mix.Tasks.ExLlm.Cache.Stats.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.ExLlm.Cache.Stats.html#run/1"},{"type":"extras","title":"ExLLM","doc":"# ExLLM\n\nA unified Elixir client for Large Language Models with intelligent test caching, comprehensive provider support, and advanced developer tooling.\n\n> ⚠️ **Alpha Quality Software**: This library is in early development. APIs may change without notice until version 1.0.0 is released. Use in production at your own risk.","ref":"readme.html"},{"type":"extras","title":"What's New in v0.7.0 - ExLLM","doc":"- **🚀 Advanced Test Caching System**: 25x faster integration tests with intelligent response caching\n- **🏷️ Comprehensive Test Tagging**: Semantic test organization with 138 skip tags replaced by meaningful tags\n- **⚡ Mix Test Aliases**: 24 new aliases for targeted testing (provider-specific, tag-based, capability-based)\n- **🤖 Smart Test Detection**: Automatic requirement checking with ExLLM.Case module\n- **📊 Cache Management**: Built-in cache analytics, TTL management, and cleanup tools","ref":"readme.html#what-s-new-in-v0-7-0"},{"type":"extras","title":"Features - ExLLM","doc":"### 🔗 **Core API**\n- **Unified Interface**: Single API for 14+ LLM providers and 300+ models\n- **Streaming Support**: Real-time streaming responses with error recovery\n- **Session Management**: Built-in conversation state tracking and persistence\n- **Function Calling**: Unified tool use interface across all providers\n- **Multimodal Support**: Vision, audio, and document processing capabilities\n\n### 📊 **Developer Experience**\n- **Intelligent Test Caching**: 25x faster integration tests with smart response caching\n- **Comprehensive Test Tagging**: Semantic organization with provider, capability, and requirement tags\n- **Mix Test Aliases**: 24 targeted testing commands (e.g., `mix test.anthropic`, `mix test.streaming`)\n- **Automatic Requirement Checking**: Dynamic test skipping with meaningful error messages\n- **Cost Tracking**: Automatic cost calculation and token usage monitoring\n\n### 🎯 **Advanced Features**\n- **Complete Gemini API**: All 15 Gemini APIs including Live API with WebSocket support\n- **OAuth2 Authentication**: Full OAuth2 support for provider APIs requiring user auth\n- **Structured Outputs**: Schema validation and retries via Instructor integration\n- **Model Discovery**: Query and compare capabilities across all providers\n- **Response Caching**: Production-ready caching with TTL, fallback strategies, and analytics\n- **Type Safety**: Comprehensive typespecs and structured data throughout","ref":"readme.html#features"},{"type":"extras","title":"Supported Providers - ExLLM","doc":"ExLLM supports **14 providers** with access to **300+ models**:\n\n- **Anthropic Claude** - Claude 4, 3.7, 3.5, and 3 series models\n- **OpenAI** - GPT-4.1, o1 reasoning models, GPT-4o, and GPT-3.5 series\n- **AWS Bedrock** - Multi-provider access (Anthropic, Amazon Nova, Meta Llama, etc.)\n- **Google Gemini** - Gemini 2.5, 2.0, and 1.5 series with multimodal support\n- **OpenRouter** - Access to 300+ models from multiple providers\n- **Groq** - Ultra-fast inference with Llama 4, DeepSeek R1, and more\n- **X.AI** - Grok models with web search and reasoning capabilities\n- **Mistral AI** - Mistral Large, Pixtral, and specialized code models\n- **Perplexity** - Search-enhanced language models\n- **Ollama** - Local model runner (any model in your installation)\n- **LM Studio** - Local model server with OpenAI-compatible API\n- **Bumblebee** - Local model inference with Elixir/Nx\n- **Mock Adapter** - For testing and development","ref":"readme.html#supported-providers"},{"type":"extras","title":"Installation - ExLLM","doc":"Add `ex_llm` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.7.0\"},\n    \n    # Optional hardware acceleration backends (choose one):\n    {:exla, \"~> 0.7\", optional: true},\n    \n    # Optional: For Apple Silicon Metal acceleration\n    {:emlx, github: \"elixir-nx/emlx\", branch: \"main\", optional: true}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - ExLLM","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"1. Configuration - ExLLM","doc":"Set your API keys as environment variables:\n\n```bash\nexport ANTHROPIC_API_KEY=\"your-anthropic-key\"\nexport OPENAI_API_KEY=\"your-openai-key\"\nexport GROQ_API_KEY=\"your-groq-key\"\n# ... other provider keys as needed\n```","ref":"readme.html#1-configuration"},{"type":"extras","title":"2. Basic Usage - ExLLM","doc":"```elixir\n# Single completion\n{:ok, response} = ExLLM.chat(:anthropic, [\n  %{role: \"user\", content: \"Explain quantum computing in simple terms\"}\n])\n\nIO.puts(response.content)\n# Cost automatically tracked: response.cost\n\n# Streaming response\nExLLM.chat_stream(:openai, [\n  %{role: \"user\", content: \"Write a short story\"}\n], fn chunk ->\n  IO.write(chunk.delta)\nend)\n\n# With session management\n{:ok, session} = ExLLM.Session.new(:groq)\n{:ok, session, response} = ExLLM.Session.chat(session, \"Hello!\")\n{:ok, session, response} = ExLLM.Session.chat(session, \"How are you?\")\n\n# Multimodal with vision\n{:ok, response} = ExLLM.chat(:gemini, [\n  %{role: \"user\", content: [\n    %{type: \"text\", text: \"What's in this image?\"},\n    %{type: \"image\", image: %{data: base64_image, media_type: \"image/jpeg\"}}\n  ]}\n])\n```","ref":"readme.html#2-basic-usage"},{"type":"extras","title":"3. Testing with Caching - ExLLM","doc":"```elixir\n# Run integration tests with automatic caching\nmix test.anthropic --include live_api\n\n# Manage test cache\nmix ex_llm.cache stats\nmix ex_llm.cache clean --older-than 7d\n```","ref":"readme.html#3-testing-with-caching"},{"type":"extras","title":"Documentation - ExLLM","doc":"📚 **[Quick Start Guide](docs/QUICKSTART.md)** - Get up and running in 5 minutes  \n📖 **[User Guide](docs/USER_GUIDE.md)** - Comprehensive documentation of all features  \n🔧 **[Logger Guide](docs/LOGGER.md)** - Debug logging and troubleshooting  \n⚡ **[Provider Capabilities](docs/PROVIDER_CAPABILITIES.md)** - Feature comparison across providers  \n🧪 **[Testing Guide](docs/TESTING.md)** - Comprehensive testing system with semantic tagging and caching","ref":"readme.html#documentation"},{"type":"extras","title":"Key Topics Covered in the User Guide - ExLLM","doc":"- **Configuration**: Environment variables, config files, and provider setup\n- **Chat Completions**: Messages, parameters, and response handling\n- **Streaming**: Real-time responses with error recovery and coordinator\n- **Session Management**: Conversation state and persistence\n- **Function Calling**: Tool use and structured interactions across providers\n- **Vision & Multimodal**: Image, audio, and document processing\n- **Cost Tracking**: Automatic cost calculation and token estimation\n- **Error Handling**: Retry logic and error recovery strategies\n- **Test Caching**: Intelligent response caching with 25x speed improvements\n- **Test Organization**: Semantic tagging and targeted test execution\n- **Model Discovery**: Query available models and capabilities\n- **OAuth2 Integration**: Complete OAuth2 flow for Gemini and other providers","ref":"readme.html#key-topics-covered-in-the-user-guide"},{"type":"extras","title":"License - ExLLM","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Contributing - ExLLM","doc":"Contributions are welcome! Please feel free to submit a Pull Request.","ref":"readme.html#contributing"},{"type":"extras","title":"Support - ExLLM","doc":"- 📖 **Documentation**: [User Guide](docs/USER_GUIDE.md)\n- 🐛 **Issues**: [GitHub Issues](https://github.com/azmaveth/ex_llm/issues)\n- 💬 **Discussions**: [GitHub Discussions](https://github.com/azmaveth/ex_llm/discussions)","ref":"readme.html#support"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.7.1] - 2025-06-14","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- **Comprehensive Documentation System** - Complete ExDoc configuration with organized structure\n  - 24 Mix test aliases for targeted testing (provider, capability, and type-based)\n  - Organized documentation into logical groups: Guides, References\n  - Complete test documentation covering semantic tagging and caching system","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated ExDoc configuration to include all public documentation files\n- Streamlined documentation structure by removing internal development docs\n- Enhanced README with current feature set and improved examples","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Resolved all ExDoc file reference warnings\n- Fixed documentation generation for publication-ready docs\n\n## [0.7.0] - 2025-06-14","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- **Advanced Test Response Caching System** - Complete caching infrastructure for integration tests\n  - Intelligent cache storage with JSON-based persistence\n  - TTL-based cache expiration and cleanup\n  - Request/response matching with fuzzy algorithms\n  - Cache statistics and performance monitoring\n  - Automatic cache key generation and indexing\n  - Smart fallback strategies for cache misses\n  - Configurable cache organization (by provider, test module, or tag)\n  - Environment-based cache configuration\n  - Mix task for cache management: `mix ex_llm.cache`","ref":"changelog.html#added-1"},{"type":"extras","title":"Enhanced - Changelog","doc":"- **Test Caching Performance** - 25x speed improvement for integration tests\n- **Cache Detection** - Automatic detection of destructive operations\n- **Response Interception** - Transparent request/response caching for HTTP calls\n- **Metadata Tracking** - Comprehensive test context and response metadata\n\n## [0.6.0] - 2025-06-14","ref":"changelog.html#enhanced"},{"type":"extras","title":"Added - Changelog","doc":"- **Comprehensive Test Tagging System** - Replaced all 138 generic `@tag :skip` with meaningful semantic tags\n  - `:live_api` - Tests that call live provider APIs\n  - `:requires_api_key` - Tests needing API keys with provider-specific checking\n  - `:requires_oauth` - Tests needing OAuth2 authentication\n  - `:requires_service` - Tests needing local services (Ollama, LM Studio)\n  - `:requires_resource` - Tests needing pre-existing resources (tuned models, corpora)\n  - `:integration` - Integration tests with external services\n  - `:external` - Tests making external network calls\n  - Provider-specific tags: `:anthropic`, `:openai`, `:gemini`, etc.\n- **Enhanced Test Caching System** - Intelligent caching based on test tags\n  - Uses `:live_api` tag to determine which tests to cache\n  - Automatic detection of destructive operations (create, delete, modify)\n  - Smart cache exclusion for corpus deletion and state-changing tests\n  - 25x speed improvement for cached integration tests (2.2s → 0.09s)\n- **Mix Test Aliases** - 24 new test aliases for targeted testing\n  - Provider-specific: `mix test.anthropic`, `mix test.openai`, etc.\n  - Tag-based: `mix test.integration`, `mix test.oauth2`, `mix test.live_api`\n  - Capability-based: `mix test.streaming`, `mix test.vision`\n- **ExLLM.Case Test Module** - Custom test case with automatic requirement checking\n  - Dynamic skipping with meaningful messages when requirements aren't met\n  - API key validation per provider\n  - OAuth2 token validation\n  - Service availability checking","ref":"changelog.html#added-2"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** Migrated from generic `:skip` tags to semantic tagging system\n- Enhanced OAuth2 test helper to use consistent `:requires_oauth` tag\n- Improved test cache detection to prevent caching destructive operations\n- Updated all provider integration tests with proper module-level tags","ref":"changelog.html#changed-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed undefined variable `service` in ExLLM.Case rescue clause\n- Fixed OpenRouter test compilation error with undefined function\n- Fixed OAuth2 tag inconsistency (now uses `:requires_oauth` everywhere)\n- Fixed test cache configuration for destructive operation detection\n\n## [0.5.0] - 2025-06-13","ref":"changelog.html#fixed-1"},{"type":"extras","title":"Added - Changelog","doc":"- **Complete Google Gemini API Implementation** - All 15 Gemini APIs now fully implemented\n  - **Live API**: Real-time bidirectional communication with WebSocket support\n    - Text, audio, and video streaming capabilities\n    - Tool/function calling in live sessions\n    - Session resumption and context compression\n    - Activity detection and management\n    - Audio transcription for input/output\n  - **Models API**: List and get model information\n  - **Content Generation API**: Chat and streaming with multimodal support\n  - **Token Counting API**: Count tokens for any content\n  - **Files API**: Upload and manage media files\n  - **Context Caching API**: Cache content for reuse across requests\n  - **Embeddings API**: Generate text embeddings\n  - **Fine-tuning API**: Create and manage custom tuned models\n  - **Permissions API**: Manage access to tuned models and corpora\n  - **Question Answering API**: Semantic search and QA\n  - **Corpus Management API**: Create and manage knowledge corpora\n  - **Document Management API**: Manage documents within corpora\n  - **Chunk Management API**: Fine-grained document chunk management\n  - **Retrieval Permissions API**: Control access to retrieval resources\n- **Gun WebSocket Library**: Added Gun dependency for Live API WebSocket support\n- **OAuth2 Authentication**: Full OAuth2 support for Gemini APIs requiring user auth\n- **Comprehensive Test Suite**: 477 tests covering all Gemini functionality","ref":"changelog.html#added-3"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated Gemini adapter to use new modular API implementation\n- Enhanced authentication to support both API keys and OAuth2 tokens\n- Improved error handling with Gemini-specific error messages\n- Updated documentation with complete Gemini API coverage","ref":"changelog.html#changed-2"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed unused variable warnings in Gemini auth module\n- Fixed Live API compilation errors with proper string escaping\n- Fixed content parsing to handle JSON response formats correctly\n\n## [0.4.2] - 2025-06-08","ref":"changelog.html#fixed-2"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** Renamed `:local` provider atom to `:bumblebee` for clarity\n  - All references to `:local` in code and documentation have been updated\n  - Update any code using `ExLLM.chat(:local, ...)` to `ExLLM.chat(:bumblebee, ...)`\n- Changed default Bumblebee model from `microsoft/phi-2` to `Qwen/Qwen3-0.6B`\n- Excluded `emlx` dependency from Hex package until it's published\n- Updated README with instructions for adding `emlx` manually for Apple Silicon support\n- Updated documentation to clarify that `instructor`, `bumblebee`, and `nx` are required dependencies\n- Clarified that `exla` and `emlx` are optional hardware acceleration backends","ref":"changelog.html#changed-3"},{"type":"extras","title":"Fixed - Changelog","doc":"- Mock adapter now properly checks for `mock_error` option in chat function\n\n## [0.4.1] - 2025-06-08","ref":"changelog.html#fixed-3"},{"type":"extras","title":"Added - Changelog","doc":"- **Response Caching System** - Cache real provider responses for offline testing and development\n  - **Automatic Response Collection**: All provider responses automatically cached when enabled\n  - **Mock Integration**: Configure Mock adapter to replay cached responses from any provider\n  - **Cache Management**: Full CRUD operations for cached responses with provider organization\n  - **Fuzzy Matching**: Robust request matching handles real-world usage variations\n  - **Environment Configuration**: Simple enable/disable via `EX_LLM_CACHE_RESPONSES` environment variable\n  - **Cost Reduction**: Reduce API costs during development by replaying cached responses\n  - **Realistic Testing**: Use authentic provider responses in tests without API calls\n  - **Streaming Support**: Cache and replay streaming responses with exact chunk reproduction\n  - **Cross-Provider Testing**: Test application compatibility across different provider response formats","ref":"changelog.html#added-4"},{"type":"extras","title":"Changed - Changelog","doc":"- Enhanced shared response builder to support more response formats (completion, image, audio, moderation)\n- Extended HTTP client with provider-specific headers for 15+ providers\n- Improved error handling with normalization and retry logic for multiple providers","ref":"changelog.html#changed-4"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed pre-push hook to exclude integration tests preventing timeouts\n- Fixed unsafe String.to_atom usage throughout codebase (Sobelow warnings)\n- Fixed length() > 0 warnings by using pattern matching\n- Fixed typing warnings for potentially nil values\n- Fixed ModelConfig runtime path resolution for test environment\n- Fixed ResponseCache JSON key atomization for proper cache loading\n- Fixed capability normalization to handle already-normalized capability names\n- Added missing model capabilities (vision for Claude-3-Opus, reasoning for XAI models)\n\n## [0.4.0] - 2025-06-06","ref":"changelog.html#fixed-4"},{"type":"extras","title":"Added - Changelog","doc":"- **Complete OpenAI API Implementation** - Full support for modern OpenAI API features\n  - **Audio Features**: Support for audio input in messages and audio output configuration\n  - **Web Search Integration**: Support for web search options in chat completions\n  - **O-Series Model Features**: Reasoning effort parameter and developer role support\n  - **Predicted Outputs**: Support for faster regeneration with prediction hints\n  - **Additional APIs**: Six new OpenAI API endpoints\n    - `moderate_content/2` - Content moderation using OpenAI's moderation API\n    - `generate_image/2` - DALL-E image generation with configurable parameters\n    - `transcribe_audio/2` - Whisper audio transcription (basic implementation)\n    - `upload_file/3` - File upload for assistants and other endpoints (basic implementation)\n    - `create_assistant/2` - Create assistants with custom instructions and tools\n    - `create_batch/2` - Batch processing for multiple requests\n  - **Enhanced Message Support**: Multiple content parts per message (text + audio/image)\n  - **Modern Request Parameters**: Support for all modern OpenAI API parameters\n    - `max_completion_tokens`, `top_p`, `frequency_penalty`, `presence_penalty`\n    - `seed`, `stop`, `service_tier`, `logprobs`, `top_logprobs`\n  - **JSON Response Formats**: JSON mode and JSON Schema structured outputs\n  - **Modern Tools API**: Full support for tools API replacing deprecated functions\n  - **Enhanced Streaming**: Tool calls and usage information in streaming responses\n  - **Enhanced Usage Tracking**: Detailed token usage with cached/reasoning/audio tokens","ref":"changelog.html#added-5"},{"type":"extras","title":"Changed - Changelog","doc":"- **MessageFormatter**: Added support for \"developer\" role for O1+ models\n- **OpenAI Adapter**: Comprehensive test coverage with 46 tests following TDD methodology\n- **Response Types**: Enhanced LLMResponse struct with new fields (refusal, logprobs, tool_calls)","ref":"changelog.html#changed-5"},{"type":"extras","title":"Technical - Changelog","doc":"- Implemented using Test-Driven Development (TDD) methodology\n- Maintains full backward compatibility with existing API\n- All features validated with comprehensive test suite\n- Proper error handling and API key validation for all new endpoints\n\n- **Ollama Configuration Management** - Generate and update local model configurations\n  - New `generate_config/1` function to create YAML config for all installed models\n  - New `update_model_config/2` function to update specific model configurations\n  - Automatic capability detection using `/api/show` endpoint\n  - Real context window sizes from model metadata\n  - Preserves existing configuration when merging\n  - Example: `ExLLM.Adapters.Ollama.generate_config(save: true)`\n\n## [0.3.2] - 2025-06-06","ref":"changelog.html#technical"},{"type":"extras","title":"Added - Changelog","doc":"- **Capability Normalization** - Automatic normalization of provider-specific capability names\n  - New `ExLLM.Capabilities` module providing unified capability interface\n  - Normalizes different provider terminologies (e.g., `tool_use` → `function_calling`)\n  - Works transparently with all capability query functions\n  - Comprehensive mappings for common capability variations\n  - Example: `find_providers_with_features([:tool_use])` works across all providers\n- Enhanced provider capability tracking with real-time API discovery\n  - New `fetch_provider_capabilities.py` script for API-based capability detection\n  - Updated `fetch_provider_models.py` with better context window detection\n  - Fixed incorrect context windows (e.g., GPT-4o now correctly shows 128,000)\n  - Automatic capability detection from model IDs\n- New capability normalization demo in example app (option 6 in Provider Capabilities Explorer)\n- **Comprehensive Documentation**\n  - New Quick Start Guide (`docs/QUICKSTART.md`) - Get up and running in 5 minutes\n  - New User Guide (`docs/USER_GUIDE.md`) - Complete documentation of all features\n  - Reorganized documentation into `docs/` directory\n  - Added prominent documentation links to README","ref":"changelog.html#added-6"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated `provider_supports?/2`, `model_supports?/3`, `find_providers_with_features/1`, and `find_models_with_features/1` to use normalized capabilities","ref":"changelog.html#changed-6"},{"type":"extras","title":"Fixed - Changelog","doc":"- Mock provider now properly supports Instructor integration for structured outputs\n- Cost formatting now consistently uses dollars with appropriate decimal places (e.g., \"$0.000324\" instead of \"$0.032¢\")\n- Anthropic provider now includes required `max_tokens` parameter when using Instructor\n- Mock provider now generates semantically meaningful embeddings for realistic similarity search\n- Fixed KeyError when using providers without pricing data (e.g., Ollama)\n- Cost tracking now properly adds cost information to chat responses\n- Ollama now properly supports function calling for compatible models\n- Made request timeouts configurable via `:timeout` option (defaults: Ollama 2min, others use client defaults)\n- Fixed MatchError in example app when displaying providers without capabilities info\n- Provider and model capability queries now accept any provider's terminology\n- Moved `LOGGER.md`, `PROVIDER_CAPABILITIES.md`, and `DROPPED.md` to `docs/` directory\n- Enhanced provider capabilities with data from API discovery scripts\n\n## [0.3.1] - 2025-06-05","ref":"changelog.html#fixed-5"},{"type":"extras","title":"Added - Changelog","doc":"- **Major Code Refactoring** - Reduced code duplication by ~40% through shared modules:\n  - `StreamingCoordinator` - Unified streaming implementation for all adapters\n    - Standardized SSE parsing and buffering\n    - Provider-agnostic chunk handling\n    - Integrated error recovery support\n    - Simplified adapter streaming implementations\n  - `RequestBuilder` - Common request construction patterns\n    - Unified parameter handling across providers\n    - Provider-specific transformations via callbacks\n    - Support for chat, embeddings, and completion endpoints\n  - `ModelFetcher` - Standardized model discovery behavior\n    - Common API fetching patterns\n    - Unified filter/parse/transform pipeline\n    - Integration with ModelLoader for caching\n  - `VisionFormatter` - Centralized vision/multimodal content handling\n    - Provider-specific image formatting (Anthropic, OpenAI, Gemini)\n    - Media type detection from file extensions and magic bytes\n    - Base64 encoding/decoding utilities\n    - Image size validation\n- Unified `ExLLM.Logger` module replacing multiple logging approaches\n  - Single consistent API for all logging needs\n  - Simple Logger-like interface: `Logger.info(\"message\")`\n  - Automatic context tracking with `with_context/2`\n  - Structured logging for LLM-specific events (requests, retries, streaming)\n  - Configurable log levels and component filtering\n  - Security features: API key and content redaction\n  - Performance tracking with automatic duration measurement","ref":"changelog.html#added-7"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** Replaced all `Logger` and `DebugLogger` usage with unified `ExLLM.Logger`\n  - All modules now use `alias ExLLM.Logger` instead of `require Logger`\n  - Consistent logging interface across the entire codebase\n  - Simplified developer experience with one logging API\n- Enhanced `HTTPClient` with unified streaming support via `post_stream/3`\n- Improved error handling consistency across all shared modules\n- Better separation of concerns in adapter implementations","ref":"changelog.html#changed-7"},{"type":"extras","title":"Technical Improvements - Changelog","doc":"- Reduced code duplication significantly across adapters\n- More maintainable and testable codebase structure\n- Easier to add new providers using shared behaviors\n- Consistent patterns for common operations\n\n## [0.3.0] - 2025-06-05","ref":"changelog.html#technical-improvements"},{"type":"extras","title":"Added - Changelog","doc":"- X.AI adapter implementation with complete feature support\n  - Full OpenAI-compatible API integration\n  - Support for all Grok models (Beta, 2, 3, Vision variants)\n  - Streaming, function calling, vision, and structured outputs\n  - Web search and reasoning capabilities\n  - Complete Instructor integration for structured outputs\n- Synced model metadata from LiteLLM (1053 models across 56 providers)\n  - New OpenAI models: GPT-4.1 series (gpt-4.1, gpt-4.1-mini, gpt-4.1-nano)\n  - New OpenAI O1 reasoning models (o1-pro, o1, o1-mini, o1-preview)\n  - New XAI Grok-3 models (grok-3, grok-3-beta, grok-3-fast, grok-3-mini variants)\n  - New model capabilities: structured_output, prompt_caching, reasoning, web_search\n  - Updated pricing and context windows for all models\n- Fetched latest models from provider APIs (606 models from 6 providers)\n  - New Anthropic models: Claude 4 Opus/Sonnet/Haiku with 32K output tokens\n  - New Groq models: DeepSeek R1 distilled models, QwQ-32B, Mistral Saba\n  - New Gemini models: Gemini 2.5 Pro/Flash, Gemini 2.0 Flash with multimodal support\n  - New OpenAI models: O3/O4 series, GPT-4.5 preview, search-enabled models\n  - Updated context windows and capabilities from live APIs\n- Groq support for structured outputs via Instructor integration","ref":"changelog.html#added-8"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated default models:\n  - OpenAI: Set to gpt-4.1-nano\n  - Anthropic: Set to claude-3-5-sonnet-latest\n- Enhanced Instructor module to support Groq provider\n- Updated example app to include Groq in structured output demos\n- Updated README.md with current model information:\n  - Anthropic: Added Claude 4 series and Claude 3.7\n  - OpenAI: Added GPT-4.1 series and O1 reasoning models\n  - Gemini: Added Gemini 2.5 and 2.0 series\n  - Groq: Added Llama 4 Scout, DeepSeek R1 Distill, and QwQ-32B\n- Task reorganization:\n  - Created docs/DROPPED.md for features that don't align with core library mission\n  - Reorganized TASKS.md with clearer priorities and focused roadmap\n  - Added refactoring tasks to reduce code duplication by ~40%","ref":"changelog.html#changed-8"},{"type":"extras","title":"Fixed - Changelog","doc":"- Instructor integration now correctly separates params and config for chat_completion\n- Advanced features demo uses correct Mock adapter method (set_stream_chunks)\n- Module reference errors in Context management demo\n\n## [0.2.1] - 2025-06-05","ref":"changelog.html#fixed-6"},{"type":"extras","title":"Added - Changelog","doc":"- Provider Capability Discovery System\n  - New `ExLLM.ProviderCapabilities` module for tracking API-level provider capabilities\n  - Provider feature discovery independent of specific models\n  - Authentication method tracking (API key, OAuth, AWS signature, etc.)\n  - Provider endpoint discovery (chat, embeddings, images, audio, etc.)\n  - Provider recommendations based on required/preferred features\n  - Provider comparison tools for feature analysis\n  - Integrated provider capability functions into main ExLLM module\n  - Added provider capability explorer to example app demo\n- Environment variable wrapper script (`scripts/run_with_env.sh`) for Claude CLI usage\n- Groq models API support (https://api.groq.com/openai/v1/models)\n- Dynamic model loading from provider APIs\n  - All adapters now fetch models dynamically from provider APIs when available\n  - Automatic fallback to YAML configuration when API is unavailable\n  - Created `ExLLM.ModelLoader` module for centralized model loading with caching\n  - Anthropic adapter now uses `/v1/models` API endpoint\n  - OpenAI adapter fetches from `/v1/models` and filters chat models\n  - Gemini adapter uses Google's models API\n  - Ollama adapter fetches from local server's `/api/tags`\n  - OpenRouter adapter uses public `/api/v1/models` API\n- OpenRouter adapter with access to 300+ models from multiple providers\n  - Support for Claude, GPT-4, Llama, PaLM, and many other model families\n  - Unified API interface for different model architectures\n  - Automatic model discovery and cost-effective access to premium models\n- External YAML configuration system for model metadata\n  - Model pricing, context windows, and capabilities stored in `config/models/*.yml`\n  - Runtime configuration loading with ETS caching for performance\n  - Separation of model data from code for easier maintenance\n  - Support for easy updates without code changes\n- OpenAI-Compatible base adapter for shared implementation\n  - Reduces code duplication across providers with OpenAI-compatible APIs\n  - Groq adapter as first implementation using the base adapter\n- Model configuration sync script from LiteLLM\n  - Python script to sync model data from LiteLLM's database\n  - Added 1048 models with pricing, context windows, and capabilities\n  - Automatic conversion from LiteLLM's JSON to ExLLM's YAML format\n- Extracted ALL provider configurations from LiteLLM\n  - Created YAML files for 56 unique providers (49 new providers)\n  - Includes Azure, Mistral, Perplexity, Together AI, Databricks, and more\n  - Ready-to-use configurations for future adapter implementations","ref":"changelog.html#added-9"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** Model configuration moved from hardcoded maps to external YAML files\n  - All providers now use `ExLLM.ModelConfig` for pricing and context window data\n  - Default models, pricing, and context windows loaded from YAML configuration\n  - Added `yaml_elixir` dependency for YAML parsing\n- Updated Bedrock adapter with comprehensive model support:\n  - Added all latest Anthropic models (Claude 4, 3.7, 3.5 series)\n  - Added Amazon Nova models (Micro, Lite, Pro, Premier)\n  - Added AI21 Labs Jamba series (1.5-large, 1.5-mini, instruct)\n  - Added Cohere Command R series (R, R+)\n  - Added DeepSeek R1 model\n  - Added Meta Llama 4 and 3.x series models\n  - Added Mistral Pixtral Large 2025-02\n  - Added Writer Palmyra X4 and X5 models\n  - Changed default model from \"claude-3-sonnet\" to \"nova-lite\" for cost efficiency\n- Updated pricing data for all Bedrock providers with per-1M token rates\n- Updated context window sizes for all new Bedrock models\n- Enhanced streaming support for all new providers (Writer, DeepSeek)\n- All adapters now use ModelConfig for consistent default model retrieval","ref":"changelog.html#changed-9"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** Refactored `ExLLM.Adapters.OpenAICompatible` base adapter\n  - Extracted common helper functions (`format_model_name/1`, `default_model_transformer/2`) as public module functions\n  - Simplified adapter implementations by removing duplicate code\n  - Added ModelLoader integration to base adapter for consistent dynamic model loading\n  - Added `filter_model/1` and `parse_model/1` callbacks for customizing model parsing","ref":"changelog.html#changed-10"},{"type":"extras","title":"Fixed - Changelog","doc":"- Anthropic models API fetch now correctly parses response structure (uses `data` field instead of `models`)\n- Python model fetch script updated to handle Anthropic's API response format\n- OpenRouter pricing parser now handles string values correctly\n- Groq adapter compilation warnings for undefined callbacks\n- DateTime serialization in MessageFormatter for session persistence\n- OpenAI adapter streaming termination handling\n- JSON double-encoding issue in HTTPClient\n- Token field name standardization across adapters (input_tokens/output_tokens)\n- Instructor integration API parameter passing\n- Context management module reference errors in example app\n- Function calling demo error handling with string keys\n- Streaming chat demo now shows token usage and cost estimates","ref":"changelog.html#fixed-7"},{"type":"extras","title":"Changed - Changelog","doc":"- Made Instructor a required dependency instead of optional\n- OpenAI default model changed to gpt-4.1-nano\n- Instructor now uses dynamic default models from YAML configs\n- Example app no longer hardcodes model names","ref":"changelog.html#changed-11"},{"type":"extras","title":"Improved - Changelog","doc":"- Code organization with shared modules to eliminate duplication:\n  - Created `ExLLM.Adapters.Shared.Validation` for API key validation\n  - All adapters now use `ModelUtils.format_model_name` for consistent formatting\n  - All adapters now use `ConfigHelper.ensure_default_model` for default models\n  - Test files updated to use `TestHelpers` consistently\n- Example app enhancements:\n  - Session management shows full conversation history\n  - Function calling demo clearly shows available tools\n  - Advanced features demo now has real implementations\n  - Cost formatting uses decimal notation instead of scientific","ref":"changelog.html#improved"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed hardcoded model names from adapters\n- Removed `model_capabilities.ex.bak` backup file\n- Removed `DUPLICATE_CODE_ANALYSIS.md` after completing all refactoring\n\n## [0.2.0] - 2025-05-25","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- OpenAI adapter with GPT-4 and GPT-3.5 support\n- Ollama adapter for local model inference\n- AWS Bedrock adapter with full multi-provider support (Anthropic, Amazon Titan, Meta Llama, Cohere, AI21, Mistral)\n  - Complete AWS credential chain support (environment vars, profiles, instance metadata, ECS task roles)\n  - Provider-specific request/response formatting\n  - Native streaming support\n  - Dynamic model listing via AWS Bedrock API\n- Google Gemini adapter with Pro, Ultra, and Nano models\n- Context management functionality to automatically handle LLM context windows\n- `ExLLM.Context` module with the following features:\n  - Automatic message truncation to fit within model context windows\n  - Multiple truncation strategies (sliding_window, smart)\n  - Context window validation\n  - Token estimation and statistics\n  - Model-specific context window sizes\n- Session management functionality for conversation state tracking\n- `ExLLM.Session` module with the following features:\n  - Conversation state management\n  - Message history tracking\n  - Token usage tracking\n  - Session persistence (save/load)\n  - Export to markdown/JSON formats\n- Local model support via Bumblebee integration\n- `ExLLM.Adapters.Local` with the following features:\n  - Support for Phi-2, Llama 2, Mistral, GPT-Neo, and Flan-T5\n  - Hardware acceleration (Metal, CUDA, ROCm, CPU)\n  - Model lifecycle management with ModelLoader GenServer\n  - Zero-cost inference (no API fees)\n  - Privacy-preserving local execution\n- New public API functions in main ExLLM module:\n  - Context management: `prepare_messages/2`, `validate_context/2`, `context_window_size/2`, `context_stats/1`\n  - Session management: `new_session/2`, `chat_with_session/2`, `save_session/2`, `load_session/1`, etc.\n- Automatic context management in `chat/3` and `stream_chat/3`\n- Optional dependencies (Bumblebee, Nx, EXLA) for local model support\n- Application supervisor for managing ModelLoader lifecycle\n- Comprehensive test coverage for all new features","ref":"changelog.html#added-10"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated `chat/3` and `stream_chat/3` to automatically apply context truncation\n- Enhanced documentation with context management and session examples\n- ExLLM is now a comprehensive all-in-one solution including cost tracking, context management, and session handling\n\n## [0.1.0] - 2025-05-24","ref":"changelog.html#changed-12"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release with unified LLM interface\n- Support for Anthropic Claude models\n- Streaming support via Server-Sent Events\n- Integrated cost tracking and calculation\n- Token estimation functionality\n- Configurable provider system\n- Comprehensive error handling","ref":"changelog.html#added-11"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 ExLLM\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"ExLLM Quick Start Guide","doc":"# ExLLM Quick Start Guide\n\nGet up and running with ExLLM in 5 minutes! This guide covers installation, basic configuration, and common usage patterns.","ref":"quickstart.html"},{"type":"extras","title":"Installation - ExLLM Quick Start Guide","doc":"Add ExLLM to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.7.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"quickstart.html#installation"},{"type":"extras","title":"Configuration - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#configuration"},{"type":"extras","title":"Environment Variables - ExLLM Quick Start Guide","doc":"Set API keys for the providers you want to use:\n\n```bash\n# Core providers\nexport ANTHROPIC_API_KEY=\"sk-ant-your-key\"\nexport OPENAI_API_KEY=\"sk-your-key\"\nexport GROQ_API_KEY=\"gsk_your-key\"\n\n# Additional providers (optional)\nexport GEMINI_API_KEY=\"your-gemini-key\"\nexport MISTRAL_API_KEY=\"your-mistral-key\"\nexport OPENROUTER_API_KEY=\"sk-or-your-key\"\nexport PERPLEXITY_API_KEY=\"pplx-your-key\"\n```","ref":"quickstart.html#environment-variables"},{"type":"extras","title":"Application Configuration (Optional) - ExLLM Quick Start Guide","doc":"```elixir\n# config/config.exs\nconfig :ex_llm,\n  # Default provider\n  default_provider: :anthropic,\n  \n  # Cost tracking\n  cost_tracking_enabled: true,\n  \n  # Test caching (speeds up tests 25x)\n  test_cache: [\n    enabled: true,\n    cache_dir: \"test/cache\",\n    ttl: 604_800_000  # 7 days in milliseconds\n  ]\n```","ref":"quickstart.html#application-configuration-optional"},{"type":"extras","title":"Basic Usage - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#basic-usage"},{"type":"extras","title":"Simple Chat Completion - ExLLM Quick Start Guide","doc":"```elixir\n# Start with a basic chat\n{:ok, response} = ExLLM.chat(:anthropic, [\n  %{role: \"user\", content: \"What is the capital of France?\"}\n])\n\nIO.puts(response.content)\n# => \"The capital of France is Paris.\"\n\n# Access additional metadata\nIO.puts(\"Model: #{response.model}\")\nIO.puts(\"Cost: $#{response.cost}\")\nIO.puts(\"Tokens used: #{response.usage.total_tokens}\")\n```","ref":"quickstart.html#simple-chat-completion"},{"type":"extras","title":"Different Providers - ExLLM Quick Start Guide","doc":"```elixir\n# Try different providers\n{:ok, response1} = ExLLM.chat(:openai, [\n  %{role: \"user\", content: \"Explain quantum computing briefly\"}\n])\n\n{:ok, response2} = ExLLM.chat(:groq, [\n  %{role: \"user\", content: \"Write a haiku about coding\"}\n])\n\n{:ok, response3} = ExLLM.chat(:gemini, [\n  %{role: \"user\", content: \"What's 2+2?\"}\n])\n```","ref":"quickstart.html#different-providers"},{"type":"extras","title":"Streaming Responses - ExLLM Quick Start Guide","doc":"```elixir\n# Stream responses in real-time\nExLLM.chat_stream(:openai, [\n  %{role: \"user\", content: \"Write a short story about a robot\"}\n], fn chunk ->\n  IO.write(chunk.delta)\nend)\n```","ref":"quickstart.html#streaming-responses"},{"type":"extras","title":"Session Management - ExLLM Quick Start Guide","doc":"```elixir\n# Maintain conversation context\n{:ok, session} = ExLLM.Session.new(:anthropic)\n\n# First message\n{:ok, session, response1} = ExLLM.Session.chat(session, \"Hi, I'm learning Elixir\")\nIO.puts(response1.content)\n\n# Continue the conversation\n{:ok, session, response2} = ExLLM.Session.chat(session, \"What are GenServers?\")\nIO.puts(response2.content)\n\n# Session automatically tracks conversation history\nIO.puts(\"Messages in session: #{length(session.messages)}\")\nIO.puts(\"Total cost: $#{session.total_cost}\")\n```","ref":"quickstart.html#session-management"},{"type":"extras","title":"Advanced Features - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#advanced-features"},{"type":"extras","title":"Multimodal (Vision) - ExLLM Quick Start Guide","doc":"```elixir\n# Analyze images (with Gemini or OpenAI)\nimage_data = File.read!(\"image.jpg\") |> Base.encode64()\n\n{:ok, response} = ExLLM.chat(:gemini, [\n  %{role: \"user\", content: [\n    %{type: \"text\", text: \"What's in this image?\"},\n    %{type: \"image\", image: %{\n      data: image_data,\n      media_type: \"image/jpeg\"\n    }}\n  ]}\n])\n\nIO.puts(response.content)\n```","ref":"quickstart.html#multimodal-vision"},{"type":"extras","title":"Function Calling - ExLLM Quick Start Guide","doc":"```elixir\n# Define tools\ntools = [\n  %{\n    type: \"function\",\n    function: %{\n      name: \"get_weather\",\n      description: \"Get current weather for a location\",\n      parameters: %{\n        type: \"object\",\n        properties: %{\n          location: %{type: \"string\", description: \"City name\"},\n          unit: %{type: \"string\", enum: [\"celsius\", \"fahrenheit\"]}\n        },\n        required: [\"location\"]\n      }\n    }\n  }\n]\n\n{:ok, response} = ExLLM.chat(:openai, [\n  %{role: \"user\", content: \"What's the weather in Paris?\"}\n], tools: tools)\n\n# Handle function calls\ncase response.function_calls do\n  [%{name: \"get_weather\", arguments: args}] ->\n    # Call your weather API here\n    weather_data = get_weather(args[\"location\"])\n    \n    # Continue conversation with function result\n    {:ok, final_response} = ExLLM.chat(:openai, [\n      %{role: \"user\", content: \"What's the weather in Paris?\"},\n      response.message,\n      %{role: \"function\", name: \"get_weather\", content: Jason.encode!(weather_data)}\n    ])\n    \n  _ ->\n    IO.puts(response.content)\nend\n```","ref":"quickstart.html#function-calling"},{"type":"extras","title":"Model Discovery - ExLLM Quick Start Guide","doc":"```elixir\n# List available models\n{:ok, models} = ExLLM.list_models(:anthropic)\n\nEnum.each(models, fn model ->\n  IO.puts(\"#{model.id} - Context: #{model.context_window} tokens\")\nend)\n\n# Get specific model info\n{:ok, model} = ExLLM.get_model(:openai, \"gpt-4o\")\nIO.puts(\"Supports streaming: #{model.capabilities.supports_streaming}\")\nIO.puts(\"Supports vision: #{model.capabilities.supports_vision}\")\n```","ref":"quickstart.html#model-discovery"},{"type":"extras","title":"Testing - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#testing"},{"type":"extras","title":"Fast Testing with Caching - ExLLM Quick Start Guide","doc":"ExLLM includes intelligent test caching for 25x faster integration tests:\n\n```bash\n# Run tests with automatic caching\nmix test\n\n# Test specific providers\nmix test.anthropic\nmix test.openai --include live_api\n\n# Manage cache\nmix ex_llm.cache stats\nmix ex_llm.cache clean --older-than 7d\n```","ref":"quickstart.html#fast-testing-with-caching"},{"type":"extras","title":"Writing Tests - ExLLM Quick Start Guide","doc":"```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case\n  \n  # Tag for automatic caching\n  @moduletag :live_api\n  @moduletag :requires_api_key\n  @moduletag provider: :anthropic\n  \n  test \"chat completion works\" do\n    {:ok, response} = ExLLM.chat(:anthropic, [\n      %{role: \"user\", content: \"Say hello\"}\n    ])\n    \n    assert response.content =~ \"hello\"\n    assert response.cost > 0\n  end\nend\n```","ref":"quickstart.html#writing-tests"},{"type":"extras","title":"Local Models - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#local-models"},{"type":"extras","title":"Ollama - ExLLM Quick Start Guide","doc":"```bash\n# Start Ollama\nollama serve\n\n# Pull a model\nollama pull llama3.2\n```\n\n```elixir\n# Use local Ollama models\n{:ok, response} = ExLLM.chat(:ollama, [\n  %{role: \"user\", content: \"Hello!\"}\n], model: \"llama3.2\")\n```","ref":"quickstart.html#ollama"},{"type":"extras","title":"LM Studio - ExLLM Quick Start Guide","doc":"```bash\n# Start LM Studio server on localhost:1234\n```\n\n```elixir\n# Use LM Studio models\n{:ok, response} = ExLLM.chat(:lmstudio, [\n  %{role: \"user\", content: \"Hello!\"}\n])\n```","ref":"quickstart.html#lm-studio"},{"type":"extras","title":"Bumblebee (Elixir Native) - ExLLM Quick Start Guide","doc":"```elixir\n# Add to deps for local inference\n{:exla, \"~> 0.7\"}  # For CPU/GPU acceleration\n\n# Use Bumblebee models\n{:ok, response} = ExLLM.chat(:bumblebee, [\n  %{role: \"user\", content: \"Hello!\"}\n], model: \"microsoft/DialoGPT-medium\")\n```","ref":"quickstart.html#bumblebee-elixir-native"},{"type":"extras","title":"Error Handling - ExLLM Quick Start Guide","doc":"```elixir\ncase ExLLM.chat(:anthropic, messages) do\n  {:ok, response} ->\n    IO.puts(response.content)\n    \n  {:error, {:api_error, %{status: 401}}} ->\n    IO.puts(\"Invalid API key\")\n    \n  {:error, {:api_error, %{status: 429}}} ->\n    IO.puts(\"Rate limited - try again later\")\n    \n  {:error, {:api_error, %{status: 400, body: body}}} ->\n    IO.puts(\"Bad request: #{inspect(body)}\")\n    \n  {:error, {:network_error, reason}} ->\n    IO.puts(\"Network error: #{reason}\")\n    \n  {:error, reason} ->\n    IO.puts(\"Other error: #{inspect(reason)}\")\nend\n```","ref":"quickstart.html#error-handling"},{"type":"extras","title":"Environment-Specific Configuration - ExLLM Quick Start Guide","doc":"","ref":"quickstart.html#environment-specific-configuration"},{"type":"extras","title":"Development - ExLLM Quick Start Guide","doc":"```elixir\n# config/dev.exs\nconfig :ex_llm,\n  log_level: :debug,\n  log_components: [:http_client, :streaming],\n  test_cache: [enabled: true]\n```","ref":"quickstart.html#development"},{"type":"extras","title":"Test - ExLLM Quick Start Guide","doc":"```elixir\n# config/test.exs\nconfig :ex_llm,\n  log_level: :warn,\n  test_cache: [\n    enabled: true,\n    cache_dir: \"test/cache\",\n    ttl: 604_800_000  # 7 days\n  ]\n```","ref":"quickstart.html#test"},{"type":"extras","title":"Production - ExLLM Quick Start Guide","doc":"```elixir\n# config/prod.exs\nconfig :ex_llm,\n  log_level: :info,\n  log_redaction: true,\n  cost_tracking_enabled: true\n```","ref":"quickstart.html#production"},{"type":"extras","title":"Next Steps - ExLLM Quick Start Guide","doc":"1. **Read the [User Guide](USER_GUIDE.md)** for comprehensive documentation\n2. **Check [Provider Capabilities](PROVIDER_CAPABILITIES.md)** to compare features\n3. **Review [Testing Guide](TESTING.md)** for advanced testing patterns and caching","ref":"quickstart.html#next-steps"},{"type":"extras","title":"Common Issues - ExLLM Quick Start Guide","doc":"### \"API key not found\"\n```bash\n# Make sure you've set the environment variable\nexport ANTHROPIC_API_KEY=\"your-key\"\n```\n\n### \"Model not found\"\n```elixir\n# Check available models\n{:ok, models} = ExLLM.list_models(:anthropic)\n```\n\n### \"Rate limited\"\n```elixir\n# Use different providers or implement backoff\nProcess.sleep(1000)\n{:ok, response} = ExLLM.chat(:groq, messages)  # Try Groq for speed\n```","ref":"quickstart.html#common-issues"},{"type":"extras","title":"Tests running slowly - ExLLM Quick Start Guide","doc":"```bash\n# Enable test caching\nexport EX_LLM_TEST_CACHE_ENABLED=true\nmix test --include live_api\n```\n\nThat's it! You're now ready to build amazing applications with ExLLM. 🚀","ref":"quickstart.html#tests-running-slowly"},{"type":"extras","title":"ExLLM User Guide","doc":"# ExLLM User Guide\n\nThis comprehensive guide covers all features and capabilities of the ExLLM library.","ref":"user_guide.html"},{"type":"extras","title":"Table of Contents - ExLLM User Guide","doc":"1. [Installation and Setup](#installation-and-setup)\n2. [Configuration](#configuration)\n3. [Basic Usage](#basic-usage)\n4. [Providers](#providers)\n5. [Chat Completions](#chat-completions)\n6. [Streaming](#streaming)\n7. [Session Management](#session-management)\n8. [Context Management](#context-management)\n9. [Function Calling](#function-calling)\n10. [Vision and Multimodal](#vision-and-multimodal)\n11. [Embeddings](#embeddings)\n12. [Structured Outputs](#structured-outputs)\n13. [Cost Tracking](#cost-tracking)\n14. [Error Handling and Retries](#error-handling-and-retries)\n15. [Caching](#caching)\n16. [Response Caching](#response-caching)\n17. [Model Discovery](#model-discovery)\n18. [Provider Capabilities](#provider-capabilities)\n19. [Logging](#logging)\n20. [Testing with Mock Adapter](#testing-with-mock-adapter)\n21. [Advanced Topics](#advanced-topics)","ref":"user_guide.html#table-of-contents"},{"type":"extras","title":"Installation and Setup - ExLLM User Guide","doc":"","ref":"user_guide.html#installation-and-setup"},{"type":"extras","title":"Adding to Your Project - ExLLM User Guide","doc":"Add ExLLM to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.4.1\"},\n    # Included dependencies (automatically installed with ex_llm):\n    # - {:instructor, \"~> 0.1.0\"} - For structured outputs\n    # - {:bumblebee, \"~> 0.5\"} - For local model inference\n    # - {:nx, \"~> 0.7\"} - For numerical computing\n    \n    # Optional hardware acceleration (choose one):\n    # {:exla, \"~> 0.7\"}  # For CUDA/ROCm GPUs\n    # {:emlx, github: \"elixir-nx/emlx\", branch: \"main\"}  # For Apple Silicon\n  ]\nend\n```\n\nRun `mix deps.get` to install the dependencies.","ref":"user_guide.html#adding-to-your-project"},{"type":"extras","title":"Optional Dependencies - ExLLM User Guide","doc":"- **Req**: HTTP client (automatically included)\n- **Jason**: JSON parser (automatically included)\n- **Instructor**: Structured outputs with schema validation (automatically included)\n- **Bumblebee**: Local model inference (automatically included)\n- **Nx**: Numerical computing (automatically included)\n- **EXLA**: CUDA/ROCm GPU acceleration (optional)\n- **EMLX**: Apple Silicon Metal acceleration (optional)","ref":"user_guide.html#optional-dependencies"},{"type":"extras","title":"Configuration - ExLLM User Guide","doc":"ExLLM supports multiple configuration methods to suit different use cases.","ref":"user_guide.html#configuration"},{"type":"extras","title":"Environment Variables - ExLLM User Guide","doc":"The simplest way to configure ExLLM:\n\n```bash\n# OpenAI\nexport OPENAI_API_KEY=\"sk-...\"\nexport OPENAI_API_BASE=\"https://api.openai.com/v1\"  # Optional custom endpoint\n\n# Anthropic\nexport ANTHROPIC_API_KEY=\"sk-ant-...\"\n\n# Google Gemini\nexport GOOGLE_API_KEY=\"...\"\n# or\nexport GEMINI_API_KEY=\"...\"\n\n# Groq\nexport GROQ_API_KEY=\"gsk_...\"\n\n# OpenRouter\nexport OPENROUTER_API_KEY=\"sk-or-...\"\n\n# X.AI\nexport XAI_API_KEY=\"xai-...\"\n\n# Mistral AI\nexport MISTRAL_API_KEY=\"...\"\n\n# Perplexity\nexport PERPLEXITY_API_KEY=\"pplx-...\"\n\n# AWS Bedrock\nexport AWS_ACCESS_KEY_ID=\"...\"\nexport AWS_SECRET_ACCESS_KEY=\"...\"\nexport AWS_REGION=\"us-east-1\"\n\n# Ollama\nexport OLLAMA_API_BASE=\"http://localhost:11434\"\n\n# LM Studio\nexport LMSTUDIO_API_BASE=\"http://localhost:1234\"\n```","ref":"user_guide.html#environment-variables"},{"type":"extras","title":"Static Configuration - ExLLM User Guide","doc":"For more control, use static configuration:\n\n```elixir\nconfig = %{\n  openai: %{\n    api_key: \"sk-...\",\n    api_base: \"https://api.openai.com/v1\",\n    default_model: \"gpt-4o\"\n  },\n  anthropic: %{\n    api_key: \"sk-ant-...\",\n    default_model: \"claude-3-5-sonnet-20241022\"\n  }\n}\n\n{:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n\n# Use with config_provider option\n{:ok, response} = ExLLM.chat(:openai, messages, config_provider: provider)\n```","ref":"user_guide.html#static-configuration"},{"type":"extras","title":"Custom Configuration Provider - ExLLM User Guide","doc":"Implement your own configuration provider:\n\n```elixir\ndefmodule MyApp.ConfigProvider do\n  @behaviour ExLLM.ConfigProvider\n  \n  def get([:openai, :api_key]), do: fetch_from_vault(\"openai_key\")\n  def get([:anthropic, :api_key]), do: fetch_from_vault(\"anthropic_key\")\n  def get(_path), do: nil\n  \n  def get_all() do\n    %{\n      openai: %{api_key: fetch_from_vault(\"openai_key\")},\n      anthropic: %{api_key: fetch_from_vault(\"anthropic_key\")}\n    }\n  end\nend\n\n# Use it\n{:ok, response} = ExLLM.chat(:openai, messages, \n  config_provider: MyApp.ConfigProvider\n)\n```","ref":"user_guide.html#custom-configuration-provider"},{"type":"extras","title":"Basic Usage - ExLLM User Guide","doc":"","ref":"user_guide.html#basic-usage"},{"type":"extras","title":"Simple Chat - ExLLM User Guide","doc":"```elixir\nmessages = [\n  %{role: \"user\", content: \"Hello, how are you?\"}\n]\n\n{:ok, response} = ExLLM.chat(:openai, messages)\nIO.puts(response.content)\n```","ref":"user_guide.html#simple-chat"},{"type":"extras","title":"Provider/Model Syntax - ExLLM User Guide","doc":"```elixir\n# Use provider/model string syntax\n{:ok, response} = ExLLM.chat(\"anthropic/claude-3-haiku-20240307\", messages)\n\n# Equivalent to\n{:ok, response} = ExLLM.chat(:anthropic, messages, \n  model: \"claude-3-haiku-20240307\"\n)\n```","ref":"user_guide.html#provider-model-syntax"},{"type":"extras","title":"Response Structure - ExLLM User Guide","doc":"```elixir\n%ExLLM.Types.LLMResponse{\n  content: \"I'm doing well, thank you!\",\n  model: \"gpt-4o\",\n  finish_reason: \"stop\",\n  usage: %{\n    input_tokens: 12,\n    output_tokens: 8,\n    total_tokens: 20\n  },\n  cost: %{\n    input_cost: 0.00006,\n    output_cost: 0.00016,\n    total_cost: 0.00022,\n    currency: \"USD\"\n  }\n}\n```","ref":"user_guide.html#response-structure"},{"type":"extras","title":"Providers - ExLLM User Guide","doc":"","ref":"user_guide.html#providers"},{"type":"extras","title":"Supported Providers - ExLLM User Guide","doc":"ExLLM supports these providers out of the box:\n\n- **:openai** - OpenAI GPT models\n- **:anthropic** - Anthropic Claude models\n- **:gemini** - Google Gemini models\n- **:groq** - Groq fast inference\n- **:mistral** - Mistral AI models\n- **:perplexity** - Perplexity search-enhanced models\n- **:ollama** - Local models via Ollama\n- **:lmstudio** - Local models via LM Studio\n- **:bedrock** - AWS Bedrock\n- **:openrouter** - OpenRouter (300+ models)\n- **:xai** - X.AI Grok models\n- **:bumblebee** - Local models via Bumblebee/NX\n- **:mock** - Mock adapter for testing","ref":"user_guide.html#supported-providers"},{"type":"extras","title":"Checking Provider Configuration - ExLLM User Guide","doc":"```elixir\n# Check if a provider is configured\nif ExLLM.configured?(:openai) do\n  {:ok, response} = ExLLM.chat(:openai, messages)\nend\n\n# Get default model for a provider\nmodel = ExLLM.default_model(:anthropic)\n# => \"claude-3-5-sonnet-20241022\"\n\n# List available models\n{:ok, models} = ExLLM.list_models(:openai)\nfor model <- models do\n  IO.puts(\"#{model.id}: #{model.context_window} tokens\")\nend\n```","ref":"user_guide.html#checking-provider-configuration"},{"type":"extras","title":"Chat Completions - ExLLM User Guide","doc":"","ref":"user_guide.html#chat-completions"},{"type":"extras","title":"Basic Options - ExLLM User Guide","doc":"```elixir\n{:ok, response} = ExLLM.chat(:openai, messages,\n  model: \"gpt-4o\",           # Specific model\n  temperature: 0.7,          # 0.0-1.0, higher = more creative\n  max_tokens: 1000,          # Max response length\n  top_p: 0.9,                # Nucleus sampling\n  frequency_penalty: 0.5,    # Reduce repetition\n  presence_penalty: 0.5,     # Encourage new topics\n  stop: [\"\\n\\n\", \"END\"],     # Stop sequences\n  seed: 12345,               # Reproducible outputs\n  timeout: 60_000            # Request timeout in ms (default: provider-specific)\n)\n```","ref":"user_guide.html#basic-options"},{"type":"extras","title":"Timeout Configuration - ExLLM User Guide","doc":"Different providers have different timeout requirements. ExLLM allows you to configure timeouts per request:\n\n```elixir\n# Ollama with function calling (can be slow)\n{:ok, response} = ExLLM.chat(:ollama, messages,\n  functions: functions,\n  timeout: 300_000  # 5 minutes\n)\n\n# Quick requests with shorter timeout\n{:ok, response} = ExLLM.chat(:openai, messages,\n  timeout: 30_000   # 30 seconds\n)\n```\n\nDefault timeouts:\n- **Ollama**: 120,000ms (2 minutes) - Local models can be slower\n- **Other providers**: Use their HTTP client defaults (typically 30-60 seconds)","ref":"user_guide.html#timeout-configuration"},{"type":"extras","title":"System Messages - ExLLM User Guide","doc":"```elixir\nmessages = [\n  %{role: \"system\", content: \"You are a helpful coding assistant.\"},\n  %{role: \"user\", content: \"How do I read a file in Elixir?\"}\n]\n\n{:ok, response} = ExLLM.chat(:openai, messages)\n```","ref":"user_guide.html#system-messages"},{"type":"extras","title":"Multi-turn Conversations - ExLLM User Guide","doc":"```elixir\nconversation = [\n  %{role: \"user\", content: \"What's the capital of France?\"},\n  %{role: \"assistant\", content: \"The capital of France is Paris.\"},\n  %{role: \"user\", content: \"What's the population?\"}\n]\n\n{:ok, response} = ExLLM.chat(:openai, conversation)\n```","ref":"user_guide.html#multi-turn-conversations"},{"type":"extras","title":"Streaming - ExLLM User Guide","doc":"","ref":"user_guide.html#streaming"},{"type":"extras","title":"Basic Streaming - ExLLM User Guide","doc":"```elixir\n{:ok, stream} = ExLLM.stream_chat(:openai, messages)\n\nfor chunk <- stream do\n  case chunk do\n    %{content: content} when content != nil ->\n      IO.write(content)\n      \n    %{finish_reason: reason} when reason != nil ->\n      IO.puts(\"\\nFinished: #{reason}\")\n      \n    _ ->\n      # Other chunk types (role, etc.)\n      :ok\n  end\nend\n```","ref":"user_guide.html#basic-streaming"},{"type":"extras","title":"Streaming with Callback - ExLLM User Guide","doc":"```elixir\n{:ok, stream} = ExLLM.stream_chat(:openai, messages,\n  on_chunk: fn chunk ->\n    if chunk.content, do: IO.write(chunk.content)\n  end\n)\n\n# Consume the stream\nEnum.to_list(stream)\n```","ref":"user_guide.html#streaming-with-callback"},{"type":"extras","title":"Collecting Streamed Response - ExLLM User Guide","doc":"```elixir\n{:ok, stream} = ExLLM.stream_chat(:openai, messages)\n\n# Collect all chunks into a single response\nfull_content = \n  stream\n  |> Enum.map(& &1.content)\n  |> Enum.reject(&is_nil/1)\n  |> Enum.join(\"\")\n```","ref":"user_guide.html#collecting-streamed-response"},{"type":"extras","title":"Stream Recovery - ExLLM User Guide","doc":"Enable automatic stream recovery for interrupted streams:\n\n```elixir\n{:ok, stream} = ExLLM.stream_chat(:openai, messages,\n  stream_recovery: true,\n  recovery_strategy: :exact  # :exact, :paragraph, or :summarize\n)\n\n# If stream is interrupted, you can resume\n{:ok, resumed_stream} = ExLLM.resume_stream(recovery_id)\n```","ref":"user_guide.html#stream-recovery"},{"type":"extras","title":"Session Management - ExLLM User Guide","doc":"Sessions provide stateful conversation management with automatic token tracking.","ref":"user_guide.html#session-management"},{"type":"extras","title":"Creating and Using Sessions - ExLLM User Guide","doc":"```elixir\n# Create a new session\nsession = ExLLM.new_session(:openai, name: \"Customer Support\")\n\n# Chat with session (automatically manages message history)\n{:ok, {response, session}} = ExLLM.chat_with_session(\n  session,\n  \"What's the weather like?\"\n)\n\n# Continue the conversation\n{:ok, {response2, session}} = ExLLM.chat_with_session(\n  session,\n  \"What should I wear?\"\n)\n\n# Check token usage\ntotal_tokens = ExLLM.session_token_usage(session)\nIO.puts(\"Total tokens used: #{total_tokens}\")\n```","ref":"user_guide.html#creating-and-using-sessions"},{"type":"extras","title":"Managing Session Messages - ExLLM User Guide","doc":"```elixir\n# Add messages manually\nsession = ExLLM.add_session_message(session, \"user\", \"Hello!\")\nsession = ExLLM.add_session_message(session, \"assistant\", \"Hi there!\")\n\n# Get message history\nmessages = ExLLM.get_session_messages(session)\nrecent_10 = ExLLM.get_session_messages(session, 10)\n\n# Clear messages but keep session metadata\nsession = ExLLM.clear_session(session)\n```","ref":"user_guide.html#managing-session-messages"},{"type":"extras","title":"Persisting Sessions - ExLLM User Guide","doc":"```elixir\n# Save session to JSON\n{:ok, json} = ExLLM.save_session(session)\nFile.write!(\"session.json\", json)\n\n# Load session from JSON\n{:ok, json} = File.read(\"session.json\")\n{:ok, restored_session} = ExLLM.load_session(json)\n```","ref":"user_guide.html#persisting-sessions"},{"type":"extras","title":"Session with Context - ExLLM User Guide","doc":"```elixir\n# Create session with default context\nsession = ExLLM.new_session(:openai,\n  name: \"Tech Support\",\n  context: %{\n    temperature: 0.3,\n    system_message: \"You are a technical support agent.\"\n  }\n)\n\n# Context is automatically applied to all chats\n{:ok, {response, session}} = ExLLM.chat_with_session(session, \"Help!\")\n```","ref":"user_guide.html#session-with-context"},{"type":"extras","title":"Context Management - ExLLM User Guide","doc":"Automatically manage conversation context to fit within model limits.","ref":"user_guide.html#context-management"},{"type":"extras","title":"Context Window Validation - ExLLM User Guide","doc":"```elixir\n# Check if messages fit in context window\ncase ExLLM.validate_context(messages, provider: :openai, model: \"gpt-4\") do\n  {:ok, token_count} ->\n    IO.puts(\"Messages use #{token_count} tokens\")\n    \n  {:error, reason} ->\n    IO.puts(\"Messages too large: #{reason}\")\nend\n\n# Get context window size for a model\nwindow_size = ExLLM.context_window_size(:anthropic, \"claude-3-opus-20240229\")\n# => 200000\n```","ref":"user_guide.html#context-window-validation"},{"type":"extras","title":"Automatic Message Truncation - ExLLM User Guide","doc":"```elixir\n# Prepare messages to fit in context window\ntruncated = ExLLM.prepare_messages(long_conversation,\n  provider: :openai,\n  model: \"gpt-4\",\n  max_tokens: 4000,           # Reserve tokens for response\n  strategy: :sliding_window,   # or :smart\n  preserve_messages: 5         # Always keep last 5 messages\n)\n```","ref":"user_guide.html#automatic-message-truncation"},{"type":"extras","title":"Truncation Strategies - ExLLM User Guide","doc":"1. **:sliding_window** - Keep most recent messages\n2. **:smart** - Preserve system messages and recent context\n\n```elixir\n# Smart truncation preserves important context\n{:ok, response} = ExLLM.chat(:openai, very_long_conversation,\n  strategy: :smart,\n  preserve_messages: 10\n)\n```","ref":"user_guide.html#truncation-strategies"},{"type":"extras","title":"Context Statistics - ExLLM User Guide","doc":"```elixir\nstats = ExLLM.context_stats(messages)\n# => %{\n#   message_count: 20,\n#   total_tokens: 1500,\n#   by_role: %{\"user\" => 10, \"assistant\" => 9, \"system\" => 1},\n#   avg_tokens_per_message: 75\n# }\n```","ref":"user_guide.html#context-statistics"},{"type":"extras","title":"Function Calling - ExLLM User Guide","doc":"Enable AI models to call functions/tools in your application.","ref":"user_guide.html#function-calling"},{"type":"extras","title":"Basic Function Calling - ExLLM User Guide","doc":"```elixir\n# Define available functions\nfunctions = [\n  %{\n    name: \"get_weather\",\n    description: \"Get current weather for a location\",\n    parameters: %{\n      type: \"object\",\n      properties: %{\n        location: %{\n          type: \"string\",\n          description: \"City and state, e.g. San Francisco, CA\"\n        },\n        unit: %{\n          type: \"string\",\n          enum: [\"celsius\", \"fahrenheit\"],\n          description: \"Temperature unit\"\n        }\n      },\n      required: [\"location\"]\n    }\n  }\n]\n\n# Let the AI decide when to call functions\n{:ok, response} = ExLLM.chat(:openai, \n  [%{role: \"user\", content: \"What's the weather in NYC?\"}],\n  functions: functions,\n  function_call: \"auto\"  # or \"none\" or %{name: \"get_weather\"}\n)\n```","ref":"user_guide.html#basic-function-calling"},{"type":"extras","title":"Handling Function Calls - ExLLM User Guide","doc":"```elixir\n# Parse function calls from response\ncase ExLLM.parse_function_calls(response, :openai) do\n  {:ok, [function_call | _]} ->\n    # AI wants to call a function\n    IO.inspect(function_call)\n    # => %ExLLM.FunctionCalling.FunctionCall{\n    #      name: \"get_weather\",\n    #      arguments: %{\"location\" => \"New York, NY\"}\n    #    }\n    \n    # Execute the function\n    result = get_weather_impl(function_call.arguments[\"location\"])\n    \n    # Format result for conversation\n    function_message = ExLLM.format_function_result(\n      %ExLLM.FunctionCalling.FunctionResult{\n        name: \"get_weather\",\n        result: result\n      },\n      :openai\n    )\n    \n    # Continue conversation with function result\n    messages = messages ++ [response_message, function_message]\n    {:ok, final_response} = ExLLM.chat(:openai, messages)\n    \n  {:ok, []} ->\n    # No function call, regular response\n    IO.puts(response.content)\nend\n```","ref":"user_guide.html#handling-function-calls"},{"type":"extras","title":"Function Execution - ExLLM User Guide","doc":"```elixir\n# Define functions with handlers\nfunctions_with_handlers = [\n  %{\n    name: \"calculate\",\n    description: \"Perform mathematical calculations\",\n    parameters: %{\n      type: \"object\",\n      properties: %{\n        expression: %{type: \"string\"}\n      },\n      required: [\"expression\"]\n    },\n    handler: fn args ->\n      # Your implementation\n      {result, _} = Code.eval_string(args[\"expression\"])\n      %{result: result}\n    end\n  }\n]\n\n# Execute function automatically\n{:ok, result} = ExLLM.execute_function(function_call, functions_with_handlers)\n```","ref":"user_guide.html#function-execution"},{"type":"extras","title":"Provider-Specific Notes - ExLLM User Guide","doc":"Different providers use different terminology:\n- OpenAI: \"functions\" and \"function_call\"\n- Anthropic: \"tools\" and \"tool_use\"\n- ExLLM normalizes these automatically","ref":"user_guide.html#provider-specific-notes"},{"type":"extras","title":"Vision and Multimodal - ExLLM User Guide","doc":"Work with images and other media types.","ref":"user_guide.html#vision-and-multimodal"},{"type":"extras","title":"Basic Image Analysis - ExLLM User Guide","doc":"```elixir\n# Create a vision message\n{:ok, message} = ExLLM.vision_message(\n  \"What's in this image?\",\n  [\"path/to/image.jpg\"]\n)\n\n# Send to vision-capable model\n{:ok, response} = ExLLM.chat(:openai, [message],\n  model: \"gpt-4o\"  # or any vision model\n)\n```","ref":"user_guide.html#basic-image-analysis"},{"type":"extras","title":"Multiple Images - ExLLM User Guide","doc":"```elixir\n{:ok, message} = ExLLM.vision_message(\n  \"Compare these images\",\n  [\n    \"image1.jpg\",\n    \"image2.jpg\",\n    \"https://example.com/image3.png\"  # URLs work too\n  ],\n  detail: :high  # :low, :high, or :auto\n)\n```","ref":"user_guide.html#multiple-images"},{"type":"extras","title":"Loading Images - ExLLM User Guide","doc":"```elixir\n# Load image with options\n{:ok, image_part} = ExLLM.load_image(\"photo.jpg\",\n  detail: :high,\n  resize: {1024, 1024}  # Optional resizing\n)\n\n# Build custom message\nmessage = %{\n  role: \"user\",\n  content: [\n    %{type: \"text\", text: \"Describe this image\"},\n    image_part\n  ]\n}\n```","ref":"user_guide.html#loading-images"},{"type":"extras","title":"Checking Vision Support - ExLLM User Guide","doc":"```elixir\n# Check if provider/model supports vision\nif ExLLM.supports_vision?(:anthropic, \"claude-3-opus-20240229\") do\n  # This model supports vision\nend\n\n# Find all vision-capable models\nvision_models = ExLLM.find_models_with_features([:vision])\n```","ref":"user_guide.html#checking-vision-support"},{"type":"extras","title":"Text Extraction from Images - ExLLM User Guide","doc":"```elixir\n# OCR-like functionality\n{:ok, text} = ExLLM.extract_text_from_image(:openai, \"document.png\",\n  model: \"gpt-4o\",\n  prompt: \"Extract all text, preserving formatting and layout\"\n)\n```","ref":"user_guide.html#text-extraction-from-images"},{"type":"extras","title":"Image Analysis - ExLLM User Guide","doc":"```elixir\n# Analyze multiple images\n{:ok, analysis} = ExLLM.analyze_images(:anthropic,\n  [\"chart1.png\", \"chart2.png\"],\n  \"Compare these charts and identify trends\",\n  model: \"claude-3-5-sonnet-20241022\"\n)\n```","ref":"user_guide.html#image-analysis"},{"type":"extras","title":"Embeddings - ExLLM User Guide","doc":"Generate vector embeddings for semantic search and similarity.","ref":"user_guide.html#embeddings"},{"type":"extras","title":"Basic Embeddings - ExLLM User Guide","doc":"```elixir\n# Generate embeddings for text\n{:ok, response} = ExLLM.embeddings(:openai, \n  [\"Hello world\", \"Goodbye world\"]\n)\n\n# Response structure\n%ExLLM.Types.EmbeddingResponse{\n  embeddings: [\n    [0.0123, -0.0456, ...],  # 1536 dimensions for text-embedding-3-small\n    [0.0789, -0.0234, ...]\n  ],\n  model: \"text-embedding-3-small\",\n  usage: %{total_tokens: 8}\n}\n```","ref":"user_guide.html#basic-embeddings"},{"type":"extras","title":"Embedding Options - ExLLM User Guide","doc":"```elixir\n{:ok, response} = ExLLM.embeddings(:openai, texts,\n  model: \"text-embedding-3-large\",\n  dimensions: 256,  # Reduce dimensions (model-specific)\n  encoding_format: \"float\"  # or \"base64\"\n)\n```","ref":"user_guide.html#embedding-options"},{"type":"extras","title":"Similarity Search - ExLLM User Guide","doc":"```elixir\n# Calculate similarity between embeddings\nsimilarity = ExLLM.cosine_similarity(embedding1, embedding2)\n# => 0.87 (1.0 = identical, 0.0 = orthogonal, -1.0 = opposite)\n\n# Find similar items\nquery_embedding = get_embedding(\"search query\")\nitems = [\n  %{id: 1, text: \"Document 1\", embedding: [...]},\n  %{id: 2, text: \"Document 2\", embedding: [...]},\n  # ...\n]\n\nresults = ExLLM.find_similar(query_embedding, items,\n  top_k: 10,\n  threshold: 0.7  # Minimum similarity\n)\n# => [\n#   %{item: %{id: 2, ...}, similarity: 0.92},\n#   %{item: %{id: 5, ...}, similarity: 0.85},\n#   ...\n# ]\n```","ref":"user_guide.html#similarity-search"},{"type":"extras","title":"Listing Embedding Models - ExLLM User Guide","doc":"```elixir\n{:ok, models} = ExLLM.list_embedding_models(:openai)\nfor model <- models do\n  IO.puts(\"#{model.name}: #{model.dimensions} dimensions\")\nend\n```","ref":"user_guide.html#listing-embedding-models"},{"type":"extras","title":"Caching Embeddings - ExLLM User Guide","doc":"```elixir\n# Enable caching for embeddings\n{:ok, response} = ExLLM.embeddings(:openai, texts,\n  cache: true,\n  cache_ttl: :timer.hours(24)\n)\n```","ref":"user_guide.html#caching-embeddings"},{"type":"extras","title":"Structured Outputs - ExLLM User Guide","doc":"Generate structured data with schema validation using Instructor integration.","ref":"user_guide.html#structured-outputs"},{"type":"extras","title":"Basic Structured Output - ExLLM User Guide","doc":"```elixir\ndefmodule EmailClassification do\n  use Ecto.Schema\n  \n  embedded_schema do\n    field :category, Ecto.Enum, values: [:personal, :work, :spam]\n    field :priority, Ecto.Enum, values: [:high, :medium, :low]\n    field :summary, :string\n  end\nend\n\n{:ok, result} = ExLLM.chat(:openai, \n  [%{role: \"user\", content: \"Classify this email: Meeting tomorrow at 3pm\"}],\n  response_model: EmailClassification,\n  max_retries: 3  # Retry on validation failure\n)\n\nIO.inspect(result)\n# => %EmailClassification{\n#      category: :work,\n#      priority: :high,\n#      summary: \"Meeting scheduled for tomorrow\"\n#    }\n```","ref":"user_guide.html#basic-structured-output"},{"type":"extras","title":"Complex Schemas - ExLLM User Guide","doc":"```elixir\ndefmodule ProductExtraction do\n  use Ecto.Schema\n  \n  embedded_schema do\n    field :name, :string\n    field :price, :decimal\n    field :currency, :string\n    field :in_stock, :boolean\n    \n    embeds_many :features, Feature do\n      field :name, :string\n      field :value, :string\n    end\n  end\n  \n  def changeset(struct, params) do\n    struct\n    |> Ecto.Changeset.cast(params, [:name, :price, :currency, :in_stock])\n    |> Ecto.Changeset.cast_embed(:features)\n    |> Ecto.Changeset.validate_required([:name, :price])\n    |> Ecto.Changeset.validate_number(:price, greater_than: 0)\n  end\nend\n\n{:ok, product} = ExLLM.chat(:anthropic,\n  [%{role: \"user\", content: \"Extract product info from: iPhone 15 Pro, $999, 256GB storage, A17 chip\"}],\n  response_model: ProductExtraction\n)\n```","ref":"user_guide.html#complex-schemas"},{"type":"extras","title":"Lists and Collections - ExLLM User Guide","doc":"```elixir\ndefmodule TodoList do\n  use Ecto.Schema\n  \n  embedded_schema do\n    embeds_many :todos, Todo do\n      field :task, :string\n      field :priority, Ecto.Enum, values: [:high, :medium, :low]\n      field :completed, :boolean, default: false\n    end\n  end\nend\n\n{:ok, todo_list} = ExLLM.chat(:openai,\n  [%{role: \"user\", content: \"Create a todo list for launching a new feature\"}],\n  response_model: TodoList\n)\n```","ref":"user_guide.html#lists-and-collections"},{"type":"extras","title":"Cost Tracking - ExLLM User Guide","doc":"ExLLM automatically tracks API costs for all operations.","ref":"user_guide.html#cost-tracking"},{"type":"extras","title":"Automatic Cost Tracking - ExLLM User Guide","doc":"```elixir\n{:ok, response} = ExLLM.chat(:openai, messages)\n\n# Cost is included in response\nIO.inspect(response.cost)\n# => %{\n#   input_cost: 0.00003,\n#   output_cost: 0.00006,\n#   total_cost: 0.00009,\n#   currency: \"USD\"\n# }\n\n# Format for display\nIO.puts(ExLLM.format_cost(response.cost.total_cost))\n# => \"$0.009¢\"\n```","ref":"user_guide.html#automatic-cost-tracking"},{"type":"extras","title":"Manual Cost Calculation - ExLLM User Guide","doc":"```elixir\nusage = %{input_tokens: 1000, output_tokens: 500}\ncost = ExLLM.calculate_cost(:openai, \"gpt-4\", usage)\n# => %{\n#   input_cost: 0.03,\n#   output_cost: 0.06,\n#   total_cost: 0.09,\n#   currency: \"USD\",\n#   per_million_input: 30.0,\n#   per_million_output: 120.0\n# }\n```","ref":"user_guide.html#manual-cost-calculation"},{"type":"extras","title":"Token Estimation - ExLLM User Guide","doc":"```elixir\n# Estimate tokens for text\ntokens = ExLLM.estimate_tokens(\"Hello, world!\")\n# => 4\n\n# Estimate for messages\ntokens = ExLLM.estimate_tokens([\n  %{role: \"user\", content: \"Hi\"},\n  %{role: \"assistant\", content: \"Hello!\"}\n])\n# => 12\n```","ref":"user_guide.html#token-estimation"},{"type":"extras","title":"Disabling Cost Tracking - ExLLM User Guide","doc":"```elixir\n{:ok, response} = ExLLM.chat(:openai, messages,\n  track_cost: false\n)\n# response.cost will be nil\n```","ref":"user_guide.html#disabling-cost-tracking"},{"type":"extras","title":"Error Handling and Retries - ExLLM User Guide","doc":"","ref":"user_guide.html#error-handling-and-retries"},{"type":"extras","title":"Automatic Retries - ExLLM User Guide","doc":"Retries are enabled by default with exponential backoff:\n\n```elixir\n{:ok, response} = ExLLM.chat(:openai, messages,\n  retry: true,           # Default: true\n  retry_count: 3,        # Default: 3 attempts\n  retry_delay: 1000,     # Default: 1 second initial delay\n  retry_backoff: :exponential,  # or :linear\n  retry_jitter: true     # Add randomness to prevent thundering herd\n)\n```","ref":"user_guide.html#automatic-retries"},{"type":"extras","title":"Error Types - ExLLM User Guide","doc":"```elixir\ncase ExLLM.chat(:openai, messages) do\n  {:ok, response} ->\n    IO.puts(response.content)\n    \n  {:error, %ExLLM.Error{type: :rate_limit} = error} ->\n    IO.puts(\"Rate limited. Retry after: #{error.retry_after}\")\n    \n  {:error, %ExLLM.Error{type: :invalid_api_key}} ->\n    IO.puts(\"Check your API key configuration\")\n    \n  {:error, %ExLLM.Error{type: :context_length_exceeded}} ->\n    IO.puts(\"Message too long for model\")\n    \n  {:error, %ExLLM.Error{type: :timeout}} ->\n    IO.puts(\"Request timed out\")\n    \n  {:error, error} ->\n    IO.inspect(error)\nend\n```","ref":"user_guide.html#error-types"},{"type":"extras","title":"Custom Retry Logic - ExLLM User Guide","doc":"```elixir\ndefmodule MyApp.RetryHandler do\n  def with_custom_retry(provider, messages, opts \\\\ []) do\n    Enum.reduce_while(1..5, nil, fn attempt, _acc ->\n      case ExLLM.chat(provider, messages, Keyword.put(opts, :retry, false)) do\n        {:ok, response} ->\n          {:halt, {:ok, response}}\n          \n        {:error, %{type: :rate_limit} = error} ->\n          wait_time = error[:retry_after] || :timer.seconds(attempt * 10)\n          Process.sleep(wait_time)\n          {:cont, nil}\n          \n        {:error, _} = error ->\n          if attempt == 5 do\n            {:halt, error}\n          else\n            Process.sleep(:timer.seconds(attempt))\n            {:cont, nil}\n          end\n      end\n    end)\n  end\nend\n```","ref":"user_guide.html#custom-retry-logic"},{"type":"extras","title":"Caching - ExLLM User Guide","doc":"Cache responses to reduce API calls and costs.","ref":"user_guide.html#caching"},{"type":"extras","title":"Basic Caching - ExLLM User Guide","doc":"```elixir\n# Enable caching globally\nApplication.put_env(:ex_llm, :cache_enabled, true)\n\n# Or per request\n{:ok, response} = ExLLM.chat(:openai, messages,\n  cache: true,\n  cache_ttl: :timer.minutes(15)  # Default: 15 minutes\n)\n\n# Same request will use cache\n{:ok, cached_response} = ExLLM.chat(:openai, messages, cache: true)\n```","ref":"user_guide.html#basic-caching"},{"type":"extras","title":"Cache Management - ExLLM User Guide","doc":"```elixir\n# Clear specific cache entry\nExLLM.Cache.delete(cache_key)\n\n# Clear all cache\nExLLM.Cache.clear()\n\n# Get cache stats\nstats = ExLLM.Cache.stats()\n# => %{size: 42, hits: 100, misses: 20}\n```","ref":"user_guide.html#cache-management"},{"type":"extras","title":"Custom Cache Keys - ExLLM User Guide","doc":"```elixir\n# Cache key is automatically generated from:\n# - Provider\n# - Messages\n# - Relevant options (model, temperature, etc.)\n\n# You can also use manual cache management\ncache_key = ExLLM.Cache.generate_cache_key(:openai, messages, options)\n```","ref":"user_guide.html#custom-cache-keys"},{"type":"extras","title":"Response Caching - ExLLM User Guide","doc":"Cache real provider responses for offline testing and development cost reduction.\n\nExLLM provides two approaches for response caching:\n\n1. **Unified Cache System** (Recommended) - Extends the runtime cache with optional disk persistence\n2. **Legacy Response Cache** - Standalone response collection system","ref":"user_guide.html#response-caching"},{"type":"extras","title":"Unified Cache System (Recommended) - ExLLM User Guide","doc":"The unified cache system extends ExLLM's runtime performance cache with optional disk persistence. This provides both speed benefits and testing capabilities from a single system.\n\n#### Enabling Unified Cache Persistence\n\n```elixir\n# Method 1: Environment variables (temporary)\nexport EX_LLM_CACHE_PERSIST=true\nexport EX_LLM_CACHE_DIR=\"/path/to/cache\"  # Optional\n\n# Method 2: Runtime configuration (recommended for tests)\nExLLM.Cache.configure_disk_persistence(true, \"/path/to/cache\")\n\n# Method 3: Application configuration\nconfig :ex_llm,\n  cache_persist_disk: true,\n  cache_disk_path: \"/tmp/ex_llm_cache\"\n```\n\n#### Automatic Response Collection with Unified Cache\n\nWhen persistence is enabled, all cached responses are automatically stored to disk:\n\n```elixir\n# Normal caching usage - responses automatically persist to disk when enabled\n{:ok, response} = ExLLM.chat(messages, provider: :openai, cache: true)\n{:ok, response} = ExLLM.chat(messages, provider: :anthropic, cache: true)\n```\n\n#### Benefits of Unified Cache System\n\n- **Zero performance impact** when persistence is disabled (default)\n- **Single configuration** controls both runtime cache and disk persistence  \n- **Natural development workflow** - enable during development, disable in production\n- **Automatic mock integration** - cached responses work seamlessly with Mock adapter","ref":"user_guide.html#unified-cache-system-recommended"},{"type":"extras","title":"Legacy Response Cache System - ExLLM User Guide","doc":"For compatibility, the original response cache system is still available:\n\n#### Enabling Legacy Response Caching\n\n```elixir\n# Enable response caching via environment variables\nexport EX_LLM_CACHE_RESPONSES=true\nexport EX_LLM_CACHE_DIR=\"/path/to/cache\"  # Optional: defaults to /tmp/ex_llm_cache\n```","ref":"user_guide.html#legacy-response-cache-system"},{"type":"extras","title":"Automatic Response Collection - ExLLM User Guide","doc":"When caching is enabled, all provider responses are automatically stored:\n\n```elixir\n# Normal usage - responses are automatically cached\n{:ok, response} = ExLLM.chat(messages, provider: :openai)\n{:ok, stream} = ExLLM.stream_chat(messages, provider: :anthropic)\n```","ref":"user_guide.html#automatic-response-collection"},{"type":"extras","title":"Cache Structure - ExLLM User Guide","doc":"Responses are organized by provider and endpoint:\n\n```\n/tmp/ex_llm_cache/\n├── openai/\n│   ├── chat.json           # Chat completions\n│   └── streaming.json      # Streaming responses\n├── anthropic/\n│   ├── chat.json           # Claude messages\n│   └── streaming.json      # Streaming responses\n└── openrouter/\n    └── chat.json           # OpenRouter responses\n```","ref":"user_guide.html#cache-structure"},{"type":"extras","title":"Manual Response Storage - ExLLM User Guide","doc":"```elixir\n# Store a specific response\nExLLM.ResponseCache.store_response(\n  \"openai\",                    # Provider\n  \"chat\",                      # Endpoint\n  %{messages: messages},       # Request data\n  %{\"choices\" => [...]}        # Response data\n)\n```","ref":"user_guide.html#manual-response-storage"},{"type":"extras","title":"Mock Adapter Integration - ExLLM User Guide","doc":"Configure the Mock adapter to replay cached responses from any provider:\n\n#### Using Unified Cache System\n\nWith the unified cache system, responses are automatically available for mock testing when disk persistence is enabled:\n\n```elixir\n# 1. Enable disk persistence during development/testing\nExLLM.Cache.configure_disk_persistence(true, \"/tmp/ex_llm_cache\")\n\n# 2. Use normal caching to collect responses\n{:ok, response} = ExLLM.chat(:openai, messages, cache: true)\n{:ok, response} = ExLLM.chat(:anthropic, messages, cache: true)\n\n# 3. Configure mock adapter to use cached responses\nExLLM.ResponseCache.configure_mock_provider(:openai)\n\n# 4. Mock calls now return authentic cached responses\n{:ok, response} = ExLLM.chat(messages, provider: :mock)\n# Returns real OpenAI response structure and content\n\n# 5. Switch to different provider responses\nExLLM.ResponseCache.configure_mock_provider(:anthropic)\n{:ok, response} = ExLLM.chat(messages, provider: :mock)\n# Now returns real Anthropic response structure\n```\n\n#### Using Legacy Response Cache\n\nFor compatibility with the original caching approach:\n\n```elixir\n# Enable legacy response caching\nexport EX_LLM_CACHE_RESPONSES=true\n\n# Use cached OpenAI responses for realistic testing\nExLLM.ResponseCache.configure_mock_provider(:openai)\n\n# Now mock calls return authentic OpenAI responses\n{:ok, response} = ExLLM.chat(messages, provider: :mock)\n# Returns real OpenAI response structure and content\n```","ref":"user_guide.html#mock-adapter-integration"},{"type":"extras","title":"Response Collection for Testing - ExLLM User Guide","doc":"Collect comprehensive test scenarios:\n\n```elixir\n# Collect responses for common test cases\nExLLM.CachingInterceptor.create_test_collection(:openai)\n\n# Collect specific scenarios\ntest_cases = [\n  {[%{role: \"user\", content: \"Hello\"}], []},\n  {[%{role: \"user\", content: \"What is 2+2?\"}], [max_tokens: 10]},\n  {[%{role: \"user\", content: \"Tell me a joke\"}], [temperature: 0.8]}\n]\n\nExLLM.CachingInterceptor.collect_test_responses(:anthropic, test_cases)\n```","ref":"user_guide.html#response-collection-for-testing"},{"type":"extras","title":"Cache Management - ExLLM User Guide","doc":"```elixir\n# List available cached providers\nproviders = ExLLM.ResponseCache.list_cached_providers()\n# => [{\"openai\", 15}, {\"anthropic\", 8}]  # {provider, response_count}\n\n# Clear cache for specific provider\nExLLM.ResponseCache.clear_provider_cache(\"openai\")\n\n# Clear all cached responses\nExLLM.ResponseCache.clear_all_cache()\n\n# Get specific cached response\ncached = ExLLM.ResponseCache.get_response(\"openai\", \"chat\", request_data)\n```","ref":"user_guide.html#cache-management-1"},{"type":"extras","title":"Configuration Options - ExLLM User Guide","doc":"```elixir\n# Environment variables\nEX_LLM_CACHE_RESPONSES=true      # Enable/disable caching\nEX_LLM_CACHE_DIR=\"/custom/path\"  # Custom cache directory\n\n# Check if caching is enabled\nExLLM.ResponseCache.caching_enabled?()\n# => true\n\n# Get current cache directory\nExLLM.ResponseCache.cache_dir()\n# => \"/tmp/ex_llm_cache\"\n```","ref":"user_guide.html#configuration-options"},{"type":"extras","title":"Use Cases - ExLLM User Guide","doc":"**Development Testing with Unified Cache:**\n```elixir\n# 1. Enable disk persistence during development\nExLLM.Cache.configure_disk_persistence(true)\n\n# 2. Use normal caching - responses get collected automatically\n{:ok, response} = ExLLM.chat(:openai, messages, cache: true)\n{:ok, response} = ExLLM.chat(:anthropic, messages, cache: true)\n\n# 3. Use cached responses in tests\nExLLM.ResponseCache.configure_mock_provider(:openai)\n# Tests now use real OpenAI response structures\n```\n\n**Development Testing with Legacy Cache:**\n```elixir\n# 1. Collect responses during development\nexport EX_LLM_CACHE_RESPONSES=true\n# Run your app normally - responses get cached\n\n# 2. Use cached responses in tests\nExLLM.ResponseCache.configure_mock_provider(:openai)\n# Tests now use real OpenAI response structures\n```\n\n**Cost Reduction:**\n```elixir\n# Unified cache approach - enable persistence temporarily\nExLLM.Cache.configure_disk_persistence(true)\n\n# Cache expensive model responses during development\n{:ok, response} = ExLLM.chat(:openai, messages,\n  cache: true,\n  model: \"gpt-4o\"  # Expensive model\n)\n# Response is cached automatically both in memory and disk\n\n# Later testing uses cached response - no API cost\nExLLM.ResponseCache.configure_mock_provider(:openai)\n{:ok, same_response} = ExLLM.chat(messages, provider: :mock)\n\n# Disable persistence for production\nExLLM.Cache.configure_disk_persistence(false)\n```\n\n**Cross-Provider Testing:**\n```elixir\n# Test how your app handles different provider response formats\nExLLM.ResponseCache.configure_mock_provider(:openai)\ntest_openai_format()\n\nExLLM.ResponseCache.configure_mock_provider(:anthropic)\ntest_anthropic_format()\n\nExLLM.ResponseCache.configure_mock_provider(:openrouter)\ntest_openrouter_format()\n```","ref":"user_guide.html#use-cases"},{"type":"extras","title":"Advanced Usage - ExLLM User Guide","doc":"**Streaming Response Caching:**\n```elixir\n# Streaming responses are automatically cached\n{:ok, stream} = ExLLM.stream_chat(messages, provider: :openai)\nchunks = Enum.to_list(stream)\n\n# Later, mock can replay the exact same stream\nExLLM.ResponseCache.configure_mock_provider(:openai)\n{:ok, cached_stream} = ExLLM.stream_chat(messages, provider: :mock)\n# Returns identical streaming chunks\n```\n\n**Interceptor Wrapping:**\n```elixir\n# Manually wrap API calls for caching\n{:ok, response} = ExLLM.CachingInterceptor.with_caching(:openai, fn ->\n  ExLLM.Adapters.OpenAI.chat(messages)\nend)\n\n# Wrap streaming calls\n{:ok, stream} = ExLLM.CachingInterceptor.with_streaming_cache(\n  :anthropic, \n  messages, \n  options, \n  fn -> ExLLM.Adapters.Anthropic.stream_chat(messages, options) end\n)\n```","ref":"user_guide.html#advanced-usage"},{"type":"extras","title":"Model Discovery - ExLLM User Guide","doc":"","ref":"user_guide.html#model-discovery"},{"type":"extras","title":"Finding Models - ExLLM User Guide","doc":"```elixir\n# Get model information\n{:ok, info} = ExLLM.get_model_info(:openai, \"gpt-4o\")\nIO.inspect(info)\n# => %ExLLM.ModelCapabilities.ModelInfo{\n#   id: \"gpt-4o\",\n#   context_window: 128000,\n#   max_output_tokens: 16384,\n#   capabilities: %{\n#     vision: %{supported: true},\n#     function_calling: %{supported: true},\n#     streaming: %{supported: true},\n#     ...\n#   }\n# }\n\n# Check specific capability\nif ExLLM.model_supports?(:openai, \"gpt-4o\", :vision) do\n  # Model supports vision\nend\n```","ref":"user_guide.html#finding-models"},{"type":"extras","title":"Model Recommendations - ExLLM User Guide","doc":"```elixir\n# Get recommendations based on requirements\nrecommendations = ExLLM.recommend_models(\n  features: [:vision, :function_calling],\n  min_context_window: 100_000,\n  max_cost_per_1k_tokens: 1.0,\n  prefer_local: false,\n  limit: 5\n)\n\nfor {provider, model, info} <- recommendations do\n  IO.puts(\"#{provider}/#{model}\")\n  IO.puts(\"  Score: #{info.score}\")\n  IO.puts(\"  Context: #{info.context_window}\")\n  IO.puts(\"  Cost: $#{info.cost_per_1k}/1k tokens\")\nend\n```","ref":"user_guide.html#model-recommendations"},{"type":"extras","title":"Finding Models by Feature - ExLLM User Guide","doc":"```elixir\n# Find all models with specific features\nmodels = ExLLM.find_models_with_features([:vision, :streaming])\n# => [\n#   {:openai, \"gpt-4o\"},\n#   {:anthropic, \"claude-3-opus-20240229\"},\n#   ...\n# ]\n\n# Group models by capability\ngrouped = ExLLM.models_by_capability(:vision)\n# => %{\n#   supported: [{:openai, \"gpt-4o\"}, ...],\n#   not_supported: [{:openai, \"gpt-3.5-turbo\"}, ...]\n# }\n```","ref":"user_guide.html#finding-models-by-feature"},{"type":"extras","title":"Comparing Models - ExLLM User Guide","doc":"```elixir\ncomparison = ExLLM.compare_models([\n  {:openai, \"gpt-4o\"},\n  {:anthropic, \"claude-3-5-sonnet-20241022\"},\n  {:gemini, \"gemini-1.5-pro\"}\n])\n\n# See feature support across models\nIO.inspect(comparison.features[:vision])\n# => [\n#   %{model: \"gpt-4o\", supported: true, details: %{...}},\n#   %{model: \"claude-3-5-sonnet\", supported: true, details: %{...}},\n#   %{model: \"gemini-1.5-pro\", supported: true, details: %{...}}\n# ]\n```","ref":"user_guide.html#comparing-models"},{"type":"extras","title":"Provider Capabilities - ExLLM User Guide","doc":"","ref":"user_guide.html#provider-capabilities"},{"type":"extras","title":"Capability Normalization - ExLLM User Guide","doc":"ExLLM automatically normalizes different provider terminologies:\n\n```elixir\n# These all work and refer to the same capability\nExLLM.provider_supports?(:openai, :function_calling)     # => true\nExLLM.provider_supports?(:anthropic, :tool_use)          # => true\nExLLM.provider_supports?(:openai, :tools)                # => true\n\n# Find providers using any terminology\nExLLM.find_providers_with_features([:tool_use])          # Works!\nExLLM.find_providers_with_features([:function_calling])  # Also works!\n```","ref":"user_guide.html#capability-normalization"},{"type":"extras","title":"Provider Discovery - ExLLM User Guide","doc":"```elixir\n# Get provider capabilities\n{:ok, caps} = ExLLM.get_provider_capabilities(:openai)\nIO.inspect(caps)\n# => %ExLLM.ProviderCapabilities.ProviderInfo{\n#   id: :openai,\n#   name: \"OpenAI\",\n#   endpoints: [:chat, :embeddings, :images, ...],\n#   features: [:streaming, :function_calling, ...],\n#   limitations: %{max_file_size: 512MB, ...}\n# }\n\n# Find providers by feature\nproviders = ExLLM.find_providers_with_features([:embeddings, :streaming])\n# => [:openai, :gemini, :bedrock, ...]\n\n# Check authentication requirements\nif ExLLM.provider_requires_auth?(:openai) do\n  # Provider needs API key\nend\n\n# Check if provider is local\nif ExLLM.is_local_provider?(:ollama) do\n  # No API costs\nend\n```","ref":"user_guide.html#provider-discovery"},{"type":"extras","title":"Provider Recommendations - ExLLM User Guide","doc":"```elixir\nrecommendations = ExLLM.recommend_providers(%{\n  required_features: [:vision, :streaming],\n  preferred_features: [:embeddings, :function_calling],\n  exclude_providers: [:mock],\n  prefer_local: false,\n  prefer_free: false\n})\n\nfor %{provider: provider, score: score, matched_features: features} <- recommendations do\n  IO.puts(\"#{provider}: #{Float.round(score, 2)}\")\n  IO.puts(\"  Features: #{Enum.join(features, \", \")}\")\nend\n```","ref":"user_guide.html#provider-recommendations"},{"type":"extras","title":"Comparing Providers - ExLLM User Guide","doc":"```elixir\ncomparison = ExLLM.compare_providers([:openai, :anthropic, :gemini])\n\n# See all features across providers\nIO.puts(\"All features: #{Enum.join(comparison.features, \", \")}\")\n\n# Check specific provider capabilities\nopenai_features = comparison.comparison.openai.features\n# => [:streaming, :function_calling, :embeddings, ...]\n```","ref":"user_guide.html#comparing-providers"},{"type":"extras","title":"Logging - ExLLM User Guide","doc":"ExLLM provides a unified logging system with security features.","ref":"user_guide.html#logging"},{"type":"extras","title":"Basic Logging - ExLLM User Guide","doc":"```elixir\nalias ExLLM.Logger\n\n# Log at different levels\nLogger.debug(\"Starting chat request\")\nLogger.info(\"Chat completed in #{duration}ms\")\nLogger.warning(\"Rate limit approaching\")\nLogger.error(\"API request failed\", error: reason)\n```","ref":"user_guide.html#basic-logging"},{"type":"extras","title":"Structured Logging - ExLLM User Guide","doc":"```elixir\n# Log with metadata\nLogger.info(\"Chat completed\",\n  provider: :openai,\n  model: \"gpt-4o\",\n  tokens: 150,\n  duration_ms: 523\n)\n\n# Context-aware logging\nLogger.with_context(request_id: \"abc123\") do\n  Logger.info(\"Processing request\")\n  # All logs in this block include request_id\nend\n```","ref":"user_guide.html#structured-logging"},{"type":"extras","title":"Security Features - ExLLM User Guide","doc":"```elixir\n# API keys are automatically redacted\nLogger.info(\"Using API key\", api_key: \"sk-1234567890\")\n# Logs: \"Using API key [api_key: REDACTED]\"\n\n# Configure content filtering\nApplication.put_env(:ex_llm, :log_redact_messages, true)\n```","ref":"user_guide.html#security-features"},{"type":"extras","title":"Configuration - ExLLM User Guide","doc":"```elixir\n# In config/config.exs\nconfig :ex_llm,\n  log_level: :info,                    # Minimum level to log\n  log_redact_keys: true,               # Redact API keys\n  log_redact_messages: false,          # Don't log message content\n  log_include_metadata: true,          # Include structured metadata\n  log_filter_components: [:cache]      # Don't log from cache component\n```\n\nSee the [Logger User Guide](LOGGER.md) for complete documentation.","ref":"user_guide.html#configuration-1"},{"type":"extras","title":"Testing with Mock Adapter - ExLLM User Guide","doc":"The mock adapter helps you test LLM integrations without making real API calls.","ref":"user_guide.html#testing-with-mock-adapter"},{"type":"extras","title":"Basic Mocking - ExLLM User Guide","doc":"```elixir\n# Start the mock adapter\n{:ok, _} = ExLLM.Adapters.Mock.start_link()\n\n# Configure mock response\n{:ok, response} = ExLLM.chat(:mock, messages,\n  mock_response: \"This is a mock response\"\n)\n\nassert response.content == \"This is a mock response\"\n```","ref":"user_guide.html#basic-mocking"},{"type":"extras","title":"Dynamic Responses - ExLLM User Guide","doc":"```elixir\n# Use a handler function\n{:ok, response} = ExLLM.chat(:mock, messages,\n  mock_handler: fn messages, _options ->\n    last_message = List.last(messages)\n    %ExLLM.Types.LLMResponse{\n      content: \"You said: #{last_message.content}\",\n      model: \"mock-model\",\n      usage: %{input_tokens: 10, output_tokens: 20}\n    }\n  end\n)\n```","ref":"user_guide.html#dynamic-responses"},{"type":"extras","title":"Simulating Errors - ExLLM User Guide","doc":"```elixir\n# Simulate specific errors\n{:error, error} = ExLLM.chat(:mock, messages,\n  mock_error: %ExLLM.Error{\n    type: :rate_limit,\n    message: \"Rate limit exceeded\",\n    retry_after: 60\n  }\n)\n```","ref":"user_guide.html#simulating-errors"},{"type":"extras","title":"Streaming Mocks - ExLLM User Guide","doc":"```elixir\n{:ok, stream} = ExLLM.stream_chat(:mock, messages,\n  mock_chunks: [\n    %{content: \"Hello\"},\n    %{content: \" world\"},\n    %{content: \"!\", finish_reason: \"stop\"}\n  ],\n  chunk_delay: 100  # Milliseconds between chunks\n)\n\nfor chunk <- stream do\n  IO.write(chunk.content || \"\")\nend\n```","ref":"user_guide.html#streaming-mocks"},{"type":"extras","title":"Request Capture - ExLLM User Guide","doc":"```elixir\n# Capture requests for assertions\nExLLM.Adapters.Mock.clear_requests()\n\n{:ok, _} = ExLLM.chat(:mock, messages,\n  capture_requests: true,\n  mock_response: \"OK\"\n)\n\nrequests = ExLLM.Adapters.Mock.get_requests()\nassert length(requests) == 1\nassert List.first(requests).messages == messages\n```","ref":"user_guide.html#request-capture"},{"type":"extras","title":"Advanced Topics - ExLLM User Guide","doc":"","ref":"user_guide.html#advanced-topics"},{"type":"extras","title":"Custom Adapters - ExLLM User Guide","doc":"Create your own adapter for unsupported providers:\n\n```elixir\ndefmodule MyApp.CustomAdapter do\n  @behaviour ExLLM.Adapter\n  \n  @impl true\n  def configured?(options) do\n    # Check if adapter is properly configured\n    config = get_config(options)\n    config[:api_key] != nil\n  end\n  \n  @impl true\n  def default_model() do\n    \"custom-model-v1\"\n  end\n  \n  @impl true\n  def chat(messages, options) do\n    # Implement chat logic\n    # Return {:ok, %ExLLM.Types.LLMResponse{}} or {:error, reason}\n  end\n  \n  @impl true\n  def stream_chat(messages, options) do\n    # Return {:ok, stream} where stream yields StreamChunk structs\n  end\n  \n  # Optional callbacks\n  @impl true\n  def list_models(options) do\n    # Return {:ok, [%ExLLM.Types.Model{}]}\n  end\n  \n  @impl true\n  def embeddings(inputs, options) do\n    # Return {:ok, %ExLLM.Types.EmbeddingResponse{}}\n  end\nend\n```","ref":"user_guide.html#custom-adapters"},{"type":"extras","title":"Stream Processing - ExLLM User Guide","doc":"Advanced stream handling:\n\n```elixir\ndefmodule StreamProcessor do\n  def process_with_buffer(provider, messages, opts) do\n    {:ok, stream} = ExLLM.stream_chat(provider, messages, opts)\n    \n    stream\n    |> Stream.scan(\"\", fn chunk, buffer ->\n      case chunk do\n        %{content: nil} -> buffer\n        %{content: text} -> buffer <> text\n      end\n    end)\n    |> Stream.each(fn buffer ->\n      # Process complete sentences\n      if String.ends_with?(buffer, \".\") do\n        IO.puts(\"\\nComplete: #{buffer}\")\n      end\n    end)\n    |> Stream.run()\n  end\nend\n```","ref":"user_guide.html#stream-processing"},{"type":"extras","title":"Token Budget Management - ExLLM User Guide","doc":"Manage token usage across multiple requests:\n\n```elixir\ndefmodule TokenBudget do\n  use GenServer\n  \n  def init(budget) do\n    {:ok, %{budget: budget, used: 0}}\n  end\n  \n  def track_usage(pid, tokens) do\n    GenServer.call(pid, {:track, tokens})\n  end\n  \n  def handle_call({:track, tokens}, _from, state) do\n    new_used = state.used + tokens\n    if new_used <= state.budget do\n      {:reply, :ok, %{state | used: new_used}}\n    else\n      {:reply, {:error, :budget_exceeded}, state}\n    end\n  end\nend\n\n# Use with ExLLM\n{:ok, budget} = GenServer.start_link(TokenBudget, 10_000)\n\n{:ok, response} = ExLLM.chat(:openai, messages)\n:ok = TokenBudget.track_usage(budget, response.usage.total_tokens)\n```","ref":"user_guide.html#token-budget-management"},{"type":"extras","title":"Multi-Provider Routing - ExLLM User Guide","doc":"Route requests to different providers based on criteria:\n\n```elixir\ndefmodule ProviderRouter do\n  def route_request(messages, requirements) do\n    cond do\n      # Use local for development\n      Mix.env() == :dev ->\n        ExLLM.chat(:ollama, messages)\n        \n      # Use Groq for speed-critical requests\n      requirements[:max_latency_ms] < 1000 ->\n        ExLLM.chat(:groq, messages)\n        \n      # Use OpenAI for complex reasoning\n      requirements[:complexity] == :high ->\n        ExLLM.chat(:openai, messages, model: \"gpt-4o\")\n        \n      # Default to Anthropic\n      true ->\n        ExLLM.chat(:anthropic, messages)\n    end\n  end\nend\n```","ref":"user_guide.html#multi-provider-routing"},{"type":"extras","title":"Batch Processing - ExLLM User Guide","doc":"Process multiple requests efficiently:\n\n```elixir\ndefmodule BatchProcessor do\n  def process_batch(items, opts \\\\ []) do\n    # Use Task.async_stream for parallel processing\n    items\n    |> Task.async_stream(\n      fn item ->\n        ExLLM.chat(opts[:provider] || :openai, [\n          %{role: \"user\", content: item}\n        ])\n      end,\n      max_concurrency: opts[:concurrency] || 5,\n      timeout: opts[:timeout] || 30_000\n    )\n    |> Enum.map(fn\n      {:ok, {:ok, response}} -> {:ok, response}\n      {:ok, {:error, reason}} -> {:error, reason}\n      {:exit, reason} -> {:error, {:timeout, reason}}\n    end)\n  end\nend\n```","ref":"user_guide.html#batch-processing"},{"type":"extras","title":"Custom Configuration Management - ExLLM User Guide","doc":"Implement advanced configuration strategies:\n\n```elixir\ndefmodule ConfigManager do\n  use GenServer\n  \n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n  \n  def init(_opts) do\n    # Load from multiple sources\n    config = %{}\n    |> load_from_env()\n    |> load_from_file()\n    |> load_from_vault()\n    |> validate_config()\n    \n    {:ok, config}\n  end\n  \n  def get_config(provider) do\n    GenServer.call(__MODULE__, {:get, provider})\n  end\n  \n  defp load_from_vault(config) do\n    # Fetch from HashiCorp Vault, AWS Secrets Manager, etc.\n    Map.merge(config, fetch_secrets())\n  end\nend\n```","ref":"user_guide.html#custom-configuration-management"},{"type":"extras","title":"Best Practices - ExLLM User Guide","doc":"1. **Always handle errors** - LLM APIs can fail for various reasons\n2. **Use streaming for long responses** - Better user experience\n3. **Enable caching for repeated queries** - Save costs\n4. **Monitor token usage** - Stay within budget\n5. **Use appropriate models** - Don't use GPT-4 for simple tasks\n6. **Implement fallbacks** - Have backup providers ready\n7. **Test with mocks** - Don't make API calls in tests\n8. **Use context management** - Handle long conversations properly\n9. **Track costs** - Monitor spending across providers\n10. **Follow rate limits** - Respect provider limitations","ref":"user_guide.html#best-practices"},{"type":"extras","title":"Troubleshooting - ExLLM User Guide","doc":"","ref":"user_guide.html#troubleshooting"},{"type":"extras","title":"Common Issues - ExLLM User Guide","doc":"1. **\"API key not found\"**\n   - Check environment variables\n   - Verify configuration provider is started\n   - Use `ExLLM.configured?/1` to debug\n\n2. **\"Context length exceeded\"**\n   - Use context management strategies\n   - Choose models with larger context windows\n   - Truncate conversation history\n\n3. **\"Rate limit exceeded\"**\n   - Enable automatic retry\n   - Implement backoff strategies\n   - Consider multiple API keys\n\n4. **\"Stream interrupted\"**\n   - Enable stream recovery\n   - Implement reconnection logic\n   - Check network stability\n\n5. **\"Invalid response format\"**\n   - Check provider documentation\n   - Verify model capabilities\n   - Use appropriate options","ref":"user_guide.html#common-issues"},{"type":"extras","title":"Debug Mode - ExLLM User Guide","doc":"Enable debug logging:\n\n```elixir\n# In config\nconfig :ex_llm, :log_level, :debug\n\n# Or at runtime\nLogger.configure(level: :debug)\n```","ref":"user_guide.html#debug-mode"},{"type":"extras","title":"Getting Help - ExLLM User Guide","doc":"- Check the [API documentation](https://hexdocs.pm/ex_llm)\n- Review [example applications](../examples/)\n- Open an issue on [GitHub](https://github.com/azmaveth/ex_llm)\n- Read provider-specific documentation","ref":"user_guide.html#getting-help"},{"type":"extras","title":"Additional Resources - ExLLM User Guide","doc":"- [Quick Start Guide](QUICKSTART.md) - Get started quickly\n- [Provider Capabilities](PROVIDER_CAPABILITIES.md) - Detailed provider information\n- [Logger Guide](LOGGER.md) - Logging system documentation\n- [API Reference](https://hexdocs.pm/ex_llm) - Complete API documentation","ref":"user_guide.html#additional-resources"},{"type":"extras","title":"Testing Guide","doc":"# Testing Guide\n\nExLLM includes a comprehensive testing system with intelligent caching, semantic tagging, and 24 specialized Mix aliases for targeted test execution.","ref":"testing.html"},{"type":"extras","title":"Quick Start - Testing Guide","doc":"```bash\n# Run all tests (fast - uses cache when available)\nmix test\n\n# Run provider-specific tests\nmix test.anthropic\nmix test.openai\nmix test.gemini\n\n# Run integration tests with live APIs\nmix test.integration --include live_api\n\n# Run tests by capability\nmix test.streaming\nmix test.vision\nmix test.oauth2\n\n# Manage test cache\nmix ex_llm.cache stats\nmix ex_llm.cache clean --older-than 7d\n```","ref":"testing.html#quick-start"},{"type":"extras","title":"Test Organization - Testing Guide","doc":"","ref":"testing.html#test-organization"},{"type":"extras","title":"Test Tags - Testing Guide","doc":"ExLLM uses semantic tags to organize tests by requirements, capabilities, and providers:\n\n#### **Requirement Tags**\n- `:requires_api_key` - Tests needing API keys with automatic provider detection\n- `:requires_oauth` - Tests needing OAuth2 authentication (e.g., Gemini APIs)\n- `:requires_service` - Tests needing local services (Ollama, LM Studio)\n- `:requires_resource` - Tests needing pre-existing resources (tuned models, corpora)\n\n#### **Test Type Tags**\n- `:live_api` - Tests that call live provider APIs\n- `:integration` - Integration tests with external services\n- `:external` - Tests making external network calls\n- `:unit` - Unit tests (isolated, no external dependencies)\n\n#### **Provider Tags**\n- `:anthropic`, `:openai`, `:gemini`, `:groq`, `:mistral`\n- `:openrouter`, `:perplexity`, `:ollama`, `:lmstudio`, `:bumblebee`\n\n#### **Capability Tags**\n- `:streaming` - Tests for streaming responses\n- `:vision` - Tests for image/vision capabilities  \n- `:multimodal` - Tests for multimodal inputs\n- `:function_calling` - Tests for tool/function calling\n- `:embedding` - Tests for embedding generation","ref":"testing.html#test-tags"},{"type":"extras","title":"Mix Test Aliases - Testing Guide","doc":"ExLLM provides 24 specialized test aliases for targeted execution:","ref":"testing.html#mix-test-aliases"},{"type":"extras","title":"Provider-Specific Tests - Testing Guide","doc":"```bash\n# Test individual providers\nmix test.anthropic       # Anthropic Claude tests\nmix test.openai          # OpenAI GPT tests  \nmix test.gemini          # Google Gemini tests\nmix test.groq            # Groq tests\nmix test.mistral         # Mistral AI tests\nmix test.openrouter      # OpenRouter tests\nmix test.perplexity      # Perplexity tests\nmix test.ollama          # Ollama local tests\nmix test.lmstudio        # LM Studio tests\nmix test.bumblebee       # Bumblebee local tests\n```","ref":"testing.html#provider-specific-tests"},{"type":"extras","title":"Test Type Aliases - Testing Guide","doc":"```bash\n# By test type\nmix test.unit            # Unit tests only\nmix test.integration     # Integration tests\nmix test.external        # Tests with external calls\nmix test.oauth2          # OAuth2 authentication tests\n```","ref":"testing.html#test-type-aliases"},{"type":"extras","title":"Capability-Based Tests - Testing Guide","doc":"```bash\n# By capability\nmix test.streaming       # Streaming response tests\nmix test.vision          # Vision/image processing tests\nmix test.multimodal      # Multimodal input tests\nmix test.function_calling # Function/tool calling tests\nmix test.embedding       # Embedding generation tests\n```","ref":"testing.html#capability-based-tests"},{"type":"extras","title":"Environment-Based Tests - Testing Guide","doc":"```bash\n# By environment needs\nmix test.live_api        # Tests calling live APIs\nmix test.local_only      # Local-only tests (no API calls)\nmix test.fast            # Fast tests (cached/mocked)\nmix test.all             # All tests including slow ones\n```","ref":"testing.html#environment-based-tests"},{"type":"extras","title":"Test Caching System - Testing Guide","doc":"ExLLM includes an advanced caching system that provides 25x speed improvements for integration tests.","ref":"testing.html#test-caching-system"},{"type":"extras","title":"How It Works - Testing Guide","doc":"1. **Automatic Detection**: Tests tagged with `:live_api` are automatically cached\n2. **Smart Exclusions**: Destructive operations (create, delete, modify) are not cached\n3. **TTL Management**: Cached responses expire after 7 days by default\n4. **Fallback Strategies**: Multiple matching algorithms for cache hits","ref":"testing.html#how-it-works"},{"type":"extras","title":"Cache Management - Testing Guide","doc":"```bash\n# View cache statistics\nmix ex_llm.cache stats\n\n# Clean old cache entries  \nmix ex_llm.cache clean --older-than 7d\n\n# Clear all cache\nmix ex_llm.cache clear\n\n# Show cache details for a provider\nmix ex_llm.cache show anthropic\n```","ref":"testing.html#cache-management"},{"type":"extras","title":"Configuration - Testing Guide","doc":"Configure caching via environment variables:\n\n```bash\n# Enable/disable caching\nexport EX_LLM_TEST_CACHE_ENABLED=true\n\n# Cache directory\nexport EX_LLM_TEST_CACHE_DIR=\"test/cache\"\n\n# TTL for cached responses (in seconds)\nexport EX_LLM_TEST_CACHE_TTL=604800  # 7 days\n\n# Cache destructive operations\nexport EX_LLM_TEST_CACHE_DESTRUCTIVE_OPS=false\n```","ref":"testing.html#configuration"},{"type":"extras","title":"Writing Tests - Testing Guide","doc":"","ref":"testing.html#writing-tests"},{"type":"extras","title":"Basic Test Structure - Testing Guide","doc":"```elixir\ndefmodule MyProviderTest do\n  use ExUnit.Case\n  \n  # Module-level tags\n  @moduletag :integration\n  @moduletag :live_api\n  @moduletag :requires_api_key\n  @moduletag provider: :anthropic\n  \n  # Import cache helpers\n  import ExLLM.TestCacheHelpers\n  \n  setup_all do\n    enable_cache_debug()\n    :ok\n  end\n  \n  setup context do\n    setup_test_cache(context)\n    on_exit(fn -> ExLLM.TestCacheDetector.clear_test_context() end)\n    :ok\n  end\n  \n  test \"basic chat completion\" do\n    {:ok, response} = ExLLM.chat(:anthropic, [\n      %{role: \"user\", content: \"Hello!\"}\n    ])\n    \n    assert response.content != \"\"\n  end\nend\n```","ref":"testing.html#basic-test-structure"},{"type":"extras","title":"Using ExLLM.Case for Automatic Requirements - Testing Guide","doc":"```elixir\ndefmodule MyProviderTest do\n  use ExLLM.Case, async: true\n  \n  @moduletag :requires_api_key\n  @moduletag provider: :openai\n  \n  test \"test with automatic API key checking\", context do\n    # Automatically skips if OPENAI_API_KEY not set\n    check_test_requirements!(context)\n    \n    {:ok, response} = ExLLM.chat(:openai, [\n      %{role: \"user\", content: \"Test\"}\n    ])\n    \n    assert response.content != \"\"\n  end\nend\n```","ref":"testing.html#using-exllm-case-for-automatic-requirements"},{"type":"extras","title":"OAuth2 Tests - Testing Guide","doc":"```elixir\ndefmodule GeminiOAuth2Test do\n  use ExLLM.Case, async: true\n  \n  @moduletag :requires_oauth\n  @moduletag provider: :gemini\n  \n  test \"OAuth2 API call\", context do\n    check_test_requirements!(context)\n    \n    # OAuth token automatically provided if available\n    oauth_token = get_oauth_token(context)\n    \n    {:ok, response} = ExLLM.Gemini.Permissions.list_permissions(\n      \"tunedModels/test\",\n      oauth_token: oauth_token\n    )\n    \n    assert is_list(response.permissions)\n  end\nend\n```","ref":"testing.html#oauth2-tests"},{"type":"extras","title":"Test Exclusions - Testing Guide","doc":"","ref":"testing.html#test-exclusions"},{"type":"extras","title":"Default Exclusions - Testing Guide","doc":"By default, these tests are excluded unless explicitly included:\n\n```bash\n# In test_helper.exs\nExUnit.configure(exclude: [\n  :live_api,           # Exclude live API calls by default\n  :requires_api_key,   # Exclude tests needing API keys\n  :requires_oauth,     # Exclude OAuth tests\n  :requires_service,   # Exclude tests needing local services\n  :integration,        # Exclude integration tests\n  :external           # Exclude external network tests\n])\n```","ref":"testing.html#default-exclusions"},{"type":"extras","title":"Running Excluded Tests - Testing Guide","doc":"```bash\n# Include specific tags\nmix test --include live_api\nmix test --include requires_api_key\nmix test --include oauth2\n\n# Include multiple tags\nmix test --include live_api --include streaming\n\n# Run only specific tags\nmix test --only provider:anthropic\nmix test --only streaming\n```","ref":"testing.html#running-excluded-tests"},{"type":"extras","title":"CI/CD Integration - Testing Guide","doc":"","ref":"testing.html#ci-cd-integration"},{"type":"extras","title":"GitHub Actions Example - Testing Guide","doc":"```yaml\nname: Tests\non: [push, pull_request]\n\njobs:\n  unit-tests:\n    runs-on: ubuntu-latest\n    steps:\n      - uses: actions/checkout@v3\n      - uses: erlef/setup-beam@v1\n        with:\n          elixir-version: '1.18'\n          otp-version: '28'\n      \n      # Unit tests only (no API keys needed)\n      - run: mix test.unit\n  \n  integration-tests:\n    runs-on: ubuntu-latest\n    if: github.event_name == 'push'\n    steps:\n      - uses: actions/checkout@v3\n      - uses: erlef/setup-beam@v1\n      \n      # Integration tests with caching\n      - run: mix test.integration --include live_api\n        env:\n          ANTHROPIC_API_KEY: ${{ secrets.ANTHROPIC_API_KEY }}\n          OPENAI_API_KEY: ${{ secrets.OPENAI_API_KEY }}\n          EX_LLM_TEST_CACHE_ENABLED: true\n```","ref":"testing.html#github-actions-example"},{"type":"extras","title":"Performance Tips - Testing Guide","doc":"1. **Use Caching**: Enable test caching for 25x faster integration tests\n2. **Tag Appropriately**: Use semantic tags for precise test selection\n3. **Run Targeted Tests**: Use Mix aliases to run only what you need\n4. **Cache Management**: Regularly clean old cache entries\n5. **Local Services**: Use Ollama/LM Studio for development without API costs","ref":"testing.html#performance-tips"},{"type":"extras","title":"Troubleshooting - Testing Guide","doc":"","ref":"testing.html#troubleshooting"},{"type":"extras","title":"Common Issues - Testing Guide","doc":"**Tests skipped with \"API key required\":**\n```bash\n# Set the required API key\nexport ANTHROPIC_API_KEY=\"your-key\"\nmix test.anthropic\n```\n\n**OAuth2 tests failing:**\n```bash\n# Setup OAuth2 first\nelixir scripts/setup_oauth2.exs\n# Then refresh token\nelixir scripts/refresh_oauth2_token.exs\n```\n\n**Cache not working:**\n```bash\n# Check cache configuration\nmix ex_llm.cache stats\n# Enable debug logging\nexport EX_LLM_LOG_LEVEL=debug\n```\n\n**Tests timing out:**\n```bash\n# Use cached responses\nexport EX_LLM_TEST_CACHE_ENABLED=true\nmix test --include live_api\n```","ref":"testing.html#common-issues"},{"type":"extras","title":"Debug Logging - Testing Guide","doc":"Enable debug logging to troubleshoot test issues:\n\n```bash\nexport EX_LLM_LOG_LEVEL=debug\nexport EX_LLM_LOG_COMPONENTS=http_client,cache,test_detector\nmix test --include live_api\n```\n\nThis comprehensive testing system ensures reliable, fast, and well-organized tests across all ExLLM functionality.","ref":"testing.html#debug-logging"},{"type":"extras","title":"ExLLM Logger User Guide","doc":"# ExLLM Logger User Guide\n\nThe ExLLM Logger provides a unified logging system with features specifically designed for debugging and monitoring LLM interactions.","ref":"logger.html"},{"type":"extras","title":"Table of Contents - ExLLM Logger User Guide","doc":"- [Quick Start](#quick-start)\n- [Configuration](#configuration)\n- [Basic Logging](#basic-logging)\n- [Context Tracking](#context-tracking)\n- [Structured Logging](#structured-logging)\n- [Security & Redaction](#security--redaction)\n- [Best Practices](#best-practices)","ref":"logger.html#table-of-contents"},{"type":"extras","title":"Quick Start - ExLLM Logger User Guide","doc":"```elixir\n# Add the logger alias in your module\nalias ExLLM.Logger\n\n# Simple logging - just like Elixir's Logger!\nLogger.info(\"Processing request\")\nLogger.error(\"Request failed\", error: reason)\nLogger.debug(\"Response received\", tokens: 150)\n```","ref":"logger.html#quick-start"},{"type":"extras","title":"Configuration - ExLLM Logger User Guide","doc":"Configure ExLLM logging in your `config/config.exs` or `config/runtime.exs`:\n\n```elixir\nconfig :ex_llm,\n  # Overall log level: :debug, :info, :warn, :error, :none\n  log_level: :info,\n  \n  # Control which components log\n  log_components: %{\n    requests: true,     # API request logging\n    responses: true,    # API response logging\n    streaming: false,   # Stream events (can be noisy)\n    retries: true,      # Retry attempts\n    cache: false,       # Cache hits/misses\n    models: true        # Model loading events\n  },\n  \n  # Security settings\n  log_redaction: %{\n    api_keys: true,     # Always redact API keys\n    content: false      # Redact message content (recommended for production)\n  }\n```","ref":"logger.html#configuration"},{"type":"extras","title":"Environment-Specific Configuration - ExLLM Logger User Guide","doc":"```elixir\n# config/dev.exs\nconfig :ex_llm,\n  log_level: :debug,\n  log_components: %{\n    requests: true,\n    responses: true,\n    streaming: true,  # Enable in dev for debugging\n    retries: true,\n    cache: true,\n    models: true\n  },\n  log_redaction: %{\n    api_keys: true,\n    content: false    # Show content in dev\n  }\n\n# config/prod.exs\nconfig :ex_llm,\n  log_level: :info,\n  log_components: %{\n    requests: false,  # Reduce noise in production\n    responses: false,\n    streaming: false,\n    retries: true,    # Keep retry logs\n    cache: false,\n    models: true\n  },\n  log_redaction: %{\n    api_keys: true,\n    content: true     # Redact content in production\n  }\n```","ref":"logger.html#environment-specific-configuration"},{"type":"extras","title":"Basic Logging - ExLLM Logger User Guide","doc":"The logger provides the same simple API as Elixir's Logger:\n\n```elixir\nalias ExLLM.Logger\n\n# Log levels\nLogger.debug(\"Detailed debugging info\")\nLogger.info(\"General information\")\nLogger.warn(\"Warning - something might be wrong\")\nLogger.error(\"Error occurred\")\n\n# With metadata\nLogger.info(\"Request completed\", \n  provider: :openai,\n  duration_ms: 250,\n  tokens: 1500\n)\n\n# Metadata is automatically included in log output\n# 10:23:45.123 [info] Request completed [provider=openai duration_ms=250 tokens=1500 ex_llm=true]\n```","ref":"logger.html#basic-logging"},{"type":"extras","title":"Context Tracking - ExLLM Logger User Guide","doc":"Add context that automatically appears in all logs within a scope:","ref":"logger.html#context-tracking"},{"type":"extras","title":"Temporary Context (within a function) - ExLLM Logger User Guide","doc":"```elixir\nLogger.with_context(provider: :anthropic, operation: :chat) do\n  Logger.info(\"Starting request\")\n  # ... do work ...\n  Logger.info(\"Request completed\", tokens: 150)\nend\n# Both logs automatically include provider=anthropic operation=chat\n```","ref":"logger.html#temporary-context-within-a-function"},{"type":"extras","title":"Persistent Context (for a process) - ExLLM Logger User Guide","doc":"```elixir\n# Set context for the rest of this process\nLogger.put_context(request_id: \"abc123\", user_id: \"user456\")\n\n# All subsequent logs include this context\nLogger.info(\"Processing\")  # Includes request_id and user_id\n\n# Clear context when done\nLogger.clear_context()\n```","ref":"logger.html#persistent-context-for-a-process"},{"type":"extras","title":"Real-World Example - ExLLM Logger User Guide","doc":"```elixir\ndefmodule MyApp.LLMService do\n  alias ExLLM.Logger\n  \n  def process_request(user_id, messages) do\n    # Set process context\n    Logger.put_context(user_id: user_id, request_id: generate_id())\n    \n    # Use scoped context for specific operations\n    Logger.with_context(provider: :openai, operation: :chat) do\n      Logger.info(\"Starting chat request\")\n      \n      case ExLLM.chat(:openai, messages) do\n        {:ok, response} ->\n          Logger.info(\"Chat completed\", \n            tokens: response.usage.total_tokens,\n            cost: response.cost.total_cost\n          )\n          {:ok, response}\n          \n        {:error, reason} ->\n          Logger.error(\"Chat failed\", error: reason)\n          {:error, reason}\n      end\n    end\n  end\nend\n```","ref":"logger.html#real-world-example"},{"type":"extras","title":"Structured Logging - ExLLM Logger User Guide","doc":"ExLLM provides specialized logging functions for common LLM operations:","ref":"logger.html#structured-logging"},{"type":"extras","title":"API Requests and Responses - ExLLM Logger User Guide","doc":"```elixir\n# Automatically redacts sensitive data\nLogger.log_request(:openai, url, body, headers)\n# Logs: API request [provider=openai url=\"...\" method=POST ...]\n\nLogger.log_response(:openai, response, duration_ms)\n# Logs: API response [provider=openai duration_ms=250 status=200 ...]\n```","ref":"logger.html#api-requests-and-responses"},{"type":"extras","title":"Streaming Events - ExLLM Logger User Guide","doc":"```elixir\nLogger.log_stream_event(:anthropic, :start, %{url: url})\nLogger.log_stream_event(:anthropic, :chunk_received, %{size: 1024})\nLogger.log_stream_event(:anthropic, :complete, %{total_chunks: 45})\nLogger.log_stream_event(:anthropic, :error, %{reason: \"timeout\"})\n```","ref":"logger.html#streaming-events"},{"type":"extras","title":"Retry Attempts - ExLLM Logger User Guide","doc":"```elixir\nLogger.log_retry(:openai, attempt, max_attempts, reason, delay_ms)\n# Logs: Retry attempt 2/3 [provider=openai reason=\"rate_limit\" delay_ms=2000 ...]\n```","ref":"logger.html#retry-attempts"},{"type":"extras","title":"Cache Operations - ExLLM Logger User Guide","doc":"```elixir\nLogger.log_cache_event(:hit, cache_key)\nLogger.log_cache_event(:miss, cache_key)\nLogger.log_cache_event(:put, cache_key, %{ttl: 300_000})\nLogger.log_cache_event(:evict, cache_key, %{reason: :expired})\n```","ref":"logger.html#cache-operations"},{"type":"extras","title":"Model Events - ExLLM Logger User Guide","doc":"```elixir\nLogger.log_model_event(:openai, :loading, %{model: \"gpt-4\"})\nLogger.log_model_event(:openai, :loaded, %{model: \"gpt-4\", size_mb: 125})\nLogger.log_model_event(:openai, :error, %{model: \"gpt-4\", error: \"not found\"})\n```","ref":"logger.html#model-events"},{"type":"extras","title":"Security & Redaction - ExLLM Logger User Guide","doc":"The logger automatically handles sensitive data based on your configuration:","ref":"logger.html#security-redaction"},{"type":"extras","title":"API Key Redaction (enabled by default) - ExLLM Logger User Guide","doc":"```elixir\n# Original\nheaders = [{\"Authorization\", \"Bearer sk-1234567890abcdef\"}]\n\n# Logged as\n# [headers=[{\"Authorization\", \"***\"}] ...]\n```","ref":"logger.html#api-key-redaction-enabled-by-default"},{"type":"extras","title":"Content Redaction (optional) - ExLLM Logger User Guide","doc":"```elixir\n# When content redaction is enabled\nbody = %{\"messages\" => [%{\"role\" => \"user\", \"content\" => \"sensitive data\"}]}\n\n# Logged as\n# [body=%{\"messages\" => \"[1 messages]\"} ...]\n```","ref":"logger.html#content-redaction-optional"},{"type":"extras","title":"URL Parameter Redaction - ExLLM Logger User Guide","doc":"```elixir\n# API keys in URLs are automatically redacted\nurl = \"https://api.example.com/v1/chat?api_key=secret123\"\n\n# Logged as\n# [url=\"https://api.example.com/v1/chat?api_key=***\" ...]\n```","ref":"logger.html#url-parameter-redaction"},{"type":"extras","title":"Best Practices - ExLLM Logger User Guide","doc":"","ref":"logger.html#best-practices"},{"type":"extras","title":"1. Use Context for Request Tracking - ExLLM Logger User Guide","doc":"```elixir\ndef handle_request(conn, params) do\n  # Set context early\n  Logger.put_context(\n    request_id: conn.assigns.request_id,\n    user_id: get_user_id(conn),\n    endpoint: conn.request_path\n  )\n  \n  # All logs in this request now have context\n  process_request(params)\nend\n```","ref":"logger.html#1-use-context-for-request-tracking"},{"type":"extras","title":"2. Use Appropriate Log Levels - ExLLM Logger User Guide","doc":"```elixir\n# Debug - Detailed information for debugging\nLogger.debug(\"Parsing response\", raw_response: response)\n\n# Info - General operational information\nLogger.info(\"Request completed\", provider: provider, duration_ms: 250)\n\n# Warn - Something unexpected but recoverable\nLogger.warn(\"Rate limit approaching\", remaining: 10, reset_at: timestamp)\n\n# Error - Something failed\nLogger.error(\"API request failed\", error: reason, provider: provider)\n```","ref":"logger.html#2-use-appropriate-log-levels"},{"type":"extras","title":"3. Structure Your Metadata - ExLLM Logger User Guide","doc":"```elixir\n# Good - structured metadata\nLogger.info(\"Operation completed\",\n  provider: :openai,\n  operation: :chat,\n  duration_ms: 250,\n  tokens: %{input: 100, output: 200, total: 300}\n)\n\n# Less useful - unstructured\nLogger.info(\"Operation completed for openai chat in 250ms with 300 tokens\")\n```","ref":"logger.html#3-structure-your-metadata"},{"type":"extras","title":"4. Configure for Your Environment - ExLLM Logger User Guide","doc":"Development:\n- Enable debug logging\n- Show all components\n- Don't redact content (easier debugging)\n\nProduction:\n- Info or warn level only\n- Disable noisy components (requests/responses)\n- Enable content redaction\n- Keep important events (retries, errors)","ref":"logger.html#4-configure-for-your-environment"},{"type":"extras","title":"5. Use Structured Logging Functions - ExLLM Logger User Guide","doc":"```elixir\n# Instead of\nLogger.info(\"Retrying request\", provider: :openai, attempt: 2, ...)\n\n# Use\nLogger.log_retry(:openai, 2, 3, \"rate_limit\", 1000)\n# Consistent structure and automatic metadata\n```","ref":"logger.html#5-use-structured-logging-functions"},{"type":"extras","title":"Filtering Logs - ExLLM Logger User Guide","doc":"You can filter ExLLM logs using the metadata tag:\n\n```elixir\n# In your Logger backend configuration\nconfig :logger, :console,\n  metadata_filter: [ex_llm: false]  # Exclude ExLLM logs\n\n# Or include only ExLLM logs\nconfig :logger, :console,\n  metadata_filter: [ex_llm: true]   # Only ExLLM logs\n```","ref":"logger.html#filtering-logs"},{"type":"extras","title":"Examples - ExLLM Logger User Guide","doc":"","ref":"logger.html#examples"},{"type":"extras","title":"Complete Request Lifecycle Logging - ExLLM Logger User Guide","doc":"```elixir\ndefmodule MyApp.AI do\n  alias ExLLM.Logger\n  \n  def generate_summary(text, user_id) do\n    # Set up context\n    Logger.with_context(\n      operation: :summary,\n      user_id: user_id,\n      request_id: UUID.uuid4()\n    ) do\n      Logger.info(\"Starting summary generation\")\n      \n      messages = [\n        %{role: \"system\", content: \"You are a helpful assistant.\"},\n        %{role: \"user\", content: \"Summarize: #{text}\"}\n      ]\n      \n      # The actual API call (logging handled internally)\n      case ExLLM.chat(:openai, messages) do\n        {:ok, response} ->\n          Logger.info(\"Summary generated\",\n            tokens: response.usage.total_tokens,\n            cost_usd: response.cost.total_cost\n          )\n          {:ok, response.content}\n          \n        {:error, reason} ->\n          Logger.error(\"Summary generation failed\", error: reason)\n          {:error, reason}\n      end\n    end\n  end\nend\n```","ref":"logger.html#complete-request-lifecycle-logging"},{"type":"extras","title":"Custom Adapter Logging - ExLLM Logger User Guide","doc":"```elixir\ndefmodule MyCustomAdapter do\n  alias ExLLM.Logger\n  \n  def make_request(messages, options) do\n    Logger.with_context(provider: :custom, operation: :chat) do\n      url = build_url(options)\n      body = build_body(messages, options)\n      headers = build_headers(options)\n      \n      # Log the request\n      Logger.log_request(:custom, url, body, headers)\n      \n      start_time = System.monotonic_time(:millisecond)\n      \n      case HTTPClient.post(url, body, headers) do\n        {:ok, response} ->\n          duration = System.monotonic_time(:millisecond) - start_time\n          Logger.log_response(:custom, response, duration)\n          parse_response(response)\n          \n        {:error, reason} ->\n          Logger.error(\"Request failed\", error: reason)\n          {:error, reason}\n      end\n    end\n  end\nend\n```","ref":"logger.html#custom-adapter-logging"},{"type":"extras","title":"Troubleshooting - ExLLM Logger User Guide","doc":"","ref":"logger.html#troubleshooting"},{"type":"extras","title":"Logs Not Appearing - ExLLM Logger User Guide","doc":"1. Check your log level configuration\n2. Verify the component is enabled in `log_components`\n3. Ensure your Logger backend is configured to show the appropriate level","ref":"logger.html#logs-not-appearing"},{"type":"extras","title":"Too Many Logs - ExLLM Logger User Guide","doc":"1. Disable noisy components (streaming, requests/responses)\n2. Increase log level to :warn or :error\n3. Use metadata filters to exclude ExLLM logs","ref":"logger.html#too-many-logs"},{"type":"extras","title":"Missing Context - ExLLM Logger User Guide","doc":"1. Ensure you're using `with_context` or `put_context`\n2. Check that context is set before logging\n3. Remember that context is process-specific","ref":"logger.html#missing-context"},{"type":"extras","title":"Summary - ExLLM Logger User Guide","doc":"The ExLLM Logger provides:\n- **Simple API** - Works like Elixir's Logger\n- **Automatic Context** - Track requests across operations\n- **Security** - Built-in redaction for sensitive data\n- **Structure** - Consistent logging for LLM operations\n- **Control** - Fine-grained configuration options\n\nUse it everywhere in your ExLLM applications for better debugging and monitoring!","ref":"logger.html#summary"},{"type":"extras","title":"Provider Capabilities Guide","doc":"# Provider Capabilities Guide\n\nThis document explains how to find and update provider capabilities in ExLLM.","ref":"provider_capabilities.html"},{"type":"extras","title":"Current Provider Capabilities - Provider Capabilities Guide","doc":"Provider capabilities are defined in `lib/ex_llm/provider_capabilities.ex`. Each provider has:\n\n- **endpoints**: API endpoints available (e.g., `:chat`, `:embeddings`, `:images`)\n- **features**: Capabilities supported (e.g., `:streaming`, `:vision`, `:function_calling`)\n- **authentication**: Auth methods supported (e.g., `:api_key`, `:oauth2`)\n- **limitations**: Known limitations and constraints","ref":"provider_capabilities.html#current-provider-capabilities"},{"type":"extras","title":"How to Find Provider Capabilities - Provider Capabilities Guide","doc":"","ref":"provider_capabilities.html#how-to-find-provider-capabilities"},{"type":"extras","title":"1. Official Documentation - Provider Capabilities Guide","doc":"Always check the official documentation for the most accurate information:\n\n- **OpenAI**: https://platform.openai.com/docs/api-reference\n  - Models: https://platform.openai.com/docs/models\n  - Capabilities: Check individual endpoint docs\n  \n- **Anthropic**: https://docs.anthropic.com/\n  - Models: https://docs.anthropic.com/en/docs/models\n  - Features: https://docs.anthropic.com/en/docs/capabilities\n  \n- **Google Gemini**: https://ai.google.dev/docs\n  - Models: https://ai.google.dev/models/gemini\n  - API capabilities: https://ai.google.dev/api/rest\n  \n- **AWS Bedrock**: https://docs.aws.amazon.com/bedrock/\n  - Model providers: https://docs.aws.amazon.com/bedrock/latest/userguide/models-supported.html\n  \n- **Groq**: https://console.groq.com/docs\n  - Models: https://console.groq.com/docs/models\n  \n- **X.AI**: https://docs.x.ai/\n  - API reference: https://docs.x.ai/api","ref":"provider_capabilities.html#1-official-documentation"},{"type":"extras","title":"2. API Discovery Endpoints - Provider Capabilities Guide","doc":"Some providers offer API endpoints to discover capabilities:\n\n```elixir\n# OpenAI - List available models\n{:ok, models} = ExLLM.list_models(:openai)\n\n# Check if a model supports specific features\n{:ok, model_info} = ExLLM.ModelCapabilities.get_capabilities(:openai, \"gpt-4o\")\n```","ref":"provider_capabilities.html#2-api-discovery-endpoints"},{"type":"extras","title":"3. Feature Mapping - Provider Capabilities Guide","doc":"Here's how common features map across providers:\n\n| Feature | OpenAI | Anthropic | Gemini | Bedrock |\n|---------|---------|-----------|---------|----------|\n| Chat | ✓ `/v1/chat/completions` | ✓ `/v1/messages` | ✓ `/v1beta/models/.../generateContent` | ✓ `/invoke` |\n| Embeddings | ✓ `/v1/embeddings` | ✗ | ✓ `/v1beta/models/.../embedContent` | ✓ (varies) |\n| Images | ✓ `/v1/images/generations` | ✗ | ✗ | ✗ (use Stable Diffusion) |\n| Vision | ✓ (GPT-4V) | ✓ (Claude 3) | ✓ (Gemini Pro Vision) | ✓ (varies) |\n| Function Calling | ✓ | ✓ (Tools) | ✓ | ✓ (varies) |\n| Streaming | ✓ | ✓ | ✓ | ✓ |\n| JSON Mode | ✓ | ✓ | ✓ | ✓ (varies) |","ref":"provider_capabilities.html#3-feature-mapping"},{"type":"extras","title":"Updating Provider Capabilities - Provider Capabilities Guide","doc":"When updating capabilities in `provider_capabilities.ex`:\n\n1. **Check Official Docs**: Always verify against the latest documentation\n2. **Test Features**: Actually test that features work as expected\n3. **Update Both Places**: Update both `endpoints` and `features` arrays\n4. **Document Limitations**: Add any constraints to the `limitations` map","ref":"provider_capabilities.html#updating-provider-capabilities"},{"type":"extras","title":"Example Update - Provider Capabilities Guide","doc":"```elixir\n# In provider_capabilities.ex\nopenai: %__MODULE__.ProviderInfo{\n  # ... other fields ...\n  endpoints: [\n    :chat,           # /v1/chat/completions\n    :embeddings,     # /v1/embeddings\n    :images,         # /v1/images/generations\n    :audio,          # /v1/audio/transcriptions, /v1/audio/translations, /v1/audio/speech\n    :moderations,    # /v1/moderations\n    :files,          # /v1/files\n    :fine_tuning,    # /v1/fine-tuning/jobs\n    :assistants,     # /v1/assistants\n    :threads,        # /v1/threads\n    :runs,           # /v1/threads/{thread_id}/runs\n    :vector_stores   # /v1/vector_stores\n  ],\n  features: [\n    :streaming,\n    :function_calling,\n    :vision,\n    :image_generation,    # DALL-E\n    :speech_synthesis,    # TTS\n    :speech_recognition,  # Whisper\n    :embeddings,\n    :fine_tuning_api,\n    :assistants_api,\n    :code_interpreter,\n    :retrieval,\n    :structured_outputs,\n    :json_mode,\n    :logprobs,\n    :seed_control,\n    # ... etc\n  ]\n}\n```","ref":"provider_capabilities.html#example-update"},{"type":"extras","title":"Testing Provider Capabilities - Provider Capabilities Guide","doc":"To verify capabilities are correctly configured:\n\n```elixir\n# Check if a provider supports a feature\nExLLM.ProviderCapabilities.supports?(:openai, :image_generation)\n# => true\n\n# Find all providers that support embeddings\nExLLM.ProviderCapabilities.find_providers_with_features([:embeddings])\n# => [:openai, :gemini, :bedrock, :ollama]\n\n# Get full capability info\n{:ok, caps} = ExLLM.ProviderCapabilities.get_capabilities(:openai)\nIO.inspect(caps.features)\n```","ref":"provider_capabilities.html#testing-provider-capabilities"},{"type":"extras","title":"Common Gotchas - Provider Capabilities Guide","doc":"1. **Feature vs Endpoint**: Some capabilities are features (`:vision`), others are endpoints (`:images`)\n2. **Provider-Specific Names**: Each provider may call the same feature differently\n3. **Beta Features**: Mark beta/experimental features in limitations\n4. **Model-Specific**: Some features depend on the specific model (e.g., vision only in GPT-4V)\n5. **Region/Tier Limits**: Some features have geographic or pricing tier restrictions","ref":"provider_capabilities.html#common-gotchas"},{"type":"extras","title":"Contributing - Provider Capabilities Guide","doc":"When adding a new provider or updating capabilities:\n\n1. Research the provider's full API surface\n2. Test each capability you add\n3. Document any special requirements\n4. Update this guide if you discover new patterns","ref":"provider_capabilities.html#contributing"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}